@@ -2,13 +2,14 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2013 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2010      IBM Corporation.  All rights reserved.
  * Copyright (c) 2010      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
@@ -177,58 +178,40 @@ OMPI_DECLSPEC int ompi_free_list_grow(ompi_free_list_t* flist, size_t num_elemen
    will not shrink the list if it is already larger than size and may
    grow it past size if necessary (it will grow in
    num_elements_per_alloc chunks) */
-OMPI_DECLSPEC int ompi_free_list_resize(ompi_free_list_t *flist, size_t size);
+OMPI_DECLSPEC int ompi_free_list_resize_mt(ompi_free_list_t *flist, size_t size);
 
-/* Allow to walk through the all allocated items. Not thread-safe, not
- * protected, this function should never be used except for debugging purposes.
- * The position should never be NULL, and it should be set to {NULL, NULL} 
- * for the first call.
- */
-typedef struct ompi_free_list_pos_t {
-    unsigned char* last_memory;
-    unsigned char* last_item;
-} ompi_free_list_pos_t;
-#define OMPI_FREE_LIST_POS_BEGINNING {NULL, NULL}
-
-OMPI_DECLSPEC int ompi_free_list_parse( ompi_free_list_t* list,
-                                        ompi_free_list_pos_t* position,
-                                        opal_list_item_t** return_item );
 /**
  * Attemp to obtain an item from a free list. 
  *
  * @param fl (IN)        Free list.
  * @param item (OUT)     Allocated item.
- * @param rc (OUT)       OMPI_SUCCESS or error status on failure.
  *
  * If the requested item is not available the free list is grown to 
  * accomodate the request - unless the max number of allocations has 
- * been reached.  If this is the case - an out of resource error is 
- * returned to the caller.
+ * been reached.  If this is the case - a NULL pointer is returned 
+ * to the caller.
  */
- 
-#define OMPI_FREE_LIST_GET(fl, item, rc) \
-{ \
-    rc = OMPI_SUCCESS; \
-    item = (ompi_free_list_item_t*) opal_atomic_lifo_pop(&((fl)->super)); \
-    if( OPAL_UNLIKELY(NULL == item) ) { \
-        if(opal_using_threads()) { \
-            opal_mutex_lock(&((fl)->fl_lock)); \
-            ompi_free_list_grow((fl), (fl)->fl_num_per_alloc); \
-            opal_mutex_unlock(&((fl)->fl_lock)); \
-        } else { \
-            ompi_free_list_grow((fl), (fl)->fl_num_per_alloc); \
-        } \
+
+#define OMPI_FREE_LIST_GET_MT(fl, item)                                 \
+    {                                                                   \
         item = (ompi_free_list_item_t*) opal_atomic_lifo_pop(&((fl)->super)); \
-        if( OPAL_UNLIKELY(NULL == item) ) rc = OMPI_ERR_TEMP_OUT_OF_RESOURCE; \
-    }  \
-} 
+        if( OPAL_UNLIKELY(NULL == item) ) {                             \
+            if(opal_using_threads()) {                                  \
+                opal_mutex_lock(&((fl)->fl_lock));                      \
+                ompi_free_list_grow((fl), (fl)->fl_num_per_alloc);      \
+                opal_mutex_unlock(&((fl)->fl_lock));                    \
+            } else {                                                    \
+                ompi_free_list_grow((fl), (fl)->fl_num_per_alloc);      \
+            }                                                           \
+            item = (ompi_free_list_item_t*) opal_atomic_lifo_pop(&((fl)->super)); \
+        }                                                               \
+    } 
 
 /**
  * Blocking call to obtain an item from a free list.
  *
  * @param fl (IN)        Free list.
  * @param item (OUT)     Allocated item.
- * @param rc (OUT)       OMPI_SUCCESS or error status on failure.
  *
  * If the requested item is not available the free list is grown to 
  * accomodate the request - unless the max number of allocations has 
@@ -236,11 +219,11 @@ OMPI_DECLSPEC int ompi_free_list_parse( ompi_free_list_t* list,
  * is returned to the list.
  */
 
-#define OMPI_FREE_LIST_WAIT(fl, item, rc)                                  \
-    rc = __ompi_free_list_wait( (fl), &(item) )
+#define OMPI_FREE_LIST_WAIT_MT(fl, item)           \
+    __ompi_free_list_wait_mt( (fl), &(item) )
 
-static inline int __ompi_free_list_wait( ompi_free_list_t* fl,
-                                         ompi_free_list_item_t** item )
+static inline void __ompi_free_list_wait_mt( ompi_free_list_t* fl,
+                                             ompi_free_list_item_t** item )
 {
     *item = (ompi_free_list_item_t*)opal_atomic_lifo_pop(&((fl)->super));
     while( NULL == *item ) {
@@ -275,7 +258,6 @@ static inline int __ompi_free_list_wait( ompi_free_list_t* fl,
         OPAL_THREAD_UNLOCK(&((fl)->fl_lock));
         *item = (ompi_free_list_item_t*)opal_atomic_lifo_pop(&((fl)->super));
     }
-    return OMPI_SUCCESS;
 } 
 
 /**
@@ -286,7 +268,7 @@ static inline int __ompi_free_list_wait( ompi_free_list_t* fl,
  *
  */
  
-#define OMPI_FREE_LIST_RETURN(fl, item)                                 \
+#define OMPI_FREE_LIST_RETURN_MT(fl, item)                              \
     do {                                                                \
         opal_list_item_t* original;                                     \
                                                                         \
