@@ -14,7 +14,7 @@
 string OMPragmaF::find_next_word() {
   while ( pline < lines.size() ) {
     string::size_type wbeg = lines[pline].find_first_not_of(" \t", ppos);
-    if ( lines[pline][wbeg] == '&' || wbeg == string::npos ) {
+    if ( wbeg == string::npos || lines[pline][wbeg] == '&' ) {
       ++pline;
       if ( pline < lines.size() ) {
 	ppos = lines[pline].find(sentinel) + slen;
@@ -89,7 +89,7 @@ namespace {
         if ( line >= outer->lines.size() ) return;
         pos = outer->lines[line].find(sentinel) + slen;
         pos = outer->lines[line].find_first_not_of(" \t", pos);
-        if ( outer->lines[line][pos] == '&' ) ++pos;
+        if ( pos != string::npos && outer->lines[line][pos] == '&' ) ++pos;
         optr = &(outer->lines[line][pos]);
         iptr = &(inner->lines[line][pos]);
       }
@@ -103,15 +103,10 @@ namespace {
 }
 
 void OMPragmaF::remove_empties() {
-  // remove comma at end of line and
-  // lines without content
+  // remove lines without content
   vector<string>::iterator it=lines.begin();
   while ( it!=lines.end() ) {
     string::size_type s = it->find(sentinel) + slen;
-    string::size_type c = it->find('!', s);
-    if ( c != string::npos ) --c;
-    string::size_type amp = it->find_last_not_of(" \t&", c);
-    if ( (*it)[amp] == ',' ) (*it)[amp] = ' ';
     string::size_type l = it->find_first_not_of(" \t&", s);
     if ( l == string::npos || (*it)[l] == '!' )
       it = lines.erase(it);
@@ -125,7 +120,7 @@ void OMPragmaF::remove_empties() {
     lines[0][slen] = ' ';
   } else {
     string::size_type l = lines[0].find_first_not_of(" \t", s+slen);
-    if ( lines[0][l] == '&' ) lines[0][l] = ' ';
+    if ( l != string::npos && lines[0][l] == '&' ) lines[0][l] = ' ';
   }
 
   // make sure last line is not a continuated line
@@ -134,7 +129,13 @@ void OMPragmaF::remove_empties() {
   string::size_type c = lines[lastline].find('!', s);
   if ( c != string::npos ) --c;
   string::size_type amp = lines[lastline].find_last_not_of(" \t", c);
-  if ( lines[lastline][amp] == '&' ) lines[lastline][amp] = ' ';
+  if ( amp != string::npos && lines[lastline][amp] == '&' )
+    lines[lastline][amp] = ' ';
+
+  // remove trailing comma
+  amp = lines[lastline].find_last_not_of(" \t", c);
+  if ( amp != string::npos && lines[lastline][amp] == ',' )
+    lines[lastline][amp] = ' ';
 }
 
 OMPragma* OMPragmaF::split_combined() {
@@ -154,9 +155,11 @@ OMPragma* OMPragmaF::split_combined() {
     string::size_type com = lines[i].find("!", s+slen);
     if ( com != string::npos ) --com;
     string::size_type amp2 = lines[i].find_last_not_of(" \t", com);
-    if ( lines[i][amp2] == '&' ) inner->lines[i][amp2] = '&';
+    if ( amp2 != string::npos && lines[i][amp2] == '&' )
+      inner->lines[i][amp2] = '&';
     string::size_type amp1 = lines[i].find_first_not_of(" \t", s+slen);
-    if ( lines[i][amp1] == '&' ) inner->lines[i][amp1] = '&';
+    if ( amp1 != string::npos && lines[i][amp1] == '&' )
+      inner->lines[i][amp1] = '&';
   }
 
   // fix pragma name
