@@ -2,7 +2,7 @@
  * VampirTrace
  * http://www.tu-dresden.de/zih/vampirtrace
  *
- * Copyright (c) 2005-2012, ZIH, TU Dresden, Federal Republic of Germany
+ * Copyright (c) 2005-2013, ZIH, TU Dresden, Federal Republic of Germany
  *
  * Copyright (c) 1998-2005, Forschungszentrum Juelich, Juelich Supercomputing
  *                          Centre, Federal Republic of Germany
@@ -13,6 +13,8 @@
 #ifndef _VT_UNIFY_DEFS_RECS_H_
 #define _VT_UNIFY_DEFS_RECS_H_
 
+#include "vt_unify_config.h"
+
 #include "vt_defs.h"
 #include "vt_inttypes.h"
 
@@ -24,8 +26,6 @@
 #include <set>
 #include <string>
 
-#include <assert.h>
-
 //
 // definition record types
 //
@@ -337,7 +337,7 @@ struct DefRec_DefProcessGroupS : DefRec_BaseS
       if( nmembers > 0 )
       {
          members = new uint32_t[nmembers];
-         assert( members );
+         vt_assert( members );
          std::copy( first, last, members );
       }
    }
