@@ -2,7 +2,7 @@
  * VampirTrace
  * http://www.tu-dresden.de/zih/vampirtrace
  *
- * Copyright (c) 2005-2012, ZIH, TU Dresden, Federal Republic of Germany
+ * Copyright (c) 2005-2013, ZIH, TU Dresden, Federal Republic of Germany
  *
  * Copyright (c) 1998-2005, Forschungszentrum Juelich, Juelich Supercomputing
  *                          Centre, Federal Republic of Germany
@@ -33,7 +33,8 @@
 #define DEFAULT_FILTER_FILE "${sysconfdir}/vt-java-default-filter.spec"
 #define MAX_FILTER_LINE_LEN 131072
 #define MAX_METHOD_NAME_LEN 300
-#define MAX_METHOD_ID_HKEY  10069
+ /* size of the method ID hash table (must be a power of two!) */
+#define MAX_METHOD_ID_HKEY  10024
 
 /* Enum for filter class types */
 typedef enum
@@ -303,7 +304,7 @@ static uint8_t check_filter(FilterClassT fclass, const char* name)
 
 static MethodIDMapT* get_methodid_map(jmethodID mid)
 {
-  size_t        hkey = (size_t)mid % MAX_METHOD_ID_HKEY;
+  size_t        hkey = (size_t)mid & (MAX_METHOD_ID_HKEY - 1);
   MethodIDMapT* curr = methodid_map[hkey];
 
   while ( curr )
@@ -318,7 +319,7 @@ static MethodIDMapT* get_methodid_map(jmethodID mid)
 
 static MethodIDMapT* add_methodid_map(jmethodID mid, uint32_t rid)
 {
-  size_t        hkey = (size_t)mid % MAX_METHOD_ID_HKEY;
+  size_t        hkey = (size_t)mid & (MAX_METHOD_ID_HKEY - 1);
   MethodIDMapT* add = (MethodIDMapT*)malloc(sizeof(MethodIDMapT));
   if ( add == NULL ) vt_error();
 
@@ -348,25 +349,25 @@ static void get_method_info(jvmtiEnv* jvmti, jmethodID method,
 
   /* check whether method is native */
   error = (*jvmti)->IsMethodNative(jvmti, method, &is_native);
-  vt_java_check_error(jvmti, error, "IsMethodNative");
+  VT_JAVA_CHECK_ERROR(jvmti, error, "IsMethodNative");
   methodInfo->is_native = (is_native) ? 1 : 0;
 
   /* check whether method is synthetic */
   error = (*jvmti)->IsMethodSynthetic(jvmti, method, &is_synthetic);
-  vt_java_check_error(jvmti, error, "IsMethodSynthetic");
+  VT_JAVA_CHECK_ERROR(jvmti, error, "IsMethodSynthetic");
   methodInfo->is_synthetic = (is_synthetic) ? 1 : 0;
 
   /* get method name */
   error = (*jvmti)->GetMethodName(jvmti, method, &method_name, NULL, NULL);
-  vt_java_check_error(jvmti, error, "GetMethodName");
+  VT_JAVA_CHECK_ERROR(jvmti, error, "GetMethodName");
 
   /* get class object that declaring method */
   error = (*jvmti)->GetMethodDeclaringClass(jvmti, method, &class);
-  vt_java_check_error(jvmti, error, "GetMethodDeclaringClass");
+  VT_JAVA_CHECK_ERROR(jvmti, error, "GetMethodDeclaringClass");
 
   /* get class signature */
   error = (*jvmti)->GetClassSignature(jvmti, class, &class_signature, NULL);
-  vt_java_check_error(jvmti, error, "GetClassSignature");
+  VT_JAVA_CHECK_ERROR(jvmti, error, "GetClassSignature");
 
   /* "demangle" class signature, if available */
 
@@ -443,7 +444,8 @@ static void get_method_info(jvmtiEnv* jvmti, jmethodID method,
     (*jvmti)->Deallocate(jvmti, (void*)lines);
 }
 
-static MethodIDMapT* register_method(jvmtiEnv* jvmti, jmethodID method)
+static MethodIDMapT* register_method(uint32_t threadid, jvmtiEnv* jvmti,
+                                     jmethodID method)
 {
   MethodIDMapT*    methodid_map;
   MethodInfoT      method_info;
@@ -464,12 +466,12 @@ static MethodIDMapT* register_method(jvmtiEnv* jvmti, jmethodID method)
 
     if ( method_info.source_filename != NULL && method_info.line_number != 0 )
     {
-      fid = vt_def_scl_file(VT_CURRENT_THREAD, method_info.source_filename);
+      fid = vt_def_scl_file(threadid, method_info.source_filename);
       lno = method_info.line_number;
     }
 
     /* register method and store region identifier */
-    rid = vt_def_region(VT_CURRENT_THREAD,
+    rid = vt_def_region(threadid,
                         method_info.long_name,
                         fid, lno, VT_NO_LNO,
                         (vt_env_java_group_classes()) ? method_info.class_name : NULL,
@@ -492,7 +494,7 @@ static void lock_agent(jvmtiEnv* jvmti)
   jvmtiError error;
 
   error = (*jvmti)->RawMonitorEnter(jvmti, vt_jvmti_agent->lock);
-  vt_java_check_error(jvmti, error, "RawMonitorEnter");
+  VT_JAVA_CHECK_ERROR(jvmti, error, "RawMonitorEnter");
 }
 
 static void unlock_agent(jvmtiEnv *jvmti)
@@ -500,7 +502,7 @@ static void unlock_agent(jvmtiEnv *jvmti)
   jvmtiError error;
 
   error = (*jvmti)->RawMonitorExit(jvmti, vt_jvmti_agent->lock);
-  vt_java_check_error(jvmti, error, "RawMonitorExit");
+  VT_JAVA_CHECK_ERROR(jvmti, error, "RawMonitorExit");
 }
 
 /* Callback for JVMTI_EVENT_VM_START */
@@ -534,22 +536,22 @@ static void JNICALL cbVMInit(jvmtiEnv* jvmti, JNIEnv* env, jthread thread)
     error =
       (*jvmti)->SetEventNotificationMode(jvmti, JVMTI_ENABLE,
                                          JVMTI_EVENT_THREAD_START, NULL);
-    vt_java_check_error(jvmti, error,
+    VT_JAVA_CHECK_ERROR(jvmti, error,
                         "SetEventNotificationMode[JVMTI_EVENT_THREAD_START]");
     error =
       (*jvmti)->SetEventNotificationMode(jvmti, JVMTI_ENABLE,
                                          JVMTI_EVENT_THREAD_END, NULL);
-    vt_java_check_error(jvmti, error,
+    VT_JAVA_CHECK_ERROR(jvmti, error,
                         "SetEventNotificationMode[JVMTI_EVENT_THREAD_END]");
     error =
       (*jvmti)->SetEventNotificationMode(jvmti, JVMTI_ENABLE,
                                          JVMTI_EVENT_METHOD_ENTRY, NULL);
-    vt_java_check_error(jvmti, error,
+    VT_JAVA_CHECK_ERROR(jvmti, error,
                         "SetEventNotificationMode[JVMTI_EVENT_METHOD_ENTRY]");
     error =
       (*jvmti)->SetEventNotificationMode(jvmti, JVMTI_ENABLE,
                                          JVMTI_EVENT_METHOD_EXIT, NULL);
-    vt_java_check_error(jvmti, error,
+    VT_JAVA_CHECK_ERROR(jvmti, error,
                         "SetEventNotificationMode[JVMTI_EVENT_METHOD_EXIT]");
 
     /* read filter specifications for threads and methods */
@@ -675,7 +677,7 @@ static void JNICALL cbMethodEntry(jvmtiEnv* jvmti, JNIEnv* env,
 
   /* get thread identifier */
   error = (*jvmti)->GetThreadLocalStorage(jvmti, thread, (void**)&threadid);
-  vt_java_check_error(jvmti, error, "GetThreadLocalStorage");
+  VT_JAVA_CHECK_ERROR(jvmti, error, "GetThreadLocalStorage");
 
   /* return immediately, if thread is excluded (no thread ID assigned) */
   if ( threadid == NULL ) return;
@@ -687,13 +689,13 @@ static void JNICALL cbMethodEntry(jvmtiEnv* jvmti, JNIEnv* env,
 
     VTTHRD_LOCK_IDS();
     if ( (methodid_map = get_methodid_map(method)) == NULL )
-      methodid_map = register_method(jvmti, method);
+      methodid_map = register_method(*threadid, jvmti, method);
     VTTHRD_UNLOCK_IDS();
   }
 
   /* write enter record, if method isn't excluded */
   if ( methodid_map->rid != VT_NO_ID )
-    vt_enter(VT_CURRENT_THREAD, &time, methodid_map->rid);
+    vt_enter(*threadid, &time, methodid_map->rid);
 }
 
 /* Callback for JVMTI_EVENT_METHOD_EXIT */
@@ -713,7 +715,7 @@ static void JNICALL cbMethodExit(jvmtiEnv* jvmti, JNIEnv* env,
 
   /* get thread identifier */
   error = (*jvmti)->GetThreadLocalStorage(jvmti, thread, (void**)&threadid);
-  vt_java_check_error(jvmti, error, "GetThreadLocalStorage");
+  VT_JAVA_CHECK_ERROR(jvmti, error, "GetThreadLocalStorage");
 
   /* return immediately, if thread is excluded (no thread ID assigned) */
   if ( threadid == NULL ) return;
@@ -723,7 +725,7 @@ static void JNICALL cbMethodExit(jvmtiEnv* jvmti, JNIEnv* env,
 
   /* write exit record, if method was entered and isn't excluded */
   if ( methodid_map != NULL && methodid_map->rid != VT_NO_ID )
-    vt_exit(VT_CURRENT_THREAD, &time);
+    vt_exit(*threadid, &time);
 }
 
 /* Load the VM Agent */
@@ -737,9 +739,6 @@ JNIEXPORT jint JNICALL Agent_OnLoad(JavaVM *vm, char *options, void *reserved)
   jvmtiCapabilities   capabilities;
   jvmtiEventCallbacks callbacks;
 
-  /* not used */
-  (void)options; (void)reserved;
-
   memset((void*)&agent, 0, sizeof(agent));
   vt_jvmti_agent = &agent;
 
@@ -756,7 +755,7 @@ JNIEXPORT jint JNICALL Agent_OnLoad(JavaVM *vm, char *options, void *reserved)
 
   /* get JVMTI's version number */
   error = (*jvmti)->GetVersionNumber(jvmti, &version);
-  vt_java_check_error(jvmti, error, "GetVersionNumber");
+  VT_JAVA_CHECK_ERROR(jvmti, error, "GetVersionNumber");
   agent.jvmti_version = version;
 
   /* set capabilities */
@@ -769,7 +768,7 @@ JNIEXPORT jint JNICALL Agent_OnLoad(JavaVM *vm, char *options, void *reserved)
   capabilities.can_get_synthetic_attribute      = 1;
 
   error = (*jvmti)->AddCapabilities(jvmti, &capabilities);
-  vt_java_check_error(jvmti, error, "AddCapabilities");
+  VT_JAVA_CHECK_ERROR(jvmti, error, "AddCapabilities");
 
   /* set event callbacks */
 
@@ -784,26 +783,26 @@ JNIEXPORT jint JNICALL Agent_OnLoad(JavaVM *vm, char *options, void *reserved)
 
   error = (*jvmti)->SetEventCallbacks(jvmti, &callbacks,
           (jint)sizeof(callbacks));
-  vt_java_check_error(jvmti, error, "SetEventCallbacks");
+  VT_JAVA_CHECK_ERROR(jvmti, error, "SetEventCallbacks");
 
   /* set event notification modes */
 
   error = (*jvmti)->SetEventNotificationMode(jvmti, JVMTI_ENABLE,
                                              JVMTI_EVENT_VM_START, NULL);
-  vt_java_check_error(jvmti, error,
+  VT_JAVA_CHECK_ERROR(jvmti, error,
                       "SetEventNotificationMode[JVMTI_EVENT_VM_START]");
   error = (*jvmti)->SetEventNotificationMode(jvmti, JVMTI_ENABLE,
                                              JVMTI_EVENT_VM_INIT, NULL);
-  vt_java_check_error(jvmti, error,
+  VT_JAVA_CHECK_ERROR(jvmti, error,
                       "SetEventNotificationMode[JVMTI_EVENT_VM_INIT]");
   error = (*jvmti)->SetEventNotificationMode(jvmti, JVMTI_ENABLE,
                                              JVMTI_EVENT_VM_DEATH, NULL);
-  vt_java_check_error(jvmti, error,
+  VT_JAVA_CHECK_ERROR(jvmti, error,
                       "SetEventNotificationMode[JVMTI_EVENT_VM_DEATH]");
 
   /* create raw monitor for this agent to protect critical sections of code */
   error = (*jvmti)->CreateRawMonitor(jvmti, "agent", &(vt_jvmti_agent->lock));
-  vt_java_check_error(jvmti, error, "CreateRawMonitor[agent]");
+  VT_JAVA_CHECK_ERROR(jvmti, error, "CreateRawMonitor[agent]");
 
   vt_cntl_msg(2, "JVMTI: VM agent loaded");
 
@@ -836,7 +835,7 @@ void vt_java_get_thread_name(jvmtiEnv* jvmti, jthread thread,
   /* get thread information */
   memset(&thread_info,0, sizeof(thread_info));
   error = (*jvmti)->GetThreadInfo(jvmti, thread, &thread_info);
-  vt_java_check_error(jvmti, error, "GetThreadInfo");
+  VT_JAVA_CHECK_ERROR(jvmti, error, "GetThreadInfo");
 
   /* copy thread name into tname, if possible */
   if ( thread_info.name != NULL ) {
@@ -848,19 +847,20 @@ void vt_java_get_thread_name(jvmtiEnv* jvmti, jthread thread,
   }
 }
 
-void vt_java_check_error(jvmtiEnv* jvmti, jvmtiError error, const char* str)
+void vt_java_error(jvmtiEnv* jvmti, jvmtiError error, const char* prefix)
 {
-  if ( jvmti == NULL ) jvmti = vt_jvmti_agent->jvmti;
-  vt_libassert(jvmti != NULL);
+  char* error_str = NULL;
 
-  if ( error != JVMTI_ERROR_NONE )
-  {
-    char* error_str = NULL;
-    (*jvmti)->GetErrorName(jvmti, error, &error_str);
-    vt_error_msg("JVMTI: %s%s%d(%s)",
-                 (str == NULL ? "" : str),
-                 (str == NULL ? " " : ": "),
-                 error,
-                 (error_str == NULL ? "Unknown" : error_str));
-  }
+  if ( jvmti == NULL )
+    jvmti = vt_jvmti_agent->jvmti;
+
+  vt_libassert(jvmti != NULL);
+  vt_libassert(error != JVMTI_ERROR_NONE);
+
+  (*jvmti)->GetErrorName(jvmti, error, &error_str);
+  vt_error_msg("JVMTI: %s%s%d(%s)",
+               (prefix == NULL ? "" : prefix),
+               (prefix == NULL ? " " : ": "),
+               error,
+               (error_str == NULL ? "Unknown" : error_str));
 }
