@@ -2,7 +2,7 @@
  * VampirTrace
  * http://www.tu-dresden.de/zih/vampirtrace
  *
- * Copyright (c) 2005-2010, ZIH, TU Dresden, Federal Republic of Germany
+ * Copyright (c) 2005-2013, ZIH, TU Dresden, Federal Republic of Germany
  *
  * Copyright (c) 1998-2005, Forschungszentrum Juelich, Juelich Supercomputing
  *                          Centre, Federal Republic of Germany
@@ -90,11 +90,11 @@ char* vt_pform_exec() {
 /* clock resolution */
 uint64_t vt_pform_clockres() {
 #if TIMER == TIMER_GETTIMEOFDAY
-  return 1e6;
+  return 1000000LL;
 #elif TIMER == TIMER_PAPI_REAL_CYC
   return vt_metric_clckrt();
 #elif TIMER == TIMER_PAPI_REAL_USEC
-  return 1e6;
+  return 1000000LL;
 #endif
 }
 
@@ -103,7 +103,7 @@ uint64_t vt_pform_wtime() {
 #if TIMER == TIMER_GETTIMEOFDAY
   struct timeval tp;
   gettimeofday(&tp, 0);
-  return ((tp.tv_sec - vt_time_base) * 1e6) + tp.tv_usec;
+  return ((tp.tv_sec - vt_time_base) * 1000000LL) + tp.tv_usec;
 #elif TIMER == TIMER_PAPI_REAL_CYC
   return vt_metric_real_cyc();
 #elif TIMER == TIMER_PAPI_REAL_USEC
