@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2013 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -11,6 +11,7 @@
  *                         All rights reserved.
  * Copyright (c) 2006      Sun Microsystems, Inc.  All rights reserved.
  * Copyright (c) 2008      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2011      NVIDIA Corporation.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -121,7 +122,7 @@ static void mca_allocator_basic_combine_prev(
         if(prev->seg_addr + prev->seg_size == seg->seg_addr) {
             prev->seg_size += seg->seg_size;
             opal_list_remove_item(&module->seg_list, &seg->seg_item.super);
-            OMPI_FREE_LIST_RETURN(&module->seg_descriptors, &seg->seg_item);
+            OMPI_FREE_LIST_RETURN_MT(&module->seg_descriptors, &seg->seg_item);
             return;
         }
     }
@@ -138,7 +139,7 @@ static void mca_allocator_basic_combine_next(
             next->seg_addr = seg->seg_addr;
             next->seg_size += seg->seg_size;
             opal_list_remove_item(&module->seg_list, &seg->seg_item.super);
-            OMPI_FREE_LIST_RETURN(&module->seg_descriptors, &seg->seg_item);
+            OMPI_FREE_LIST_RETURN_MT(&module->seg_descriptors, &seg->seg_item);
             return;
         }
     }
@@ -190,7 +191,7 @@ void *mca_allocator_basic_alloc(
         } else if (seg->seg_size == size) {
             addr = seg->seg_addr;
             opal_list_remove_item(&module->seg_list, &item->super);
-            OMPI_FREE_LIST_RETURN(&module->seg_descriptors, item);
+            OMPI_FREE_LIST_RETURN_MT(&module->seg_descriptors, item);
             OPAL_THREAD_UNLOCK(&module->seg_lock);
             *(size_t*)addr = size;
             return addr+sizeof(size_t);
@@ -198,7 +199,7 @@ void *mca_allocator_basic_alloc(
     }
 
     /* request additional block */
-    allocated_size = (unsigned char)size;
+    allocated_size = size;
     if(NULL == (addr = (unsigned char *)module->seg_alloc(module->super.alc_mpool, &allocated_size, registration))) {
         OPAL_THREAD_UNLOCK(&module->seg_lock);
         return NULL;
@@ -206,9 +207,8 @@ void *mca_allocator_basic_alloc(
 
     /* create a segment for any extra allocation */
     if(allocated_size > size) {
-        int rc;
-        OMPI_FREE_LIST_GET(&module->seg_descriptors, item, rc);
-        if(rc != OMPI_SUCCESS) {
+        OMPI_FREE_LIST_GET_MT(&module->seg_descriptors, item);
+        if(NULL == item) {
             OPAL_THREAD_UNLOCK(&module->seg_lock);
             return NULL;
         }
@@ -277,7 +277,6 @@ void mca_allocator_basic_free(
     ompi_free_list_item_t *item;
     unsigned char* addr = (unsigned char*)ptr - sizeof(size_t);
     size_t size = *(size_t*)addr;
-    int rc;
     OPAL_THREAD_LOCK(&module->seg_lock);
 
     /* maintain the free list in sorted order by address */
@@ -310,13 +309,12 @@ void mca_allocator_basic_free(
             /* insert before larger entry */
             } else {
                 mca_allocator_basic_segment_t* new_seg;
-                OMPI_FREE_LIST_GET(&module->seg_descriptors, item, rc);
-                if(rc != OMPI_SUCCESS) {
+                OMPI_FREE_LIST_GET_MT(&module->seg_descriptors, item);
+                if(NULL == item) {
                     OPAL_THREAD_UNLOCK(&module->seg_lock);
                     return;
                 }
-                new_seg = (
-mca_allocator_basic_segment_t*)item;
+                new_seg = (mca_allocator_basic_segment_t*)item;
                 new_seg->seg_addr = addr;
                 new_seg->seg_size = size;
                 opal_list_insert_pos(&module->seg_list, &seg->seg_item.super, &item->super);
@@ -327,8 +325,8 @@ mca_allocator_basic_segment_t*)item;
     }
 
     /* append to the end of the list */
-    OMPI_FREE_LIST_GET(&module->seg_descriptors, item, rc);
-    if(rc != OMPI_SUCCESS) {
+    OMPI_FREE_LIST_GET_MT(&module->seg_descriptors, item);
+    if(NULL == item) {
         OPAL_THREAD_UNLOCK(&module->seg_lock);
         return;
     }
