@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -10,6 +11,8 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2014      Los Alamos National Security, LLC. All rights
+ *                         reseved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -20,7 +23,6 @@
 #include "ompi_config.h"
 #include "ompi/mca/allocator/allocator.h"
 #include "ompi/constants.h"
-#include "opal/mca/base/mca_base_param.h"
 #include "ompi/mca/allocator/bucket/allocator_bucket_alloc.h"
 #include "ompi/mca/mpool/mpool.h" 
 
@@ -46,8 +48,17 @@ static int mca_allocator_num_buckets;
 
 int mca_allocator_bucket_finalize(struct mca_allocator_base_module_t* allocator)
 {
+    mca_allocator_bucket_t *bucket = (mca_allocator_bucket_t *) allocator;
+
     mca_allocator_bucket_cleanup(allocator);
+
+    for (int i = 0 ; i < bucket->num_buckets ; ++i) {
+        OBJ_DESTRUCT(&bucket->buckets[i].lock);
+    }
+
+    free (bucket->buckets);
     free(allocator);
+
     return(OMPI_SUCCESS);
 }
 
@@ -80,10 +91,16 @@ struct mca_allocator_base_module_t* mca_allocator_bucket_module_init(
     return((mca_allocator_base_module_t *) allocator);
 }
 
-int mca_allocator_bucket_module_open(void) {
+static int mca_allocator_bucket_module_register(void) {
+    mca_allocator_num_buckets = 30;
+    (void) mca_base_component_var_register(&mca_allocator_bucket_component.allocator_version,
+                                           "num_buckets", NULL, MCA_BASE_VAR_TYPE_INT, NULL, 0,
+                                           MCA_BASE_VAR_FLAG_SETTABLE, OPAL_INFO_LVL_9,
+                                           MCA_BASE_VAR_SCOPE_LOCAL, &mca_allocator_num_buckets);
+    return(OMPI_SUCCESS);
+}
 
-    int id = mca_base_param_register_int("allocator","bucket","num_buckets", NULL,30);
-    mca_base_param_lookup_int(id,&mca_allocator_num_buckets);
+int mca_allocator_bucket_module_open(void) {
     return(OMPI_SUCCESS);
 }
 
@@ -117,7 +134,9 @@ mca_allocator_base_component_t mca_allocator_bucket_component = {
     OMPI_MINOR_VERSION,
     OMPI_RELEASE_VERSION,
     mca_allocator_bucket_module_open,  /* module open */
-    mca_allocator_bucket_module_close  /* module close */
+    mca_allocator_bucket_module_close, /* module close */
+    NULL,
+    mca_allocator_bucket_module_register
   },
   {
       /* The component is checkpoint ready */
