@@ -23,7 +23,9 @@
 #include "ompi/mca/bml/bml.h"
 #include "bml_base_btl.h"
 #include "opal/util/crc.h"
-
+#if OPAL_ENABLE_DEBUG_RELIABILITY
+#include "opal/util/alfg.h"
+#endif /* OPAL_ENABLE_DEBUG_RELIABILITY */
 static void mca_bml_base_btl_array_construct(mca_bml_base_btl_array_t* array)
 {
     array->bml_btls = NULL;
@@ -69,9 +71,10 @@ int mca_bml_base_btl_array_reserve(mca_bml_base_btl_array_t* array, size_t size)
 
 #if OPAL_ENABLE_DEBUG_RELIABILITY
 
-extern double mca_bml_base_error_rate_floor;
-extern double mca_bml_base_error_rate_ceiling;
-extern int    mca_bml_base_error_count;
+extern int mca_bml_base_error_rate_floor;
+extern int mca_bml_base_error_rate_ceiling;
+extern int  mca_bml_base_error_count;
+extern opal_rng_buff_t mca_bml_base_rand_buff;
 
 struct mca_bml_base_context_t {
     size_t index;
@@ -102,9 +105,10 @@ int mca_bml_base_send( mca_bml_base_btl_t* bml_btl,
 { 
     des->des_context = (void*)bml_btl; 
     if(mca_bml_base_error_count <= 0 && mca_bml_base_error_rate_ceiling > 0) {
-        mca_bml_base_error_count = (int) ((mca_bml_base_error_rate_ceiling * rand())/(RAND_MAX+1.0));
-        if(mca_bml_base_error_count < mca_bml_base_error_rate_floor) { 
-            mca_bml_base_error_count = mca_bml_base_error_rate_floor;
+      mca_bml_base_error_count = (int) (((double) mca_bml_base_error_rate_ceiling * 
+                  opal_rand(&mca_bml_base_rand_buff))/(UINT32_MAX+1.0));
+        if(mca_bml_base_error_count < (double) mca_bml_base_error_rate_floor) { 
+          mca_bml_base_error_count = (double) mca_bml_base_error_rate_floor;
         }
         if(mca_bml_base_error_count % 2) {
             /* local completion - network "drops" packet */
@@ -117,7 +121,8 @@ int mca_bml_base_send( mca_bml_base_btl_t* bml_btl,
                 malloc(sizeof(mca_bml_base_context_t));
             if(NULL != ctx) {
                 opal_output(0, "%s:%d: corrupting data\n", __FILE__, __LINE__);
-                ctx->index = (size_t) ((des->des_src[0].seg_len * rand() * 1.0) / (RAND_MAX + 1.0));
+                ctx->index = (size_t) ((des->des_src[0].seg_len * 
+                            opal_rand(&mca_bml_base_rand_buff) * 1.0) / (UINT32_MAX + 1.0));
                 ctx->cbfunc = des->des_cbfunc;
                 ctx->cbdata = des->des_cbdata;
                 ((unsigned char*)des->des_src[0].seg_addr.pval)[ctx->index] ^= ~0;
