@@ -10,6 +10,8 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2006-2007 Sun Microsystems, Inc.  All rights reserved.
+ * Copyright (c) 2012      Los Alamos National Security, LLC.
+ *                         All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -22,15 +24,12 @@
 #include <stdio.h>
 #include <stdarg.h>
 
+#include "opal/util/show_help.h"
+#include "ompi/mca/rte/rte.h"
+
 #include "base.h"
 #include "btl_base_error.h"
 
-#include "orte/util/show_help.h"
-#include "orte/util/proc_info.h"
-#include "orte/types.h"
-#include "orte/util/name_fns.h"
-#include "orte/runtime/orte_globals.h"
-
 int mca_btl_base_verbose = -1;
 
 int mca_btl_base_err(const char* fmt, ...)
@@ -63,10 +62,10 @@ void mca_btl_base_error_no_nics(const char* transport,
     char *procid;
     if (mca_btl_base_warn_component_unused) {
         /* print out no-nic warning if user told us to */
-        asprintf(&procid, "%s", ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
+        asprintf(&procid, "%s", OMPI_NAME_PRINT(OMPI_PROC_MY_NAME));
 
-        orte_show_help("help-mpi-btl-base.txt", "btl:no-nics",
-                       true, procid, transport, orte_process_info.nodename,
+        opal_show_help("help-mpi-btl-base.txt", "btl:no-nics",
+                       true, procid, transport, ompi_process_info.nodename,
                        nic_name);
         free(procid);
     }
