@@ -1,8 +1,9 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2013 The University of Tennessee and The University
+ * Copyright (c) 2004-2007 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -10,7 +11,10 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2007      Sun Microsystems, Inc.  All rights reserved.
- * Copyright (c) 2013 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2012-2014 Los Alamos National Security, LLC.
+ *                         All rights reserved.
+ * Copyright (c) 2014      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -22,14 +26,16 @@
 
 #include "opal/util/argv.h"
 #include "opal/util/output.h"
-#include "orte/util/show_help.h"
+#include "opal/util/show_help.h"
 #include "opal/mca/mca.h"
 #include "opal/mca/base/base.h"
 #include "opal/mca/base/mca_base_component_repository.h"
+#include "opal/runtime/opal.h"
+
+#include "ompi/mca/rte/rte.h"
+
 #include "ompi/mca/btl/btl.h"
 #include "ompi/mca/btl/base/base.h"
-#include "orte/mca/errmgr/errmgr.h"
-#include "opal/runtime/opal.h"
 
 OBJ_CLASS_INSTANCE( mca_btl_base_selected_module_t,
                     opal_list_item_t,
@@ -48,9 +54,7 @@ int mca_btl_base_select(bool enable_progress_threads,
                         bool enable_mpi_threads)
 {
     int i, num_btls;
-    opal_list_t btls_to_close;
-    opal_list_item_t *item;
-    mca_base_component_list_item_t *cli;
+    mca_base_component_list_item_t *cli, *next;
     mca_btl_base_component_t *component;
     mca_btl_base_module_t **modules;
     mca_btl_base_selected_module_t *sm;
@@ -58,16 +62,10 @@ int mca_btl_base_select(bool enable_progress_threads,
     char** include = opal_argv_split(mca_btl_base_include, ',');
     char** exclude = opal_argv_split(mca_btl_base_exclude, ',');
 
-    OBJ_CONSTRUCT(&btls_to_close, opal_list_t);
-
     /* Traverse the list of opened modules; call their init
        functions. */
 
-    item  = opal_list_get_first(&mca_btl_base_components_opened);
-    while(item != opal_list_get_end(&mca_btl_base_components_opened)) {
-        opal_list_item_t *next = opal_list_get_next(item);
-        cli = (mca_base_component_list_item_t *) item;
-
+    OPAL_LIST_FOREACH_SAFE(cli, next, &ompi_btl_base_framework.framework_components, mca_base_component_list_item_t) {
         component = (mca_btl_base_component_t *) cli->cli_component;
 
         /* if there is an include list - item must be in the list to be included */
@@ -82,7 +80,7 @@ int mca_btl_base_select(bool enable_progress_threads,
                 argv++;
             }
             if(found == false) {
-                goto close_component_and_continue;
+                continue;
             }
 
             /* otherwise - check the exclude list to see if this item has been specifically excluded */
@@ -97,19 +95,18 @@ int mca_btl_base_select(bool enable_progress_threads,
                 argv++;
             }
             if(found == true) {
-                goto close_component_and_continue;
+                continue;
             }
         }
 
-        opal_output_verbose(10, mca_btl_base_output, 
+        opal_output_verbose(10, ompi_btl_base_framework.framework_output, 
                             "select: initializing %s component %s",
                             component->btl_version.mca_type_name,
                             component->btl_version.mca_component_name);
         if (NULL == component->btl_init) {
-            opal_output_verbose(10, mca_btl_base_output,
+            opal_output_verbose(10, ompi_btl_base_framework.framework_output,
                                 "select: no init function; ignoring component %s",
                                 component->btl_version.mca_component_name);
-            goto close_component_and_continue;
         } else {
             modules = component->btl_init(&num_btls, enable_progress_threads,
                                           enable_mpi_threads);
@@ -118,23 +115,23 @@ int mca_btl_base_select(bool enable_progress_threads,
                list and remove it from the component repository */
 
             if (NULL == modules) {
-                opal_output_verbose(10, mca_btl_base_output,
+                opal_output_verbose(10, ompi_btl_base_framework.framework_output,
                                     "select: init of component %s returned failure",
                                     component->btl_version.mca_component_name);
-                goto close_component_and_continue;
+
+                opal_list_remove_item(&ompi_btl_base_framework.framework_components, &cli->super);
+                OBJ_RELEASE(cli);
+                mca_base_component_close((mca_base_component_t *) component,
+                                         ompi_btl_base_framework.framework_output);
             } 
 
             /* Otherwise, it initialized properly.  Save it. */
 
             else {
-                opal_output_verbose(10, mca_btl_base_output,
+                opal_output_verbose(10, ompi_btl_base_framework.framework_output,
                                     "select: init of component %s returned success",
                                     component->btl_version.mca_component_name);
 
-                if (opal_profile) {
-                    opal_output(0, "btl:%s", component->btl_version.mca_component_name);
-                }
-                
                 for (i = 0; i < num_btls; ++i) {
                     sm = OBJ_NEW(mca_btl_base_selected_module_t);
                     if (NULL == sm) {
@@ -146,35 +143,16 @@ int mca_btl_base_select(bool enable_progress_threads,
                                      (opal_list_item_t*) sm);
                 }
                 free(modules);
-                item = next;
-                continue;  /* The only succesful path for this component */
             }
         }
-      close_component_and_continue:
-        opal_output_verbose(10, mca_btl_base_output,
-                            "select: component %s unloaded",
-                            component->btl_version.mca_component_name);
-
-        opal_list_remove_item(&mca_btl_base_components_opened, item);
-        opal_list_append(&btls_to_close, item);
-        item = next;
-     }
-
-    /* Close/release all the components that said they don't want to
-       run */
-    if (!opal_list_is_empty(&btls_to_close)) {
-        mca_base_components_close(mca_btl_base_output,
-                                  &btls_to_close, NULL, false);
     }
-    OBJ_DESTRUCT(&btls_to_close);
 
     /* Finished querying all components.  Check for the bozo case. */
 
     if (0 == opal_list_get_size(&mca_btl_base_modules_initialized)) {
-        orte_show_help("help-mca-base.txt", "find-available:none-found", true,
+        opal_show_help("help-mca-base.txt", "find-available:none-found", true,
                        "btl");
-        orte_errmgr.abort(1, NULL);
+        ompi_rte_abort(1, NULL);
     }
-
     return OMPI_SUCCESS;
 }
