@@ -55,8 +55,8 @@
 
 BEGIN_C_DECLS
 
-#define HAVE_XRC (1 == OMPI_HAVE_CONNECTX_XRC)
-#define ENABLE_DYNAMIC_SL (1 == OMPI_ENABLE_DYNAMIC_SL)
+#define HAVE_XRC (OMPI_HAVE_CONNECTX_XRC || OMPI_HAVE_CONNECTX_XRC_DOMAINS)
+#define ENABLE_DYNAMIC_SL OMPI_ENABLE_DYNAMIC_SL
 
 #define MCA_BTL_IB_LEAVE_PINNED 1
 #define IB_DEFAULT_GID_PREFIX 0xfe80000000000000ll
@@ -288,8 +288,6 @@ struct mca_btl_openib_component_t {
     unsigned int cq_poll_progress;
     unsigned int cq_poll_batch;
     unsigned int eager_rdma_poll_ratio;
-    /** Whether we want fork support or not */
-    int want_fork_support;
     int rdma_qp;
     int credits_qp; /* qp used for software flow control */
     bool cpc_explicitly_defined;
