@@ -10,13 +10,18 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2009 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2006-2007 Los Alamos National Security, LLC.  All rights
+ * Copyright (c) 2006-2013 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2006-2012 Los Alamos National Security, LLC.  All rights
  *                         reserved.
  * Copyright (c) 2006-2007 Voltaire All rights reserved.
  * Copyright (c) 2006-2009 Mellanox Technologies, Inc.  All rights reserved.
- * Copyright (c) 2009      IBM Corporation.  All rights reserved.
+ * Copyright (c) 2010-2011 IBM Corporation.  All rights reserved.
  * Copyright (c) 2010-2011 Oracle and/or its affiliates.  All rights reserved
+ * Copyright (c) 2013      Intel, Inc. All rights reserved
+ * Copyright (c) 2013      NVIDIA Corporation.  All rights reserved.
+ * Copyright (c) 2014      Bull SAS.  All rights reserved.
+ * Copyright (c) 2015      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  *
  * $COPYRIGHT$
  *
@@ -27,15 +32,16 @@
 
 #include "ompi_config.h"
 
+#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
+#endif
 #include <time.h>
 #include <errno.h>
 #include <string.h>
 
 #include "opal_stdint.h"
 #include "opal/util/output.h"
-
-#include "orte/util/show_help.h"
+#include "opal/util/show_help.h"
 
 #include "ompi/types.h"
 #include "ompi/class/ompi_free_list.h"
@@ -195,7 +201,7 @@ OBJ_CLASS_INSTANCE(mca_btl_openib_endpoint_t,
  */
 static mca_btl_openib_qp_t *endpoint_alloc_qp(void)
 {
-    mca_btl_openib_qp_t *qp = calloc(1, sizeof(mca_btl_openib_qp_t));
+    mca_btl_openib_qp_t *qp = (mca_btl_openib_qp_t *) calloc(1, sizeof(mca_btl_openib_qp_t));
     if(!qp) {
         BTL_ERROR(("Failed to allocate memory for qp"));
         return NULL;
@@ -311,7 +317,7 @@ void mca_btl_openib_endpoint_init(mca_btl_openib_module_t *btl,
     ep->rem_info.rem_lid = remote_proc_info->pm_port_info.lid;
     ep->rem_info.rem_subnet_id = remote_proc_info->pm_port_info.subnet_id;
     ep->rem_info.rem_mtu = remote_proc_info->pm_port_info.mtu;
-    opal_output(-1, "Got remote LID, subnet, MTU: %d, 0x%" PRIx64 ", %d", 
+    opal_output(-1, "Got remote LID, subnet, MTU: %d, 0x%" PRIx64 ", %d",
                 ep->rem_info.rem_lid,
                 ep->rem_info.rem_subnet_id,
                 ep->rem_info.rem_mtu);
@@ -319,7 +325,8 @@ void mca_btl_openib_endpoint_init(mca_btl_openib_module_t *btl,
     ep->rem_info.rem_vendor_id = (remote_proc_info->pm_port_info).vendor_id;
     ep->rem_info.rem_vendor_part_id = (remote_proc_info->pm_port_info).vendor_part_id;
 
-    ep->rem_info.rem_transport_type = (remote_proc_info->pm_port_info).transport_type;
+    ep->rem_info.rem_transport_type =
+         (mca_btl_openib_transport_type_t) (remote_proc_info->pm_port_info).transport_type;
 
     for (qp = 0; qp < mca_btl_openib_component.num_qps; qp++) {
         endpoint_init_qp(ep, qp);
@@ -346,7 +353,11 @@ static void mca_btl_openib_endpoint_construct(mca_btl_base_endpoint_t* endpoint)
     }
 
     endpoint->ib_addr = NULL;
+#if OMPI_HAVE_CONNECTX_XRC_DOMAINS
+    endpoint->xrc_recv_qp = NULL;
+#else
     endpoint->xrc_recv_qp_num = 0;
+#endif
     endpoint->endpoint_btl = 0;
     endpoint->endpoint_proc = 0;
     endpoint->endpoint_local_cpc = NULL;
@@ -457,12 +468,24 @@ static void mca_btl_openib_endpoint_destruct(mca_btl_base_endpoint_t* endpoint)
 
     /* unregister xrc recv qp */
 #if HAVE_XRC
+#if OMPI_HAVE_CONNECTX_XRC_DOMAINS
+    if (NULL != endpoint->xrc_recv_qp) {
+        if(ibv_destroy_qp(endpoint->xrc_recv_qp)) {
+            BTL_ERROR(("Failed to unregister XRC recv QP:%d\n", endpoint->xrc_recv_qp->qp_num));
+        } else {
+            BTL_VERBOSE(("Unregistered XRC Recv QP:%d\n", endpoint->xrc_recv_qp->qp_num));
+        }
+    }
+#else
     if (0 != endpoint->xrc_recv_qp_num) {
         if(ibv_unreg_xrc_rcv_qp(endpoint->endpoint_btl->device->xrc_domain,
                     endpoint->xrc_recv_qp_num)) {
             BTL_ERROR(("Failed to unregister XRC recv QP:%d\n", endpoint->xrc_recv_qp_num));
+        } else {
+            BTL_VERBOSE(("Unregistered XRC Recv QP:%d\n", endpoint->xrc_recv_qp_num));
         }
     }
+#endif
 #endif
 
     OBJ_DESTRUCT(&endpoint->endpoint_lock);
@@ -505,13 +528,14 @@ static void cts_sent(mca_btl_base_module_t* btl,
     /* Nothing to do/empty function (we can't pass in a NULL pointer
        for the des_cbfunc) */
     OPAL_OUTPUT((-1, "CTS send to %s completed",
-                 ep->endpoint_proc->proc_ompi->proc_hostname));
+                 (NULL == ep->endpoint_proc->proc_ompi->proc_hostname) ?
+                 "unknown" : ep->endpoint_proc->proc_ompi->proc_hostname));
 }
 
 /*
  * Send CTS control fragment
  */
-void mca_btl_openib_endpoint_send_cts(mca_btl_openib_endpoint_t *endpoint) 
+void mca_btl_openib_endpoint_send_cts(mca_btl_openib_endpoint_t *endpoint)
 {
     mca_btl_openib_send_control_frag_t *sc_frag;
     mca_btl_base_descriptor_t *base_des;
@@ -520,7 +544,8 @@ void mca_btl_openib_endpoint_send_cts(mca_btl_openib_endpoint_t *endpoint)
     mca_btl_openib_control_header_t *ctl_hdr;
 
     OPAL_OUTPUT((-1, "SENDING CTS to %s on qp index %d (QP num %d)",
-                 endpoint->endpoint_proc->proc_ompi->proc_hostname,
+                 (NULL == endpoint->endpoint_proc->proc_ompi->proc_hostname) ?
+                 "unknown" : endpoint->endpoint_proc->proc_ompi->proc_hostname,
                  mca_btl_openib_component.credits_qp,
                  endpoint->qps[mca_btl_openib_component.credits_qp].qp->lcl_qp->qp_num));
     sc_frag = alloc_control_frag(endpoint->endpoint_btl);
@@ -542,15 +567,15 @@ void mca_btl_openib_endpoint_send_cts(mca_btl_openib_endpoint_t *endpoint)
     base_des->des_cbdata = NULL;
     base_des->des_flags |= MCA_BTL_DES_FLAGS_PRIORITY|MCA_BTL_DES_SEND_ALWAYS_CALLBACK;
     base_des->order = mca_btl_openib_component.credits_qp;
-    openib_frag->segment.seg_len = sizeof(mca_btl_openib_control_header_t);
+    openib_frag->segment.base.seg_len = sizeof(mca_btl_openib_control_header_t);
     com_frag->endpoint = endpoint;
 
-    sc_frag->hdr->tag = MCA_BTL_TAG_BTL;
+    sc_frag->hdr->tag = MCA_BTL_TAG_IB;
     sc_frag->hdr->cm_seen = 0;
     sc_frag->hdr->credits = 0;
 
     ctl_hdr = (mca_btl_openib_control_header_t*)
-        openib_frag->segment.seg_addr.pval;
+        openib_frag->segment.base.seg_addr.pval;
     ctl_hdr->type = MCA_BTL_OPENIB_CONTROL_CTS;
 
     /* Send the fragment */
@@ -590,7 +615,8 @@ void mca_btl_openib_endpoint_cpc_complete(mca_btl_openib_endpoint_t *endpoint)
         transport_type_ib_p = (IBV_TRANSPORT_IB == endpoint->endpoint_btl->device->ib_dev->transport_type);
 #endif
         OPAL_OUTPUT((-1, "cpc_complete to peer %s: is IB %d, initiatior %d, cts received: %d",
-                     endpoint->endpoint_proc->proc_ompi->proc_hostname,
+                     (NULL == endpoint->endpoint_proc->proc_ompi->proc_hostname) ?
+                     "unknown" : endpoint->endpoint_proc->proc_ompi->proc_hostname,
                      transport_type_ib_p,
                      endpoint->endpoint_initiator,
                      endpoint->endpoint_cts_received));
@@ -603,13 +629,15 @@ void mca_btl_openib_endpoint_cpc_complete(mca_btl_openib_endpoint_t *endpoint)
                mark us as connected */
             if (endpoint->endpoint_cts_received) {
                 OPAL_OUTPUT((-1, "cpc_complete to %s -- already got CTS, so marking endpoint as complete",
-                             endpoint->endpoint_proc->proc_ompi->proc_hostname));
+                             (NULL == endpoint->endpoint_proc->proc_ompi->proc_hostname) ?
+                             "unknown" : endpoint->endpoint_proc->proc_ompi->proc_hostname));
                 mca_btl_openib_endpoint_connected(endpoint);
             }
         }
 
         OPAL_OUTPUT((-1, "cpc_complete to %s -- done",
-                     endpoint->endpoint_proc->proc_ompi->proc_hostname));
+                     (NULL == endpoint->endpoint_proc->proc_ompi->proc_hostname) ?
+                     "unknown" : endpoint->endpoint_proc->proc_ompi->proc_hostname));
         return;
     }
 
@@ -668,8 +696,8 @@ void mca_btl_openib_endpoint_connected(mca_btl_openib_endpoint_t *endpoint)
         while(master && !opal_list_is_empty(&endpoint->ib_addr->pending_ep)) {
             ep_item = opal_list_remove_first(&endpoint->ib_addr->pending_ep);
             ep = (mca_btl_openib_endpoint_t *)ep_item;
-            if (OMPI_SUCCESS != 
-                ompi_btl_openib_connect_base_start(endpoint->endpoint_local_cpc, 
+            if (OMPI_SUCCESS !=
+                ompi_btl_openib_connect_base_start(endpoint->endpoint_local_cpc,
                                                    ep)) {
                 BTL_ERROR(("Failed to connect pending endpoint\n"));
             }
@@ -686,8 +714,9 @@ void mca_btl_openib_endpoint_connected(mca_btl_openib_endpoint_t *endpoint)
         frag = to_send_frag(frag_item);
         /* We need to post this one */
 
-        if(OMPI_ERROR == mca_btl_openib_endpoint_post_send(endpoint, frag))
-            BTL_ERROR(("Error posting send"));
+        if (OMPI_ERROR == mca_btl_openib_endpoint_post_send(endpoint, frag)) {
+		     BTL_ERROR(("Error posting send"));
+		}
     }
     OPAL_THREAD_UNLOCK(&endpoint->endpoint_lock);
 
@@ -777,14 +806,14 @@ void mca_btl_openib_endpoint_send_credits(mca_btl_openib_endpoint_t* endpoint,
         to_base_frag(frag)->base.des_cbdata = NULL;
         to_base_frag(frag)->base.des_flags |= MCA_BTL_DES_SEND_ALWAYS_CALLBACK;;
         to_com_frag(frag)->endpoint = endpoint;
-        frag->hdr->tag = MCA_BTL_TAG_BTL;
-        to_base_frag(frag)->segment.seg_len =
+        frag->hdr->tag = MCA_BTL_TAG_IB;
+        to_base_frag(frag)->segment.base.seg_len =
             sizeof(mca_btl_openib_rdma_credits_header_t);
     }
 
     assert(frag->qp_idx == qp);
     credits_hdr = (mca_btl_openib_rdma_credits_header_t*)
-        to_base_frag(frag)->segment.seg_addr.pval;
+        to_base_frag(frag)->segment.base.seg_addr.pval;
     if(OMPI_SUCCESS == acquire_eager_rdma_send_credit(endpoint)) {
         do_rdma = true;
     } else {
@@ -869,16 +898,16 @@ static int mca_btl_openib_endpoint_send_eager_rdma(
     to_base_frag(frag)->base.des_cbdata = NULL;
     to_base_frag(frag)->base.des_flags |= MCA_BTL_DES_FLAGS_PRIORITY|MCA_BTL_DES_SEND_ALWAYS_CALLBACK;
     to_base_frag(frag)->base.order = mca_btl_openib_component.credits_qp;
-    to_base_frag(frag)->segment.seg_len =
+    to_base_frag(frag)->segment.base.seg_len =
         sizeof(mca_btl_openib_eager_rdma_header_t);
     to_com_frag(frag)->endpoint = endpoint;
 
-    frag->hdr->tag = MCA_BTL_TAG_BTL;
-    rdma_hdr = (mca_btl_openib_eager_rdma_header_t*)to_base_frag(frag)->segment.seg_addr.pval;
+    frag->hdr->tag = MCA_BTL_TAG_IB;
+    rdma_hdr = (mca_btl_openib_eager_rdma_header_t*)to_base_frag(frag)->segment.base.seg_addr.pval;
     rdma_hdr->control.type = MCA_BTL_OPENIB_CONTROL_RDMA;
     rdma_hdr->rkey = endpoint->eager_rdma_local.reg->mr->rkey;
     rdma_hdr->rdma_start.lval = ompi_ptr_ptol(endpoint->eager_rdma_local.base.pval);
-    BTL_VERBOSE(("sending rkey %" PRIu32 ", rdma_start.lval %" PRIx64 
+    BTL_VERBOSE(("sending rkey %" PRIu32 ", rdma_start.lval %" PRIx64
                  ", pval %p, ival %" PRIu32 " type %d and sizeof(rdma_hdr) %d\n",
                  rdma_hdr->rkey,
                  rdma_hdr->rdma_start.lval,
@@ -899,7 +928,7 @@ static int mca_btl_openib_endpoint_send_eager_rdma(
                      ));
     }
     rc = mca_btl_openib_endpoint_send(endpoint, frag);
-    if (OMPI_SUCCESS == rc ||OMPI_ERR_RESOURCE_BUSY == rc)
+    if (OMPI_SUCCESS == rc || OMPI_ERR_RESOURCE_BUSY == rc)
         return OMPI_SUCCESS;
 
     MCA_BTL_IB_FRAG_RETURN(frag);
@@ -944,7 +973,7 @@ void mca_btl_openib_endpoint_connect_eager_rdma(
     flag |= MCA_MPOOL_FLAGS_SO_MEM;
 #endif
 
-     buf = openib_btl->super.btl_mpool->mpool_alloc(openib_btl->super.btl_mpool,
+    buf = (char *) openib_btl->super.btl_mpool->mpool_alloc(openib_btl->super.btl_mpool,
             openib_btl->eager_rdma_frag_size *
             mca_btl_openib_component.eager_rdma_num,
             mca_btl_openib_component.buffer_alignment,
@@ -964,7 +993,7 @@ void mca_btl_openib_endpoint_connect_eager_rdma(
         mca_btl_openib_frag_init_data_t init_data;
 
         item = (ompi_free_list_item_t*)&headers_buf[i];
-        item->registration = (void*)endpoint->eager_rdma_local.reg;
+        item->registration = (mca_mpool_base_registration_t *)endpoint->eager_rdma_local.reg;
         item->ptr = buf + i * openib_btl->eager_rdma_frag_size;
         OBJ_CONSTRUCT(item, mca_btl_openib_recv_frag_t);
 
@@ -976,7 +1005,7 @@ void mca_btl_openib_endpoint_connect_eager_rdma(
         to_base_frag(frag)->type = MCA_BTL_OPENIB_FRAG_EAGER_RDMA;
         to_com_frag(frag)->endpoint = endpoint;
         frag->ftr = (mca_btl_openib_footer_t*)
-            ((char*)to_base_frag(frag)->segment.seg_addr.pval +
+            ((char*)to_base_frag(frag)->segment.base.seg_addr.pval +
              mca_btl_openib_component.eager_limit);
 
         MCA_BTL_OPENIB_RDMA_MAKE_REMOTE(frag->ftr);
@@ -1044,9 +1073,9 @@ void *mca_btl_openib_endpoint_invoke_error(void *context)
 
     /* If we didn't find a BTL, then just bail :-( */
     if (NULL == btl || NULL == btl->error_cb) {
-        orte_show_help("help-mpi-btl-openib.txt",
+        opal_show_help("help-mpi-btl-openib.txt",
                        "cannot raise btl error", true,
-                       orte_process_info.nodename,
+                       ompi_process_info.nodename,
                        __FILE__, __LINE__);
         exit(1);
     }
