@@ -1,18 +1,19 @@
 /*
  * Copyright (c) 2007-2008 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2009      Mellanox Technologies.  All rights reserved.
+ * Copyright (c) 2013      Mellanox Technologies, Inc.
+ *                         All rights reserved.
  *
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
 #ifndef BTL_OPENIB_CONNECT_BASE_H
 #define BTL_OPENIB_CONNECT_BASE_H
 
-#include "connect/connect.h"
+#include "ompi/mca/btl/openib/connect/connect.h"
 
 #ifdef OMPI_HAVE_RDMAOE
 #define BTL_OPENIB_CONNECT_BASE_CHECK_IF_NOT_IB(btl)                       \
@@ -74,13 +75,13 @@ int ompi_btl_openib_connect_base_get_cpc_index
 ompi_btl_openib_connect_base_component_t *
     ompi_btl_openib_connect_base_get_cpc_byindex(uint8_t index);
 
-/* 
+/*
  * Allocate a CTS frag
  */
 int ompi_btl_openib_connect_base_alloc_cts(
         struct mca_btl_base_endpoint_t *endpoint);
 
-/* 
+/*
  * Free a CTS frag
  */
 int ompi_btl_openib_connect_base_free_cts(
