@@ -1,11 +1,15 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
- * Copyright (c) 2007-2009 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2007 Mellanox Technologies, Inc.  All rights reserved.
+ * Copyright (c) 2007-2013 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2007      Mellanox Technologies, Inc.  All rights reserved.
+ * Copyright (c) 2012-2013 Los Alamos National Security, LLC.  All rights
+ *                         reserved. 
+ * Copyright (c) 2013      Intel, Inc. All rights reserved
  *
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 #include "ompi_config.h"
@@ -13,32 +17,29 @@
 #include "btl_openib.h"
 #include "btl_openib_proc.h"
 #include "connect/base.h"
-#include "connect/btl_openib_connect_oob.h"
 #include "connect/btl_openib_connect_empty.h"
-#if HAVE_XRC
-#include "connect/btl_openib_connect_xoob.h"
-#endif
 #if OMPI_HAVE_RDMACM && OPAL_HAVE_THREADS
 #include "connect/btl_openib_connect_rdmacm.h"
 #endif
+#if OMPI_HAVE_UDCM && OPAL_HAVE_THREADS
+#include "connect/btl_openib_connect_udcm.h"
+#endif
 
-#include "orte/util/show_help.h"
 #include "opal/util/argv.h"
 #include "opal/util/output.h"
+#include "opal/util/show_help.h"
 
 /*
  * Array of all possible connection functions
  */
 static ompi_btl_openib_connect_base_component_t *all[] = {
-    &ompi_btl_openib_connect_oob,
+    /* Always have an entry here so that the CP indexes will always be
+       the same: OOB has been removed, so use the "empty" CPC */
+    &ompi_btl_openib_connect_empty,
 
     /* Always have an entry here so that the CP indexes will always be
-       the same: if XRC is not available, use the "empty" CPC */
-#if HAVE_XRC
-    &ompi_btl_openib_connect_xoob,
-#else
+       the same: XOOB has been removed, so use the "empty" CPC */
     &ompi_btl_openib_connect_empty,
-#endif
 
     /* Always have an entry here so that the CP indexes will always be
        the same: if RDMA CM is not available, use the "empty" CPC */
@@ -48,11 +49,24 @@ static ompi_btl_openib_connect_base_component_t *all[] = {
     &ompi_btl_openib_connect_empty,
 #endif
 
+    /* Always have an entry here so that the CP indexes will always be
+       the same: if UD CM is not enabled, use the "empty" CPC */
+#if OMPI_HAVE_UDCM && OPAL_HAVE_THREADS
+    &ompi_btl_openib_connect_udcm,
+#else
+    &ompi_btl_openib_connect_empty,
+#endif
+
     NULL
 };
-static ompi_btl_openib_connect_base_component_t **available = NULL;
+
+/* increase this count if any more cpcs are added */
+static ompi_btl_openib_connect_base_component_t *available[5];
 static int num_available = 0;
 
+static char *btl_openib_cpc_include;
+static char *btl_openib_cpc_exclude;
+
 /*
  * Register MCA parameters
  */
@@ -60,7 +74,6 @@ int ompi_btl_openib_connect_base_register(void)
 {
     int i, j, save;
     char **temp = NULL, *string = NULL, *all_cpc_names = NULL;
-    char *cpc_include = NULL, *cpc_exclude = NULL;
 
     /* Make an MCA parameter to select which connect module to use */
     for (i = 0; NULL != all[i]; ++i) {
@@ -75,32 +88,36 @@ int ompi_btl_openib_connect_base_register(void)
              "Method used to select OpenFabrics connections (valid values: %s)",
              all_cpc_names);
 
-    mca_base_param_reg_string(&mca_btl_openib_component.super.btl_version,
-                              "cpc_include", string, false, false, 
-                              NULL, &cpc_include);
+    btl_openib_cpc_include = NULL;
+    (void) mca_base_component_var_register(&mca_btl_openib_component.super.btl_version,
+                                           "cpc_include", string, MCA_BASE_VAR_TYPE_STRING,
+                                           NULL, 0, 0, OPAL_INFO_LVL_9,
+                                           MCA_BASE_VAR_SCOPE_READONLY,
+                                           &btl_openib_cpc_include);
     free(string);
 
     asprintf(&string,
              "Method used to exclude OpenFabrics connections (valid values: %s)",
              all_cpc_names);
 
-    mca_base_param_reg_string(&mca_btl_openib_component.super.btl_version,
-                              "cpc_exclude", string, false, false, 
-                              NULL, &cpc_exclude);
+    btl_openib_cpc_exclude = NULL;
+    (void) mca_base_component_var_register(&mca_btl_openib_component.super.btl_version,
+                                           "cpc_exclude", string, MCA_BASE_VAR_TYPE_STRING,
+                                           NULL, 0, 0, OPAL_INFO_LVL_9,
+                                           MCA_BASE_VAR_SCOPE_READONLY,
+                                           &btl_openib_cpc_exclude);
     free(string);
 
     /* Parse the if_[in|ex]clude paramters to come up with a list of
        CPCs that are available */
-    available = calloc(1, sizeof(all));
-
     /* If we have an "include" list, then find all those CPCs and put
        them in available[] */
-    if (NULL != cpc_include) {
+    if (NULL != btl_openib_cpc_include) {
         mca_btl_openib_component.cpc_explicitly_defined = true;
-        temp = opal_argv_split(cpc_include, ',');
+        temp = opal_argv_split(btl_openib_cpc_include, ',');
         for (save = j = 0; NULL != temp[j]; ++j) {
             for (i = 0; NULL != all[i]; ++i) {
-                if (0 == strcmp(temp[j], all[i]->cbc_name)) { 
+                if (0 == strcmp(temp[j], all[i]->cbc_name)) {
                     opal_output(-1, "include: saving %s", all[i]->cbc_name);
                     available[save++] = all[i];
                     ++num_available;
@@ -108,10 +125,10 @@ int ompi_btl_openib_connect_base_register(void)
                 }
             }
             if (NULL == all[i]) {
-                orte_show_help("help-mpi-btl-openib-cpc-base.txt",
+                opal_show_help("help-mpi-btl-openib-cpc-base.txt",
                                "cpc name not found", true,
-                               "include", orte_process_info.nodename,
-                               "include", cpc_include, temp[j], 
+                               "include", ompi_process_info.nodename,
+                               "include", btl_openib_cpc_include, temp[j],
                                all_cpc_names);
                 opal_argv_free(temp);
                 free(all_cpc_names);
@@ -123,21 +140,21 @@ int ompi_btl_openib_connect_base_register(void)
 
     /* Otherwise, if we have an "exclude" list, take all the CPCs that
        are not in that list and put them in available[] */
-    else if (NULL != cpc_exclude) {
+    else if (NULL != btl_openib_cpc_exclude) {
         mca_btl_openib_component.cpc_explicitly_defined = true;
-        temp = opal_argv_split(cpc_exclude, ',');
+        temp = opal_argv_split(btl_openib_cpc_exclude, ',');
         /* First: error check -- ensure that all the names are valid */
         for (j = 0; NULL != temp[j]; ++j) {
             for (i = 0; NULL != all[i]; ++i) {
-                if (0 == strcmp(temp[j], all[i]->cbc_name)) { 
+                if (0 == strcmp(temp[j], all[i]->cbc_name)) {
                     break;
                 }
             }
             if (NULL == all[i]) {
-                orte_show_help("help-mpi-btl-openib-cpc-base.txt",
+                opal_show_help("help-mpi-btl-openib-cpc-base.txt",
                                "cpc name not found", true,
-                               "exclude", orte_process_info.nodename,
-                               "exclude", cpc_exclude, temp[j], 
+                               "exclude", ompi_process_info.nodename,
+                               "exclude", btl_openib_cpc_exclude, temp[j],
                                all_cpc_names);
                 opal_argv_free(temp);
                 free(all_cpc_names);
@@ -159,13 +176,13 @@ int ompi_btl_openib_connect_base_register(void)
             }
         }
         opal_argv_free(temp);
-    } 
+    }
 
     /* If there's no include/exclude list, copy all[] into available[] */
     else {
         opal_output(-1, "no include or exclude: saving all");
         memcpy(available, all, sizeof(all));
-        num_available = (sizeof(all) / 
+        num_available = (sizeof(all) /
                          sizeof(ompi_btl_openib_connect_base_module_t *)) - 1;
     }
 
@@ -177,6 +194,7 @@ int ompi_btl_openib_connect_base_register(void)
         }
     }
 
+    free (all_cpc_names);
     return OMPI_SUCCESS;
 }
 
@@ -229,7 +247,7 @@ int ompi_btl_openib_connect_base_select_for_local_port(mca_btl_openib_module_t *
     int i, rc, cpc_index, len;
     ompi_btl_openib_connect_base_module_t **cpcs;
 
-    cpcs = calloc(num_available, 
+    cpcs = (ompi_btl_openib_connect_base_module_t **) calloc(num_available,
                   sizeof(ompi_btl_openib_connect_base_module_t *));
     if (NULL == cpcs) {
         return OMPI_ERR_OUT_OF_RESOURCE;
@@ -241,7 +259,7 @@ int ompi_btl_openib_connect_base_select_for_local_port(mca_btl_openib_module_t *
     for (len = 1, i = 0; NULL != available[i]; ++i) {
         len += strlen(available[i]->cbc_name) + 2;
     }
-    msg = malloc(len);
+    msg = (char *) malloc(len);
     if (NULL == msg) {
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
@@ -279,9 +297,9 @@ int ompi_btl_openib_connect_base_select_for_local_port(mca_btl_openib_module_t *
 
     /* If we got an empty array, then no CPCs were eligible.  Doh! */
     if (0 == cpc_index) {
-        orte_show_help("help-mpi-btl-openib-cpc-base.txt",
+        opal_show_help("help-mpi-btl-openib-cpc-base.txt",
                        "no cpcs for port", true,
-                       orte_process_info.nodename,
+                       ompi_process_info.nodename,
                        ibv_get_device_name(btl->device->ib_dev),
                        btl->port_num, msg);
         free(cpcs);
@@ -298,7 +316,7 @@ int ompi_btl_openib_connect_base_select_for_local_port(mca_btl_openib_module_t *
     return OMPI_SUCCESS;
 }
 
-/* 
+/*
  * This function is invoked when determining whether we have a CPC in
  * common with a specific remote port.  We already know that the
  * subnet ID is the same between a specific local port and the target
@@ -385,7 +403,7 @@ int ompi_btl_openib_connect_base_get_cpc_index(ompi_btl_openib_connect_base_comp
 ompi_btl_openib_connect_base_component_t *
 ompi_btl_openib_connect_base_get_cpc_byindex(uint8_t index)
 {
-    return (index >= (sizeof(all) / 
+    return (index >= (sizeof(all) /
                       sizeof(ompi_btl_openib_connect_base_module_t *))) ?
         NULL : all[index];
 }
@@ -408,8 +426,8 @@ int ompi_btl_openib_connect_base_alloc_cts(mca_btl_base_endpoint_t *endpoint)
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
 
-    endpoint->endpoint_cts_mr = 
-        ibv_reg_mr(endpoint->endpoint_btl->device->ib_pd, 
+    endpoint->endpoint_cts_mr =
+        ibv_reg_mr(endpoint->endpoint_btl->device->ib_pd,
                    fli->ptr, length,
                    IBV_ACCESS_LOCAL_WRITE | IBV_ACCESS_REMOTE_WRITE |
                    IBV_ACCESS_REMOTE_READ);
@@ -425,18 +443,19 @@ int ompi_btl_openib_connect_base_alloc_cts(mca_btl_base_endpoint_t *endpoint)
        from underneath us. */
 
     /* Copy the lkey where it needs to go */
-    endpoint->endpoint_cts_frag.super.sg_entry.lkey = 
-        endpoint->endpoint_cts_frag.super.super.segment.seg_key.key32[0] = 
+    endpoint->endpoint_cts_frag.super.sg_entry.lkey =
+        endpoint->endpoint_cts_frag.super.super.segment.key =
         endpoint->endpoint_cts_mr->lkey;
     endpoint->endpoint_cts_frag.super.sg_entry.length = length;
 
     /* Construct the rest of the recv_frag_t */
     OBJ_CONSTRUCT(&(endpoint->endpoint_cts_frag), mca_btl_openib_recv_frag_t);
-    endpoint->endpoint_cts_frag.super.super.base.order = 
+    endpoint->endpoint_cts_frag.super.super.base.order =
         mca_btl_openib_component.credits_qp;
     endpoint->endpoint_cts_frag.super.endpoint = endpoint;
     OPAL_OUTPUT((-1, "Got a CTS frag for peer %s, addr %p, length %d, lkey %d",
-                 endpoint->endpoint_proc->proc_ompi->proc_hostname,
+                 (NULL == endpoint->endpoint_proc->proc_ompi->proc_hostname) ?
+                 "unknown" : endpoint->endpoint_proc->proc_ompi->proc_hostname,
                  (void*) endpoint->endpoint_cts_frag.super.sg_entry.addr,
                  endpoint->endpoint_cts_frag.super.sg_entry.length,
                  endpoint->endpoint_cts_frag.super.sg_entry.lkey));
@@ -490,13 +509,9 @@ void ompi_btl_openib_connect_base_finalize(void)
 {
     int i;
 
-    if (NULL != available) {
-        for (i = 0; NULL != available[i]; ++i) {
-            if (NULL != available[i]->cbc_finalize) {
-                available[i]->cbc_finalize();
-            }
+    for (i = 0 ; i < num_available ; ++i) {
+        if (NULL != available[i]->cbc_finalize) {
+            available[i]->cbc_finalize();
         }
-        free(available);
-        available = NULL;
     }
 }
