@@ -1,9 +1,8 @@
-
 /*
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2009 The University of Tennessee and The University
+ * Copyright (c) 2004-2012 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -11,9 +10,10 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2006-2007 Voltaire. All rights reserved.
- * Copyright (c) 2009      Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2011      Los Alamos National Security, LLC.
- *                         All rights reserved.
+ * Copyright (c) 2009-2010 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2010-2013 Los Alamos National Security, LLC.  
+ *                         All rights reserved. 
+ * Copyright (c) 2010-2012 IBM Corporation.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -36,39 +36,14 @@
 #ifdef HAVE_SCHED_H
 #include <sched.h>
 #endif  /* HAVE_SCHED_H */
-#ifdef HAVE_SYS_IOCTL_H
-#include <sys/ioctl.h>
-#endif  /* HAVE_SYS_IOCTL_H */
-#ifdef HAVE_SYS_MMAN_H
-#include <sys/mman.h>
-#endif  /* HAVE_SYS_MMAN_H */
-#ifdef HAVE_FCNTL_H
-#include <fcntl.h>
-#endif  /* HAVE_FCNTL_H */
 #if OMPI_BTL_SM_HAVE_KNEM
 #include "knem_io.h"
-#endif  /* OMPI_BTL_SM_HAVE_KNEM */
+#endif /* OMPI_BTL_SM_HAVE_KNEM */
 
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif  /* HAVE_SYS_TYPES_H */
-#ifdef HAVE_SYS_SOCKET_H
-#include <sys/socket.h>
-#endif  /* HAVE_SYS_SOCKET_H */
-#ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>
-#endif  /* HAVE_NETINET_IN_H */
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif  /* HAVE_UNISTD_H */
+#include "opal/util/bit_ops.h"
 #include "opal/class/opal_free_list.h"
-#include "ompi/class/ompi_free_list.h"
-#include "opal/event/event.h"
-#include "opal/threads/threads.h"
-#include "ompi/mca/btl/btl.h"
-#include "ompi/mca/btl/base/base.h"
 
-#include "ompi/mca/mpool/mpool.h"
+#include "ompi/mca/btl/btl.h"
 #include "ompi/mca/common/sm/common_sm.h"
 
 BEGIN_C_DECLS
@@ -138,7 +113,7 @@ typedef struct sm_fifo_t sm_fifo_t;
  * Shared Memory resource managment
  */
 
-#if OPAL_ENABLE_PROGRESS_THREADS == 1
+#if OMPI_ENABLE_PROGRESS_THREADS == 1
 #define DATA (char)0
 #define DONE (char)1
 #endif
@@ -155,10 +130,10 @@ struct mca_btl_sm_component_t {
     int sm_free_list_num;              /**< initial size of free lists */
     int sm_free_list_max;              /**< maximum size of free lists */
     int sm_free_list_inc;              /**< number of elements to alloc when growing free lists */
-    int32_t sm_max_procs;              /**< upper limit on the number of processes using the shared memory pool */
+    int sm_max_procs;                  /**< upper limit on the number of processes using the shared memory pool */
     int sm_extra_procs;                /**< number of extra procs to allow */
     char* sm_mpool_name;               /**< name of shared memory pool module */
-    mca_mpool_base_module_t **sm_mpools; /**< shared memory pools (one for each memory node */
+    mca_mpool_base_module_t **sm_mpools; /**< shared memory pools (one for each memory node) */
     mca_mpool_base_module_t *sm_mpool; /**< mpool on local node */
     void* sm_mpool_base;               /**< base address of shared memory pool */
     size_t eager_limit;                /**< first fragment size */
@@ -174,8 +149,8 @@ struct mca_btl_sm_component_t {
                                           but this one, in process private memory, is
                                           a real virtual address */
     uint16_t *mem_nodes;               /**< cached copy of mem nodes of each local rank */
-    size_t fifo_size;                  /**< number of FIFO queue entries */
-    size_t fifo_lazy_free;             /**< number of reads before lazy fifo free is triggered */
+    unsigned int fifo_size;            /**< number of FIFO queue entries */
+    unsigned int fifo_lazy_free;       /**< number of reads before lazy fifo free is triggered */
     int nfifos;                        /**< number of FIFOs per receiver */
     int32_t num_smp_procs;             /**< current number of smp procs on this host */
     int32_t my_smp_rank;               /**< My SMP process rank.  Used for accessing
@@ -194,7 +169,7 @@ struct mca_btl_sm_component_t {
     int mem_node;
     int num_mem_nodes;
     
-#if OPAL_ENABLE_PROGRESS_THREADS == 1
+#if OMPI_ENABLE_PROGRESS_THREADS == 1
     char sm_fifo_path[PATH_MAX];   /**< path to fifo used to signal this process */
     int  sm_fifo_fd;               /**< file descriptor corresponding to opened fifo */
     opal_thread_t sm_fifo_thread;
@@ -207,7 +182,7 @@ struct mca_btl_sm_component_t {
 #if OMPI_BTL_SM_HAVE_KNEM
     /* Knem capabilities info */
     struct knem_cmd_info knem_info;
-#endif
+#endif /* OMPI_BTL_SM_HAVE_KNEM */
 
     /** MCA: should we be using knem or not?  neg=try but continue if
         not available, 0=don't try, 1=try and fail if not available */
@@ -215,7 +190,7 @@ struct mca_btl_sm_component_t {
 
     /** MCA: minimal message size (bytes) to offload on DMA engine
         when using knem */
-    uint32_t knem_dma_min;
+    unsigned int knem_dma_min;
 
     /** MCA: how many simultaneous ongoing knem operations to
         support */
@@ -224,6 +199,16 @@ struct mca_btl_sm_component_t {
     /** If we want DMA and DMA is supported, this will be loaded with
         KNEM_FLAG_DMA.  Otherwise, it'll be 0. */
     int knem_dma_flag;
+
+    /** MCA: should we be using CMA or not?
+        0 = no, 1 = yes */
+    int use_cma;
+
+    /* /// well-known file names for sm and sm mpool init /// */
+    char *sm_mpool_ctl_file_name;
+    char *sm_mpool_rndv_file_name;
+    char *sm_ctl_file_name;
+    char *sm_rndv_file_name;
 };
 typedef struct mca_btl_sm_component_t mca_btl_sm_component_t;
 OMPI_MODULE_DECLSPEC extern mca_btl_sm_component_t mca_btl_sm_component;
@@ -256,15 +241,11 @@ struct mca_btl_sm_t {
 
     /* Number of status items currently in use */
     int knem_status_num_used;
-#endif
+#endif /* OMPI_BTL_SM_HAVE_KNEM */
 };
 typedef struct mca_btl_sm_t mca_btl_sm_t;
 OMPI_MODULE_DECLSPEC extern mca_btl_sm_t mca_btl_sm;
 
-
-
-
-
 struct btl_sm_pending_send_item_t
 {
     opal_free_list_item_t super;
@@ -293,9 +274,7 @@ static inline int sm_fifo_init(int fifo_size, mca_mpool_base_module_t *mpool,
     int i, qsize;
 
     /* figure out the queue size (a power of two that is at least 1) */
-    qsize = 1;
-    while ( qsize < fifo_size )
-        qsize <<= 1;
+    qsize = opal_next_poweroftwo_inclusive (fifo_size);
 
     /* allocate the queue in the receiver's address space */
     fifo->queue_recv = (volatile void **)mpool->mpool_alloc(
@@ -339,7 +318,7 @@ static inline int sm_fifo_write(void *value, sm_fifo_t *fifo)
 
     /* otherwise, write to the slot and advance the head index */
     q[fifo->head] = value;
-    opal_atomic_wmb(); 
+    opal_atomic_wmb();
     fifo->head = (fifo->head + 1) & fifo->mask;
     return OMPI_SUCCESS;
 }
@@ -376,30 +355,6 @@ static inline void *sm_fifo_read(sm_fifo_t *fifo)
     return value;
 }
 
-/**
- * Register shared memory module parameters with the MCA framework
- */
-extern int mca_btl_sm_component_open(void);
-
-/**
- * Any final cleanup before being unloaded.
- */
-extern int mca_btl_sm_component_close(void);
-
-/**
- * SM module initialization.
- *
- * @param num_btls (OUT)                  Number of BTLs returned in BTL array.
- * @param enable_progress_threads (IN)    Flag indicating whether BTL is allowed to have progress threads
- * @param enable_mpi_threads (IN)         Flag indicating whether BTL must support multilple simultaneous invocations from different threads
- *
- */
-extern mca_btl_base_module_t** mca_btl_sm_component_init(
-    int *num_btls,
-    bool enable_progress_threads,
-    bool enable_mpi_threads
-);
-
 /**
  * shared memory component progress.
  */
@@ -545,21 +500,14 @@ extern int mca_btl_sm_send(
     mca_btl_base_tag_t tag
 );
 
-#if OMPI_BTL_SM_HAVE_KNEM
+#if OMPI_BTL_SM_HAVE_KNEM || OMPI_BTL_SM_HAVE_CMA
 /*
- * Synchronous knem get
+ * Synchronous knem/cma get
  */
 extern int mca_btl_sm_get_sync(
 		struct mca_btl_base_module_t* btl,
 		struct mca_btl_base_endpoint_t* endpoint,
 		struct mca_btl_base_descriptor_t* des );
-/*
- * Asynchronous knem get
- */
-extern int mca_btl_sm_get_async(
-		struct mca_btl_base_module_t* btl,
-		struct mca_btl_base_endpoint_t* endpoint,
-		struct mca_btl_base_descriptor_t* des );
 
 extern struct mca_btl_base_descriptor_t* mca_btl_sm_prepare_dst(
 		struct mca_btl_base_module_t* btl,
@@ -570,7 +518,22 @@ extern struct mca_btl_base_descriptor_t* mca_btl_sm_prepare_dst(
 		size_t reserve,
 		size_t* size,
 		uint32_t flags);
-#endif
+#endif /* OMPI_BTL_SM_HAVE_KNEM || OMPI_BTL_SM_HAVE_CMA */
+
+#if OMPI_BTL_SM_HAVE_KNEM
+/*
+ * Asynchronous knem get
+ */
+extern int mca_btl_sm_get_async(
+                struct mca_btl_base_module_t* btl,
+                struct mca_btl_base_endpoint_t* endpoint,
+                struct mca_btl_base_descriptor_t* des );
+
+#endif /* OMPI_BTL_SM_HAVE_KNEM */
+
+extern void mca_btl_sm_dump(struct mca_btl_base_module_t* btl,
+                            struct mca_btl_base_endpoint_t* endpoint,
+                            int verbose);
 
 /**
  * Fault Tolerance Event Notification Function
@@ -579,11 +542,11 @@ extern struct mca_btl_base_descriptor_t* mca_btl_sm_prepare_dst(
  */
 int mca_btl_sm_ft_event(int state);
 
-#if OPAL_ENABLE_PROGRESS_THREADS == 1
+#if OMPI_ENABLE_PROGRESS_THREADS == 1
 void mca_btl_sm_component_event_thread(opal_object_t*);
 #endif
 
-#if OPAL_ENABLE_PROGRESS_THREADS == 1
+#if OMPI_ENABLE_PROGRESS_THREADS == 1
 #define MCA_BTL_SM_SIGNAL_PEER(peer) \
 { \
     unsigned char cmd = DATA; \
