@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2009 The University of Tennessee and The University
+ * Copyright (c) 2004-2013 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -23,9 +23,7 @@
 #ifndef MCA_BTL_SM_SEND_FRAG_H
 #define MCA_BTL_SM_SEND_FRAG_H
 
-#include <sys/types.h>
 #include "ompi_config.h"
-#include "ompi/class/ompi_free_list.h"
 #include "btl_sm.h"
 
 
@@ -47,12 +45,20 @@ struct mca_btl_sm_hdr_t {
 };
 typedef struct mca_btl_sm_hdr_t mca_btl_sm_hdr_t;
 
+struct mca_btl_sm_segment_t {
+    mca_btl_base_segment_t base;
+#if OMPI_BTL_SM_HAVE_KNEM || OMPI_BTL_SM_HAVE_CMA
+    uint64_t key;
+#endif /* OMPI_BTL_SM_HAVE_KNEM || OMPI_BTL_SM_HAVE_CMA */
+};
+typedef struct mca_btl_sm_segment_t mca_btl_sm_segment_t;
+
 /**
  * shared memory send fragment derived type.
  */
 struct mca_btl_sm_frag_t {
     mca_btl_base_descriptor_t base;
-    mca_btl_base_segment_t segment;
+    mca_btl_sm_segment_t segment;
     struct mca_btl_base_endpoint_t *endpoint;
     size_t size;
     /* pointer written to the FIFO, this is the base of the shared memory region */
@@ -70,30 +76,30 @@ OBJ_CLASS_DECLARATION(mca_btl_sm_frag1_t);
 OBJ_CLASS_DECLARATION(mca_btl_sm_frag2_t);
 OBJ_CLASS_DECLARATION(mca_btl_sm_user_t);
 
-#define MCA_BTL_SM_FRAG_ALLOC_EAGER(frag, rc)                           \
+#define MCA_BTL_SM_FRAG_ALLOC_EAGER(frag)                               \
 {                                                                       \
     ompi_free_list_item_t* item;                                        \
-    OMPI_FREE_LIST_GET(&mca_btl_sm_component.sm_frags_eager, item, rc); \
+    OMPI_FREE_LIST_GET_MT(&mca_btl_sm_component.sm_frags_eager, item);     \
     frag = (mca_btl_sm_frag_t*)item;                                    \
 }
 
-#define MCA_BTL_SM_FRAG_ALLOC_MAX(frag, rc)                             \
+#define MCA_BTL_SM_FRAG_ALLOC_MAX(frag)                                 \
 {                                                                       \
     ompi_free_list_item_t* item;                                        \
-    OMPI_FREE_LIST_GET(&mca_btl_sm_component.sm_frags_max, item, rc);   \
+    OMPI_FREE_LIST_GET_MT(&mca_btl_sm_component.sm_frags_max, item);       \
     frag = (mca_btl_sm_frag_t*)item;                                    \
 }
 
-#define MCA_BTL_SM_FRAG_ALLOC_USER(frag, rc)                             \
+#define MCA_BTL_SM_FRAG_ALLOC_USER(frag)                                \
 {                                                                       \
 	ompi_free_list_item_t* item;                                        \
-	OMPI_FREE_LIST_GET(&mca_btl_sm_component.sm_frags_user, item, rc);   \
+	OMPI_FREE_LIST_GET_MT(&mca_btl_sm_component.sm_frags_user, item);      \
 	frag = (mca_btl_sm_frag_t*)item;                                    \
 }
 
 
 #define MCA_BTL_SM_FRAG_RETURN(frag)                                      \
 {                                                                         \
-    OMPI_FREE_LIST_RETURN(frag->my_list, (ompi_free_list_item_t*)(frag)); \
+    OMPI_FREE_LIST_RETURN_MT(frag->my_list, (ompi_free_list_item_t*)(frag)); \
 }
 #endif
