@@ -1,8 +1,9 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2009 The University of Tennessee and The University
+ * Copyright (c) 2004-2014 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -12,6 +13,9 @@
  * Copyright (c) 2007-2012 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
  * Copyright (c) 2009      Oak Ridge National Laboratory
+ * Copyright (c) 2012-2013 Los Alamos National Security, LLC.  All rights
+ *                         reserved. 
+ * Copyright (c) 2013      NVIDIA Corporation.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -40,7 +44,7 @@
 #ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
 #endif
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
 #  ifdef HAVE_NETDB_H
 #  include <netdb.h>
 #  endif
@@ -48,16 +52,12 @@
 #include <ctype.h>
 #include <limits.h>
 
-#include "opal/event/event.h"
+#include "opal/mca/event/event.h"
 #include "opal/util/if.h"
 #include "opal/util/output.h"
 #include "opal/util/argv.h"
 #include "opal/util/net.h"
-#include "opal/mca/base/mca_base_param.h"
-
-#include "orte/types.h"
-#include "orte/util/show_help.h"
-#include "orte/mca/ess/ess.h"
+#include "opal/util/show_help.h"
 
 #include "ompi/constants.h"
 #include "ompi/mca/btl/btl.h"
@@ -70,6 +70,9 @@
 #include "btl_tcp_proc.h"
 #include "btl_tcp_frag.h"
 #include "btl_tcp_endpoint.h" 
+#if OPAL_CUDA_SUPPORT
+#include "ompi/mca/common/cuda/common_cuda.h"
+#endif /* OPAL_CUDA_SUPPORT */
 
 
 /* 
@@ -79,6 +82,7 @@ static int mca_btl_tcp_component_register(void);
 static int mca_btl_tcp_component_open(void);
 static int mca_btl_tcp_component_close(void);
 
+static char *mca_btl_tcp_if_seq_string;
 
 mca_btl_tcp_component_t mca_btl_tcp_component = {
     {
@@ -114,25 +118,46 @@ mca_btl_tcp_component_t mca_btl_tcp_component = {
 static inline char* mca_btl_tcp_param_register_string(
         const char* param_name, 
         const char* help_string,
-        const char* default_value)
+        const char* default_value,
+        int level,
+        char **storage)
 {
-    char *value;
-    mca_base_param_reg_string(&mca_btl_tcp_component.super.btl_version,
-                              param_name, help_string, false, false,
-                              default_value, &value);
-    return value;
+    *storage = (char *) default_value;
+    (void) mca_base_component_var_register(&mca_btl_tcp_component.super.btl_version,
+                                           param_name, help_string, MCA_BASE_VAR_TYPE_STRING,
+                                           NULL, 0, 0, level,
+                                           MCA_BASE_VAR_SCOPE_READONLY, storage);
+    return *storage;
 }
 
 static inline int mca_btl_tcp_param_register_int(
         const char* param_name, 
         const char* help_string,
-        int default_value)
+        int default_value,
+        int level,
+        int *storage)
 {
-    int value;
-    mca_base_param_reg_int(&mca_btl_tcp_component.super.btl_version,
-                           param_name, help_string, false, false, 
-                           default_value, &value);
-    return value;
+    *storage = default_value;
+    (void) mca_base_component_var_register(&mca_btl_tcp_component.super.btl_version,
+                                           param_name, help_string, MCA_BASE_VAR_TYPE_INT,
+                                           NULL, 0, 0, level,
+                                           MCA_BASE_VAR_SCOPE_READONLY, storage);
+    return *storage;
+}
+
+static inline unsigned int mca_btl_tcp_param_register_uint(
+        const char* param_name, 
+        const char* help_string,
+        unsigned int default_value,
+        int level,
+        unsigned int *storage)
+{
+    *storage = default_value;
+    (void) mca_base_component_var_register(&mca_btl_tcp_component.super.btl_version,
+                                           param_name, help_string, MCA_BASE_VAR_TYPE_UNSIGNED_INT,
+                                           NULL, 0, 0, level,
+                                           MCA_BASE_VAR_SCOPE_READONLY, storage);
+    return *storage;
 }
 
 
@@ -146,25 +171,8 @@ struct mca_btl_tcp_event_t {
 };
 typedef struct mca_btl_tcp_event_t mca_btl_tcp_event_t;
 
-static void mca_btl_tcp_event_construct(mca_btl_tcp_event_t* event)
-{
-    OPAL_THREAD_LOCK(&mca_btl_tcp_component.tcp_lock);
-    opal_list_append(&mca_btl_tcp_component.tcp_events, &event->item);
-    OPAL_THREAD_UNLOCK(&mca_btl_tcp_component.tcp_lock);
-}
-
-static void mca_btl_tcp_event_destruct(mca_btl_tcp_event_t* event)
-{
-    OPAL_THREAD_LOCK(&mca_btl_tcp_component.tcp_lock);
-    opal_list_remove_item(&mca_btl_tcp_component.tcp_events, &event->item);
-    OPAL_THREAD_UNLOCK(&mca_btl_tcp_component.tcp_lock);
-}
-
-OBJ_CLASS_INSTANCE(
-    mca_btl_tcp_event_t,
-    opal_list_item_t,
-    mca_btl_tcp_event_construct,
-    mca_btl_tcp_event_destruct);
+OBJ_CLASS_INSTANCE( mca_btl_tcp_event_t, opal_list_item_t,
+                    NULL, NULL);
 
 
 /*
@@ -173,6 +181,25 @@ OBJ_CLASS_INSTANCE(
 static void mca_btl_tcp_component_recv_handler(int, short, void*);
 static void mca_btl_tcp_component_accept_handler(int, short, void*);
 
+static int mca_btl_tcp_component_verify(void)
+{
+    if( mca_btl_tcp_component.tcp_port_min > USHRT_MAX ) {
+        opal_show_help("help-mpi-btl-tcp.txt", "invalid minimum port",
+                       true, "v4", ompi_process_info.nodename,
+                       mca_btl_tcp_component.tcp_port_min );
+        mca_btl_tcp_component.tcp_port_min = 1024;
+    }
+#if OPAL_ENABLE_IPV6
+    if( mca_btl_tcp_component.tcp6_port_min > USHRT_MAX ) {
+        opal_show_help("help-mpi-btl-tcp.txt", "invalid minimum port",
+                       true, "v6", ompi_process_info.nodename,
+                       mca_btl_tcp_component.tcp6_port_min );
+        mca_btl_tcp_component.tcp6_port_min = 1024;
+    }
+#endif
+
+    return OMPI_SUCCESS;
+}
 
 /*
  *  Called by MCA framework to open the component, registers
@@ -184,76 +211,59 @@ static int mca_btl_tcp_component_register(void)
     char* message;
 
     /* register TCP component parameters */
-    mca_btl_tcp_component.tcp_num_links =
-        mca_btl_tcp_param_register_int("links", NULL, 1);
-    mca_btl_tcp_component.tcp_if_include =
-        mca_btl_tcp_param_register_string("if_include", "Comma-delimited list of devices or CIDR notation of networks to use for MPI communication (e.g., \"eth0,eth1\" or \"192.168.0.0/16,10.1.4.0/24\").  Mutually exclusive with btl_tcp_if_exclude.", "");
-    mca_btl_tcp_component.tcp_if_exclude =
-        mca_btl_tcp_param_register_string("if_exclude", "Comma-delimited list of devices and/or CIDR notation of networks to NOT use for MPI communication -- all devices not matching these specifications will be used (e.g., \"eth0,192.168.0.0/16\").  If set to a non-default value, it is mutually exclusive with btl_tcp_if_include.", 
-#ifndef __WINDOWS__
-        "127.0.0.1/8,sppp"
-#else
-        ""
-#endif
-        );
+    mca_btl_tcp_param_register_uint("links", NULL, 1, OPAL_INFO_LVL_4, &mca_btl_tcp_component.tcp_num_links);
+    mca_btl_tcp_param_register_string("if_include", "Comma-delimited list of devices and/or CIDR notation of networks to use for MPI communication (e.g., \"eth0,192.168.0.0/16\").  Mutually exclusive with btl_tcp_if_exclude.", "", OPAL_INFO_LVL_1, &mca_btl_tcp_component.tcp_if_include);
+    mca_btl_tcp_param_register_string("if_exclude", "Comma-delimited list of devices and/or CIDR notation of networks to NOT use for MPI communication -- all devices not matching these specifications will be used (e.g., \"eth0,192.168.0.0/16\").  If set to a non-default value, it is mutually exclusive with btl_tcp_if_include.", 
+                                      "127.0.0.1/8,sppp",
+                                      OPAL_INFO_LVL_1, &mca_btl_tcp_component.tcp_if_exclude);
+
+    mca_btl_tcp_param_register_int ("free_list_num", NULL, 8, OPAL_INFO_LVL_5,  &mca_btl_tcp_component.tcp_free_list_num);
+    mca_btl_tcp_param_register_int ("free_list_max", NULL, -1, OPAL_INFO_LVL_5,  &mca_btl_tcp_component.tcp_free_list_max);
+    mca_btl_tcp_param_register_int ("free_list_inc", NULL, 32, OPAL_INFO_LVL_5,  &mca_btl_tcp_component.tcp_free_list_inc);
+    mca_btl_tcp_param_register_int ("sndbuf", NULL, 128*1024, OPAL_INFO_LVL_4, &mca_btl_tcp_component.tcp_sndbuf);
+    mca_btl_tcp_param_register_int ("rcvbuf", NULL, 128*1024, OPAL_INFO_LVL_4, &mca_btl_tcp_component.tcp_rcvbuf);
+    mca_btl_tcp_param_register_int ("endpoint_cache",
+        "The size of the internal cache for each TCP connection. This cache is"
+        " used to reduce the number of syscalls, by replacing them with memcpy."
+        " Every read will read the expected data plus the amount of the"
+                                    " endpoint_cache", 30*1024, OPAL_INFO_LVL_4, &mca_btl_tcp_component.tcp_endpoint_cache);
+    mca_btl_tcp_param_register_int ("use_nagle", "Whether to use Nagle's algorithm or not (using Nagle's algorithm may increase short message latency)", 0, OPAL_INFO_LVL_4, &mca_btl_tcp_component.tcp_not_use_nodelay);
+    mca_btl_tcp_param_register_int( "port_min_v4", 
+                                    "The minimum port where the TCP BTL will try to bind (default 1024)",
+                                    1024, OPAL_INFO_LVL_2, &mca_btl_tcp_component.tcp_port_min);
 
-    mca_btl_tcp_component.tcp_free_list_num =
-        mca_btl_tcp_param_register_int ("free_list_num", NULL, 8);
-    mca_btl_tcp_component.tcp_free_list_max =
-        mca_btl_tcp_param_register_int ("free_list_max", NULL, -1);
-    mca_btl_tcp_component.tcp_free_list_inc =
-        mca_btl_tcp_param_register_int ("free_list_inc", NULL, 32);
-    mca_btl_tcp_component.tcp_sndbuf =
-        mca_btl_tcp_param_register_int ("sndbuf", NULL, 128*1024);
-    mca_btl_tcp_component.tcp_rcvbuf =
-        mca_btl_tcp_param_register_int ("rcvbuf", NULL, 128*1024);
-    mca_btl_tcp_component.tcp_endpoint_cache =
-        mca_btl_tcp_param_register_int ("endpoint_cache",
-            "The size of the internal cache for each TCP connection. This cache is"
-            " used to reduce the number of syscalls, by replacing them with memcpy."
-            " Every read will read the expected data plus the amount of the"
-            " endpoint_cache", 30*1024);
-    mca_btl_tcp_component.tcp_use_nodelay =
-        !mca_btl_tcp_param_register_int ("use_nagle", "Whether to use Nagle's algorithm or not (using Nagle's algorithm may increase short message latency)", 0);
-    mca_btl_tcp_component.tcp_port_min =
-        mca_btl_tcp_param_register_int( "port_min_v4",
-            "The minimum port where the TCP BTL will try to bind (default 1024)", 1024 );
-    if( mca_btl_tcp_component.tcp_port_min > USHRT_MAX ) {
-        orte_show_help("help-mpi-btl-tcp.txt", "invalid minimum port",
-                       true, "v4", orte_process_info.nodename,
-                       mca_btl_tcp_component.tcp_port_min );
-        mca_btl_tcp_component.tcp_port_min = 1024;
-    }
     asprintf( &message, 
               "The number of ports where the TCP BTL will try to bind (default %d)."
               " This parameter together with the port min, define a range of ports"
               " where Open MPI will open sockets.",
               (0x1 << 16) - mca_btl_tcp_component.tcp_port_min - 1 );
-    mca_btl_tcp_component.tcp_port_range =
-        mca_btl_tcp_param_register_int( "port_range_v4", message,
-                                        (0x1 << 16) - mca_btl_tcp_component.tcp_port_min - 1);
+    mca_btl_tcp_param_register_int( "port_range_v4", message,
+                                    (0x1 << 16) - mca_btl_tcp_component.tcp_port_min - 1,
+                                    OPAL_INFO_LVL_2, &mca_btl_tcp_component.tcp_port_range);
     free(message);
-#if OPAL_WANT_IPV6
-    mca_btl_tcp_component.tcp6_port_min =
-        mca_btl_tcp_param_register_int( "port_min_v6",
-            "The minimum port where the TCP BTL will try to bind (default 1024)", 1024 );
-    if( mca_btl_tcp_component.tcp6_port_min > USHRT_MAX ) {
-        orte_show_help("help-mpi-btl-tcp.txt", "invalid minimum port",
-                       true, "v6", orte_process_info.nodename,
-                       mca_btl_tcp_component.tcp6_port_min );
-        mca_btl_tcp_component.tcp6_port_min = 1024;
-    }
+#if OPAL_ENABLE_IPV6
+    mca_btl_tcp_param_register_int( "port_min_v6",
+                                    "The minimum port where the TCP BTL will try to bind (default 1024)", 1024,
+                                    OPAL_INFO_LVL_2, & mca_btl_tcp_component.tcp6_port_min );
     asprintf( &message, 
               "The number of ports where the TCP BTL will try to bind (default %d)."
               " This parameter together with the port min, define a range of ports"
               " where Open MPI will open sockets.",
               (0x1 << 16) - mca_btl_tcp_component.tcp6_port_min - 1 );
-    mca_btl_tcp_component.tcp6_port_range =
-        mca_btl_tcp_param_register_int( "port_range_v6", message,
-                                        (0x1 << 16) - mca_btl_tcp_component.tcp6_port_min - 1);
+    mca_btl_tcp_param_register_int( "port_range_v6", message,
+                                    (0x1 << 16) - mca_btl_tcp_component.tcp6_port_min - 1,
+                                    OPAL_INFO_LVL_2, &mca_btl_tcp_component.tcp6_port_range );
     free(message);
 #endif
 
+    mca_btl_tcp_component.report_all_unfound_interfaces = false;
+    (void) mca_base_component_var_register(&mca_btl_tcp_component.super.btl_version,
+                                           "warn_all_unfound_interfaces",
+                                           "Issue a warning for all unfound interfaces included in if_exclude",
+                                           MCA_BASE_VAR_TYPE_BOOL,
+                                           NULL, 0, 0, OPAL_INFO_LVL_2,
+                                           MCA_BASE_VAR_SCOPE_READONLY, &mca_btl_tcp_component.report_all_unfound_interfaces);
+
     mca_btl_tcp_module.super.btl_exclusivity =  MCA_BTL_EXCLUSIVITY_LOW + 100;
     mca_btl_tcp_module.super.btl_eager_limit = 64*1024;
     mca_btl_tcp_module.super.btl_rndv_eager_limit = 64*1024;
@@ -266,28 +276,29 @@ static int mca_btl_tcp_component_register(void)
                                        MCA_BTL_FLAGS_NEED_CSUM |
                                        MCA_BTL_FLAGS_NEED_ACK |
                                        MCA_BTL_FLAGS_HETEROGENEOUS_RDMA;
+    mca_btl_tcp_module.super.btl_seg_size = sizeof (mca_btl_base_segment_t);
     mca_btl_tcp_module.super.btl_bandwidth = 100;
     mca_btl_tcp_module.super.btl_latency = 100;
 
     mca_btl_base_param_register(&mca_btl_tcp_component.super.btl_version,
                                 &mca_btl_tcp_module.super);
 
-    mca_btl_tcp_component.tcp_disable_family =
-        mca_btl_tcp_param_register_int ("disable_family", NULL, 0);
+    mca_btl_tcp_param_register_int ("disable_family", NULL, 0, OPAL_INFO_LVL_2,  &mca_btl_tcp_component.tcp_disable_family);
 
     /* Register a list of interfaces to use in sequence */
-    message = mca_btl_tcp_param_register_string("if_seq", 
-                                                "If specified, a comma-delimited list of TCP interfaces.  Interfaces will be assigned, one to each MPI process, in a round-robin fashion on each server.  For example, if the list is \"eth0,eth1\" and four MPI processes are run on a single server, then local ranks 0 and 2 will use eth0 and local ranks 1 and 3 will use eth1.", NULL);
+    mca_btl_tcp_param_register_string("if_seq", 
+                                      "If specified, a comma-delimited list of TCP interfaces.  Interfaces will be assigned, one to each MPI process, in a round-robin fashion on each server.  For example, if the list is \"eth0,eth1\" and four MPI processes are run on a single server, then local ranks 0 and 2 will use eth0 and local ranks 1 and 3 will use eth1.", NULL, OPAL_INFO_LVL_9, &mca_btl_tcp_if_seq_string);
+
     mca_btl_tcp_component.tcp_if_seq = NULL;
-    if (NULL != message && '\0' != *message) {
-        char **argv = opal_argv_split(message, ',');
+    if (NULL != mca_btl_tcp_if_seq_string && '\0' != *mca_btl_tcp_if_seq_string) {
+        char **argv = opal_argv_split(mca_btl_tcp_if_seq_string, ',');
 
         if (NULL != argv && '\0' != *(argv[0])) {
             int if_index, rc, count;
-            orte_node_rank_t node_rank;
+            ompi_node_rank_t node_rank;
             char name[256];
 
-            node_rank = orte_ess.get_node_rank(ORTE_PROC_MY_NAME);
+            node_rank = ompi_process_info.my_node_rank;
 
             /* Now that we've got that local rank, take the
                corresponding entry from the tcp_if_seq list (wrapping
@@ -309,35 +320,33 @@ static int mca_btl_tcp_component_register(void)
                 }
             }
             if (if_index < 0) {
-                orte_show_help("help-mpi-btl-tcp.txt", 
+                opal_show_help("help-mpi-btl-tcp.txt", 
                                "invalid if_inexclude",
                                true, "if_seq",
-                               orte_process_info.nodename,
+                               ompi_process_info.nodename,
                                mca_btl_tcp_component.tcp_if_seq,
                                "Interface does not exist");
-                return OMPI_ERR_BAD_PARAM;
+                free(mca_btl_tcp_component.tcp_if_seq);
+                mca_btl_tcp_component.tcp_if_seq = NULL;
+            } else {
+                BTL_VERBOSE(("Node rank %d using TCP interface %s",
+                             node_rank, mca_btl_tcp_component.tcp_if_seq));
             }
-            BTL_VERBOSE(("Node rank %d using TCP interface %s",
-                         node_rank, mca_btl_tcp_component.tcp_if_seq));
         }
     }
 
-    return OMPI_SUCCESS;
+    return mca_btl_tcp_component_verify();
 }
 
 static int mca_btl_tcp_component_open(void)
 {
-#ifdef __WINDOWS__
-    WSADATA win_sock_data;
-    if( WSAStartup(MAKEWORD(2,2), &win_sock_data) != 0 ) {
-        BTL_ERROR(("failed to initialise windows sockets:%d", WSAGetLastError()));
+    if (OMPI_SUCCESS != mca_btl_tcp_component_verify()) {
         return OMPI_ERROR;
     }
-#endif
 
     /* initialize state */
     mca_btl_tcp_component.tcp_listen_sd = -1;
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
     mca_btl_tcp_component.tcp6_listen_sd = -1;
 #endif
     mca_btl_tcp_component.tcp_num_btls=0;
@@ -347,7 +356,6 @@ static int mca_btl_tcp_component_open(void)
     /* initialize objects */ 
     OBJ_CONSTRUCT(&mca_btl_tcp_component.tcp_lock, opal_mutex_t);
     OBJ_CONSTRUCT(&mca_btl_tcp_component.tcp_procs, opal_hash_table_t);
-    OBJ_CONSTRUCT(&mca_btl_tcp_component.tcp_events, opal_list_t);
     OBJ_CONSTRUCT(&mca_btl_tcp_component.tcp_frag_eager, ompi_free_list_t);
     OBJ_CONSTRUCT(&mca_btl_tcp_component.tcp_frag_max, ompi_free_list_t);
     OBJ_CONSTRUCT(&mca_btl_tcp_component.tcp_frag_user, ompi_free_list_t);
@@ -355,7 +363,7 @@ static int mca_btl_tcp_component_open(void)
 
     /* if_include and if_exclude need to be mutually exclusive */
     if (OPAL_SUCCESS != 
-        mca_base_param_check_exclusive_string(
+        mca_base_var_check_exclusive("ompi",
         mca_btl_tcp_component.super.btl_version.mca_type_name,
         mca_btl_tcp_component.super.btl_version.mca_component_name,
         "if_include",
@@ -377,30 +385,19 @@ static int mca_btl_tcp_component_open(void)
 
 static int mca_btl_tcp_component_close(void)
 {
-    opal_list_item_t* item;
-    opal_list_item_t* next;
-
-    if(NULL != mca_btl_tcp_component.tcp_if_include) {
-        free(mca_btl_tcp_component.tcp_if_include);
-        mca_btl_tcp_component.tcp_if_include = NULL;
-    }
-    if(NULL != mca_btl_tcp_component.tcp_if_exclude) {
-       free(mca_btl_tcp_component.tcp_if_exclude);
-       mca_btl_tcp_component.tcp_if_exclude = NULL;
-    }
     if (NULL != mca_btl_tcp_component.tcp_if_seq) {
         free(mca_btl_tcp_component.tcp_if_seq);
     }
 
     if (NULL != mca_btl_tcp_component.tcp_btls)
         free(mca_btl_tcp_component.tcp_btls);
- 
+  
     if (mca_btl_tcp_component.tcp_listen_sd >= 0) {
         opal_event_del(&mca_btl_tcp_component.tcp_recv_event);
         CLOSE_THE_SOCKET(mca_btl_tcp_component.tcp_listen_sd);
         mca_btl_tcp_component.tcp_listen_sd = -1;
     }
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
     if (mca_btl_tcp_component.tcp6_listen_sd >= 0) {
         opal_event_del(&mca_btl_tcp_component.tcp6_recv_event);
         CLOSE_THE_SOCKET(mca_btl_tcp_component.tcp6_listen_sd);
@@ -408,29 +405,16 @@ static int mca_btl_tcp_component_close(void)
     }
 #endif
 
-    /* cleanup any pending events */
-    OPAL_THREAD_LOCK(&mca_btl_tcp_component.tcp_lock);
-    for(item =  opal_list_get_first(&mca_btl_tcp_component.tcp_events);
-        item != opal_list_get_end(&mca_btl_tcp_component.tcp_events); 
-        item = next) {
-        mca_btl_tcp_event_t* event = (mca_btl_tcp_event_t*)item;
-        next = opal_list_get_next(item);
-        opal_event_del(&event->event);
-        OBJ_RELEASE(event);
-    }
-    OPAL_THREAD_UNLOCK(&mca_btl_tcp_component.tcp_lock);
-
     /* release resources */
     OBJ_DESTRUCT(&mca_btl_tcp_component.tcp_procs);
-    OBJ_DESTRUCT(&mca_btl_tcp_component.tcp_events);
     OBJ_DESTRUCT(&mca_btl_tcp_component.tcp_frag_eager);
     OBJ_DESTRUCT(&mca_btl_tcp_component.tcp_frag_max);
     OBJ_DESTRUCT(&mca_btl_tcp_component.tcp_frag_user);
     OBJ_DESTRUCT(&mca_btl_tcp_component.tcp_lock);
 
-#ifdef __WINDOWS__
-    WSACleanup();
-#endif
+#if OPAL_CUDA_SUPPORT
+    mca_common_cuda_fini();
+#endif /* OPAL_CUDA_SUPPORT */
 
     return OMPI_SUCCESS;
 }
@@ -464,11 +448,11 @@ static int mca_btl_tcp_create(int if_kindex, const char* if_name)
 
         /* allow user to specify interface bandwidth */
         sprintf(param, "bandwidth_%s", if_name);
-        btl->super.btl_bandwidth = mca_btl_tcp_param_register_int(param, NULL, btl->super.btl_bandwidth);
+        mca_btl_tcp_param_register_uint(param, NULL, btl->super.btl_bandwidth, OPAL_INFO_LVL_5, &btl->super.btl_bandwidth);
 
         /* allow user to override/specify latency ranking */
         sprintf(param, "latency_%s", if_name);
-        btl->super.btl_latency = mca_btl_tcp_param_register_int(param, NULL, btl->super.btl_latency);
+        mca_btl_tcp_param_register_uint(param, NULL, btl->super.btl_latency, OPAL_INFO_LVL_5,  &btl->super.btl_latency);
         if( i > 0 ) {
             btl->super.btl_bandwidth >>= 1;
             btl->super.btl_latency   <<= 1;
@@ -476,11 +460,11 @@ static int mca_btl_tcp_create(int if_kindex, const char* if_name)
 
         /* allow user to specify interface bandwidth */
         sprintf(param, "bandwidth_%s:%d", if_name, i);
-        btl->super.btl_bandwidth = mca_btl_tcp_param_register_int(param, NULL, btl->super.btl_bandwidth);
+        mca_btl_tcp_param_register_uint(param, NULL, btl->super.btl_bandwidth, OPAL_INFO_LVL_5, &btl->super.btl_bandwidth);
 
         /* allow user to override/specify latency ranking */
         sprintf(param, "latency_%s:%d", if_name, i);
-        btl->super.btl_latency = mca_btl_tcp_param_register_int(param, NULL, btl->super.btl_latency);
+        mca_btl_tcp_param_register_uint(param, NULL, btl->super.btl_latency, OPAL_INFO_LVL_5, &btl->super.btl_latency);
 #if 0 && OPAL_ENABLE_DEBUG
         BTL_OUTPUT(("interface %s instance %i: bandwidth %d latency %d\n", if_name, i,
                     btl->super.btl_bandwidth, btl->super.btl_latency));
@@ -494,7 +478,7 @@ static int mca_btl_tcp_create(int if_kindex, const char* if_name)
  * (a.b.c.d/e), resolve them to an interface name (Currently only
  * supporting IPv4).  If unresolvable, warn and remove.
  */
-static char **split_and_resolve(char **orig_str, char *name)
+static char **split_and_resolve(char **orig_str, char *name, bool reqd)
 {
     int i, ret, save, if_index;
     char **argv, *str, *tmp;
@@ -523,8 +507,8 @@ static char **split_and_resolve(char **orig_str, char *name)
         tmp = strdup(argv[i]);
         str = strchr(argv[i], '/');
         if (NULL == str) {
-            orte_show_help("help-mpi-btl-tcp.txt", "invalid if_inexclude",
-                           true, name, orte_process_info.nodename, 
+            opal_show_help("help-mpi-btl-tcp.txt", "invalid if_inexclude",
+                           true, name, ompi_process_info.nodename, 
                            tmp, "Invalid specification (missing \"/\")");
             free(argv[i]);
             free(tmp);
@@ -540,13 +524,13 @@ static char **split_and_resolve(char **orig_str, char *name)
         free(argv[i]);
 
         if (1 != ret) {
-            orte_show_help("help-mpi-btl-tcp.txt", "invalid if_inexclude",
-                           true, name, orte_process_info.nodename, tmp,
+            opal_show_help("help-mpi-btl-tcp.txt", "invalid if_inexclude",
+                           true, name, ompi_process_info.nodename, tmp,
                            "Invalid specification (inet_pton() failed)");
             free(tmp);
             continue;
         }
-        opal_output_verbose(20, mca_btl_base_output, 
+        opal_output_verbose(20, ompi_btl_base_framework.framework_output, 
                             "btl: tcp: Searching for %s address+prefix: %s / %u",
                             name,
                             opal_net_get_hostname((struct sockaddr*) &argv_inaddr),
@@ -567,9 +551,11 @@ static char **split_and_resolve(char **orig_str, char *name)
         
         /* If we didn't find a match, keep trying */
         if (if_index < 0) {
-            orte_show_help("help-mpi-btl-tcp.txt", "invalid if_inexclude",
-                           true, name, orte_process_info.nodename, tmp,
-                           "Did not find interface matching this subnet");
+            if (reqd || mca_btl_tcp_component.report_all_unfound_interfaces) {
+                opal_show_help("help-mpi-btl-tcp.txt", "invalid if_inexclude",
+                               true, name, ompi_process_info.nodename, tmp,
+                               "Did not find interface matching this subnet");
+            }
             free(tmp);
             continue;
         }
@@ -577,7 +563,7 @@ static char **split_and_resolve(char **orig_str, char *name)
         /* We found a match; get the name and replace it in the
            argv */
         opal_ifindextoname(if_index, if_name, sizeof(if_name));
-        opal_output_verbose(20, mca_btl_base_output, 
+        opal_output_verbose(20, ompi_btl_base_framework.framework_output, 
                             "btl: tcp: Found match: %s (%s)",
                             opal_net_get_hostname((struct sockaddr*) &if_inaddr),
                             if_name);
@@ -673,7 +659,7 @@ static int mca_btl_tcp_component_create_instances(void)
 
     /* if the user specified an interface list - use these exclusively */
     argv = include = split_and_resolve(&mca_btl_tcp_component.tcp_if_include,
-                                       "include");
+                                       "include", true);
     while(argv && *argv) {
         char* if_name = *argv;
         int if_index = opal_ifnametokindex(if_name);
@@ -698,7 +684,7 @@ static int mca_btl_tcp_component_create_instances(void)
      * a BTL for each interface that was not excluded.
     */
     exclude = split_and_resolve(&mca_btl_tcp_component.tcp_if_exclude,
-                                "exclude");
+                                "exclude", false);
     {
         int i;
         for(i = 0; i < kif_count; i++) {
@@ -711,7 +697,7 @@ static int mca_btl_tcp_component_create_instances(void)
             /* check to see if this interface exists in the exclude list */
             argv = exclude;
             while(argv && *argv) {
-                if(strcmp(*argv,if_name) == 0)
+                if(strncmp(*argv,if_name,strlen(*argv)) == 0)
                     break;
                 argv++;
             }
@@ -753,7 +739,7 @@ static int mca_btl_tcp_component_create_listen(uint16_t af_family)
 
     mca_btl_tcp_set_socket_options(sd);
 
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
     {
         struct addrinfo hints, *res = NULL;
         int error;
@@ -809,19 +795,19 @@ static int mca_btl_tcp_component_create_listen(uint16_t af_family)
         
         range = mca_btl_tcp_component.tcp_port_range;
         port = mca_btl_tcp_component.tcp_port_min;
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
         if (AF_INET6 == af_family) {
             range = mca_btl_tcp_component.tcp6_port_range;
             port = mca_btl_tcp_component.tcp6_port_min;
         }
-#endif  /* OPAL_WANT_IPV6 */
+#endif  /* OPAL_ENABLE_IPV6 */
 
         for( index = 0;  index < range; index++ ) {
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
             ((struct sockaddr_in6*) &inaddr)->sin6_port = htons(port + index);
 #else
             ((struct sockaddr_in*) &inaddr)->sin_port = htons(port + index);
-#endif  /* OPAL_WANT_IPV6 */
+#endif  /* OPAL_ENABLE_IPV6 */
             if(bind(sd, (struct sockaddr*)&inaddr, addrlen) < 0) {
                 if( (EADDRINUSE == opal_socket_errno) || (EADDRNOTAVAIL == opal_socket_errno) ) {
                     continue;
@@ -838,13 +824,13 @@ static int mca_btl_tcp_component_create_listen(uint16_t af_family)
                        mca_btl_tcp_component.tcp_port_min,
                        mca_btl_tcp_component.tcp_port_min + range));
         }
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
         if (AF_INET6 == af_family) {
             BTL_ERROR(("bind6() failed: no port available in the range [%d..%d]",
                        mca_btl_tcp_component.tcp6_port_min,
                        mca_btl_tcp_component.tcp6_port_min + range));
         }
-#endif  /* OPAL_WANT_IPV6 */
+#endif  /* OPAL_ENABLE_IPV6 */
         CLOSE_THE_SOCKET(sd);
         return OMPI_ERROR;
     }
@@ -861,7 +847,7 @@ static int mca_btl_tcp_component_create_listen(uint16_t af_family)
         mca_btl_tcp_component.tcp_listen_port = ((struct sockaddr_in*) &inaddr)->sin_port;
         mca_btl_tcp_component.tcp_listen_sd = sd;
     }
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
     if (AF_INET6 == af_family) {
         mca_btl_tcp_component.tcp6_listen_port = ((struct sockaddr_in6*) &inaddr)->sin6_port;
         mca_btl_tcp_component.tcp6_listen_sd = sd;
@@ -894,16 +880,16 @@ static int mca_btl_tcp_component_create_listen(uint16_t af_family)
 
     /* register listen port */
     if (AF_INET == af_family) {
-        opal_event_set( &mca_btl_tcp_component.tcp_recv_event,
+        opal_event_set(opal_event_base, &mca_btl_tcp_component.tcp_recv_event,
                         mca_btl_tcp_component.tcp_listen_sd,
                         OPAL_EV_READ|OPAL_EV_PERSIST,
                         mca_btl_tcp_component_accept_handler,
                         0 );
         opal_event_add(&mca_btl_tcp_component.tcp_recv_event, 0);
     }
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
     if (AF_INET6 == af_family) {
-        opal_event_set( &mca_btl_tcp_component.tcp6_recv_event,
+        opal_event_set(opal_event_base, &mca_btl_tcp_component.tcp6_recv_event,
                         mca_btl_tcp_component.tcp6_listen_sd,
                         OPAL_EV_READ|OPAL_EV_PERSIST,
                         mca_btl_tcp_component_accept_handler,
@@ -976,7 +962,7 @@ static int mca_btl_tcp_component_exchange(void)
                          opal_ifindextokindex (index);
                      current_addr++;
                  }
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
                  if ((AF_INET6 == my_ss.ss_family) &&
                      (6 != mca_btl_tcp_component.tcp_disable_family)) {
                      memcpy(&addrs[current_addr].addr_inet,
@@ -1027,7 +1013,7 @@ mca_btl_base_module_t** mca_btl_tcp_component_init(int *num_btl_modules,
                          mca_btl_tcp_component.tcp_free_list_max,
                          mca_btl_tcp_component.tcp_free_list_inc,
                          NULL );
-                                                                                                                                  
+
     ompi_free_list_init_new( &mca_btl_tcp_component.tcp_frag_max,
                          sizeof (mca_btl_tcp_frag_max_t) + 
                          mca_btl_tcp_module.super.btl_max_send_size,
@@ -1038,7 +1024,7 @@ mca_btl_base_module_t** mca_btl_tcp_component_init(int *num_btl_modules,
                          mca_btl_tcp_component.tcp_free_list_max,
                          mca_btl_tcp_component.tcp_free_list_inc,
                          NULL );
-                                                                                                                                  
+
     ompi_free_list_init_new( &mca_btl_tcp_component.tcp_frag_user,
                          sizeof (mca_btl_tcp_frag_user_t),
                          opal_cache_line_size,
@@ -1048,7 +1034,7 @@ mca_btl_base_module_t** mca_btl_tcp_component_init(int *num_btl_modules,
                          mca_btl_tcp_component.tcp_free_list_max,
                          mca_btl_tcp_component.tcp_free_list_inc,
                          NULL );
-                                                                                                                                  
+
     /* create a BTL TCP module for selected interfaces */
     if(OMPI_SUCCESS != (ret = mca_btl_tcp_component_create_instances() )) {
         return 0;
@@ -1058,9 +1044,10 @@ mca_btl_base_module_t** mca_btl_tcp_component_init(int *num_btl_modules,
     if(OMPI_SUCCESS != (ret = mca_btl_tcp_component_create_listen(AF_INET) )) {
         return 0;
     }
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
     if((ret = mca_btl_tcp_component_create_listen(AF_INET6)) != OMPI_SUCCESS) {
-        if (!(OMPI_ERR_IN_ERRNO == ret && EAFNOSUPPORT == opal_socket_errno)) {
+        if (!(OMPI_ERR_IN_ERRNO == ret &&
+              EAFNOSUPPORT == opal_socket_errno)) {
             opal_output (0, "mca_btl_tcp_component: IPv6 listening socket failed\n");
             return 0;
         }
@@ -1078,20 +1065,15 @@ mca_btl_base_module_t** mca_btl_tcp_component_init(int *num_btl_modules,
         return NULL;
     }
 
+#if OPAL_CUDA_SUPPORT
+    mca_common_cuda_stage_one_init();
+#endif /* OPAL_CUDA_SUPPORT */
+
     memcpy(btls, mca_btl_tcp_component.tcp_btls, mca_btl_tcp_component.tcp_num_btls*sizeof(mca_btl_tcp_module_t*));
     *num_btl_modules = mca_btl_tcp_component.tcp_num_btls;
     return btls;
 }
 
-/*
- *  TCP module control
- */
-
-int mca_btl_tcp_component_control(int param, void* value, size_t size)
-{
-    return OMPI_SUCCESS;
-}
-
 
 /**
  * Called by the event engine when the listening socket has
@@ -1103,7 +1085,7 @@ static void mca_btl_tcp_component_accept_handler( int incoming_sd,
                                                   void* unused )
 {
     while(true) {
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
         struct sockaddr_in6 addr;
 #else
         struct sockaddr_in addr;
@@ -1123,9 +1105,8 @@ static void mca_btl_tcp_component_accept_handler( int incoming_sd,
         mca_btl_tcp_set_socket_options(sd);
 
         /* wait for receipt of peers process identifier to complete this connection */
-         
         event = OBJ_NEW(mca_btl_tcp_event_t);
-        opal_event_set(&event->event, sd, OPAL_EV_READ, mca_btl_tcp_component_recv_handler, event);
+        opal_event_set(opal_event_base, &event->event, sd, OPAL_EV_READ, mca_btl_tcp_component_recv_handler, event);
         opal_event_add(&event->event, 0);
     }
 }
@@ -1139,7 +1120,7 @@ static void mca_btl_tcp_component_accept_handler( int incoming_sd,
  */
 static void mca_btl_tcp_component_recv_handler(int sd, short flags, void* user)
 {
-    orte_process_name_t guid;
+    ompi_process_name_t guid;
     struct sockaddr_storage addr;
     int retval;
     mca_btl_tcp_proc_t* btl_proc;
@@ -1154,7 +1135,7 @@ static void mca_btl_tcp_component_recv_handler(int sd, short flags, void* user)
         CLOSE_THE_SOCKET(sd);
         return;
     }
-    ORTE_PROCESS_NAME_NTOH(guid);
+    OMPI_PROCESS_NAME_NTOH(guid);
 
     /* now set socket up to be non-blocking */
     if((flags = fcntl(sd, F_GETFL, 0)) < 0) {
@@ -1184,9 +1165,6 @@ static void mca_btl_tcp_component_recv_handler(int sd, short flags, void* user)
     }
 
     /* are there any existing peer instances will to accept this connection */
-    if(mca_btl_tcp_proc_accept(btl_proc, (struct sockaddr*)&addr, sd) == false) {
-        CLOSE_THE_SOCKET(sd);
-        return;
-    }
+    (void)mca_btl_tcp_proc_accept(btl_proc, (struct sockaddr*)&addr, sd);
 }
 
