@@ -40,4 +40,8 @@
 #define MCA_COLL_BASE_TAG_SCAN -23
 #define MCA_COLL_BASE_TAG_SCATTER -24
 #define MCA_COLL_BASE_TAG_SCATTERV -25
+#define MCA_COLL_BASE_TAG_NONBLOCKING_BASE -26
+#define MCA_COLL_BASE_TAG_NONBLOCKING_END ((-1 * INT_MAX/2) + 1)
+#define MCA_COLL_BASE_TAG_HCOLL_BASE (-1 * INT_MAX/2)
+#define MCA_COLL_BASE_TAG_HCOLL_END (-1 * INT_MAX)
 #endif /* MCA_COLL_BASE_TAGS_H */
