@@ -3,7 +3,7 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2007 The University of Tennessee and The University
+ * Copyright (c) 2004-2014 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -12,6 +12,7 @@
  *                         All rights reserved.
  * Copyright (c) 2007-2008 University of Houston. All rights reserved.
  * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
+ * Copyright (c) 2012 Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -24,6 +25,8 @@
 
 #include <stdio.h>
 
+#include "opal/mca/hwloc/base/base.h"
+
 #include "mpi.h"
 #include "ompi/communicator/communicator.h"
 #include "ompi/group/group.h"
@@ -98,12 +101,8 @@ mca_coll_hierarch_comm_query(struct ompi_communicator_t *comm, int *priority )
 	return NULL;
     }
 
-    /* This module only works with ob1 pml respectively btl's. Opt out
-       of we use cm/mtl's. */
-    if ( strcmp ( mca_pml_base_selected_component.pmlm_version.mca_component_name, "ob1")) {
-	return NULL;
-    }
-
+    /* This module only works when the BTLs are alive.  If they aren't, time to exit. */
+    if (!mca_bml_base_inited()) return NULL;
 
     size = ompi_comm_size(comm);
     if (size < 3) {
@@ -424,8 +423,8 @@ int mca_coll_hierarch_get_llr ( mca_coll_hierarch_module_t *hierarch_module )
     ncount = mca_coll_hierarch_count_lleaders ( hierarch_module->hier_num_colorarr, 
 						hierarch_module->hier_colorarr);
     hierarch_module->hier_num_lleaders = ncount;
-    hierarch_module->hier_llr = (int *) malloc ( hierarch_module->hier_num_lleaders * sizeof(int));
-    hierarch_module->hier_max_offset = (int *) calloc ( 1, hierarch_module->hier_num_lleaders * sizeof(int));
+    hierarch_module->hier_llr = (int *) malloc ( (size_t)hierarch_module->hier_num_lleaders * sizeof(int));
+    hierarch_module->hier_max_offset = (int *) calloc ( 1, (size_t)hierarch_module->hier_num_lleaders * sizeof(int));
     if ( ( NULL == hierarch_module->hier_llr) || ( NULL == hierarch_module->hier_max_offset )) {
 	return OMPI_ERR_OUT_OF_RESOURCE;
     }
@@ -608,6 +607,7 @@ mca_coll_hierarch_checkfor_component ( struct ompi_communicator_t *comm,
     struct mca_bml_base_btl_array_t *bml_btl_array=NULL;
     mca_bml_base_btl_t *bml_btl=NULL;
     mca_btl_base_component_t *btl=NULL;
+    mca_bml_base_endpoint_t *endpoint;
 
     int i, size, rc;
 
@@ -644,11 +644,12 @@ mca_coll_hierarch_checkfor_component ( struct ompi_communicator_t *comm,
             continue;
         }
 	
+        endpoint = (mca_bml_base_endpoint_t*) procs[i]->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_BML];
         if ( use_rdma ) {
-            bml_btl_array = &(procs[i]->proc_bml->btl_rdma);
+            bml_btl_array = &(endpoint->btl_rdma);
         }
         else {
-            bml_btl_array = &(procs[i]->proc_bml->btl_send);
+            bml_btl_array = &(endpoint->btl_send);
         }
         bml_btl = mca_bml_base_btl_array_get_index ( bml_btl_array, 0 );
         btl = bml_btl->btl->btl_component;
