@@ -26,11 +26,14 @@
 #define TWO_LEVELS 2
 
 #include "ompi_config.h"
+#include "ompi/constants.h"
 
 #include "mpi.h"
+#include "opal/class/opal_pointer_array.h"
 #include "opal/mca/mca.h"
 #include "ompi/mca/coll/coll.h"
 #include "ompi/request/request.h"
+#include "ompi/communicator/communicator.h"
 
 BEGIN_C_DECLS
 
@@ -45,6 +48,17 @@ extern int mca_coll_hierarch_verbose_param;
 extern int mca_coll_hierarch_use_rdma_param;
 extern int mca_coll_hierarch_ignore_sm_param;
 extern int mca_coll_hierarch_detection_alg_param;
+extern int mca_coll_hierarch_bcast_alg_param;
+extern int mca_coll_hierarch_segsize_param;
+
+
+#define COLL_HIERARCH_SEG_BCAST_ALG   0
+#define COLL_HIERARCH_SEG1_BCAST_ALG  1
+#define COLL_HIERARCH_SEG2_BCAST_ALG  2
+#define COLL_HIERARCH_SEG3_BCAST_ALG  3
+#define COLL_HIERARCH_BASIC_BCAST_ALG 4
+
+
 
 #define HIER_DEFAULT_NUM_LLEAD 5
 /*
