@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2014 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -21,7 +21,9 @@
 #include "coll_inter.h"
 
 #include "mpi.h"
-#include "ompi/datatype/datatype.h"
+#include "ompi/datatype/ompi_datatype.h"
+#include "ompi/request/request.h"
+#include "ompi/communicator/communicator.h"
 #include "ompi/constants.h"
 #include "ompi/mca/coll/coll.h"
 #include "ompi/mca/coll/base/coll_tags.h"
@@ -49,7 +51,7 @@ mca_coll_inter_allgatherv_inter(void *sbuf, int scount,
     MPI_Aint incr;
     MPI_Aint extent;
     MPI_Aint lb;
-    ompi_datatype_t *ndtype;
+    ompi_datatype_t *ndtype = NULL;
     ompi_request_t *req[2];
 
     rank = ompi_comm_rank(comm);
@@ -58,12 +60,10 @@ mca_coll_inter_allgatherv_inter(void *sbuf, int scount,
 
     if (0 == rank) {
 	count = (int *)malloc(sizeof(int) * size_local);
-	if (NULL == count) {
-	    return OMPI_ERR_OUT_OF_RESOURCE;
-	}
 	displace = (int *)malloc(sizeof(int) * size_local);
-	if (NULL == displace) {
-	    return OMPI_ERR_OUT_OF_RESOURCE;
+	if ((NULL == count) || (NULL == displace)) {
+            err = OMPI_ERR_OUT_OF_RESOURCE;
+            goto exit;
 	}
     }
     /* Local gather to get the scount of each process */
@@ -72,7 +72,7 @@ mca_coll_inter_allgatherv_inter(void *sbuf, int scount,
 						 0, comm->c_local_comm,
                                                  comm->c_local_comm->c_coll.coll_gather_module);
     if (OMPI_SUCCESS != err) {
-	return err;
+        goto exit;
     }
     if(0 == rank) {
 	displace[0] = 0;
@@ -80,18 +80,20 @@ mca_coll_inter_allgatherv_inter(void *sbuf, int scount,
 	    displace[i] = displace[i-1] + count[i-1];
 	}
 	/* Perform the gatherv locally with the first process as root */
-	err = ompi_ddt_get_extent(sdtype, &lb, &extent);
+	err = ompi_datatype_get_extent(sdtype, &lb, &extent);
 	if (OMPI_SUCCESS != err) {
-	    return OMPI_ERROR;
+            err = OMPI_ERROR;
+            goto exit;
 	}
 	incr = 0;
 	for (i = 0; i < size_local; i++) {
 	    incr = incr + extent*count[i];
 	}
 	if ( incr > 0 ) {
-	    ptmp = (char*)malloc(incr); 
+	    ptmp = (char*)malloc(incr);
 	    if (NULL == ptmp) {
-		return OMPI_ERR_OUT_OF_RESOURCE;
+                err = OMPI_ERR_OUT_OF_RESOURCE;
+                goto exit;
 	    }
 	}
     }
@@ -100,11 +102,11 @@ mca_coll_inter_allgatherv_inter(void *sbuf, int scount,
 						  sdtype,0, comm->c_local_comm,
                                                   comm->c_local_comm->c_coll.coll_gatherv_module);
     if (OMPI_SUCCESS != err) {
-	return err;
+        goto exit;
     }
-    
-    ompi_ddt_create_indexed(size,rcounts,disps,rdtype,&ndtype);
-    ompi_ddt_commit(&ndtype);
+
+    ompi_datatype_create_indexed(size,rcounts,disps,rdtype,&ndtype);
+    ompi_datatype_commit(&ndtype);
 
     if (0 == rank) { 
 	for (i = 0; i < size_local; i++) {
@@ -115,7 +117,7 @@ mca_coll_inter_allgatherv_inter(void *sbuf, int scount,
                                  MCA_COLL_BASE_TAG_ALLGATHERV, comm,
                                  &(req[0])));
         if (OMPI_SUCCESS != err) {
-            return err;
+            goto exit;
         }
 
         err = MCA_PML_CALL(isend(ptmp, total, sdtype, 0,
@@ -123,12 +125,12 @@ mca_coll_inter_allgatherv_inter(void *sbuf, int scount,
                                  MCA_PML_BASE_SEND_STANDARD,
                                  comm, &(req[1])));
         if (OMPI_SUCCESS != err) {
-            return err;
+            goto exit;
         }
 
         err = ompi_request_wait_all(2, req, MPI_STATUSES_IGNORE);
         if (OMPI_SUCCESS != err) {
-            return err;
+            goto exit;
 	}
     }
     
@@ -136,19 +138,18 @@ mca_coll_inter_allgatherv_inter(void *sbuf, int scount,
     err = comm->c_local_comm->c_coll.coll_bcast(rbuf, 1, ndtype, 
 						0, comm->c_local_comm,
                                                 comm->c_local_comm->c_coll.coll_bcast_module);
-    if (OMPI_SUCCESS != err) {
-            return err;
+  exit:
+    if( NULL != ndtype ) {
+        ompi_datatype_destroy(&ndtype);
     }
-
-    ompi_ddt_destroy(&ndtype);
     if (NULL != ptmp) {
-	free(ptmp);
+        free(ptmp);
     }
     if (NULL != displace) {
-	free(displace);
+        free(displace);
     }
     if (NULL != count) {
-	free(count);
+        free(count);
     }
 
     return err;
