@@ -10,8 +10,6 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008-2009 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2011      Los Alamos National Security, LLC.
- *                         All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -28,7 +26,6 @@
 #include "mpi.h"
 #include "opal/mca/mca.h"
 #include "opal/datatype/opal_convertor.h"
-#include "orte/types.h"
 #include "ompi/mca/coll/coll.h"
 #include "ompi/mca/common/sm/common_sm.h"
 
@@ -144,7 +141,7 @@ BEGIN_C_DECLS
     typedef struct mca_coll_sm_comm_t {
         /* Meta data that we get back from the common mmap allocation
            function */
-        mca_common_sm_module_t *mcb_mmap;
+        mca_common_sm_module_t *sm_bootstrap_meta;
 
         /** Pointer to my barrier control pages (odd index pages are
             "in", even index pages are "out") */
