@@ -10,9 +10,11 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
- * Copyright (c) 2009      Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2011      Los Alamos National Security, LLC.
- *                         All rights reserved.
+ * Copyright (c) 2009-2013 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2010-2012 Los Alamos National Security, LLC.  
+ *                         All rights reserved. 
+ * Copyright (c) 2014      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -48,17 +50,14 @@
 
 #include "mpi.h"
 #include "opal_stdint.h"
-#include "opal/mca/maffinity/maffinity.h"
-#include "opal/mca/maffinity/base/base.h"
+#include "opal/mca/hwloc/base/base.h"
 #include "opal/util/os_path.h"
 
-#include "orte/util/proc_info.h"
-#include "orte/util/name_fns.h"
-
 #include "ompi/communicator/communicator.h"
 #include "ompi/group/group.h"
 #include "ompi/mca/coll/coll.h"
 #include "ompi/mca/coll/base/base.h"
+#include "ompi/mca/rte/rte.h"
 #include "ompi/proc/proc.h"
 #include "coll_sm.h"
 
@@ -98,10 +97,11 @@ static void mca_coll_sm_module_destruct(mca_coll_sm_module_t *module)
 
     if (NULL != c) {
         /* Munmap the per-communicator shmem data segment */
-        if (NULL != c->mcb_mmap) {
+        if (NULL != c->sm_bootstrap_meta) {
             /* Ignore any errors -- what are we going to do about
                them? */
-            mca_common_sm_fini(c->mcb_mmap);
+            mca_common_sm_fini(c->sm_bootstrap_meta);
+            OBJ_RELEASE(c->sm_bootstrap_meta);
         }
         free(c);
     }
@@ -136,13 +136,18 @@ int mca_coll_sm_init_query(bool enable_progress_threads,
        then don't bother going any further. */
     if (NULL == (my_proc = ompi_proc_local()) ||
         NULL == (procs = ompi_proc_all(&size))) {
-        opal_output_verbose(10, mca_coll_base_output,
+        opal_output_verbose(10, ompi_coll_base_framework.framework_output,
                             "coll:sm:init_query: weirdness on procs; disqualifying myself");
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
+    /* if no session directory was created, then we cannot be used */
+    if (NULL == ompi_process_info.job_session_dir) {
+        return OMPI_ERR_OUT_OF_RESOURCE;
+    }
     if (size <= 1) {
-        opal_output_verbose(10, mca_coll_base_output,
+        opal_output_verbose(10, ompi_coll_base_framework.framework_output,
                             "coll:sm:init_query: comm size too small; disqualifying myself");
+        free(procs);
         return OMPI_ERR_NOT_AVAILABLE;
     }
     for (i = 0; i < size; ++i) {
@@ -152,16 +157,16 @@ int mca_coll_sm_init_query(bool enable_progress_threads,
             break;
         }
     }
+    free(procs);
     if (i >= size) {
-        opal_output_verbose(10, mca_coll_base_output,
+        opal_output_verbose(10, ompi_coll_base_framework.framework_output,
                             "coll:sm:init_query: no other local procs; disqualifying myself");
         return OMPI_ERR_NOT_AVAILABLE;
     }
-    free(procs);
 
     /* Don't do much here because we don't really want to allocate any
        shared memory until this component is selected to be used. */
-    opal_output_verbose(10, mca_coll_base_output,
+    opal_output_verbose(10, ompi_coll_base_framework.framework_output,
                         "coll:sm:init_query: pick me! pick me!");
     return OMPI_SUCCESS;
 }
@@ -182,7 +187,7 @@ mca_coll_sm_comm_query(struct ompi_communicator_t *comm, int *priority)
        are not on this node, then we don't want to run */
     if (OMPI_COMM_IS_INTER(comm) || 1 == ompi_comm_size(comm) ||
         !have_local_peers(comm->c_local_group, ompi_comm_size(comm))) {
-        opal_output_verbose(10, mca_coll_base_output,
+        opal_output_verbose(10, ompi_coll_base_framework.framework_output,
                             "coll:sm:comm_query (%d/%s): intercomm, comm is too small, or not all peers local; disqualifying myself", comm->c_contextid, comm->c_name);
 	return NULL;
     }
@@ -191,13 +196,17 @@ mca_coll_sm_comm_query(struct ompi_communicator_t *comm, int *priority)
      * than or equal to 0, then the module is unavailable. */
     *priority = mca_coll_sm_component.sm_priority;
     if (mca_coll_sm_component.sm_priority <= 0) {
-        opal_output_verbose(10, mca_coll_base_output,
+        opal_output_verbose(10, ompi_coll_base_framework.framework_output,
                             "coll:sm:comm_query (%d/%s): priority too low; disqualifying myself", comm->c_contextid, comm->c_name);
 	return NULL;
     }
 
-    /* All is good -- return a module */
     sm_module = OBJ_NEW(mca_coll_sm_module_t);
+    if (NULL == sm_module) {
+        return NULL;
+    }
+
+    /* All is good -- return a module */
     sm_module->super.coll_module_enable = sm_module_enable;
     sm_module->super.ft_event        = mca_coll_sm_ft_event;
     sm_module->super.coll_allgather  = NULL;
@@ -217,7 +226,7 @@ mca_coll_sm_comm_query(struct ompi_communicator_t *comm, int *priority)
     sm_module->super.coll_scatter    = NULL;
     sm_module->super.coll_scatterv   = NULL;
 
-    opal_output_verbose(10, mca_coll_base_output,
+    opal_output_verbose(10, ompi_coll_base_framework.framework_output,
                         "coll:sm:comm_query (%d/%s): pick me! pick me!", 
                         comm->c_contextid, comm->c_name);
     return &(sm_module->super);
@@ -232,7 +241,7 @@ static int sm_module_enable(mca_coll_base_module_t *module,
 {
     if (NULL == comm->c_coll.coll_reduce ||
         NULL == comm->c_coll.coll_reduce_module) {
-        opal_output_verbose(10, mca_coll_base_output,
+        opal_output_verbose(10, ompi_coll_base_framework.framework_output,
                             "coll:sm:enable (%d/%s): no underlying reduce; disqualifying myself",
                             comm->c_contextid, comm->c_name);
         return OMPI_ERROR;
@@ -252,7 +261,9 @@ int ompi_coll_sm_lazy_enable(mca_coll_base_module_t *module,
     mca_coll_sm_comm_t *data = NULL;
     size_t control_size, frag_size;
     mca_coll_sm_component_t *c = &mca_coll_sm_component;
-    opal_maffinity_base_segment_t *maffinity;
+#if OPAL_HAVE_HWLOC
+    opal_hwloc_base_memory_segment_t *maffinity;
+#endif
     int parent, min_child, max_child, num_children;
     unsigned char *base = NULL;
     const int num_barrier_buffers = 2;
@@ -263,17 +274,19 @@ int ompi_coll_sm_lazy_enable(mca_coll_base_module_t *module,
     }
     sm_module->enabled = true;
 
+#if OPAL_HAVE_HWLOC
     /* Get some space to setup memory affinity (just easier to try to
        alloc here to handle the error case) */
-    maffinity = (opal_maffinity_base_segment_t*)
-        malloc(sizeof(opal_maffinity_base_segment_t) * 
+    maffinity = (opal_hwloc_base_memory_segment_t*)
+        malloc(sizeof(opal_hwloc_base_memory_segment_t) * 
                c->sm_comm_num_segments * 3);
     if (NULL == maffinity) {
-        opal_output_verbose(10, mca_coll_base_output,
+        opal_output_verbose(10, ompi_coll_base_framework.framework_output,
                             "coll:sm:enable (%d/%s): malloc failed (1)", 
                             comm->c_contextid, comm->c_name);
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
+#endif
 
     /* Allocate data to hang off the communicator.  The memory we
        alloc will be laid out as follows:
@@ -295,8 +308,10 @@ int ompi_coll_sm_lazy_enable(mca_coll_base_module_t *module,
                 (sizeof(mca_coll_sm_tree_node_t) +
                  (sizeof(mca_coll_sm_tree_node_t*) * c->sm_tree_degree))));
     if (NULL == data) {
+#if OPAL_HAVE_HWLOC
         free(maffinity);
-        opal_output_verbose(10, mca_coll_base_output,
+#endif
+        opal_output_verbose(10, ompi_coll_base_framework.framework_output,
                             "coll:sm:enable (%d/%s): malloc failed (2)", 
                             comm->c_contextid, comm->c_name);
         return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
@@ -360,7 +375,9 @@ int ompi_coll_sm_lazy_enable(mca_coll_base_module_t *module,
     /* Attach to this communicator's shmem data segment */
     if (OMPI_SUCCESS != (ret = bootstrap_comm(comm, sm_module))) {
         free(data);
+#if OPAL_HAVE_HWLOC
         free(maffinity);
+#endif
         sm_module->sm_comm_data = NULL;
         return ret;
     }
@@ -374,7 +391,7 @@ int ompi_coll_sm_lazy_enable(mca_coll_base_module_t *module,
        children are contiguous, so having the first pointer and the
        num_children from the mcb_tree data is sufficient). */
     control_size = c->sm_control_size;
-    base = data->mcb_mmap->module_data_addr;
+    base = data->sm_bootstrap_meta->module_data_addr;
     data->mcb_barrier_control_me = (uint32_t*)
         (base + (rank * control_size * num_barrier_buffers * 2));
     if (data->mcb_tree[rank].mcstn_parent) {
@@ -406,10 +423,11 @@ int ompi_coll_sm_lazy_enable(mca_coll_base_module_t *module,
        that they're marked as unused. */
     j = 0;
     if (0 == rank) {
+#if OPAL_HAVE_HWLOC
         maffinity[j].mbs_start_addr = base;
         maffinity[j].mbs_len = c->sm_control_size * 
             c->sm_comm_num_in_use_flags;
-
+#endif
         /* Set the op counts to 1 (actually any nonzero value will do)
            so that the first time children/leaf processes come
            through, they don't see a value of 0 and think that the
@@ -434,11 +452,12 @@ int ompi_coll_sm_lazy_enable(mca_coll_base_module_t *module,
             (((char*) data->mcb_data_index[i].mcbmi_control) + 
              control_size);
 
+#if OPAL_HAVE_HWLOC
         /* Memory affinity: control */
 
         maffinity[j].mbs_len = c->sm_control_size;
         maffinity[j].mbs_start_addr = (void *)
-            (data->mcb_data_index[i].mcbmi_control +
+            (((char*) data->mcb_data_index[i].mcbmi_control) +
              (rank * c->sm_control_size));
         ++j;
 
@@ -446,15 +465,18 @@ int ompi_coll_sm_lazy_enable(mca_coll_base_module_t *module,
 
         maffinity[j].mbs_len = c->sm_fragment_size;
         maffinity[j].mbs_start_addr = 
-            data->mcb_data_index[i].mcbmi_data +
+            ((char*) data->mcb_data_index[i].mcbmi_data) +
             (rank * c->sm_control_size);
         ++j;
+#endif
     }
 
+#if OPAL_HAVE_HWLOC
     /* Setup memory affinity so that the pages that belong to this
        process are local to this process */
-    opal_maffinity_base_set(maffinity, j);
+    opal_hwloc_base_memory_set(maffinity, j);
     free(maffinity);
+#endif
 
     /* Zero out the control structures that belong to this process */
     memset(data->mcb_barrier_control_me, 0, 
@@ -470,25 +492,26 @@ int ompi_coll_sm_lazy_enable(mca_coll_base_module_t *module,
     OBJ_RETAIN(sm_module->previous_reduce_module);
 
     /* Indicate that we have successfully attached and setup */
-    opal_atomic_add(&(data->mcb_mmap->module_seg->seg_inited), 1);
+    opal_atomic_add(&(data->sm_bootstrap_meta->module_seg->seg_inited), 1);
 
     /* Wait for everyone in this communicator to attach and setup */
-    opal_output_verbose(10, mca_coll_base_output,
+    opal_output_verbose(10, ompi_coll_base_framework.framework_output,
                         "coll:sm:enable (%d/%s): waiting for peers to attach",
                         comm->c_contextid, comm->c_name);
-    SPIN_CONDITION(size == data->mcb_mmap->module_seg->seg_inited, seg_init_exit);
+    SPIN_CONDITION(size == data->sm_bootstrap_meta->module_seg->seg_inited, seg_init_exit);
 
     /* Once we're all here, remove the mmap file; it's not needed anymore */
     if (0 == rank) {
-        unlink(data->mcb_mmap->shmem_ds.seg_name);
-        opal_output_verbose(10, mca_coll_base_output,
+        unlink(data->sm_bootstrap_meta->shmem_ds.seg_name);
+        opal_output_verbose(10, ompi_coll_base_framework.framework_output,
                             "coll:sm:enable (%d/%s): removed mmap file %s", 
-                            comm->c_contextid, comm->c_name, data->mcb_mmap->shmem_ds.seg_name);
+                            comm->c_contextid, comm->c_name,
+                            data->sm_bootstrap_meta->shmem_ds.seg_name);
     }
 
     /* All done */
 
-    opal_output_verbose(10, mca_coll_base_output,
+    opal_output_verbose(10, ompi_coll_base_framework.framework_output,
                         "coll:sm:enable (%d/%s): success!", 
                         comm->c_contextid, comm->c_name);
     return OMPI_SUCCESS;
@@ -523,7 +546,7 @@ static int bootstrap_comm(ompi_communicator_t *comm,
     int num_in_use = c->sm_comm_num_in_use_flags;
     int frag_size = c->sm_fragment_size;
     int control_size = c->sm_control_size;
-    orte_process_name_t *lowest_name = NULL;
+    ompi_process_name_t *lowest_name = NULL;
     size_t size;
     ompi_proc_t *proc;
 
@@ -535,25 +558,25 @@ static int bootstrap_comm(ompi_communicator_t *comm,
     lowest_name = &(proc->proc_name);
     for (i = 1; i < comm_size; ++i) {
         proc = ompi_group_peer_lookup(comm->c_local_group, i);
-        if (orte_util_compare_name_fields(ORTE_NS_CMP_ALL, 
+        if (ompi_rte_compare_name_fields(OMPI_RTE_CMP_ALL, 
                                           &(proc->proc_name),
                                           lowest_name) < 0) {
             lowest_name = &(proc->proc_name);
         }
     }
     asprintf(&shortpath, "coll-sm-cid-%d-name-%s.mmap", comm->c_contextid,
-             ORTE_NAME_PRINT(lowest_name));
+             OMPI_NAME_PRINT(lowest_name));
     if (NULL == shortpath) {
-        opal_output_verbose(10, mca_coll_base_output,
+        opal_output_verbose(10, ompi_coll_base_framework.framework_output,
                             "coll:sm:enable:bootstrap comm (%d/%s): asprintf failed", 
                             comm->c_contextid, comm->c_name);
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
-    fullpath = opal_os_path(false, orte_process_info.job_session_dir,
+    fullpath = opal_os_path(false, ompi_process_info.job_session_dir,
                             shortpath, NULL);
     free(shortpath);
     if (NULL == fullpath) {
-        opal_output_verbose(10, mca_coll_base_output,
+        opal_output_verbose(10, ompi_coll_base_framework.framework_output,
                             "coll:sm:enable:bootstrap comm (%d/%s): opal_os_path failed", 
                             comm->c_contextid, comm->c_name);
         return OMPI_ERR_OUT_OF_RESOURCE;
@@ -584,16 +607,17 @@ static int bootstrap_comm(ompi_communicator_t *comm,
         (num_in_use * control_size) +
         (num_segments * (comm_size * control_size * 2)) +
         (num_segments * (comm_size * frag_size));
-    opal_output_verbose(10, mca_coll_base_output,
+    opal_output_verbose(10, ompi_coll_base_framework.framework_output,
                         "coll:sm:enable:bootstrap comm (%d/%s): attaching to %" PRIsize_t " byte mmap: %s",
                         comm->c_contextid, comm->c_name, size, fullpath);
-    data->mcb_mmap =
+    data->sm_bootstrap_meta =
         mca_common_sm_init_group(comm->c_local_group, size, fullpath,
-                                sizeof(mca_common_sm_seg_header_t),
-                                sizeof(void*));
-    if (NULL == data->mcb_mmap) {
-        opal_output_verbose(10, mca_coll_base_output,
-                            "coll:sm:enable:bootstrap comm (%d/%s): common_sm_mmap_init_group failed", 
+                                 sizeof(mca_common_sm_seg_header_t),
+                                 getpagesize());
+    free(fullpath);
+    if (NULL == data->sm_bootstrap_meta) {
+        opal_output_verbose(10, ompi_coll_base_framework.framework_output,
+                            "coll:sm:enable:bootstrap comm (%d/%s): mca_common_sm_init_group failed", 
                             comm->c_contextid, comm->c_name);
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
