@@ -1,14 +1,20 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2012 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2013      Los Alamos National Security, LLC. All Rights
+ *                         reserved.
+ * Copyright (c) 2014      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2015      Intel, Inc. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -29,19 +35,133 @@
 #include "coll_tuned_topo.h"
 #include "coll_tuned_util.h"
 
+/* alltoall algorithm variables */
+static int coll_tuned_alltoall_algorithm_count = 5;
+static int coll_tuned_alltoall_forced_algorithm = 0;
+static int coll_tuned_alltoall_segment_size = 0;
+static int coll_tuned_alltoall_max_requests;
+static int coll_tuned_alltoall_tree_fanout;
+static int coll_tuned_alltoall_chain_fanout;
+
+/* valid values for coll_tuned_alltoall_forced_algorithm */
+static mca_base_var_enum_value_t alltoall_algorithms[] = {
+    {0, "ignore"},
+    {1, "linear"},
+    {2, "pairwise"},
+    {3, "modified_bruck"},
+    {4, "linear_sync"},
+    {5, "two_proc"},
+    {0, NULL}
+};
+
+/* MPI_IN_PLACE all to all algorithm. TODO: implement a better one. */
+static int
+mca_coll_tuned_alltoall_intra_basic_inplace(void *rbuf, int rcount,
+                                            struct ompi_datatype_t *rdtype,
+                                            struct ompi_communicator_t *comm,
+                                            mca_coll_base_module_t *module)
+{
+    mca_coll_tuned_module_t *tuned_module = (mca_coll_tuned_module_t*) module;
+    int i, j, size, rank, err=MPI_SUCCESS;
+    MPI_Request *preq;
+    char *tmp_buffer;
+    size_t max_size;
+    ptrdiff_t ext;
+
+    /* Initialize. */
+
+    size = ompi_comm_size(comm);
+    rank = ompi_comm_rank(comm);
+
+    /* If only one process, we're done. */
+    if (1 == size) {
+        return MPI_SUCCESS;
+    }
+
+    /* Find the largest receive amount */
+    ompi_datatype_type_extent (rdtype, &ext);
+    max_size = ext * rcount;
+
+    /* Allocate a temporary buffer */
+    tmp_buffer = calloc (max_size, 1);
+    if (NULL == tmp_buffer) {
+      return OMPI_ERR_OUT_OF_RESOURCE;
+    }
+
+    /* in-place alltoall slow algorithm (but works) */
+    for (i = 0 ; i < size ; ++i) {
+        for (j = i+1 ; j < size ; ++j) {
+            /* Initiate all send/recv to/from others. */
+            preq = tuned_module->tuned_data->mcct_reqs;
+
+            if (i == rank) {
+                /* Copy the data into the temporary buffer */
+                err = ompi_datatype_copy_content_same_ddt (rdtype, rcount, tmp_buffer,
+                                                       (char *) rbuf + j * max_size);
+                if (MPI_SUCCESS != err) { goto error_hndl; }
+
+                /* Exchange data with the peer */
+                err = MCA_PML_CALL(irecv ((char *) rbuf + max_size * j, rcount, rdtype,
+                                          j, MCA_COLL_BASE_TAG_ALLTOALL, comm, preq++));
+                if (MPI_SUCCESS != err) { goto error_hndl; }
+
+                err = MCA_PML_CALL(isend ((char *) tmp_buffer,  rcount, rdtype,
+                                          j, MCA_COLL_BASE_TAG_ALLTOALL, MCA_PML_BASE_SEND_STANDARD,
+                                          comm, preq++));
+                if (MPI_SUCCESS != err) { goto error_hndl; }
+            } else if (j == rank) {
+                /* Copy the data into the temporary buffer */
+                err = ompi_datatype_copy_content_same_ddt (rdtype, rcount, tmp_buffer,
+                                                       (char *) rbuf + i * max_size);
+                if (MPI_SUCCESS != err) { goto error_hndl; }
+
+                /* Exchange data with the peer */
+                err = MCA_PML_CALL(irecv ((char *) rbuf + max_size * i, rcount, rdtype,
+                                          i, MCA_COLL_BASE_TAG_ALLTOALL, comm, preq++));
+                if (MPI_SUCCESS != err) { goto error_hndl; }
+
+                err = MCA_PML_CALL(isend ((char *) tmp_buffer,  rcount, rdtype,
+                                          i, MCA_COLL_BASE_TAG_ALLTOALL, MCA_PML_BASE_SEND_STANDARD,
+                                          comm, preq++));
+                if (MPI_SUCCESS != err) { goto error_hndl; }
+            } else {
+                continue;
+            }
+
+            /* Wait for the requests to complete */
+            err = ompi_request_wait_all (2, tuned_module->tuned_data->mcct_reqs, MPI_STATUSES_IGNORE);
+            if (MPI_SUCCESS != err) { goto error_hndl; }
+
+            /* Free the requests. */
+            mca_coll_tuned_free_reqs(tuned_module->tuned_data->mcct_reqs, 2);
+        }
+    }
+
+ error_hndl:
+    /* Free the temporary buffer */
+    free (tmp_buffer);
+
+    /* All done */
+
+    return err;
+}
+
 int ompi_coll_tuned_alltoall_intra_pairwise(void *sbuf, int scount, 
                                             struct ompi_datatype_t *sdtype,
                                             void* rbuf, int rcount,
                                             struct ompi_datatype_t *rdtype,
                                             struct ompi_communicator_t *comm,
-					    mca_coll_base_module_t *module)
+                                            mca_coll_base_module_t *module)
 {
-    int line = -1, err = 0;
-    int rank, size, step;
-    int sendto, recvfrom;
+    int line = -1, err = 0, rank, size, step, sendto, recvfrom;
     void * tmpsend, *tmprecv;
     ptrdiff_t lb, sext, rext;
 
+    if (MPI_IN_PLACE == sbuf) {
+        return mca_coll_tuned_alltoall_intra_basic_inplace (rbuf, rcount, rdtype,
+                                                            comm, module);
+    }
+
     size = ompi_comm_size(comm);
     rank = ompi_comm_rank(comm);
 
@@ -62,8 +182,8 @@ int ompi_coll_tuned_alltoall_intra_pairwise(void *sbuf, int scount,
         recvfrom = (rank + size - step) % size;
 
         /* Determine sending and receiving locations */
-        tmpsend = (char*)sbuf + sendto * sext * scount;
-        tmprecv = (char*)rbuf + recvfrom * rext * rcount;
+        tmpsend = (char*)sbuf + (ptrdiff_t)sendto * sext * (ptrdiff_t)scount;
+        tmprecv = (char*)rbuf + (ptrdiff_t)recvfrom * rext * (ptrdiff_t)rcount;
 
         /* send and receive */
         err = ompi_coll_tuned_sendrecv( tmpsend, scount, sdtype, sendto, 
@@ -77,9 +197,9 @@ int ompi_coll_tuned_alltoall_intra_pairwise(void *sbuf, int scount,
     return MPI_SUCCESS;
  
  err_hndl:
-    OPAL_OUTPUT((ompi_coll_tuned_stream,
+    opal_output_verbose(COLL_TUNED_VERBOSITY, ompi_coll_tuned_stream,
                  "%s:%4d\tError occurred %d, rank %2d", __FILE__, line, 
-                 err, rank));
+                 err, rank);
     return err;
 }
 
@@ -89,21 +209,23 @@ int ompi_coll_tuned_alltoall_intra_bruck(void *sbuf, int scount,
                                          void* rbuf, int rcount,
                                          struct ompi_datatype_t *rdtype,
                                          struct ompi_communicator_t *comm,
-					 mca_coll_base_module_t *module)
+                                         mca_coll_base_module_t *module)
 {
-    int i, k, line = -1;
-    int rank, size;
+    int i, k, line = -1, rank, size, err = 0, weallocated = 0;
     int sendto, recvfrom, distance, *displs = NULL, *blen = NULL;
     char *tmpbuf = NULL, *tmpbuf_free = NULL;
     ptrdiff_t rlb, slb, tlb, sext, rext, tsext;
-    int err = 0;
-    int weallocated = 0;
     struct ompi_datatype_t *new_ddt;
 #ifdef blahblah
     mca_coll_tuned_module_t *tuned_module = (mca_coll_tuned_module_t*) module;
     mca_coll_tuned_comm_t *data = tuned_module->tuned_data;
 #endif
 
+    if (MPI_IN_PLACE == sbuf) {
+        return mca_coll_tuned_alltoall_intra_basic_inplace (rbuf, rcount, rdtype,
+                                                            comm, module);
+    }
+
     size = ompi_comm_size(comm);
     rank = ompi_comm_rank(comm);
 
@@ -141,23 +263,23 @@ int ompi_coll_tuned_alltoall_intra_bruck(void *sbuf, int scount,
 #endif
 
     /* tmp buffer allocation for message data */
-    tmpbuf_free = (char *) malloc(tsext + (scount * size - 1) * sext);
+    tmpbuf_free = (char *) malloc(tsext + ((ptrdiff_t)scount * (ptrdiff_t)size - 1) * sext);
     if (tmpbuf_free == NULL) { line = __LINE__; err = -1; goto err_hndl; }
     tmpbuf = tmpbuf_free - slb;
 
     /* Step 1 - local rotation - shift up by rank */
     err = ompi_datatype_copy_content_same_ddt (sdtype, 
-                                          (int32_t) ((size - rank) * scount),
-                                          tmpbuf, 
-                                          ((char*) sbuf) + rank * scount * sext);
+                                               (int32_t) ((ptrdiff_t)(size - rank) * (ptrdiff_t)scount),
+                                               tmpbuf, 
+                                               ((char*) sbuf) + (ptrdiff_t)rank * (ptrdiff_t)scount * sext);
     if (err<0) {
         line = __LINE__; err = -1; goto err_hndl;
     }
 
     if (rank != 0) {
-        err = ompi_datatype_copy_content_same_ddt (sdtype, (int32_t) (rank * scount),
-                                              tmpbuf + (size - rank) * scount* sext, 
-                                              (char*) sbuf);
+        err = ompi_datatype_copy_content_same_ddt (sdtype, (ptrdiff_t)rank * (ptrdiff_t)scount,
+                                                   tmpbuf + (ptrdiff_t)(size - rank) * (ptrdiff_t)scount* sext, 
+                                                   (char*) sbuf);
         if (err<0) {
             line = __LINE__; err = -1; goto err_hndl;
         }
@@ -173,7 +295,7 @@ int ompi_coll_tuned_alltoall_intra_bruck(void *sbuf, int scount,
         /* create indexed datatype */
         for (i = 1; i < size; i++) {
             if (( i & distance) == distance) {
-                displs[k] = i * scount; 
+                displs[k] = (ptrdiff_t)i * (ptrdiff_t)scount; 
                 blen[k] = scount;
                 k++;
             }
@@ -206,8 +328,8 @@ int ompi_coll_tuned_alltoall_intra_bruck(void *sbuf, int scount,
     for (i = 0; i < size; i++) {
 
         err = ompi_datatype_copy_content_same_ddt (rdtype, (int32_t) rcount,
-                                              ((char*)rbuf) + (((rank - i + size) % size) * rcount * rext), 
-                                              tmpbuf + i * rcount * rext);
+                                                   ((char*)rbuf) + ((ptrdiff_t)((rank - i + size) % size) * (ptrdiff_t)rcount * rext), 
+                                                   tmpbuf + (ptrdiff_t)i * (ptrdiff_t)rcount * rext);
         if (err < 0) { line = __LINE__; err = -1; goto err_hndl;  }
     }
 
@@ -220,9 +342,9 @@ int ompi_coll_tuned_alltoall_intra_bruck(void *sbuf, int scount,
     return OMPI_SUCCESS;
 
  err_hndl:
-    OPAL_OUTPUT((ompi_coll_tuned_stream,
+    opal_output_verbose(COLL_TUNED_VERBOSITY, ompi_coll_tuned_stream,
                  "%s:%4d\tError occurred %d, rank %2d", __FILE__, line, err, 
-                 rank));
+                 rank);
     if (tmpbuf != NULL) free(tmpbuf_free);
     if (weallocated) {
         if (displs != NULL) free(displs);
@@ -253,21 +375,20 @@ int ompi_coll_tuned_alltoall_intra_linear_sync(void *sbuf, int scount,
                                                void* rbuf, int rcount,
                                                struct ompi_datatype_t *rdtype,
                                                struct ompi_communicator_t *comm,
-					       mca_coll_base_module_t *module,
+                                               mca_coll_base_module_t *module,
                                                int max_outstanding_reqs)
 {
-    int line = -1, error;
-    int ri, si;
-    int rank;
-    int size;
-    int nreqs, nrreqs, nsreqs, total_reqs;
-    char *psnd;
-    char *prcv;
-    ptrdiff_t slb, sext;
-    ptrdiff_t rlb, rext;
+    int line, error, ri, si, rank, size, nreqs, nrreqs, nsreqs, total_reqs;
+    char *psnd, *prcv;
+    ptrdiff_t slb, sext, rlb, rext;
 
     ompi_request_t **reqs = NULL;
 
+    if (MPI_IN_PLACE == sbuf) {
+        return mca_coll_tuned_alltoall_intra_basic_inplace (rbuf, rcount, rdtype,
+                                                            comm, module);
+    }
+
     /* Initialize. */
 
     size = ompi_comm_size(comm);
@@ -276,7 +397,6 @@ int ompi_coll_tuned_alltoall_intra_linear_sync(void *sbuf, int scount,
     OPAL_OUTPUT((ompi_coll_tuned_stream,
                  "ompi_coll_tuned_alltoall_intra_linear_sync rank %d", rank));
 
-
     error = ompi_datatype_get_extent(sdtype, &slb, &sext);
     if (OMPI_SUCCESS != error) {
         return error;
@@ -291,8 +411,8 @@ int ompi_coll_tuned_alltoall_intra_linear_sync(void *sbuf, int scount,
 
     /* simple optimization */
 
-    psnd = ((char *) sbuf) + (rank * sext);
-    prcv = ((char *) rbuf) + (rank * rext);
+    psnd = ((char *) sbuf) + (ptrdiff_t)rank * sext;
+    prcv = ((char *) rbuf) + (ptrdiff_t)rank * rext;
 
     error = ompi_datatype_sndrcv(psnd, scount, sdtype, prcv, rcount, rdtype);
     if (MPI_SUCCESS != error) {
@@ -311,7 +431,7 @@ int ompi_coll_tuned_alltoall_intra_linear_sync(void *sbuf, int scount,
                    (size - 1) : (max_outstanding_reqs));
     reqs = (ompi_request_t**) malloc( 2 * total_reqs * 
                                       sizeof(ompi_request_t*));
-    if (NULL == reqs) { error = -1; goto error_hndl; }
+    if (NULL == reqs) { error = -1; line = __LINE__; goto error_hndl; }
     
     prcv = (char *) rbuf;
     psnd = (char *) sbuf;
@@ -319,67 +439,67 @@ int ompi_coll_tuned_alltoall_intra_linear_sync(void *sbuf, int scount,
     /* Post first batch or ireceive and isend requests  */
     for (nreqs = 0, nrreqs = 0, ri = (rank + 1) % size; nreqs < total_reqs; 
          ri = (ri + 1) % size, ++nreqs, ++nrreqs) {
-       error =
-          MCA_PML_CALL(irecv
-                       (prcv + (ri * rext), rcount, rdtype, ri,
-                        MCA_COLL_BASE_TAG_ALLTOALL, comm, &reqs[nreqs]));
-       if (MPI_SUCCESS != error) { line = __LINE__; goto error_hndl; }
+        error =
+            MCA_PML_CALL(irecv
+                         (prcv + (ptrdiff_t)ri * rext, rcount, rdtype, ri,
+                          MCA_COLL_BASE_TAG_ALLTOALL, comm, &reqs[nreqs]));
+        if (MPI_SUCCESS != error) { line = __LINE__; goto error_hndl; }
     }
     for ( nsreqs = 0, si =  (rank + size - 1) % size; nreqs < 2 * total_reqs; 
           si = (si + size - 1) % size, ++nreqs, ++nsreqs) {
-       error =
-          MCA_PML_CALL(isend
-                       (psnd + (si * sext), scount, sdtype, si,
-                        MCA_COLL_BASE_TAG_ALLTOALL,
-                        MCA_PML_BASE_SEND_STANDARD, comm, &reqs[nreqs]));
-       if (MPI_SUCCESS != error) { line = __LINE__; goto error_hndl; }
+        error =
+            MCA_PML_CALL(isend
+                         (psnd + (ptrdiff_t)si * sext, scount, sdtype, si,
+                          MCA_COLL_BASE_TAG_ALLTOALL,
+                          MCA_PML_BASE_SEND_STANDARD, comm, &reqs[nreqs]));
+        if (MPI_SUCCESS != error) { line = __LINE__; goto error_hndl; }
     }
 
     /* Wait for requests to complete */
     if (nreqs == 2 * (size - 1)) {
-       /* Optimization for the case when all requests have been posted  */
-       error = ompi_request_wait_all(nreqs, reqs, MPI_STATUSES_IGNORE);
-       if (MPI_SUCCESS != error) { line = __LINE__; goto error_hndl; }
+        /* Optimization for the case when all requests have been posted  */
+        error = ompi_request_wait_all(nreqs, reqs, MPI_STATUSES_IGNORE);
+        if (MPI_SUCCESS != error) { line = __LINE__; goto error_hndl; }
        
     } else {
-       /* As requests complete, replace them with corresponding requests:
-          - wait for any request to complete, mark the request as 
-            MPI_REQUEST_NULL
-          - If it was a receive request, replace it with new irecv request 
-            (if any)
-          - if it was a send request, replace it with new isend request (if any)
-       */
-       int ncreqs = 0;
-       while (ncreqs < 2 * (size - 1)) {
-          int completed;
-          error = ompi_request_wait_any(2 * total_reqs, reqs, &completed,
-                                        MPI_STATUS_IGNORE);
-          if (MPI_SUCCESS != error) { line = __LINE__; goto error_hndl; }
-          reqs[completed] = MPI_REQUEST_NULL;
-          ncreqs++;
-          if (completed < total_reqs) {
-             if (nrreqs < (size - 1)) {
-                error = 
-                   MCA_PML_CALL(irecv
-                                (prcv + (ri * rext), rcount, rdtype, ri,
-                                 MCA_COLL_BASE_TAG_ALLTOALL, comm, 
-                                 &reqs[completed]));
-                if (MPI_SUCCESS != error) { line = __LINE__; goto error_hndl; }
-                ++nrreqs;
-                ri = (ri + 1) % size;
-             }
-          } else {
-             if (nsreqs < (size - 1)) {
-                error = MCA_PML_CALL(isend
-                                     (psnd + (si * sext), scount, sdtype, si,
-                                      MCA_COLL_BASE_TAG_ALLTOALL,
-                                      MCA_PML_BASE_SEND_STANDARD, comm,
+        /* As requests complete, replace them with corresponding requests:
+           - wait for any request to complete, mark the request as 
+           MPI_REQUEST_NULL
+           - If it was a receive request, replace it with new irecv request 
+           (if any)
+           - if it was a send request, replace it with new isend request (if any)
+        */
+        int ncreqs = 0;
+        while (ncreqs < 2 * (size - 1)) {
+            int completed;
+            error = ompi_request_wait_any(2 * total_reqs, reqs, &completed,
+                                          MPI_STATUS_IGNORE);
+            if (MPI_SUCCESS != error) { line = __LINE__; goto error_hndl; }
+            reqs[completed] = MPI_REQUEST_NULL;
+            ncreqs++;
+            if (completed < total_reqs) {
+                if (nrreqs < (size - 1)) {
+                    error = 
+                        MCA_PML_CALL(irecv
+                                     (prcv + (ptrdiff_t)ri * rext, rcount, rdtype, ri,
+                                      MCA_COLL_BASE_TAG_ALLTOALL, comm, 
                                       &reqs[completed]));
-                ++nsreqs;
-                si = (si + size - 1) % size; 
-             }
-          }
-       }
+                    if (MPI_SUCCESS != error) { line = __LINE__; goto error_hndl; }
+                    ++nrreqs;
+                    ri = (ri + 1) % size;
+                }
+            } else {
+                if (nsreqs < (size - 1)) {
+                    error = MCA_PML_CALL(isend
+                                         (psnd + (ptrdiff_t)si * sext, scount, sdtype, si,
+                                          MCA_COLL_BASE_TAG_ALLTOALL,
+                                          MCA_PML_BASE_SEND_STANDARD, comm,
+                                          &reqs[completed]));
+                    ++nsreqs;
+                    si = (si + size - 1) % size; 
+                }
+            }
+        }
     }
 
     /* Free the reqs */
@@ -389,9 +509,9 @@ int ompi_coll_tuned_alltoall_intra_linear_sync(void *sbuf, int scount,
     return MPI_SUCCESS;
 
  error_hndl:
-    OPAL_OUTPUT((ompi_coll_tuned_stream,
+    opal_output_verbose(COLL_TUNED_VERBOSITY, ompi_coll_tuned_stream,
                  "%s:%4d\tError occurred %d, rank %2d", __FILE__, line, error, 
-                 rank));
+                 rank);
     if (NULL != reqs) free(reqs);
     return error;
 }
@@ -402,14 +522,17 @@ int ompi_coll_tuned_alltoall_intra_two_procs(void *sbuf, int scount,
                                              void* rbuf, int rcount,
                                              struct ompi_datatype_t *rdtype,
                                              struct ompi_communicator_t *comm,
-					     mca_coll_base_module_t *module)
+                                             mca_coll_base_module_t *module)
 {
-    int line = -1, err = 0;
-    int rank;
-    int remote;
+    int line = -1, err = 0, rank, remote;
     void * tmpsend, *tmprecv;
     ptrdiff_t sext, rext, lb;
 
+    if (MPI_IN_PLACE == sbuf) {
+        return mca_coll_tuned_alltoall_intra_basic_inplace (rbuf, rcount, rdtype,
+                                                            comm, module);
+    }
+
     rank = ompi_comm_rank(comm);
 
     OPAL_OUTPUT((ompi_coll_tuned_stream,
@@ -424,8 +547,8 @@ int ompi_coll_tuned_alltoall_intra_two_procs(void *sbuf, int scount,
     /* exchange data */
     remote  = rank ^ 1;
 
-    tmpsend = (char*)sbuf + remote * sext * scount;
-    tmprecv = (char*)rbuf + remote * rext * rcount;
+    tmpsend = (char*)sbuf + (ptrdiff_t)remote * sext * (ptrdiff_t)scount;
+    tmprecv = (char*)rbuf + (ptrdiff_t)remote * rext * (ptrdiff_t)rcount;
 
     /* send and receive */
     err = ompi_coll_tuned_sendrecv ( tmpsend, scount, sdtype, remote, 
@@ -436,19 +559,19 @@ int ompi_coll_tuned_alltoall_intra_two_procs(void *sbuf, int scount,
     if (err != MPI_SUCCESS) { line = __LINE__; goto err_hndl;  }
 
     /* ddt sendrecv your own data */
-    err = ompi_datatype_sndrcv((char*) sbuf + rank * sext * scount, 
-                          (int32_t) scount, sdtype, 
-                          (char*) rbuf + rank * rext * rcount, 
-                          (int32_t) rcount, rdtype);
+    err = ompi_datatype_sndrcv((char*) sbuf + (ptrdiff_t)rank * sext * (ptrdiff_t)scount, 
+                               (int32_t) scount, sdtype, 
+                               (char*) rbuf + (ptrdiff_t)rank * rext * (ptrdiff_t)rcount, 
+                               (int32_t) rcount, rdtype);
     if (err != MPI_SUCCESS) { line = __LINE__; goto err_hndl;  }
 
     /* done */
     return MPI_SUCCESS;
 
  err_hndl:
-    OPAL_OUTPUT((ompi_coll_tuned_stream,
+    opal_output_verbose(COLL_TUNED_VERBOSITY, ompi_coll_tuned_stream,
                  "%s:%4d\tError occurred %d, rank %2d", __FILE__, line, err,
-                 rank));
+                 rank);
     return err;
 }
 
@@ -473,26 +596,20 @@ int ompi_coll_tuned_alltoall_intra_basic_linear(void *sbuf, int scount,
                                                 void* rbuf, int rcount,
                                                 struct ompi_datatype_t *rdtype,
                                                 struct ompi_communicator_t *comm,
-						mca_coll_base_module_t *module)
+                                                mca_coll_base_module_t *module)
 {
-    int i;
-    int rank;
-    int size;
-    int err;
-    int nreqs;
-    char *psnd;
-    char *prcv;
-    MPI_Aint lb;
-    MPI_Aint sndinc;
-    MPI_Aint rcvinc;
-
-    ompi_request_t **req;
-    ompi_request_t **sreq;
-    ompi_request_t **rreq;
-
+    int i, rank, size, err, nreqs;
+    char *psnd, *prcv;
+    MPI_Aint lb, sndinc, rcvinc;
+    ompi_request_t **req, **sreq, **rreq;
     mca_coll_tuned_module_t *tuned_module = (mca_coll_tuned_module_t*) module;
     mca_coll_tuned_comm_t *data = tuned_module->tuned_data;
 
+    if (MPI_IN_PLACE == sbuf) {
+        return mca_coll_tuned_alltoall_intra_basic_inplace (rbuf, rcount, rdtype,
+                                                            comm, module);
+    }
+
     /* Initialize. */
 
     size = ompi_comm_size(comm);
@@ -516,8 +633,8 @@ int ompi_coll_tuned_alltoall_intra_basic_linear(void *sbuf, int scount,
 
     /* simple optimization */
 
-    psnd = ((char *) sbuf) + (rank * sndinc);
-    prcv = ((char *) rbuf) + (rank * rcvinc);
+    psnd = ((char *) sbuf) + (ptrdiff_t)rank * sndinc;
+    prcv = ((char *) rbuf) + (ptrdiff_t)rank * rcvinc;
 
     err = ompi_datatype_sndrcv(psnd, scount, sdtype, prcv, rcount, rdtype);
     if (MPI_SUCCESS != err) {
@@ -544,7 +661,7 @@ int ompi_coll_tuned_alltoall_intra_basic_linear(void *sbuf, int scount,
          i = (i + 1) % size, ++rreq, ++nreqs) {
         err =
             MCA_PML_CALL(irecv_init
-                         (prcv + (i * rcvinc), rcount, rdtype, i,
+                         (prcv + (ptrdiff_t)i * rcvinc, rcount, rdtype, i,
                           MCA_COLL_BASE_TAG_ALLTOALL, comm, rreq));
         if (MPI_SUCCESS != err) {
             ompi_coll_tuned_free_reqs(req, rreq - req);
@@ -560,7 +677,7 @@ int ompi_coll_tuned_alltoall_intra_basic_linear(void *sbuf, int scount,
          i = (i + size - 1) % size, ++sreq, ++nreqs) {
         err =
             MCA_PML_CALL(isend_init
-                         (psnd + (i * sndinc), scount, sdtype, i,
+                         (psnd + (ptrdiff_t)i * sndinc, scount, sdtype, i,
                           MCA_COLL_BASE_TAG_ALLTOALL,
                           MCA_PML_BASE_SEND_STANDARD, comm, sreq));
         if (MPI_SUCCESS != err) {
@@ -605,72 +722,84 @@ int ompi_coll_tuned_alltoall_intra_basic_linear(void *sbuf, int scount,
 
 int ompi_coll_tuned_alltoall_intra_check_forced_init (coll_tuned_force_algorithm_mca_param_indices_t *mca_param_indices)
 {
-    int max_alg = 5, requested_alg, max_requests;
+    mca_base_var_enum_t*new_enum;
 
-    ompi_coll_tuned_forced_max_algorithms[ALLTOALL] = max_alg;
+    ompi_coll_tuned_forced_max_algorithms[ALLTOALL] = coll_tuned_alltoall_algorithm_count;
 
-    mca_base_param_reg_int (&mca_coll_tuned_component.super.collm_version,
-                            "alltoall_algorithm_count",
-                            "Number of alltoall algorithms available",
-                            false, true, max_alg, NULL);
+    (void) mca_base_component_var_register(&mca_coll_tuned_component.super.collm_version,
+                                           "alltoall_algorithm_count",
+                                           "Number of alltoall algorithms available",
+                                           MCA_BASE_VAR_TYPE_INT, NULL, 0,
+                                           MCA_BASE_VAR_FLAG_DEFAULT_ONLY,
+                                           OPAL_INFO_LVL_5,
+                                           MCA_BASE_VAR_SCOPE_CONSTANT,
+                                           &coll_tuned_alltoall_algorithm_count);
 
-    mca_param_indices->algorithm_param_index
-        = mca_base_param_reg_int(&mca_coll_tuned_component.super.collm_version,
-                                 "alltoall_algorithm",
-                                 "Which alltoall algorithm is used. Can be locked down to choice of: 0 ignore, 1 basic linear, 2 pairwise, 3: modified bruck, 4: linear with sync, 5:two proc only.",
-                                 false, false, 0, NULL);
+    /* MPI_T: This variable should eventually be bound to a communicator */
+    coll_tuned_alltoall_forced_algorithm = 0;
+    (void) mca_base_var_enum_create("coll_tuned_alltoall_algorithms", alltoall_algorithms, &new_enum);
+    mca_param_indices->algorithm_param_index =
+        mca_base_component_var_register(&mca_coll_tuned_component.super.collm_version,
+                                        "alltoall_algorithm",
+                                        "Which alltoall algorithm is used. Can be locked down to choice of: 0 ignore, 1 basic linear, 2 pairwise, 3: modified bruck, 4: linear with sync, 5:two proc only.",
+                                        MCA_BASE_VAR_TYPE_INT, new_enum, 0, 0,
+                                        OPAL_INFO_LVL_5,
+                                        MCA_BASE_VAR_SCOPE_READONLY,
+                                        &coll_tuned_alltoall_forced_algorithm);
+    OBJ_RELEASE(new_enum);
     if (mca_param_indices->algorithm_param_index < 0) {
         return mca_param_indices->algorithm_param_index;
     }
-    mca_base_param_lookup_int(mca_param_indices->algorithm_param_index, &(requested_alg));
-    if( 0 > requested_alg || requested_alg > max_alg ) {
-        if( 0 == ompi_comm_rank( MPI_COMM_WORLD ) ) {
-            opal_output( 0, "Alltoall algorithm #%d is not available (range [0..%d]). Switching back to ignore(0)\n",
-                         requested_alg, max_alg );
-        }
-        mca_base_param_set_int( mca_param_indices->algorithm_param_index, 0);
-    }
-    
-    mca_param_indices->segsize_param_index
-        = mca_base_param_reg_int(&mca_coll_tuned_component.super.collm_version,
-                                 "alltoall_algorithm_segmentsize",
-                                 "Segment size in bytes used by default for alltoall algorithms. Only has meaning if algorithm is forced and supports segmenting. 0 bytes means no segmentation.",
-                                 false, false, 0, NULL);
-    
-    mca_param_indices->tree_fanout_param_index
-        = mca_base_param_reg_int(&mca_coll_tuned_component.super.collm_version,
-                                 "alltoall_algorithm_tree_fanout",
-                                 "Fanout for n-tree used for alltoall algorithms. Only has meaning if algorithm is forced and supports n-tree topo based operation.",
-                                 false, false, 
-                                 ompi_coll_tuned_init_tree_fanout, /* get system wide default */
-                                 NULL);
 
-    mca_param_indices->chain_fanout_param_index
-        = mca_base_param_reg_int(&mca_coll_tuned_component.super.collm_version,
-                                 "alltoall_algorithm_chain_fanout",
-                                 "Fanout for chains used for alltoall algorithms. Only has meaning if algorithm is forced and supports chain topo based operation.",
-                                 false, false, 
-                                 ompi_coll_tuned_init_chain_fanout, /* get system wide default */
-                                 NULL);
+    coll_tuned_alltoall_segment_size = 0;
+    mca_param_indices->segsize_param_index =
+        mca_base_component_var_register(&mca_coll_tuned_component.super.collm_version,
+                                        "alltoall_algorithm_segmentsize",
+                                        "Segment size in bytes used by default for alltoall algorithms. Only has meaning if algorithm is forced and supports segmenting. 0 bytes means no segmentation.",
+                                        MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
+                                        OPAL_INFO_LVL_5,
+                                        MCA_BASE_VAR_SCOPE_READONLY,
+                                        &coll_tuned_alltoall_segment_size);
 
-    mca_param_indices->max_requests_param_index
-       = mca_base_param_reg_int(&mca_coll_tuned_component.super.collm_version,
-                                "alltoall_algorithm_max_requests",
-                                "Maximum number of outstanding send or recv requests.  Only has meaning for synchronized algorithms.",
-                                false, false, 
-                                ompi_coll_tuned_init_max_requests, /* get system wide default */
-                                NULL);
+    coll_tuned_alltoall_tree_fanout = ompi_coll_tuned_init_tree_fanout; /* get system wide default */
+    mca_param_indices->tree_fanout_param_index =
+        mca_base_component_var_register(&mca_coll_tuned_component.super.collm_version,
+                                        "alltoall_algorithm_tree_fanout",
+                                        "Fanout for n-tree used for alltoall algorithms. Only has meaning if algorithm is forced and supports n-tree topo based operation.",
+                                        MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
+                                        OPAL_INFO_LVL_5,
+                                        MCA_BASE_VAR_SCOPE_READONLY,
+                                        &coll_tuned_alltoall_tree_fanout);
+
+    coll_tuned_alltoall_chain_fanout = ompi_coll_tuned_init_chain_fanout; /* get system wide default */
+    mca_param_indices->chain_fanout_param_index = 
+      mca_base_component_var_register(&mca_coll_tuned_component.super.collm_version,
+                                      "alltoall_algorithm_chain_fanout",
+                                      "Fanout for chains used for alltoall algorithms. Only has meaning if algorithm is forced and supports chain topo based operation.",
+                                      MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
+                                      OPAL_INFO_LVL_5,
+                                      MCA_BASE_VAR_SCOPE_READONLY,
+                                      &coll_tuned_alltoall_chain_fanout);
+
+    coll_tuned_alltoall_max_requests = 0; /* no limit for alltoall by default */
+    mca_param_indices->max_requests_param_index = 
+      mca_base_component_var_register(&mca_coll_tuned_component.super.collm_version,
+                                      "alltoall_algorithm_max_requests",
+                                      "Maximum number of outstanding send or recv requests.  Only has meaning for synchronized algorithms.",
+                                      MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
+                                      OPAL_INFO_LVL_5,
+                                      MCA_BASE_VAR_SCOPE_READONLY,
+                                      &coll_tuned_alltoall_max_requests);
     if (mca_param_indices->max_requests_param_index < 0) {
-        return mca_param_indices->algorithm_param_index;
+        return mca_param_indices->max_requests_param_index;
     }
-    mca_base_param_lookup_int(mca_param_indices->max_requests_param_index, &(max_requests));
-    if( max_requests <= 1 ) {
+
+    if (coll_tuned_alltoall_max_requests < 0) {
         if( 0 == ompi_comm_rank( MPI_COMM_WORLD ) ) {
             opal_output( 0, "Maximum outstanding requests must be positive number greater than 1.  Switching to system level default %d \n",
                          ompi_coll_tuned_init_max_requests );
         }
-        mca_base_param_set_int( mca_param_indices->max_requests_param_index, 
-                                ompi_coll_tuned_init_max_requests);
+        coll_tuned_alltoall_max_requests = 0;
     }
 
     return (MPI_SUCCESS);
@@ -683,7 +812,7 @@ int ompi_coll_tuned_alltoall_intra_do_forced(void *sbuf, int scount,
                                              void* rbuf, int rcount,
                                              struct ompi_datatype_t *rdtype,
                                              struct ompi_communicator_t *comm,
-					     mca_coll_base_module_t *module)
+                                             mca_coll_base_module_t *module)
 {
     mca_coll_tuned_module_t *tuned_module = (mca_coll_tuned_module_t*) module;
     mca_coll_tuned_comm_t *data = tuned_module->tuned_data;
@@ -712,7 +841,7 @@ int ompi_coll_tuned_alltoall_intra_do_this(void *sbuf, int scount,
                                            void* rbuf, int rcount,
                                            struct ompi_datatype_t *rdtype,
                                            struct ompi_communicator_t *comm,
-					   mca_coll_base_module_t *module,
+                                           mca_coll_base_module_t *module,
                                            int algorithm, int faninout, int segsize, 
                                            int max_requests)
 {
