@@ -1,8 +1,9 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2012 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -10,6 +11,12 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
+ * Copyright (c) 2013      Los Alamos National Security, LLC. All Rights
+ *                         reserved.
+ * Copyright (c) 2013      FUJITSU LIMITED.  All rights reserved.
+ * Copyright (c) 2014      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2015      Intel, Inc. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -30,6 +37,114 @@
 #include "coll_tuned_topo.h"
 #include "coll_tuned_util.h"
 
+/* alltoallv algorithm variables */
+static int coll_tuned_alltoallv_algorithm_count = 2;
+static int coll_tuned_alltoallv_forced_algorithm = 0;
+
+/* valid values for coll_tuned_alltoallv_forced_algorithm */
+static mca_base_var_enum_value_t alltoallv_algorithms[] = {
+    {0, "ignore"},
+    {1, "basic_linear"},
+    {2, "pairwise"},
+    {0, NULL}
+};
+
+static int
+mca_coll_tuned_alltoallv_intra_basic_inplace(void *rbuf, const int *rcounts, const int *rdisps,
+                                             struct ompi_datatype_t *rdtype,
+                                             struct ompi_communicator_t *comm,
+                                             mca_coll_base_module_t *module)
+{
+    mca_coll_tuned_module_t *tuned_module = (mca_coll_tuned_module_t*) module;
+    int i, j, size, rank, err=MPI_SUCCESS;
+    MPI_Request *preq;
+    char *tmp_buffer;
+    size_t max_size, rdtype_size;
+    ptrdiff_t ext;
+
+    /* Initialize. */
+
+    size = ompi_comm_size(comm);
+    rank = ompi_comm_rank(comm);
+    ompi_datatype_type_size(rdtype, &rdtype_size);
+
+    /* If only one process, we're done. */
+    if (1 == size || 0 == rdtype_size) {
+        return MPI_SUCCESS;
+    }
+
+    /* Find the largest receive amount */
+    ompi_datatype_type_extent (rdtype, &ext);
+    for (i = 0, max_size = 0 ; i < size ; ++i) {
+        size_t size = ext * rcounts[i];
+
+        max_size = size > max_size ? size : max_size;
+    }
+
+    /* Allocate a temporary buffer */
+    tmp_buffer = calloc (max_size, 1);
+    if (NULL == tmp_buffer) {
+        return OMPI_ERR_OUT_OF_RESOURCE;
+    }
+
+    /* in-place alltoallv slow algorithm (but works) */
+    for (i = 0 ; i < size ; ++i) {
+        for (j = i+1 ; j < size ; ++j) {
+            /* Initiate all send/recv to/from others. */
+            preq = tuned_module->tuned_data->mcct_reqs;
+
+            if (i == rank && rcounts[j]) {
+                /* Copy the data into the temporary buffer */
+                err = ompi_datatype_copy_content_same_ddt (rdtype, rcounts[j],
+                                                           tmp_buffer, (char *) rbuf + rdisps[j] * ext);
+                if (MPI_SUCCESS != err) { goto error_hndl; }
+
+                /* Exchange data with the peer */
+                err = MCA_PML_CALL(irecv ((char *) rbuf + rdisps[j] * ext, rcounts[j], rdtype,
+                                          j, MCA_COLL_BASE_TAG_ALLTOALLV, comm, preq++));
+                if (MPI_SUCCESS != err) { goto error_hndl; }
+
+                err = MCA_PML_CALL(isend ((void *) tmp_buffer,  rcounts[j], rdtype,
+                                          j, MCA_COLL_BASE_TAG_ALLTOALLV, MCA_PML_BASE_SEND_STANDARD,
+                                          comm, preq++));
+                if (MPI_SUCCESS != err) { goto error_hndl; }
+            } else if (j == rank && rcounts[i]) {
+                /* Copy the data into the temporary buffer */
+                err = ompi_datatype_copy_content_same_ddt (rdtype, rcounts[i],
+                                                           tmp_buffer, (char *) rbuf + rdisps[i] * ext);
+                if (MPI_SUCCESS != err) { goto error_hndl; }
+
+                /* Exchange data with the peer */
+                err = MCA_PML_CALL(irecv ((char *) rbuf + rdisps[i] * ext, rcounts[i], rdtype,
+                                          i, MCA_COLL_BASE_TAG_ALLTOALLV, comm, preq++));
+                if (MPI_SUCCESS != err) { goto error_hndl; }
+
+                err = MCA_PML_CALL(isend ((void *) tmp_buffer,  rcounts[i], rdtype,
+                                          i, MCA_COLL_BASE_TAG_ALLTOALLV, MCA_PML_BASE_SEND_STANDARD,
+                                          comm, preq++));
+                if (MPI_SUCCESS != err) { goto error_hndl; }
+            } else {
+                continue;
+            }
+
+            /* Wait for the requests to complete */
+            err = ompi_request_wait_all (2, tuned_module->tuned_data->mcct_reqs, MPI_STATUSES_IGNORE);
+            if (MPI_SUCCESS != err) { goto error_hndl; }
+
+            /* Free the requests. */
+            mca_coll_tuned_free_reqs(tuned_module->tuned_data->mcct_reqs, 2);
+        }
+    }
+
+ error_hndl:
+    /* Free the temporary buffer */
+    free (tmp_buffer);
+
+    /* All done */
+
+    return err;
+}
+
 int
 ompi_coll_tuned_alltoallv_intra_pairwise(void *sbuf, int *scounts, int *sdisps,
                                          struct ompi_datatype_t *sdtype,
@@ -38,12 +153,15 @@ ompi_coll_tuned_alltoallv_intra_pairwise(void *sbuf, int *scounts, int *sdisps,
                                          struct ompi_communicator_t *comm,
                                          mca_coll_base_module_t *module)
 {
-    int line = -1, err = 0;
-    int rank, size, step;
-    int sendto, recvfrom;
+    int line = -1, err = 0, rank, size, step = 0, sendto, recvfrom;
     void *psnd, *prcv;
     ptrdiff_t sext, rext;
 
+    if (MPI_IN_PLACE == sbuf) {
+        return mca_coll_tuned_alltoallv_intra_basic_inplace (rbuf, rcounts, rdisps,
+                                                             rdtype, comm, module);
+    }
+
     size = ompi_comm_size(comm);
     rank = ompi_comm_rank(comm);
 
@@ -53,32 +171,16 @@ ompi_coll_tuned_alltoallv_intra_pairwise(void *sbuf, int *scounts, int *sdisps,
     ompi_datatype_type_extent(sdtype, &sext);
     ompi_datatype_type_extent(rdtype, &rext);
 
-    psnd = ((char *) sbuf) + (sdisps[rank] * sext);
-    prcv = ((char *) rbuf) + (rdisps[rank] * rext);
-
-    if (0 != scounts[rank]) {
-        err = ompi_datatype_sndrcv(psnd, scounts[rank], sdtype,
-                              prcv, rcounts[rank], rdtype);
-        if (MPI_SUCCESS != err) {
-            return err;
-        }
-    }
-
-    /* If only one process, we're done. */
-    if (1 == size) {
-        return MPI_SUCCESS;
-    }
-
-    /* Perform pairwise exchange starting from 1 since local exhange is done */
-    for (step = 1; step < size + 1; step++) {
+   /* Perform pairwise exchange starting from 1 since local exhange is done */
+    for (step = 0; step < size; step++) {
 
         /* Determine sender and receiver for this step. */
         sendto  = (rank + step) % size;
         recvfrom = (rank + size - step) % size;
 
         /* Determine sending and receiving locations */
-        psnd = (char*)sbuf + sdisps[sendto] * sext;
-        prcv = (char*)rbuf + rdisps[recvfrom] * rext;
+        psnd = (char*)sbuf + (ptrdiff_t)sdisps[sendto] * sext;
+        prcv = (char*)rbuf + (ptrdiff_t)rdisps[recvfrom] * rext;
 
         /* send and receive */
         err = ompi_coll_tuned_sendrecv( psnd, scounts[sendto], sdtype, sendto, 
@@ -86,15 +188,15 @@ ompi_coll_tuned_alltoallv_intra_pairwise(void *sbuf, int *scounts, int *sdisps,
                                         prcv, rcounts[recvfrom], rdtype, recvfrom, 
                                         MCA_COLL_BASE_TAG_ALLTOALLV,
                                         comm, MPI_STATUS_IGNORE, rank);
-        if (err != MPI_SUCCESS) { line = __LINE__; goto err_hndl;  }
+        if (MPI_SUCCESS != err) { line = __LINE__; goto err_hndl;  }
     }
 
     return MPI_SUCCESS;
  
  err_hndl:
-    OPAL_OUTPUT((ompi_coll_tuned_stream,
-                 "%s:%4d\tError occurred %d, rank %2d", __FILE__, line, 
-                 err, rank));
+    opal_output_verbose(COLL_TUNED_VERBOSITY, ompi_coll_tuned_stream,
+                 "%s:%4d\tError occurred %d, rank %2d at step %d", __FILE__, line, 
+                 err, rank, step);
     return err;
 }
 
@@ -115,14 +217,18 @@ ompi_coll_tuned_alltoallv_intra_basic_linear(void *sbuf, int *scounts, int *sdis
                                             struct ompi_communicator_t *comm,
                                             mca_coll_base_module_t *module)
 {
-    int i, size, rank, err;
+    int i, size, rank, err, nreqs;
     char *psnd, *prcv;
-    int nreqs;
     ptrdiff_t sext, rext;
     MPI_Request *preq;
     mca_coll_tuned_module_t *tuned_module = (mca_coll_tuned_module_t*) module;
     mca_coll_tuned_comm_t *data = tuned_module->tuned_data;
 
+    if (MPI_IN_PLACE == sbuf) {
+        return  mca_coll_tuned_alltoallv_intra_basic_inplace (rbuf, rcounts, rdisps,
+                                                              rdtype, comm, module);
+    }
+
     size = ompi_comm_size(comm);
     rank = ompi_comm_rank(comm);
 
@@ -133,8 +239,8 @@ ompi_coll_tuned_alltoallv_intra_basic_linear(void *sbuf, int *scounts, int *sdis
     ompi_datatype_type_extent(rdtype, &rext);
 
     /* Simple optimization - handle send to self first */
-    psnd = ((char *) sbuf) + (sdisps[rank] * sext);
-    prcv = ((char *) rbuf) + (rdisps[rank] * rext);
+    psnd = ((char *) sbuf) + (ptrdiff_t)sdisps[rank] * sext;
+    prcv = ((char *) rbuf) + (ptrdiff_t)rdisps[rank] * rext;
     if (0 != scounts[rank]) {
         err = ompi_datatype_sndrcv(psnd, scounts[rank], sdtype,
                               prcv, rcounts[rank], rdtype);
@@ -158,7 +264,7 @@ ompi_coll_tuned_alltoallv_intra_basic_linear(void *sbuf, int *scounts, int *sdis
             continue;
         }
 
-        prcv = ((char *) rbuf) + (rdisps[i] * rext);
+        prcv = ((char *) rbuf) + (ptrdiff_t)rdisps[i] * rext;
         err = MCA_PML_CALL(irecv_init(prcv, rcounts[i], rdtype,
                                       i, MCA_COLL_BASE_TAG_ALLTOALLV, comm,
                                       preq++));
@@ -175,7 +281,7 @@ ompi_coll_tuned_alltoallv_intra_basic_linear(void *sbuf, int *scounts, int *sdis
             continue;
         }
 
-        psnd = ((char *) sbuf) + (sdisps[i] * sext);
+        psnd = ((char *) sbuf) + (ptrdiff_t)sdisps[i] * sext;
         err = MCA_PML_CALL(isend_init(psnd, scounts[i], sdtype,
                                       i, MCA_COLL_BASE_TAG_ALLTOALLV,
                                       MCA_PML_BASE_SEND_STANDARD, comm,
@@ -217,34 +323,36 @@ ompi_coll_tuned_alltoallv_intra_basic_linear(void *sbuf, int *scounts, int *sdis
 int ompi_coll_tuned_alltoallv_intra_check_forced_init(coll_tuned_force_algorithm_mca_param_indices_t
                                                       *mca_param_indices)
 {
-    int max_alg = 2, requested_alg;
+    mca_base_var_enum_t *new_enum;
 
-    ompi_coll_tuned_forced_max_algorithms[ALLTOALLV] = max_alg;
+    ompi_coll_tuned_forced_max_algorithms[ALLTOALLV] = coll_tuned_alltoallv_algorithm_count;
 
-    mca_base_param_reg_int (&mca_coll_tuned_component.super.collm_version,
-                            "alltoallv_algorithm_count",
-                            "Number of alltoallv algorithms available",
-                            false, true, max_alg, NULL);
+    (void) mca_base_component_var_register(&mca_coll_tuned_component.super.collm_version,
+                                           "alltoallv_algorithm_count",
+                                           "Number of alltoallv algorithms available",
+                                           MCA_BASE_VAR_TYPE_INT, NULL, 0,
+                                           MCA_BASE_VAR_FLAG_DEFAULT_ONLY,
+                                           OPAL_INFO_LVL_5,
+                                           MCA_BASE_VAR_SCOPE_CONSTANT,
+                                           &coll_tuned_alltoallv_algorithm_count);
 
-    mca_param_indices->algorithm_param_index
-        = mca_base_param_reg_int(&mca_coll_tuned_component.super.collm_version,
-                                 "alltoallv_algorithm",
-                                 "Which alltoallv algorithm is used. "
-                                 "Can be locked down to choice of: 0 ignore, "
-                                 "1 basic linear, 2 pairwise.",
-                                 false, false, 0, NULL);
+    /* MPI_T: This variable should eventually be bound to a communicator */
+    coll_tuned_alltoallv_forced_algorithm = 0;
+    (void) mca_base_var_enum_create("coll_tuned_alltoallv_algorithms", alltoallv_algorithms, &new_enum);
+    mca_param_indices->algorithm_param_index =
+        mca_base_component_var_register(&mca_coll_tuned_component.super.collm_version,
+                                        "alltoallv_algorithm",
+                                        "Which alltoallv algorithm is used. "
+                                        "Can be locked down to choice of: 0 ignore, "
+                                        "1 basic linear, 2 pairwise.",
+                                        MCA_BASE_VAR_TYPE_INT, new_enum, 0, 0,
+                                        OPAL_INFO_LVL_5,
+                                        MCA_BASE_VAR_SCOPE_READONLY,
+                                        &coll_tuned_alltoallv_forced_algorithm);
+    OBJ_RELEASE(new_enum);
     if (mca_param_indices->algorithm_param_index < 0) {
         return mca_param_indices->algorithm_param_index;
     }
-    mca_base_param_lookup_int(mca_param_indices->algorithm_param_index, &(requested_alg));
-    if (0 > requested_alg || requested_alg > max_alg) {
-        if (0 == ompi_comm_rank( MPI_COMM_WORLD)) {
-            opal_output(0, "Alltoallv algorithm #%d is not available (range [0..%d]). "
-                        "Switching back to ignore(0)\n",
-                        requested_alg, max_alg );
-        }
-        mca_base_param_set_int(mca_param_indices->algorithm_param_index, 0);
-    }
     
     return (MPI_SUCCESS);
 }
@@ -305,16 +413,16 @@ int ompi_coll_tuned_alltoallv_intra_do_this(void *sbuf, int *scounts, int *sdisp
     switch (algorithm) {
     case (0):
         return ompi_coll_tuned_alltoallv_intra_dec_fixed(sbuf, scounts, sdisps, sdtype,
-                                                        rbuf, rcounts, rdisps, rdtype,
-                                                        comm, module);
+                                                         rbuf, rcounts, rdisps, rdtype,
+                                                         comm, module);
     case (1):
         return ompi_coll_tuned_alltoallv_intra_basic_linear(sbuf, scounts, sdisps, sdtype,
-                                                           rbuf, rcounts, rdisps, rdtype,
-                                                           comm, module);
+                                                            rbuf, rcounts, rdisps, rdtype,
+                                                            comm, module);
     case (2):
         return ompi_coll_tuned_alltoallv_intra_pairwise(sbuf, scounts, sdisps, sdtype,
-                                                       rbuf, rcounts, rdisps, rdtype,
-                                                       comm, module);
+                                                        rbuf, rcounts, rdisps, rdtype,
+                                                        comm, module);
     default:
         OPAL_OUTPUT((ompi_coll_tuned_stream,
                      "coll:tuned:alltoall_intra_do_this attempt to select "
