@@ -1,8 +1,9 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2014 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -10,6 +11,9 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
+ * Copyright (c) 2013      Los Alamos National Security, LLC. All Rights
+ *                         reserved.
+ * Copyright (c) 2015      Intel, Inc. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -20,6 +24,7 @@
 #include "ompi_config.h"
 
 #include "mpi.h"
+#include "opal/util/bit_ops.h"
 #include "ompi/constants.h"
 #include "ompi/communicator/communicator.h"
 #include "ompi/mca/coll/coll.h"
@@ -29,6 +34,79 @@
 #include "coll_tuned_topo.h"
 #include "coll_tuned_util.h"
 
+/* barrier algorithm variables */
+static int coll_tuned_barrier_algorithm_count = 6;
+static int coll_tuned_barrier_forced_algorithm = 0;
+
+/* valid values for coll_tuned_barrier_forced_algorithm */
+static mca_base_var_enum_value_t barrier_algorithms[] = {
+    {0, "ignore"},
+    {1, "linear"},
+    {2, "double_ring"},
+    {3, "recursive_doubling"},
+    {4, "bruck"},
+    {5, "two_proc"},
+    {6, "tree"},
+    {0, NULL}
+};
+
+/**
+ * A quick version of the MPI_Sendreceive implemented for the barrier.
+ * No actual data is moved across the wire, we use 0-byte messages to
+ * signal a two peer synchronization.
+ */
+static inline int
+ompi_coll_tuned_sendrecv_zero(int dest, int stag,
+                              int source, int rtag,
+                              MPI_Comm comm)
+
+{
+    int err, line = 0;
+    ompi_request_t* reqs[2];
+    ompi_status_public_t statuses[2];
+
+    /* post new irecv */
+    err = MCA_PML_CALL(irecv( NULL, 0, MPI_BYTE, source, rtag,
+                              comm, &reqs[0]));
+    if (err != MPI_SUCCESS) { line = __LINE__; goto error_handler; }
+
+    /* send data to children */
+    err = MCA_PML_CALL(isend( NULL, 0, MPI_BYTE, dest, stag,
+                              MCA_PML_BASE_SEND_STANDARD, comm, &reqs[1]));
+    if (err != MPI_SUCCESS) { line = __LINE__; goto error_handler; }
+
+    err = ompi_request_wait_all( 2, reqs, statuses );
+    if (err != MPI_SUCCESS) { line = __LINE__; goto error_handler; }
+
+    return (MPI_SUCCESS);
+
+ error_handler:
+    /* As we use wait_all we will get MPI_ERR_IN_STATUS which is not an error
+     * code that we can propagate up the stack. Instead, look for the real
+     * error code from the MPI_ERROR in the status.
+     */
+    if( MPI_ERR_IN_STATUS == err ) {
+        /* At least we know the error was detected during the wait_all */
+        int err_index = 1;
+        if( MPI_SUCCESS == statuses[0].MPI_ERROR ) {
+            err_index = 0;
+        }
+        err = statuses[err_index].MPI_ERROR;
+        opal_output_verbose(COLL_TUNED_VERBOSITY, ompi_coll_tuned_stream,
+                            "%s:%d: Error %d occurred in the %s"
+                            " stage of ompi_coll_tuned_sendrecv_zero\n",
+                            __FILE__, line, err, (0 == err_index ? "receive" : "send"));
+    } else {
+        /* Error discovered during the posting of the irecv or isend,
+         * and no status is available.
+         */
+        opal_output_verbose(COLL_TUNED_VERBOSITY, ompi_coll_tuned_stream,
+                            "%s:%d: Error %d occurred\n",
+                            __FILE__, line, err);
+    }
+    return err;
+}
+
 /*
  * Barrier is ment to be a synchronous operation, as some BTLs can mark 
  * a request done before its passed to the NIC and progress might not be made 
@@ -49,12 +127,9 @@
  *
  */
 int ompi_coll_tuned_barrier_intra_doublering(struct ompi_communicator_t *comm,
-					     mca_coll_base_module_t *module)
+                                             mca_coll_base_module_t *module)
 {
-    int rank, size;
-    int err=0, line=0;
-    int left, right;
-
+    int rank, size, err = 0, line = 0, left, right;
 
     rank = ompi_comm_rank(comm);
     size = ompi_comm_size(comm);
@@ -110,8 +185,9 @@ int ompi_coll_tuned_barrier_intra_doublering(struct ompi_communicator_t *comm,
     return MPI_SUCCESS;
 
  err_hndl:
-    OPAL_OUTPUT((ompi_coll_tuned_stream,"%s:%4d\tError occurred %d, rank %2d", 
-                 __FILE__, line, err, rank));
+    opal_output_verbose(COLL_TUNED_VERBOSITY, ompi_coll_tuned_stream,
+                        "%s:%4d\tError occurred %d, rank %2d", 
+                        __FILE__, line, err, rank);
     return err;
 }
 
@@ -121,11 +197,9 @@ int ompi_coll_tuned_barrier_intra_doublering(struct ompi_communicator_t *comm,
  */
 
 int ompi_coll_tuned_barrier_intra_recursivedoubling(struct ompi_communicator_t *comm,
-						    mca_coll_base_module_t *module)
+                                                    mca_coll_base_module_t *module)
 {
-    int rank, size, adjsize;
-    int err, line;
-    int mask, remote;
+    int rank, size, adjsize, err, line, mask, remote;
 
     rank = ompi_comm_rank(comm);
     size = ompi_comm_size(comm);
@@ -134,7 +208,7 @@ int ompi_coll_tuned_barrier_intra_recursivedoubling(struct ompi_communicator_t *
                  rank));
 
     /* do nearest power of 2 less than size calc */
-    for( adjsize = 1; adjsize <= size; adjsize <<= 1 );
+    adjsize = opal_next_poweroftwo(size);
     adjsize >>= 1;
 
     /* if size is not exact power of two, perform an extra step */
@@ -142,11 +216,9 @@ int ompi_coll_tuned_barrier_intra_recursivedoubling(struct ompi_communicator_t *
         if (rank >= adjsize) {
             /* send message to lower ranked node */
             remote = rank - adjsize;
-            err = ompi_coll_tuned_sendrecv_actual(NULL, 0, MPI_BYTE, remote,
-                                                  MCA_COLL_BASE_TAG_BARRIER,
-                                                  NULL, 0, MPI_BYTE, remote,
-                                                  MCA_COLL_BASE_TAG_BARRIER,
-                                                  comm, MPI_STATUS_IGNORE);
+            err = ompi_coll_tuned_sendrecv_zero(remote, MCA_COLL_BASE_TAG_BARRIER,
+                                                remote, MCA_COLL_BASE_TAG_BARRIER,
+                                                comm);
             if (err != MPI_SUCCESS) { line = __LINE__; goto err_hndl;}
 
         } else if (rank < (size - adjsize)) {
@@ -169,11 +241,9 @@ int ompi_coll_tuned_barrier_intra_recursivedoubling(struct ompi_communicator_t *
             if (remote >= adjsize) continue;
 
             /* post receive from the remote node */
-            err = ompi_coll_tuned_sendrecv_actual(NULL, 0, MPI_BYTE, remote,
-                                                  MCA_COLL_BASE_TAG_BARRIER,
-                                                  NULL, 0, MPI_BYTE, remote,
-                                                  MCA_COLL_BASE_TAG_BARRIER,
-                                                  comm, MPI_STATUS_IGNORE);
+            err = ompi_coll_tuned_sendrecv_zero(remote, MCA_COLL_BASE_TAG_BARRIER,
+                                                remote, MCA_COLL_BASE_TAG_BARRIER,
+                                                comm);
             if (err != MPI_SUCCESS) { line = __LINE__; goto err_hndl;}
         }
     }
@@ -194,8 +264,9 @@ int ompi_coll_tuned_barrier_intra_recursivedoubling(struct ompi_communicator_t *
     return MPI_SUCCESS;
 
  err_hndl:
-    OPAL_OUTPUT((ompi_coll_tuned_stream,"%s:%4d\tError occurred %d, rank %2d",
-                 __FILE__, line, err, rank));
+    opal_output_verbose(COLL_TUNED_VERBOSITY, ompi_coll_tuned_stream,
+                        "%s:%4d\tError occurred %d, rank %2d",
+                        __FILE__, line, err, rank);
     return err;
 }
 
@@ -205,11 +276,9 @@ int ompi_coll_tuned_barrier_intra_recursivedoubling(struct ompi_communicator_t *
  */
 
 int ompi_coll_tuned_barrier_intra_bruck(struct ompi_communicator_t *comm,
-					mca_coll_base_module_t *module)
+                                        mca_coll_base_module_t *module)
 {
-    int rank, size;
-    int distance, to, from;
-    int err, line = 0;
+    int rank, size, distance, to, from, err, line = 0;
 
     rank = ompi_comm_rank(comm);
     size = ompi_comm_size(comm);
@@ -222,19 +291,18 @@ int ompi_coll_tuned_barrier_intra_bruck(struct ompi_communicator_t *comm,
         to   = (rank + distance) % size;
 
         /* send message to lower ranked node */
-        err = ompi_coll_tuned_sendrecv_actual(NULL, 0, MPI_BYTE, to, 
-                                              MCA_COLL_BASE_TAG_BARRIER,
-                                              NULL, 0, MPI_BYTE, from, 
-                                              MCA_COLL_BASE_TAG_BARRIER,
-                                              comm, MPI_STATUS_IGNORE);
+        err = ompi_coll_tuned_sendrecv_zero(to, MCA_COLL_BASE_TAG_BARRIER,
+                                            from, MCA_COLL_BASE_TAG_BARRIER,
+                                            comm);
         if (err != MPI_SUCCESS) { line = __LINE__; goto err_hndl;}
     }
 
     return MPI_SUCCESS;
 
  err_hndl:
-    OPAL_OUTPUT((ompi_coll_tuned_stream,"%s:%4d\tError occurred %d, rank %2d", 
-                 __FILE__, line, err, rank));
+    opal_output_verbose(COLL_TUNED_VERBOSITY, ompi_coll_tuned_stream,
+                        "%s:%4d\tError occurred %d, rank %2d", 
+                        __FILE__, line, err, rank);
     return err;
 }
 
@@ -244,7 +312,7 @@ int ompi_coll_tuned_barrier_intra_bruck(struct ompi_communicator_t *comm,
  */
 /* special case for two processes */
 int ompi_coll_tuned_barrier_intra_two_procs(struct ompi_communicator_t *comm,
-					    mca_coll_base_module_t *module)
+                                            mca_coll_base_module_t *module)
 {
     int remote, err;
 
@@ -253,11 +321,9 @@ int ompi_coll_tuned_barrier_intra_two_procs(struct ompi_communicator_t *comm,
                  "ompi_coll_tuned_barrier_intra_two_procs rank %d", remote));
     remote = (remote + 1) & 0x1;
 
-    err = ompi_coll_tuned_sendrecv_actual(NULL, 0, MPI_BYTE, remote, 
-                                          MCA_COLL_BASE_TAG_BARRIER, 
-                                          NULL, 0, MPI_BYTE, remote, 
-                                          MCA_COLL_BASE_TAG_BARRIER,
-                                          comm, MPI_STATUS_IGNORE);
+    err = ompi_coll_tuned_sendrecv_zero(remote, MCA_COLL_BASE_TAG_BARRIER, 
+                                        remote, MCA_COLL_BASE_TAG_BARRIER,
+                                        comm);
     return (err);
 }
 
@@ -277,14 +343,14 @@ int ompi_coll_tuned_barrier_intra_two_procs(struct ompi_communicator_t *comm,
 /* copied function (with appropriate renaming) starts here */
 
 static int ompi_coll_tuned_barrier_intra_basic_linear(struct ompi_communicator_t *comm,
-						      mca_coll_base_module_t *module)
+                                                      mca_coll_base_module_t *module)
 {
-    int i, err;
-    int size = ompi_comm_size(comm);
-    int rank = ompi_comm_rank(comm);
+    int i, err, rank, size;
+
+    rank = ompi_comm_rank(comm);
+    size = ompi_comm_size(comm);
 
     /* All non-root send & receive zero-length message. */
-
     if (rank > 0) {
         err = MCA_PML_CALL(send (NULL, 0, MPI_BYTE, 0, 
                                  MCA_COLL_BASE_TAG_BARRIER,
@@ -344,8 +410,7 @@ static int ompi_coll_tuned_barrier_intra_basic_linear(struct ompi_communicator_t
 int ompi_coll_tuned_barrier_intra_tree(struct ompi_communicator_t *comm,
                                        mca_coll_base_module_t *module)
 {
-    int rank, size, depth;
-    int err, jump, partner;
+    int rank, size, depth, err, jump, partner;
 
     rank = ompi_comm_rank(comm);
     size = ompi_comm_size(comm);
@@ -354,7 +419,7 @@ int ompi_coll_tuned_barrier_intra_tree(struct ompi_communicator_t *comm,
                  rank));
 
     /* Find the nearest power of 2 of the communicator size. */
-    for(depth = 1; depth < size; depth <<= 1 );
+    depth = opal_next_poweroftwo_inclusive(size);
 
     for (jump=1; jump<depth; jump<<=1) {
         partner = rank ^ jump;
@@ -375,7 +440,7 @@ int ompi_coll_tuned_barrier_intra_tree(struct ompi_communicator_t *comm,
         }
     }
     
-    depth>>=1;
+    depth >>= 1;
     for (jump = depth; jump>0; jump>>=1) {
         partner = rank ^ jump;
         if (!(partner & (jump-1)) && partner < size) {
@@ -412,32 +477,34 @@ int ompi_coll_tuned_barrier_intra_tree(struct ompi_communicator_t *comm,
 
 int ompi_coll_tuned_barrier_intra_check_forced_init (coll_tuned_force_algorithm_mca_param_indices_t *mca_param_indices)
 {
-    int max_alg = 6, requested_alg;
+    mca_base_var_enum_t *new_enum;
 
-    ompi_coll_tuned_forced_max_algorithms[BARRIER] = max_alg;
+    ompi_coll_tuned_forced_max_algorithms[BARRIER] = coll_tuned_barrier_algorithm_count;
 
-    mca_base_param_reg_int (&mca_coll_tuned_component.super.collm_version,
-                            "barrier_algorithm_count",
-                            "Number of barrier algorithms available",
-                            false, true, max_alg, NULL);
+    (void) mca_base_component_var_register(&mca_coll_tuned_component.super.collm_version,
+                                           "barrier_algorithm_count",
+                                           "Number of barrier algorithms available",
+                                           MCA_BASE_VAR_TYPE_INT, NULL, 0,
+                                           MCA_BASE_VAR_FLAG_DEFAULT_ONLY,
+                                           OPAL_INFO_LVL_5,
+                                           MCA_BASE_VAR_SCOPE_CONSTANT,
+                                           &coll_tuned_barrier_algorithm_count);
 
-    mca_param_indices->algorithm_param_index = 
-       mca_base_param_reg_int(&mca_coll_tuned_component.super.collm_version,
-                              "barrier_algorithm",
-                              "Which barrier algorithm is used. Can be locked down to choice of: 0 ignore, 1 linear, 2 double ring, 3: recursive doubling 4: bruck, 5: two proc only, 6: tree",
-                              false, false, 0, NULL);
+    /* MPI_T: This variable should eventually be bound to a communicator */
+    coll_tuned_barrier_forced_algorithm = 0;
+    (void) mca_base_var_enum_create("coll_tuned_barrier_algorithms", barrier_algorithms, &new_enum);
+    mca_param_indices->algorithm_param_index =
+        mca_base_component_var_register(&mca_coll_tuned_component.super.collm_version,
+                                        "barrier_algorithm",
+                                        "Which barrier algorithm is used. Can be locked down to choice of: 0 ignore, 1 linear, 2 double ring, 3: recursive doubling 4: bruck, 5: two proc only, 6: tree",
+                                        MCA_BASE_VAR_TYPE_INT, new_enum, 0, 0,
+                                        OPAL_INFO_LVL_5,
+                                        MCA_BASE_VAR_SCOPE_READONLY,
+                                        &coll_tuned_barrier_forced_algorithm);
+    OBJ_RELEASE(new_enum);
     if (mca_param_indices->algorithm_param_index < 0) {
         return mca_param_indices->algorithm_param_index;
     }
-    mca_base_param_lookup_int(mca_param_indices->algorithm_param_index, 
-                              &(requested_alg));
-    if( 0 > requested_alg || requested_alg > max_alg ) {
-        if( 0 == ompi_comm_rank( MPI_COMM_WORLD ) ) {
-            opal_output( 0, "Barrier algorithm #%d is not available (range [0..%d]). Switching back to ignore(0)\n",
-                         requested_alg, max_alg );
-        }
-        mca_base_param_set_int( mca_param_indices->algorithm_param_index, 0);
-    }
 
     return (MPI_SUCCESS);
 }
