@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2012 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -21,15 +21,12 @@
 
 #include "mpi.h"
 #include "ompi/constants.h"
-#include "ompi/datatype/datatype.h"
+#include "ompi/datatype/ompi_datatype.h"
 #include "ompi/communicator/communicator.h"
-#include "opal/mca/base/mca_base_param.h"
 #include "ompi/mca/coll/base/base.h"
 #include "ompi/mca/coll/coll.h"
 #include "ompi/mca/coll/base/coll_tags.h"
 #include "coll_tuned.h"
-#include "ompi/mca/pml/pml.h"
-#include "opal/util/bit_ops.h"
 
 #include "coll_tuned.h"
 
@@ -58,7 +55,7 @@ ompi_coll_tuned_allreduce_intra_dec_dynamic (void *sbuf, void *rbuf, int count,
                                              struct ompi_datatype_t *dtype,
                                              struct ompi_op_t *op,
                                              struct ompi_communicator_t *comm,
-					     mca_coll_base_module_t *module)
+                                             mca_coll_base_module_t *module)
 {
     mca_coll_tuned_module_t *tuned_module = (mca_coll_tuned_module_t*) module;
     mca_coll_tuned_comm_t *data = tuned_module->tuned_data;
@@ -71,26 +68,26 @@ ompi_coll_tuned_allreduce_intra_dec_dynamic (void *sbuf, void *rbuf, int count,
         int alg, faninout, segsize, ignoreme;
         size_t dsize;
 
-        ompi_ddt_type_size (dtype, &dsize);
+        ompi_datatype_type_size (dtype, &dsize);
         dsize *= count;
 
         alg = ompi_coll_tuned_get_target_method_params (data->com_rules[ALLREDUCE], 
                                                         dsize, &faninout, &segsize, &ignoreme);
 
         if (alg) {
-	    /* we have found a valid choice from the file based rules for this message size */
+            /* we have found a valid choice from the file based rules for this message size */
             return ompi_coll_tuned_allreduce_intra_do_this (sbuf, rbuf, count, dtype, op,
-							    comm, module,
+                                                            comm, module,
                                                             alg, faninout, segsize);
         } /* found a method */
     } /*end if any com rules to check */
 
     if (data->user_forced[ALLREDUCE].algorithm) {
         return ompi_coll_tuned_allreduce_intra_do_forced (sbuf, rbuf, count, dtype, op,
-							  comm, module);
+                                                          comm, module);
     }
     return ompi_coll_tuned_allreduce_intra_dec_fixed (sbuf, rbuf, count, dtype, op,
-						      comm, module);
+                                                      comm, module);
 }
 
 /*
@@ -106,7 +103,7 @@ int ompi_coll_tuned_alltoall_intra_dec_dynamic(void *sbuf, int scount,
                                                void* rbuf, int rcount, 
                                                struct ompi_datatype_t *rdtype, 
                                                struct ompi_communicator_t *comm,
-					       mca_coll_base_module_t *module)
+                                               mca_coll_base_module_t *module)
 {
     mca_coll_tuned_module_t *tuned_module = (mca_coll_tuned_module_t*) module;
     mca_coll_tuned_comm_t *data = tuned_module->tuned_data;
@@ -120,31 +117,30 @@ int ompi_coll_tuned_alltoall_intra_dec_dynamic(void *sbuf, int scount,
         int alg, faninout, segsize, max_requests;
         size_t dsize;
 
-        ompi_ddt_type_size (sdtype, &dsize);
+        ompi_datatype_type_size (sdtype, &dsize);
         comsize = ompi_comm_size(comm);
-        dsize *= comsize * scount;
+        dsize *= (ptrdiff_t)comsize * (ptrdiff_t)scount;
 
         alg = ompi_coll_tuned_get_target_method_params (data->com_rules[ALLTOALL], 
                                                         dsize, &faninout, &segsize, &max_requests);
 
         if (alg) {
-	    /* we have found a valid choice from the file based rules for this message size */
+            /* we have found a valid choice from the file based rules for this message size */
             return ompi_coll_tuned_alltoall_intra_do_this (sbuf, scount, sdtype,
-							   rbuf, rcount, rdtype,
-							   comm, module,
+                                                           rbuf, rcount, rdtype,
+                                                           comm, module,
                                                            alg, faninout, segsize, max_requests);
         } /* found a method */
     } /*end if any com rules to check */
 
-
     if (data->user_forced[ALLTOALL].algorithm) {
         return ompi_coll_tuned_alltoall_intra_do_forced (sbuf, scount, sdtype,
-							 rbuf, rcount, rdtype,
-							 comm, module);
+                                                         rbuf, rcount, rdtype,
+                                                         comm, module);
     }
     return ompi_coll_tuned_alltoall_intra_dec_fixed (sbuf, scount, sdtype,
-						     rbuf, rcount, rdtype,
-						     comm, module);
+                                                     rbuf, rcount, rdtype,
+                                                     comm, module);
 }
 
 /*
@@ -165,16 +161,27 @@ int ompi_coll_tuned_alltoallv_intra_dec_dynamic(void *sbuf, int *scounts, int *s
 
     OPAL_OUTPUT((ompi_coll_tuned_stream, "ompi_coll_tuned_alltoallv_intra_dec_dynamic"));
 
-    /* 
-     * BEGIN - File Based Rules
-     *
-     * Here is where we would check to see if we have some file based
-     * rules.  Currently, we do not, so move on to seeing if the user
-     * specified a specific algorithm.  If not, then use the fixed
-     * decision code to decide.
-     *
-     * END - File Based Rules
+    /**
+     * check to see if we have some filebased rules. As we don't have global
+     * knowledge about the total amount of data, use the first available rule.
+     * This allow the users to specify the alltoallv algorithm to be used only
+     * based on the communicator size.
      */
+    if (data->com_rules[ALLTOALLV]) {
+        int alg, faninout, segsize, max_requests;
+
+        alg = ompi_coll_tuned_get_target_method_params (data->com_rules[ALLTOALLV], 
+                                                        0, &faninout, &segsize, &max_requests);
+
+        if (alg) {
+            /* we have found a valid choice from the file based rules for this message size */
+            return ompi_coll_tuned_alltoallv_intra_do_this (sbuf, scounts, sdisps, sdtype,
+                                                            rbuf, rcounts, rdisps, rdtype,
+                                                            comm, module,
+                                                            alg);
+        } /* found a method */
+    } /*end if any com rules to check */
+
     if (data->user_forced[ALLTOALLV].algorithm) {
         return ompi_coll_tuned_alltoallv_intra_do_forced(sbuf, scounts, sdisps, sdtype,
                                                          rbuf, rcounts, rdisps, rdtype,
@@ -193,7 +200,7 @@ int ompi_coll_tuned_alltoallv_intra_dec_dynamic(void *sbuf, int *scounts, int *s
  *    Returns:    - MPI_SUCCESS or error code (passed from the barrier implementation)
  */
 int ompi_coll_tuned_barrier_intra_dec_dynamic(struct ompi_communicator_t *comm,
-					      mca_coll_base_module_t *module)
+                                              mca_coll_base_module_t *module)
 {
     mca_coll_tuned_module_t *tuned_module = (mca_coll_tuned_module_t*) module;
     mca_coll_tuned_comm_t *data = tuned_module->tuned_data;
@@ -209,7 +216,7 @@ int ompi_coll_tuned_barrier_intra_dec_dynamic(struct ompi_communicator_t *comm,
                                                         0, &faninout, &segsize, &ignoreme);
 
         if (alg) {
-	    /* we have found a valid choice from the file based rules for this message size */
+            /* we have found a valid choice from the file based rules for this message size */
             return ompi_coll_tuned_barrier_intra_do_this (comm, module,
                                                           alg, faninout, segsize);
         } /* found a method */
@@ -231,7 +238,7 @@ int ompi_coll_tuned_barrier_intra_dec_dynamic(struct ompi_communicator_t *comm,
 int ompi_coll_tuned_bcast_intra_dec_dynamic(void *buff, int count,
                                             struct ompi_datatype_t *datatype, int root,
                                             struct ompi_communicator_t *comm,
-					    mca_coll_base_module_t *module)
+                                            mca_coll_base_module_t *module)
 {
     mca_coll_tuned_module_t *tuned_module = (mca_coll_tuned_module_t*) module;
     mca_coll_tuned_comm_t *data = tuned_module->tuned_data;
@@ -244,16 +251,16 @@ int ompi_coll_tuned_bcast_intra_dec_dynamic(void *buff, int count,
         int alg, faninout, segsize, ignoreme;
         size_t dsize;
 
-        ompi_ddt_type_size (datatype, &dsize);
+        ompi_datatype_type_size (datatype, &dsize);
         dsize *= count;
 
         alg = ompi_coll_tuned_get_target_method_params (data->com_rules[BCAST], 
                                                         dsize, &faninout, &segsize, &ignoreme);
 
         if (alg) {
-	    /* we have found a valid choice from the file based rules for this message size */
+            /* we have found a valid choice from the file based rules for this message size */
             return ompi_coll_tuned_bcast_intra_do_this (buff, count, datatype, root,
-							comm, module,
+                                                        comm, module,
                                                         alg, faninout, segsize);
         } /* found a method */
     } /*end if any com rules to check */
@@ -261,10 +268,10 @@ int ompi_coll_tuned_bcast_intra_dec_dynamic(void *buff, int count,
 
     if (data->user_forced[BCAST].algorithm) {
         return ompi_coll_tuned_bcast_intra_do_forced (buff, count, datatype, root,
-						      comm, module);
+                                                      comm, module);
     }
     return ompi_coll_tuned_bcast_intra_dec_fixed (buff, count, datatype, root,
-						  comm, module);
+                                                  comm, module);
 }
 
 /*
@@ -279,7 +286,7 @@ int ompi_coll_tuned_reduce_intra_dec_dynamic( void *sendbuf, void *recvbuf,
                                               int count, struct ompi_datatype_t* datatype,
                                               struct ompi_op_t* op, int root,
                                               struct ompi_communicator_t* comm,
-					      mca_coll_base_module_t *module)
+                                              mca_coll_base_module_t *module)
 {
     mca_coll_tuned_module_t *tuned_module = (mca_coll_tuned_module_t*) module;
     mca_coll_tuned_comm_t *data = tuned_module->tuned_data;
@@ -293,17 +300,17 @@ int ompi_coll_tuned_reduce_intra_dec_dynamic( void *sendbuf, void *recvbuf,
         int alg, faninout, segsize, max_requests;
         size_t dsize;
 
-        ompi_ddt_type_size (datatype, &dsize);
+        ompi_datatype_type_size (datatype, &dsize);
         dsize *= count;
 
         alg = ompi_coll_tuned_get_target_method_params (data->com_rules[REDUCE], 
                                                         dsize, &faninout, &segsize, &max_requests);
 
         if (alg) {
-	    /* we have found a valid choice from the file based rules for this message size */
+            /* we have found a valid choice from the file based rules for this message size */
             return  ompi_coll_tuned_reduce_intra_do_this (sendbuf, recvbuf, count, datatype, 
                                                           op, root,
-							  comm, module,
+                                                          comm, module,
                                                           alg, faninout, 
                                                           segsize, 
                                                           max_requests);
@@ -312,12 +319,12 @@ int ompi_coll_tuned_reduce_intra_dec_dynamic( void *sendbuf, void *recvbuf,
 
     if (data->user_forced[REDUCE].algorithm) {
         return ompi_coll_tuned_reduce_intra_do_forced (sendbuf, recvbuf, count, datatype,
-						       op, root,
-						       comm, module);
+                                                       op, root,
+                                                       comm, module);
     }
     return ompi_coll_tuned_reduce_intra_dec_fixed (sendbuf, recvbuf, count, datatype,
-						   op, root,
-						   comm, module);
+                                                   op, root,
+                                                   comm, module);
 }
 
 /*
@@ -334,7 +341,7 @@ int ompi_coll_tuned_reduce_scatter_intra_dec_dynamic(void *sbuf, void *rbuf,
                                                      struct ompi_datatype_t *dtype,
                                                      struct ompi_op_t *op,
                                                      struct ompi_communicator_t *comm,
-						     mca_coll_base_module_t *module)
+                                                     mca_coll_base_module_t *module)
 {
     mca_coll_tuned_module_t *tuned_module = (mca_coll_tuned_module_t*) module;
     mca_coll_tuned_comm_t *data = tuned_module->tuned_data;
@@ -349,17 +356,17 @@ int ompi_coll_tuned_reduce_scatter_intra_dec_dynamic(void *sbuf, void *rbuf,
         size_t dsize;
         size = ompi_comm_size(comm);
         for (i = 0, count = 0; i < size; i++) { count += rcounts[i];}
-        ompi_ddt_type_size (dtype, &dsize);
+        ompi_datatype_type_size (dtype, &dsize);
         dsize *= count;
 
         alg = ompi_coll_tuned_get_target_method_params (data->com_rules[REDUCESCATTER], 
                                                         dsize, &faninout, 
                                                         &segsize, &ignoreme);
         if (alg) { 
-           /* we have found a valid choice from the file based rules for this message size */
+            /* we have found a valid choice from the file based rules for this message size */
             return  ompi_coll_tuned_reduce_scatter_intra_do_this (sbuf, rbuf, rcounts,
-								  dtype, op,
-								  comm, module,
+                                                                  dtype, op,
+                                                                  comm, module,
                                                                   alg, faninout, 
                                                                   segsize);
         } /* found a method */
@@ -371,8 +378,8 @@ int ompi_coll_tuned_reduce_scatter_intra_dec_dynamic(void *sbuf, void *rbuf,
                                                                comm, module);
     }
     return ompi_coll_tuned_reduce_scatter_intra_dec_fixed (sbuf, rbuf, rcounts,
-							   dtype, op,
-							   comm, module);
+                                                           dtype, op,
+                                                           comm, module);
 }
 
 /*
@@ -389,49 +396,49 @@ int ompi_coll_tuned_allgather_intra_dec_dynamic(void *sbuf, int scount,
                                                 void* rbuf, int rcount, 
                                                 struct ompi_datatype_t *rdtype, 
                                                 struct ompi_communicator_t *comm,
-						mca_coll_base_module_t *module)
+                                                mca_coll_base_module_t *module)
 {
     mca_coll_tuned_module_t *tuned_module = (mca_coll_tuned_module_t*) module;
     mca_coll_tuned_comm_t *data = tuned_module->tuned_data;
 
     OPAL_OUTPUT((ompi_coll_tuned_stream, 
-		 "ompi_coll_tuned_allgather_intra_dec_dynamic"));
+                 "ompi_coll_tuned_allgather_intra_dec_dynamic"));
    
     if (data->com_rules[ALLGATHER]) {
-	/* We have file based rules:
-	   - calculate message size and other necessary information */
-	int comsize;
-	int alg, faninout, segsize, ignoreme;
-	size_t dsize;
+        /* We have file based rules:
+           - calculate message size and other necessary information */
+        int comsize;
+        int alg, faninout, segsize, ignoreme;
+        size_t dsize;
       
-	ompi_ddt_type_size (sdtype, &dsize);
-	comsize = ompi_comm_size(comm);
-	dsize *= comsize * scount;
+        ompi_datatype_type_size (sdtype, &dsize);
+        comsize = ompi_comm_size(comm);
+        dsize *= (ptrdiff_t)comsize * (ptrdiff_t)scount;
       
-	alg = ompi_coll_tuned_get_target_method_params (data->com_rules[ALLGATHER], 
-							dsize, &faninout, &segsize, &ignoreme);
-	if (alg) { 
-	    /* we have found a valid choice from the file based rules for 
-	       this message size */
-	    return ompi_coll_tuned_allgather_intra_do_this (sbuf, scount, sdtype,
-							    rbuf, rcount, rdtype,
-							    comm, module,
-							    alg, faninout, segsize);
-	}
+        alg = ompi_coll_tuned_get_target_method_params (data->com_rules[ALLGATHER], 
+                                                        dsize, &faninout, &segsize, &ignoreme);
+        if (alg) { 
+            /* we have found a valid choice from the file based rules for 
+               this message size */
+            return ompi_coll_tuned_allgather_intra_do_this (sbuf, scount, sdtype,
+                                                            rbuf, rcount, rdtype,
+                                                            comm, module,
+                                                            alg, faninout, segsize);
+        }
     } 
 
     /* We do not have file based rules */
     if (data->user_forced[ALLGATHER].algorithm) {
-	/* User-forced algorithm */
-	return ompi_coll_tuned_allgather_intra_do_forced (sbuf, scount, sdtype, 
-							  rbuf, rcount, rdtype, 
-							  comm, module);
+        /* User-forced algorithm */
+        return ompi_coll_tuned_allgather_intra_do_forced (sbuf, scount, sdtype, 
+                                                          rbuf, rcount, rdtype, 
+                                                          comm, module);
     }
 
     /* Use default decision */
     return ompi_coll_tuned_allgather_intra_dec_fixed (sbuf, scount, sdtype, 
-						      rbuf, rcount, rdtype, 
-						      comm, module);
+                                                      rbuf, rcount, rdtype, 
+                                                      comm, module);
 }
 
 /*
@@ -449,100 +456,146 @@ int ompi_coll_tuned_allgatherv_intra_dec_dynamic(void *sbuf, int scount,
                                                  int *rdispls,
                                                  struct ompi_datatype_t *rdtype, 
                                                  struct ompi_communicator_t *comm,
-						 mca_coll_base_module_t *module)
+                                                 mca_coll_base_module_t *module)
 {
     mca_coll_tuned_module_t *tuned_module = (mca_coll_tuned_module_t*) module;
     mca_coll_tuned_comm_t *data = tuned_module->tuned_data;
    
     OPAL_OUTPUT((ompi_coll_tuned_stream, 
-		 "ompi_coll_tuned_allgatherv_intra_dec_dynamic"));
+                 "ompi_coll_tuned_allgatherv_intra_dec_dynamic"));
    
     if (data->com_rules[ALLGATHERV]) {
-	/* We have file based rules:
-	   - calculate message size and other necessary information */
-	int comsize, i;
-	int alg, faninout, segsize, ignoreme;
-	size_t dsize, total_size;
+        /* We have file based rules:
+           - calculate message size and other necessary information */
+        int comsize, i;
+        int alg, faninout, segsize, ignoreme;
+        size_t dsize, total_size;
 
-	comsize = ompi_comm_size(comm);      
-	ompi_ddt_type_size (sdtype, &dsize);
-	total_size = 0;
-	for (i = 0; i < comsize; i++) { total_size += dsize * rcounts[i]; }
+        comsize = ompi_comm_size(comm);      
+        ompi_datatype_type_size (sdtype, &dsize);
+        total_size = 0;
+        for (i = 0; i < comsize; i++) { total_size += dsize * rcounts[i]; }
       
-	alg = ompi_coll_tuned_get_target_method_params (data->com_rules[ALLGATHERV], 
-							total_size, &faninout, &segsize, &ignoreme);
-	if (alg) { 
-	    /* we have found a valid choice from the file based rules for 
-	       this message size */
-	    return ompi_coll_tuned_allgatherv_intra_do_this (sbuf, scount, sdtype,
-							     rbuf, rcounts, 
-							     rdispls, rdtype,
-							     comm, module,
-							     alg, faninout, segsize);
-	}
+        alg = ompi_coll_tuned_get_target_method_params (data->com_rules[ALLGATHERV], 
+                                                        total_size, &faninout, &segsize, &ignoreme);
+        if (alg) { 
+            /* we have found a valid choice from the file based rules for 
+               this message size */
+            return ompi_coll_tuned_allgatherv_intra_do_this (sbuf, scount, sdtype,
+                                                             rbuf, rcounts, 
+                                                             rdispls, rdtype,
+                                                             comm, module,
+                                                             alg, faninout, segsize);
+        }
     } 
 
     /* We do not have file based rules */
     if (data->user_forced[ALLGATHERV].algorithm) {
-	/* User-forced algorithm */
-	return ompi_coll_tuned_allgatherv_intra_do_forced (sbuf, scount, sdtype, 
-							   rbuf, rcounts, 
-							   rdispls, rdtype, 
-							   comm, module);
+        /* User-forced algorithm */
+        return ompi_coll_tuned_allgatherv_intra_do_forced (sbuf, scount, sdtype, 
+                                                           rbuf, rcounts, 
+                                                           rdispls, rdtype, 
+                                                           comm, module);
     }
 
     /* Use default decision */
     return ompi_coll_tuned_allgatherv_intra_dec_fixed (sbuf, scount, sdtype, 
-						       rbuf, rcounts, 
-						       rdispls, rdtype, 
-						       comm, module);
+                                                       rbuf, rcounts, 
+                                                       rdispls, rdtype, 
+                                                       comm, module);
 }
 
 int ompi_coll_tuned_gather_intra_dec_dynamic(void *sbuf, int scount, 
-					     struct ompi_datatype_t *sdtype,
-					     void* rbuf, int rcount, 
-					     struct ompi_datatype_t *rdtype, 
-					     int root,
-					     struct ompi_communicator_t *comm,
-					     mca_coll_base_module_t *module)
+                                             struct ompi_datatype_t *sdtype,
+                                             void* rbuf, int rcount, 
+                                             struct ompi_datatype_t *rdtype, 
+                                             int root,
+                                             struct ompi_communicator_t *comm,
+                                             mca_coll_base_module_t *module)
 {
     mca_coll_tuned_module_t *tuned_module = (mca_coll_tuned_module_t*) module;
     mca_coll_tuned_comm_t *data = tuned_module->tuned_data;
 
     OPAL_OUTPUT((ompi_coll_tuned_stream,
-		 "ompi_coll_tuned_gather_intra_dec_dynamic"));
+                 "ompi_coll_tuned_gather_intra_dec_dynamic"));
+
+    /**
+     * check to see if we have some filebased rules.
+     */
+    if (data->com_rules[GATHER]) {
+        int comsize, alg, faninout, segsize, max_requests;
+        size_t dsize;
+
+        comsize = ompi_comm_size(comm);      
+        ompi_datatype_type_size (sdtype, &dsize);
+        dsize *= comsize;
+
+        alg = ompi_coll_tuned_get_target_method_params (data->com_rules[GATHER], 
+                                                        dsize, &faninout, &segsize, &max_requests);
+
+        if (alg) {
+            /* we have found a valid choice from the file based rules for this message size */
+            return ompi_coll_tuned_gather_intra_do_this (sbuf, scount, sdtype,
+                                                         rbuf, rcount, rdtype,
+                                                         root, comm, module,
+                                                         alg, faninout, segsize);
+        } /* found a method */
+    } /*end if any com rules to check */
 
     if (data->user_forced[GATHER].algorithm) {
-	return ompi_coll_tuned_gather_intra_do_forced (sbuf, scount, sdtype,
-						       rbuf, rcount, rdtype,
-						       root, comm, module);
+        return ompi_coll_tuned_gather_intra_do_forced (sbuf, scount, sdtype,
+                                                       rbuf, rcount, rdtype,
+                                                       root, comm, module);
     }
 
     return ompi_coll_tuned_gather_intra_dec_fixed (sbuf, scount, sdtype, 
-						   rbuf, rcount, rdtype, 
-						   root, comm, module);
+                                                   rbuf, rcount, rdtype, 
+                                                   root, comm, module);
 }
 
 int ompi_coll_tuned_scatter_intra_dec_dynamic(void *sbuf, int scount, 
-					      struct ompi_datatype_t *sdtype,
-					      void* rbuf, int rcount, 
-					      struct ompi_datatype_t *rdtype, 
-					      int root, struct ompi_communicator_t *comm,
-					      mca_coll_base_module_t *module)
+                                              struct ompi_datatype_t *sdtype,
+                                              void* rbuf, int rcount, 
+                                              struct ompi_datatype_t *rdtype, 
+                                              int root, struct ompi_communicator_t *comm,
+                                              mca_coll_base_module_t *module)
 {
     mca_coll_tuned_module_t *tuned_module = (mca_coll_tuned_module_t*) module;
     mca_coll_tuned_comm_t *data = tuned_module->tuned_data;
 
     OPAL_OUTPUT((ompi_coll_tuned_stream,
-		 "ompi_coll_tuned_scatter_intra_dec_dynamic"));
+                 "ompi_coll_tuned_scatter_intra_dec_dynamic"));
+
+    /**
+     * check to see if we have some filebased rules.
+     */
+    if (data->com_rules[SCATTER]) {
+        int comsize, alg, faninout, segsize, max_requests;
+        size_t dsize;
+
+        comsize = ompi_comm_size(comm);      
+        ompi_datatype_type_size (sdtype, &dsize);
+        dsize *= comsize;
+
+        alg = ompi_coll_tuned_get_target_method_params (data->com_rules[SCATTER], 
+                                                        dsize, &faninout, &segsize, &max_requests);
+
+        if (alg) {
+            /* we have found a valid choice from the file based rules for this message size */
+            return ompi_coll_tuned_scatter_intra_do_this (sbuf, scount, sdtype,
+                                                          rbuf, rcount, rdtype,
+                                                          root, comm, module,
+                                                          alg, faninout, segsize);
+        } /* found a method */
+    } /*end if any com rules to check */
 
     if (data->user_forced[SCATTER].algorithm) {
-	return ompi_coll_tuned_scatter_intra_do_forced (sbuf, scount, sdtype,
-							rbuf, rcount, rdtype,
-							root, comm, module);
+        return ompi_coll_tuned_scatter_intra_do_forced (sbuf, scount, sdtype,
+                                                        rbuf, rcount, rdtype,
+                                                        root, comm, module);
     }
 
     return ompi_coll_tuned_scatter_intra_dec_fixed (sbuf, scount, sdtype, 
-						    rbuf, rcount, rdtype, 
-						    root, comm, module);
+                                                    rbuf, rcount, rdtype, 
+                                                    root, comm, module);
 }
