@@ -1,14 +1,18 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2009 The University of Tennessee and The University
+ * Copyright (c) 2004-2014 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2013      Los Alamos National Security, LLC. All Rights
+ *                         reserved.
+ * Copyright (c) 2015      Intel, Inc. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -29,6 +33,26 @@
 #include "coll_tuned.h"
 #include "coll_tuned_topo.h"
 
+/* reduce algorithm variables */
+static int coll_tuned_reduce_algorithm_count = 6;
+static int coll_tuned_reduce_forced_algorithm = 0;
+static int coll_tuned_reduce_segment_size = 0;
+static int coll_tuned_reduce_max_requests;
+static int coll_tuned_reduce_tree_fanout;
+static int coll_tuned_reduce_chain_fanout;
+
+/* valid values for coll_tuned_reduce_forced_algorithm */
+static mca_base_var_enum_value_t reduce_algorithms[] = {
+    {0, "ignore"},
+    {1, "linear"},
+    {2, "chain"},
+    {3, "pipeline"},
+    {4, "binary"},
+    {5, "binomial"},
+    {6, "in-order_binary"},
+    {0, NULL}
+};
+
 /**
  * This is a generic implementation of the reduce protocol. It used the tree
  * provided as an argument and execute all operations using a segment of
@@ -62,15 +86,17 @@ int ompi_coll_tuned_reduce_generic( void* sendbuf, void* recvbuf, int original_c
      */
     ompi_datatype_get_extent( datatype, &lower_bound, &extent );
     ompi_datatype_type_size( datatype, &typelng );
-    num_segments = (original_count + count_by_segment - 1) / count_by_segment;
-    segment_increment = count_by_segment * extent;
+    num_segments = (int)(((size_t)original_count + (size_t)count_by_segment - (size_t)1) / (size_t)count_by_segment);
+    segment_increment = (ptrdiff_t)count_by_segment * extent;
 
     sendtmpbuf = (char*) sendbuf; 
     if( sendbuf == MPI_IN_PLACE ) { 
         sendtmpbuf = (char *)recvbuf; 
     }
 
-    OPAL_OUTPUT((ompi_coll_tuned_stream, "coll:tuned:reduce_generic count %d, msg size %ld, segsize %ld, max_requests %d", original_count, (unsigned long)(num_segments * segment_increment), (unsigned long)segment_increment, max_outstanding_reqs));
+    OPAL_OUTPUT((ompi_coll_tuned_stream, "coll:tuned:reduce_generic count %d, msg size %ld, segsize %ld, max_requests %d",
+                 original_count, (unsigned long)((ptrdiff_t)num_segments * (ptrdiff_t)segment_increment),
+                 (unsigned long)segment_increment, max_outstanding_reqs));
 
     rank = ompi_comm_rank(comm);
 
@@ -87,7 +113,7 @@ int ompi_coll_tuned_reduce_generic( void* sendbuf, void* recvbuf, int original_c
         if( (NULL == accumbuf) || (root != rank) ) {
             /* Allocate temporary accumulator buffer. */
             accumbuf_free = (char*)malloc(true_extent + 
-                                          (original_count - 1) * extent);
+                                          (ptrdiff_t)(original_count - 1) * extent);
             if (accumbuf_free == NULL) { 
                 line = __LINE__; ret = -1; goto error_hndl; 
             }
@@ -102,7 +128,7 @@ int ompi_coll_tuned_reduce_generic( void* sendbuf, void* recvbuf, int original_c
                                                 (char*)sendtmpbuf);
         }
         /* Allocate two buffers for incoming segments */
-        real_segment_size = true_extent + (count_by_segment - 1) * extent;
+        real_segment_size = true_extent + (ptrdiff_t)(count_by_segment - 1) * extent;
         inbuf_free[0] = (char*) malloc(real_segment_size);
         if( inbuf_free[0] == NULL ) { 
             line = __LINE__; ret = -1; goto error_hndl; 
@@ -127,7 +153,7 @@ int ompi_coll_tuned_reduce_generic( void* sendbuf, void* recvbuf, int original_c
             /* recvcount - number of elements in current segment */
             recvcount = count_by_segment;
             if( segindex == (num_segments-1) )
-                recvcount = original_count - count_by_segment * segindex;
+                recvcount = original_count - (ptrdiff_t)count_by_segment * (ptrdiff_t)segindex;
 
             /* for each child */
             for( i = 0; i < tree->tree_nextsize; i++ ) {
@@ -151,7 +177,7 @@ int ompi_coll_tuned_reduce_generic( void* sendbuf, void* recvbuf, int original_c
                          */
                         if( (ompi_op_is_commute(op)) &&
                             !((MPI_IN_PLACE == sendbuf) && (rank == tree->tree_root)) ) {
-                            local_recvbuf = accumbuf + segindex * segment_increment;
+                            local_recvbuf = accumbuf + (ptrdiff_t)segindex * (ptrdiff_t)segment_increment;
                         }
                     }
 
@@ -178,19 +204,19 @@ int ompi_coll_tuned_reduce_generic( void* sendbuf, void* recvbuf, int original_c
                     if( 1 == i ) {
                         if( (ompi_op_is_commute(op)) && 
                             !((MPI_IN_PLACE == sendbuf) && (rank == tree->tree_root)) ) {
-                            local_op_buffer = sendtmpbuf + segindex * segment_increment;
+                            local_op_buffer = sendtmpbuf + (ptrdiff_t)segindex * (ptrdiff_t)segment_increment;
                         }
                     }
                     /* apply operation */
                     ompi_op_reduce(op, local_op_buffer, 
-                                   accumbuf + segindex * segment_increment, 
+                                   accumbuf + (ptrdiff_t)segindex * (ptrdiff_t)segment_increment, 
                                    recvcount, datatype );
                 } else if ( segindex > 0 ) {
-                    void* accumulator = accumbuf + (segindex-1) * segment_increment;
+                    void* accumulator = accumbuf + (ptrdiff_t)(segindex-1) * (ptrdiff_t)segment_increment;
                     if( tree->tree_nextsize <= 1 ) {
                         if( (ompi_op_is_commute(op)) &&
                             !((MPI_IN_PLACE == sendbuf) && (rank == tree->tree_root)) ) {
-                            local_op_buffer = sendtmpbuf + (segindex-1) * segment_increment;
+                            local_op_buffer = sendtmpbuf + (ptrdiff_t)(segindex-1) * (ptrdiff_t)segment_increment;
                         }
                     }
                     ompi_op_reduce(op, local_op_buffer, accumulator, prevcount, 
@@ -251,7 +277,7 @@ int ompi_coll_tuned_reduce_generic( void* sendbuf, void* recvbuf, int original_c
                     count_by_segment = original_count;
                 }
                 ret = MCA_PML_CALL( send((char*)sendbuf + 
-                                         segindex * segment_increment,
+                                         (ptrdiff_t)segindex * (ptrdiff_t)segment_increment,
                                          count_by_segment, datatype,
                                          tree->tree_prev, 
                                          MCA_COLL_BASE_TAG_REDUCE,
@@ -281,7 +307,7 @@ int ompi_coll_tuned_reduce_generic( void* sendbuf, void* recvbuf, int original_c
             /* post first group of requests */
             for (segindex = 0; segindex < max_outstanding_reqs; segindex++) {
                 ret = MCA_PML_CALL( isend((char*)sendbuf +
-                                          segindex * segment_increment,
+                                          (ptrdiff_t)segindex * (ptrdiff_t)segment_increment,
                                           count_by_segment, datatype,
                                           tree->tree_prev, 
                                           MCA_COLL_BASE_TAG_REDUCE,
@@ -302,7 +328,7 @@ int ompi_coll_tuned_reduce_generic( void* sendbuf, void* recvbuf, int original_c
                     count_by_segment = original_count;
                 }
                 ret = MCA_PML_CALL( isend((char*)sendbuf + 
-                                          segindex * segment_increment, 
+                                          (ptrdiff_t)segindex * (ptrdiff_t)segment_increment, 
                                           count_by_segment, datatype, 
                                           tree->tree_prev, 
                                           MCA_COLL_BASE_TAG_REDUCE, 
@@ -326,9 +352,9 @@ int ompi_coll_tuned_reduce_generic( void* sendbuf, void* recvbuf, int original_c
     return OMPI_SUCCESS;
 
  error_hndl:  /* error handler */
-    OPAL_OUTPUT (( ompi_coll_tuned_stream, 
-                   "ERROR_HNDL: node %d file %s line %d error %d\n", 
-                   rank, __FILE__, line, ret ));
+    opal_output_verbose(COLL_TUNED_VERBOSITY, ompi_coll_tuned_stream,
+                        "ERROR_HNDL: node %d file %s line %d error %d\n", 
+                        rank, __FILE__, line, ret );
     if( inbuf_free[0] != NULL ) free(inbuf_free[0]);
     if( inbuf_free[1] != NULL ) free(inbuf_free[1]);
     if( accumbuf_free != NULL ) free(accumbuf);
@@ -481,9 +507,7 @@ int ompi_coll_tuned_reduce_intra_in_order_binary( void *sendbuf, void *recvbuf,
                                                   uint32_t segsize,
                                                   int max_outstanding_reqs  )
 {
-    int ret;
-    int rank, size, io_root;
-    int segcount = count;
+    int ret, rank, size, io_root, segcount = count;
     void *use_this_sendbuf = NULL, *use_this_recvbuf = NULL;
     size_t typelng;
     mca_coll_tuned_module_t *tuned_module = (mca_coll_tuned_module_t*) module;
@@ -521,7 +545,7 @@ int ompi_coll_tuned_reduce_intra_in_order_binary( void *sendbuf, void *recvbuf,
         ompi_datatype_get_true_extent(datatype, &tlb, &text);
 
         if ((root == rank) && (MPI_IN_PLACE == sendbuf)) {
-            tmpbuf = (char *) malloc(text + (count - 1) * ext);
+            tmpbuf = (char *) malloc(text + (ptrdiff_t)(count - 1) * ext);
             if (NULL == tmpbuf) {
                 return MPI_ERR_INTERN;
             }
@@ -530,7 +554,7 @@ int ompi_coll_tuned_reduce_intra_in_order_binary( void *sendbuf, void *recvbuf,
                                                 (char*)recvbuf);
             use_this_sendbuf = tmpbuf;
         } else if (io_root == rank) {
-            tmpbuf = (char *) malloc(text + (count - 1) * ext);
+            tmpbuf = (char *) malloc(text + (ptrdiff_t)(count - 1) * ext);
             if (NULL == tmpbuf) {
                 return MPI_ERR_INTERN;
             }
@@ -601,10 +625,8 @@ ompi_coll_tuned_reduce_intra_basic_linear(void *sbuf, void *rbuf, int count,
 {
     int i, rank, err, size;
     ptrdiff_t true_lb, true_extent, lb, extent;
-    char *free_buffer = NULL;
-    char *pml_buffer = NULL;
-    char *inplace_temp = NULL;
-    char *inbuf;
+    char *free_buffer = NULL, *pml_buffer = NULL;
+    char *inplace_temp = NULL, *inbuf;
 
     /* Initialize */
 
@@ -631,7 +653,7 @@ ompi_coll_tuned_reduce_intra_basic_linear(void *sbuf, void *rbuf, int count,
 
     if (MPI_IN_PLACE == sbuf) {
         sbuf = rbuf;
-        inplace_temp = (char*)malloc(true_extent + (count - 1) * extent);
+        inplace_temp = (char*)malloc(true_extent + (ptrdiff_t)(count - 1) * extent);
         if (NULL == inplace_temp) {
             return OMPI_ERR_OUT_OF_RESOURCE;
         }
@@ -639,9 +661,10 @@ ompi_coll_tuned_reduce_intra_basic_linear(void *sbuf, void *rbuf, int count,
     }
 
     if (size > 1) {
-        free_buffer = (char*)malloc(true_extent + (count - 1) * extent);
+        free_buffer = (char*)malloc(true_extent + (ptrdiff_t)(count - 1) * extent);
         if (NULL == free_buffer) {
-            return OMPI_ERR_OUT_OF_RESOURCE;
+            err = OMPI_ERR_OUT_OF_RESOURCE;
+            goto exit;
         }
         pml_buffer = free_buffer - lb;
     }
@@ -657,10 +680,7 @@ ompi_coll_tuned_reduce_intra_basic_linear(void *sbuf, void *rbuf, int count,
                                 MPI_STATUS_IGNORE));
     }
     if (MPI_SUCCESS != err) {
-        if (NULL != free_buffer) {
-            free(free_buffer);
-        }
-        return err;
+        goto exit;
     }
 
     /* Loop receiving and calling reduction function (C or Fortran). */
@@ -673,10 +693,7 @@ ompi_coll_tuned_reduce_intra_basic_linear(void *sbuf, void *rbuf, int count,
                                     MCA_COLL_BASE_TAG_REDUCE, comm,
                                     MPI_STATUS_IGNORE));
             if (MPI_SUCCESS != err) {
-                if (NULL != free_buffer) {
-                    free(free_buffer);
-                }
-                return err;
+                goto exit;
             }
 
             inbuf = pml_buffer;
@@ -689,6 +706,11 @@ ompi_coll_tuned_reduce_intra_basic_linear(void *sbuf, void *rbuf, int count,
     if (NULL != inplace_temp) {
         err = ompi_datatype_copy_content_same_ddt(dtype, count, (char*)sbuf, 
                                                   inplace_temp);
+    }
+    err = MPI_SUCCESS;
+
+  exit:
+    if (NULL != inplace_temp) {
         free(inplace_temp);
     }
     if (NULL != free_buffer) {
@@ -696,7 +718,7 @@ ompi_coll_tuned_reduce_intra_basic_linear(void *sbuf, void *rbuf, int count,
     }
 
     /* All done */
-    return MPI_SUCCESS;
+    return err;
 }
 
 /* copied function (with appropriate renaming) ends here */
@@ -713,72 +735,86 @@ ompi_coll_tuned_reduce_intra_basic_linear(void *sbuf, void *rbuf, int count,
  * perms module does not call this they call the forced_getvalues routine 
  * instead.
  */
+
 int ompi_coll_tuned_reduce_intra_check_forced_init (coll_tuned_force_algorithm_mca_param_indices_t *mca_param_indices)
 {
-    int rc, requested_alg, max_alg = 6, max_requests;
+    mca_base_var_enum_t*new_enum;
 
-    ompi_coll_tuned_forced_max_algorithms[REDUCE] = max_alg;
+    ompi_coll_tuned_forced_max_algorithms[REDUCE] = coll_tuned_reduce_algorithm_count;
 
-    rc = mca_base_param_reg_int (&mca_coll_tuned_component.super.collm_version,
-                                 "reduce_algorithm_count",
-                                 "Number of reduce algorithms available",
-                                 false, true, max_alg, NULL);
+    (void) mca_base_component_var_register(&mca_coll_tuned_component.super.collm_version,
+                                           "reduce_algorithm_count",
+                                           "Number of reduce algorithms available",
+                                           MCA_BASE_VAR_TYPE_INT, NULL, 0,
+                                           MCA_BASE_VAR_FLAG_DEFAULT_ONLY,
+                                           OPAL_INFO_LVL_5,
+                                           MCA_BASE_VAR_SCOPE_CONSTANT,
+                                           &coll_tuned_reduce_algorithm_count);
 
-    mca_param_indices->algorithm_param_index
-        = mca_base_param_reg_int(&mca_coll_tuned_component.super.collm_version,
-                                 "reduce_algorithm",
-                                 "Which reduce algorithm is used. Can be locked down to choice of: 0 ignore, 1 linear, 2 chain, 3 pipeline, 4 binary, 5 binomial, 6 in-order binary",
-                                 false, false, 0, NULL);
+    /* MPI_T: This variable should eventually be bound to a communicator */
+    coll_tuned_reduce_forced_algorithm = 0;
+    (void) mca_base_var_enum_create("coll_tuned_reduce_algorithms", reduce_algorithms, &new_enum);
+    mca_param_indices->algorithm_param_index =
+        mca_base_component_var_register(&mca_coll_tuned_component.super.collm_version,
+                                        "reduce_algorithm",
+                                        "Which reduce algorithm is used. Can be locked down to choice of: 0 ignore, 1 linear, 2 chain, 3 pipeline, 4 binary, 5 binomial, 6 in-order binary",
+                                        MCA_BASE_VAR_TYPE_INT, new_enum, 0, 0,
+                                        OPAL_INFO_LVL_5,
+                                        MCA_BASE_VAR_SCOPE_READONLY,
+                                        &coll_tuned_reduce_forced_algorithm);
+    OBJ_RELEASE(new_enum);
     if (mca_param_indices->algorithm_param_index < 0) {
         return mca_param_indices->algorithm_param_index;
     }
-    mca_base_param_lookup_int(mca_param_indices->algorithm_param_index, &(requested_alg));
-    if( 0 > requested_alg || requested_alg > max_alg ) {
-        if( 0 == ompi_comm_rank( MPI_COMM_WORLD ) ) {
-            opal_output( 0, "Reduce algorithm #%d is not available (range [0..%d]). Switching back to ignore(0)\n",
-                         requested_alg, max_alg );
-        }
-        mca_base_param_set_int( mca_param_indices->algorithm_param_index, 0);
-    }
 
-    mca_param_indices->segsize_param_index
-        = mca_base_param_reg_int(&mca_coll_tuned_component.super.collm_version,
-                                 "reduce_algorithm_segmentsize",
-                                 "Segment size in bytes used by default for reduce algorithms. Only has meaning if algorithm is forced and supports segmenting. 0 bytes means no segmentation.",
-                                 false, false, 0, NULL);
+    coll_tuned_reduce_segment_size = 0;
+    mca_param_indices->segsize_param_index =
+        mca_base_component_var_register(&mca_coll_tuned_component.super.collm_version,
+                                        "reduce_algorithm_segmentsize",
+                                        "Segment size in bytes used by default for reduce algorithms. Only has meaning if algorithm is forced and supports segmenting. 0 bytes means no segmentation.",
+                                        MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
+                                        OPAL_INFO_LVL_5,
+                                        MCA_BASE_VAR_SCOPE_READONLY,
+                                        &coll_tuned_reduce_segment_size);
 
-    mca_param_indices->tree_fanout_param_index
-        = mca_base_param_reg_int(&mca_coll_tuned_component.super.collm_version,
-                                 "reduce_algorithm_tree_fanout",
-                                 "Fanout for n-tree used for reduce algorithms. Only has meaning if algorithm is forced and supports n-tree topo based operation.",
-                                 false, false,
-                                 ompi_coll_tuned_init_tree_fanout, /* get system wide default */
-                                 NULL);
+    coll_tuned_reduce_tree_fanout = ompi_coll_tuned_init_tree_fanout; /* get system wide default */
+    mca_param_indices->tree_fanout_param_index =
+        mca_base_component_var_register(&mca_coll_tuned_component.super.collm_version,
+                                        "reduce_algorithm_tree_fanout",
+                                        "Fanout for n-tree used for reduce algorithms. Only has meaning if algorithm is forced and supports n-tree topo based operation.",
+                                        MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
+                                        OPAL_INFO_LVL_5,
+                                        MCA_BASE_VAR_SCOPE_READONLY,
+                                        &coll_tuned_reduce_tree_fanout);
 
-    mca_param_indices->chain_fanout_param_index
-        = mca_base_param_reg_int(&mca_coll_tuned_component.super.collm_version,
-                                 "reduce_algorithm_chain_fanout",
-                                 "Fanout for chains used for reduce algorithms. Only has meaning if algorithm is forced and supports chain topo based operation.",
-                                 false, false,
-                                 ompi_coll_tuned_init_chain_fanout, /* get system wide default */
-                                 NULL);
-
-    mca_param_indices->max_requests_param_index
-        = mca_base_param_reg_int(&mca_coll_tuned_component.super.collm_version,
-                                 "reduce_algorithm_max_requests",
-                                 "Maximum number of outstanding send requests on leaf nodes. 0 means no limit.",
-                                 false, false, 0, /* no limit for reduce by default */
-                                 NULL);
+    coll_tuned_reduce_chain_fanout = ompi_coll_tuned_init_chain_fanout; /* get system wide default */
+    mca_param_indices->chain_fanout_param_index = 
+      mca_base_component_var_register(&mca_coll_tuned_component.super.collm_version,
+                                      "reduce_algorithm_chain_fanout",
+                                      "Fanout for chains used for reduce algorithms. Only has meaning if algorithm is forced and supports chain topo based operation.",
+                                      MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
+                                      OPAL_INFO_LVL_5,
+                                      MCA_BASE_VAR_SCOPE_READONLY,
+                                      &coll_tuned_reduce_chain_fanout);
 
+    coll_tuned_reduce_max_requests = 0; /* no limit for reduce by default */
+    mca_param_indices->max_requests_param_index = 
+      mca_base_component_var_register(&mca_coll_tuned_component.super.collm_version,
+                                      "reduce_algorithm_max_requests",
+                                      "Maximum number of outstanding send requests on leaf nodes. 0 means no limit.",
+                                      MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
+                                      OPAL_INFO_LVL_5,
+                                      MCA_BASE_VAR_SCOPE_READONLY,
+                                      &coll_tuned_reduce_max_requests);
     if (mca_param_indices->max_requests_param_index < 0) {
         return mca_param_indices->max_requests_param_index;
     }
-    mca_base_param_lookup_int(mca_param_indices->max_requests_param_index, &(max_requests));
-    if( max_requests < 0 ) {
+
+    if (coll_tuned_reduce_max_requests < 0) {
         if( 0 == ompi_comm_rank( MPI_COMM_WORLD ) ) {
             opal_output( 0, "Maximum outstanding requests must be positive number or 0.  Initializing to 0 (no limit).\n" );
         }
-        mca_base_param_set_int( mca_param_indices->max_requests_param_index, 0);
+        coll_tuned_reduce_max_requests = 0;
     }
 
     return (MPI_SUCCESS);
