@@ -19,12 +19,10 @@
 #include "ompi_config.h"
 
 #include "mpi.h"
+#include "opal/util/bit_ops.h"
 #include "ompi/constants.h"
-#include "ompi/datatype/datatype.h"
 #include "ompi/communicator/communicator.h"
-#include "ompi/mca/coll/coll.h"
 #include "ompi/mca/coll/base/coll_tags.h"
-#include "ompi/mca/pml/pml.h"
 #include "coll_tuned.h"
 #include "coll_tuned_topo.h"
 
@@ -64,20 +62,25 @@ static int calculate_num_nodes_up_to_level( int fanout, int level )
 
 /*
  * And now the building functions.
+ *
+ * An example for fanout = 2, comm_size = 7
+ *
+ *              0           <-- delta = 1 (fanout^0)
+ *            /   \
+ *           1     2        <-- delta = 2 (fanout^1)
+ *          / \   / \
+ *         3   5 4   6      <-- delta = 4 (fanout^2)
  */
 
 ompi_coll_tree_t*
 ompi_coll_tuned_topo_build_tree( int fanout,
-                             struct ompi_communicator_t* comm,
-                             int root )
+                                 struct ompi_communicator_t* comm,
+                                 int root )
 {
-    int rank, size;
-    int schild, sparent;
+    int rank, size, schild, sparent, shiftedrank, i;
     int level; /* location of my rank in the tree structure of size */
     int delta; /* number of nodes on my level */
     int slimit; /* total number of nodes on levels above me */ 
-    int shiftedrank;
-    int i;
     ompi_coll_tree_t* tree;
 
     OPAL_OUTPUT((ompi_coll_tuned_stream, "coll:tuned:topo_build_tree Building fo %d rt %d", fanout, root));
@@ -169,6 +172,126 @@ ompi_coll_tuned_topo_build_tree( int fanout,
     return tree;
 }
 
+/*
+ * Constructs in-order binary tree which can be used for non-commutative reduce 
+ * operations.
+ * Root of this tree is always rank (size-1) and fanout is 2.
+ * Here are some of the examples of this tree:
+ * size == 2     size == 3     size == 4                size == 9
+ *      1             2             3                        8
+ *     /             / \          /   \                    /   \
+ *    0             1  0         2     1                  7     3
+ *                                    /                 /  \   / \
+ *                                   0                 6    5 2   1
+ *                                                         /     /
+ *                                                        4     0
+ */
+ompi_coll_tree_t*
+ompi_coll_tuned_topo_build_in_order_bintree( struct ompi_communicator_t* comm )
+{
+    int rank, size, myrank, rightsize, delta, parent, lchild, rchild;
+    ompi_coll_tree_t* tree;
+
+    /* 
+     * Get size and rank of the process in this communicator 
+     */
+    size = ompi_comm_size(comm);
+    rank = ompi_comm_rank(comm);
+
+    tree = (ompi_coll_tree_t*)malloc(sizeof(ompi_coll_tree_t));
+    if (!tree) {
+        OPAL_OUTPUT((ompi_coll_tuned_stream,
+                     "coll:tuned:topo_build_tree PANIC::out of memory"));
+        return NULL;
+    }
+
+    tree->tree_root     = MPI_UNDEFINED;
+    tree->tree_nextsize = MPI_UNDEFINED;
+
+    /* 
+     * Initialize tree
+     */
+    tree->tree_fanout   = 2;
+    tree->tree_bmtree   = 0;
+    tree->tree_root     = size - 1;
+    tree->tree_prev     = -1;
+    tree->tree_nextsize = 0;
+    tree->tree_next[0]  = -1;
+    tree->tree_next[1]  = -1;
+    OPAL_OUTPUT((ompi_coll_tuned_stream, 
+                 "coll:tuned:topo_build_in_order_tree Building fo %d rt %d", 
+                 tree->tree_fanout, tree->tree_root));
+
+    /* 
+     * Build the tree
+     */
+    myrank = rank;
+    parent = size - 1;
+    delta = 0;
+
+    while ( 1 ) {
+        /* Compute the size of the right subtree */
+        rightsize = size >> 1;
+
+        /* Determine the left and right child of this parent  */
+        lchild = -1;
+        rchild = -1;
+        if (size - 1 > 0) {
+            lchild = parent - 1;
+            if (lchild > 0) { 
+                rchild = rightsize - 1;
+            }
+        }
+       
+        /* The following cases are possible: myrank can be 
+           - a parent,
+           - belong to the left subtree, or
+           - belong to the right subtee
+           Each of the cases need to be handled differently.
+        */
+          
+        if (myrank == parent) {
+            /* I am the parent:
+               - compute real ranks of my children, and exit the loop. */
+            if (lchild >= 0) tree->tree_next[0] = lchild + delta;
+            if (rchild >= 0) tree->tree_next[1] = rchild + delta;
+            break;
+        }
+        if (myrank > rchild) {
+            /* I belong to the left subtree:
+               - If I am the left child, compute real rank of my parent
+               - Iterate down through tree: 
+               compute new size, shift ranks down, and update delta.
+            */
+            if (myrank == lchild) {
+                tree->tree_prev = parent + delta;
+            }
+            size = size - rightsize - 1;
+            delta = delta + rightsize;
+            myrank = myrank - rightsize;
+            parent = size - 1;
+
+        } else {
+            /* I belong to the right subtree:
+               - If I am the right child, compute real rank of my parent
+               - Iterate down through tree:  
+               compute new size and parent, 
+               but the delta and rank do not need to change.
+            */
+            if (myrank == rchild) {
+                tree->tree_prev = parent + delta;
+            }
+            size = rightsize;
+            parent = rchild;
+        }
+    }
+    
+    if (tree->tree_next[0] >= 0) { tree->tree_nextsize = 1; }
+    if (tree->tree_next[1] >= 0) { tree->tree_nextsize += 1; }
+
+    return tree;
+}
+
 int ompi_coll_tuned_topo_destroy_tree( ompi_coll_tree_t** tree )
 {
     ompi_coll_tree_t *ptr;
@@ -185,18 +308,24 @@ int ompi_coll_tuned_topo_destroy_tree( ompi_coll_tree_t** tree )
     return OMPI_SUCCESS;
 }
 
+/*
+ * 
+ * Here are some of the examples of this tree:
+ * size == 2                   size = 4                 size = 8
+ *      0                           0                        0
+ *     /                            | \                    / | \
+ *    1                             2  1                  4  2  1
+ *                                     |                     |  |\
+ *                                     3                     6  5 3
+ *                                                                |
+ *                                                                7
+ */
 ompi_coll_tree_t*
 ompi_coll_tuned_topo_build_bmtree( struct ompi_communicator_t* comm,
-                        int root )
+                                   int root )
 {
-    int childs = 0;
-    int rank;
-    int size;
-    int mask = 1;
-    int index;
-    int remote;
+    int childs = 0, rank, size, mask = 1, index, remote, i;
     ompi_coll_tree_t *bmtree;
-    int i;
 
     OPAL_OUTPUT((ompi_coll_tuned_stream,"coll:tuned:topo:build_bmtree rt %d", root));
 
@@ -218,13 +347,13 @@ ompi_coll_tuned_topo_build_bmtree( struct ompi_communicator_t* comm,
 
     bmtree->tree_root     = MPI_UNDEFINED;
     bmtree->tree_nextsize = MPI_UNDEFINED;
-    for(i=0;i<MAXTREEFANOUT;i++) {
+    for( i = 0;i < MAXTREEFANOUT; i++ ) {
         bmtree->tree_next[i] = -1;
     }
 
     if( index < 0 ) index += size;
 
-    while( mask <= index ) mask <<= 1;
+    mask = opal_next_poweroftwo(index);
 
     /* Now I can compute my father rank */
     if( root == rank ) {
@@ -253,17 +382,85 @@ ompi_coll_tuned_topo_build_bmtree( struct ompi_communicator_t* comm,
     return bmtree;
 }
 
-
-ompi_coll_chain_t*
-ompi_coll_tuned_topo_build_chain( int fanout,
-                       struct ompi_communicator_t* comm,
-                       int root )
+/*
+ * Constructs in-order binomial tree which can be used for gather/scatter
+ * operations.
+ * 
+ * Here are some of the examples of this tree:
+ * size == 2                   size = 4                 size = 8
+ *      0                           0                        0
+ *     /                          / |                      / | \
+ *    1                          1  2                     1  2  4
+ *                                  |                        |  | \
+ *                                  3                        3  5  6
+ *                                                                 |
+ *                                                                 7
+ */
+ompi_coll_tree_t*
+ompi_coll_tuned_topo_build_in_order_bmtree( struct ompi_communicator_t* comm,
+                                            int root )
 {
-    int rank, size;
-    int srank; /* shifted rank */
-    int i,maxchainlen;
-    int mark,head,len;
-    ompi_coll_chain_t *chain;
+    int childs = 0, rank, vrank, size, mask = 1, remote, i;
+    ompi_coll_tree_t *bmtree;
+
+    OPAL_OUTPUT((ompi_coll_tuned_stream,"coll:tuned:topo:build_in_order_bmtree rt %d", root));
+
+    /* 
+     * Get size and rank of the process in this communicator 
+     */
+    size = ompi_comm_size(comm);
+    rank = ompi_comm_rank(comm);
+
+    vrank = (rank - root + size) % size;
+
+    bmtree = (ompi_coll_tree_t*)malloc(sizeof(ompi_coll_tree_t));
+    if (!bmtree) {
+        OPAL_OUTPUT((ompi_coll_tuned_stream,"coll:tuned:topo:build_bmtree PANIC out of memory"));
+        return NULL;
+    }
+
+    bmtree->tree_bmtree   = 1;
+    bmtree->tree_root     = MPI_UNDEFINED;
+    bmtree->tree_nextsize = MPI_UNDEFINED;
+    for(i=0;i<MAXTREEFANOUT;i++) {
+        bmtree->tree_next[i] = -1;
+    }
+
+    if (root == rank) {
+        bmtree->tree_prev = root;
+    }
+
+    while (mask < size) {
+        remote = vrank ^ mask;
+        if (remote < vrank) {
+            bmtree->tree_prev = (remote + root) % size;
+            break;
+        } else if (remote < size) {
+            bmtree->tree_next[childs] = (remote + root) % size;
+            childs++;
+            if (childs==MAXTREEFANOUT) {
+                OPAL_OUTPUT((ompi_coll_tuned_stream,
+                             "coll:tuned:topo:build_bmtree max fanout incorrect %d needed %d",
+                             MAXTREEFANOUT, childs));
+                return NULL;
+            }
+        }
+        mask <<= 1;
+    }
+    bmtree->tree_nextsize = childs;
+    bmtree->tree_root     = root;
+
+    return bmtree;
+}
+
+
+ompi_coll_tree_t*
+ompi_coll_tuned_topo_build_chain( int fanout,
+                                  struct ompi_communicator_t* comm,
+                                  int root )
+{
+    int i, maxchainlen, mark, head, len, rank, size, srank /* shifted rank */;
+    ompi_coll_tree_t *chain;
 
     OPAL_OUTPUT((ompi_coll_tuned_stream,"coll:tuned:topo:build_chain fo %d rt %d", fanout, root));
 
@@ -285,28 +482,25 @@ ompi_coll_tuned_topo_build_chain( int fanout,
     /*
      * Allocate space for topology arrays if needed 
      */
-    chain = (ompi_coll_chain_t*)malloc( sizeof(ompi_coll_chain_t) );
+    chain = (ompi_coll_tree_t*)malloc( sizeof(ompi_coll_tree_t) );
     if (!chain) {
         OPAL_OUTPUT((ompi_coll_tuned_stream,"coll:tuned:topo:build_chain PANIC out of memory"));
         fflush(stdout);
         return NULL;
     }
-    chain->chain_root     = MPI_UNDEFINED;
-    chain->chain_nextsize = -1;
-    chain->chain_numchain = -1;
-    for(i=0;i<fanout;i++) chain->chain_next[i] = -1;
+    chain->tree_root     = MPI_UNDEFINED;
+    chain->tree_nextsize = -1;
+    for(i=0;i<fanout;i++) chain->tree_next[i] = -1;
 
     /* 
      * Set root & numchain
      */
-    chain->chain_root = root;
+    chain->tree_root = root;
     if( (size - 1) < fanout ) { 
-        chain->chain_numchain = size-1;
-        chain->chain_nextsize = size-1;
+        chain->tree_nextsize = size-1;
         fanout = size-1;
     } else {
-        chain->chain_numchain = fanout;
-        chain->chain_nextsize = fanout;
+        chain->tree_nextsize = fanout;
     }
     
     /*
@@ -319,25 +513,24 @@ ompi_coll_tuned_topo_build_chain( int fanout,
      * Special case - fanout == 1
      */
     if( fanout == 1 ) {
-        if( srank == 0 ) chain->chain_prev = -1;
-        else chain->chain_prev = (srank-1+root)%size;
+        if( srank == 0 ) chain->tree_prev = -1;
+        else chain->tree_prev = (srank-1+root)%size;
 
         if( (srank + 1) >= size) {
-            chain->chain_next[0] = -1;
-            chain->chain_nextsize = 0;
+            chain->tree_next[0] = -1;
+            chain->tree_nextsize = 0;
         } else {
-            chain->chain_next[0] = (srank+1+root)%size;
-            chain->chain_nextsize = 1;
+            chain->tree_next[0] = (srank+1+root)%size;
+            chain->tree_nextsize = 1;
         }
         return chain;
     }
 
     /* Let's handle the case where there is just one node in the communicator */
     if( size == 1 ) {
-        chain->chain_next[0] = -1;
-        chain->chain_nextsize = 0;
-        chain->chain_prev = -1;
-        chain->chain_numchain = 0;
+        chain->tree_next[0] = -1;
+        chain->tree_nextsize = 0;
+        chain->tree_prev = -1;
         return chain;
     }
     /*
@@ -367,20 +560,20 @@ ompi_coll_tuned_topo_build_chain( int fanout,
         }
 
         if( srank == head ) {
-            chain->chain_prev = 0; /*root*/
+            chain->tree_prev = 0; /*root*/
         } else {
-            chain->chain_prev = srank-1; /* rank -1 */
+            chain->tree_prev = srank-1; /* rank -1 */
         }
         if( srank == (head + len - 1) ) {
-            chain->chain_next[0] = -1;
-            chain->chain_nextsize = 0;
+            chain->tree_next[0] = -1;
+            chain->tree_nextsize = 0;
         } else {
             if( (srank + 1) < size ) {
-                chain->chain_next[0] = srank+1;
-                chain->chain_nextsize = 1;
+                chain->tree_next[0] = srank+1;
+                chain->tree_nextsize = 1;
             } else {
-                chain->chain_next[0] = -1;
-                chain->chain_nextsize = 0;    
+                chain->tree_next[0] = -1;
+                chain->tree_nextsize = 0;    
             }
         }
     }
@@ -389,62 +582,37 @@ ompi_coll_tuned_topo_build_chain( int fanout,
      * Unshift values 
      */
     if( rank == root ) {
-        chain->chain_prev = -1;
-        chain->chain_next[0] = (root+1)%size;
+        chain->tree_prev = -1;
+        chain->tree_next[0] = (root+1)%size;
         for( i = 1; i < fanout; i++ ) {
-            chain->chain_next[i] = chain->chain_next[i-1] + maxchainlen;
+            chain->tree_next[i] = chain->tree_next[i-1] + maxchainlen;
             if( i > mark ) {
-                chain->chain_next[i]--;
+                chain->tree_next[i]--;
             }
-            chain->chain_next[i] %= size;
+            chain->tree_next[i] %= size;
         }
-        chain->chain_nextsize = fanout;
+        chain->tree_nextsize = fanout;
     } else {
-        chain->chain_prev = (chain->chain_prev+root)%size;
-        if( chain->chain_next[0] != -1 ) {
-            chain->chain_next[0] = (chain->chain_next[0]+root)%size;
+        chain->tree_prev = (chain->tree_prev+root)%size;
+        if( chain->tree_next[0] != -1 ) {
+            chain->tree_next[0] = (chain->tree_next[0]+root)%size;
         }
     }
 
     return chain;
 }
 
-int ompi_coll_tuned_topo_destroy_chain( ompi_coll_chain_t** chain )
-{
-    ompi_coll_chain_t *ptr;
-
-    if ((!chain)||(!*chain)) {
-        return OMPI_SUCCESS;
-    }
-
-    ptr = *chain;
-
-    free (ptr);
-    *chain = NULL;   /* mark chain as gone */
-
-    return OMPI_SUCCESS;
-}
-
-
 int ompi_coll_tuned_topo_dump_tree (ompi_coll_tree_t* tree, int rank)
 {
-int i;
-OPAL_OUTPUT((ompi_coll_tuned_stream,"coll:tuned:topo:topo_dump_tree %1d tree root %d fanout %d BM %1d nextsize %d prev %d", rank, 
-        tree->tree_root, tree->tree_bmtree, tree->tree_fanout, tree->tree_nextsize, tree->tree_prev));
-if (tree->tree_nextsize) {
-    for (i=0;i<tree->tree_nextsize;i++) OPAL_OUTPUT((ompi_coll_tuned_stream,"[%1d] %d", i, tree->tree_next[i]));
-}
-return (0);
-}
+    int i;
 
-int ompi_coll_tuned_topo_dump_chain (ompi_coll_chain_t* chain, int rank)
-{
-int i;
-OPAL_OUTPUT((ompi_coll_tuned_stream,"coll:tuned:topo:topo_dump_chain %1d chain root %d fanout %d nextsize %d prev %d\n", rank, 
-        chain->chain_root, chain->chain_numchain, chain->chain_nextsize, chain->chain_prev));
-if (chain->chain_nextsize) {
-    for (i=0;i<chain->chain_nextsize;i++) OPAL_OUTPUT((ompi_coll_tuned_stream,"[%1d] %d ", i, chain->chain_next[i]));
+    OPAL_OUTPUT((ompi_coll_tuned_stream, "coll:tuned:topo:topo_dump_tree %1d tree root %d"
+                 " fanout %d BM %1d nextsize %d prev %d",
+                 rank, tree->tree_root, tree->tree_bmtree, tree->tree_fanout,
+                 tree->tree_nextsize, tree->tree_prev));
+    if( tree->tree_nextsize ) {
+        for( i = 0; i < tree->tree_nextsize; i++ )
+            OPAL_OUTPUT((ompi_coll_tuned_stream,"[%1d] %d", i, tree->tree_next[i]));
+    }
+    return (0);
 }
-return (0);
-}
-
