@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2008 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2010 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
@@ -20,6 +20,7 @@
 
 #include "ompi_config.h"
 
+#include "opal/mca/base/base.h"
 #include "ompi/constants.h"
 
 #include "ompi/mca/crcp/crcp.h"
@@ -30,16 +31,6 @@
 
 BEGIN_C_DECLS
 
-    /**
-     * Initialize the CRCP MCA framework
-     *
-     * @retval OMPI_SUCCESS Upon success
-     * @retval OMPI_ERROR   Upon failures
-     * 
-     * This function is invoked during ompi_init();
-     */
-    OMPI_DECLSPEC int ompi_crcp_base_open(void);
-    
     /**
      * Select an available component.
      *
@@ -51,14 +42,10 @@ BEGIN_C_DECLS
     OMPI_DECLSPEC int ompi_crcp_base_select(void);
     
     /**
-     * Finalize the CRCP MCA framework
-     *
-     * @retval OMPI_SUCCESS Upon success
-     * @retval OMPI_ERROR   Upon failures
-     * 
-     * This function is invoked during ompi_finalize();
+     * Quiesce Interface (For MPI Ext.)
      */
-    OMPI_DECLSPEC int ompi_crcp_base_close(void);
+    OMPI_DECLSPEC int ompi_crcp_base_quiesce_start(MPI_Info *info);
+    OMPI_DECLSPEC int ompi_crcp_base_quiesce_end(MPI_Info *info);
 
     /**
      * 'None' component functions
@@ -72,6 +59,10 @@ BEGIN_C_DECLS
     int ompi_crcp_base_module_init(void);
     int ompi_crcp_base_module_finalize(void);
 
+    /* Quiesce Interface */
+    int ompi_crcp_base_none_quiesce_start(MPI_Info *info);
+    int ompi_crcp_base_none_quiesce_end(MPI_Info *info);
+
     /* PML Interface */
     ompi_crcp_base_pml_state_t* ompi_crcp_base_none_pml_enable( bool enable, ompi_crcp_base_pml_state_t* );
 
@@ -198,10 +189,7 @@ BEGIN_C_DECLS
     ompi_crcp_base_none_btl_ft_event(int state,
                                      ompi_crcp_base_btl_state_t*);
 
-    /* Utility Functions */
-
-    OMPI_DECLSPEC extern int  ompi_crcp_base_output;
-    OMPI_DECLSPEC extern opal_list_t ompi_crcp_base_components_available;
+    OMPI_DECLSPEC extern mca_base_framework_t ompi_crcp_base_framework;
     OMPI_DECLSPEC extern ompi_crcp_base_component_t ompi_crcp_base_selected_component;
     OMPI_DECLSPEC extern ompi_crcp_base_module_t ompi_crcp;
 
