@@ -1,14 +1,18 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2008 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2011 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2013      Intel, Inc. All rights reserved
+ * Copyright (c) 2013      Los Alamos National Security, LLC.  All rights
+ *                         reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -30,35 +34,11 @@
 #include "opal/mca/mca.h"
 #include "opal/mca/base/base.h"
 
-
 #include "ompi/info/info.h"
 #include "ompi/communicator/communicator.h"
 
 BEGIN_C_DECLS
 
-/* OMPI port definitions */
-/* carry over the INVALID def */
-#define OMPI_RML_TAG_INVALID                        ORTE_RML_TAG_INVALID
-/* define a starting point to avoid conflicts */
-#define OMPI_RML_TAG_BASE                           ORTE_RML_TAG_MAX
-
-#define OMPI_RML_TAG_UDAPL                          OMPI_RML_TAG_BASE+1
-#define OMPI_RML_TAG_OPENIB                         OMPI_RML_TAG_BASE+2
-#define OMPI_RML_TAG_XOPENIB                        OMPI_RML_TAG_BASE+3
-#define OMPI_RML_TAG_COMM_CID_INTRA                 OMPI_RML_TAG_BASE+4
-#define OMPI_RML_TAG_XOOB                           OMPI_RML_TAG_BASE+5
-#define OMPI_RML_TAG_SM_BACK_FILE_CREATED           OMPI_RML_TAG_BASE+6
-#define OMPI_CRCP_COORD_BOOKMARK_TAG                OMPI_RML_TAG_BASE+7
-#define OMPI_COMM_JOIN_TAG                          OMPI_RML_TAG_BASE+8
-
-/* support for shared memory collectives */
-#define OMPI_RML_TAG_COLL_SM2_BACK_FILE_CREATED     OMPI_RML_TAG_BASE+9
-
-#define OMPI_RML_TAG_DYNAMIC                        OMPI_RML_TAG_BASE+200
-
-
-
-
 /*
  * Initialize a module
  */
@@ -68,23 +48,23 @@ typedef int (*ompi_dpm_base_module_init_fn_t)(void);
  * Connect/accept communications
  */
 typedef int (*ompi_dpm_base_module_connect_accept_fn_t)(ompi_communicator_t *comm, int root,
-                                                        char *port, bool send_first,
+                                                        const char *port, bool send_first,
                                                         ompi_communicator_t **newcomm);
 
 /**
  * Executes internally a disconnect on all dynamic communicators
  * in case the user did not disconnect them.
  */
-typedef void (*ompi_dpm_base_module_disconnect_fn_t)(ompi_communicator_t *comm);
+typedef int (*ompi_dpm_base_module_disconnect_fn_t)(ompi_communicator_t *comm);
 
 /*
  * Dynamically spawn processes
  */
-typedef int (*ompi_dpm_base_module_spawn_fn_t)(int count, char **array_of_commands,
-                                               char ***array_of_argv,
-                                               int *array_of_maxprocs,
-                                               MPI_Info *array_of_info,
-                                               char *port_name);
+typedef int (*ompi_dpm_base_module_spawn_fn_t)(int count, char const *array_of_commands[],
+                                               char **array_of_argv[],
+                                               const int array_of_maxprocs[],
+                                               const MPI_Info array_of_info[],
+                                               const char *port_name);
 
 /*
  * This routine checks, whether an application has been spawned
@@ -114,26 +94,26 @@ typedef void (*ompi_dpm_base_module_mark_dyncomm_fn_t)(ompi_communicator_t *comm
  * specified tag is valid, then it will be used to form the port. Otherwise,
  * a dynamically assigned tag that is unique to this request will be provided
  */
-typedef int (*ompi_dpm_base_module_open_port_fn_t)(char *port_name, orte_rml_tag_t tag);
+typedef int (*ompi_dpm_base_module_open_port_fn_t)(char *port_name, ompi_rml_tag_t tag);
 
 /*
  * Converts an opaque port string to a RML process nane and tag.
  */
-typedef int (*ompi_dpm_base_module_parse_port_name_t)(char *port_name,
+typedef int (*ompi_dpm_base_module_parse_port_name_t)(const char *port_name,
                                                       char **hnp_uri, char **rml_uri,
-                                                      orte_rml_tag_t *tag);
+                                                      ompi_rml_tag_t *tag);
 
 /* 
  * Update the routed component to make sure that the RML can send messages to
  * the remote port
  */
-typedef int (*ompi_dpm_base_module_route_to_port_t)(char *rml_uri, orte_process_name_t *rproc);
+typedef int (*ompi_dpm_base_module_route_to_port_t)(char *rml_uri, ompi_process_name_t *rproc);
 
 
 /*
  * Close a port
  */
-typedef int (*ompi_dpm_base_module_close_port_fn_t)(char *port_name);
+typedef int (*ompi_dpm_base_module_close_port_fn_t)(const char *port_name);
 
 /*
  * Finalize a module
