@@ -1,4 +1,4 @@
-/* -*- Mode: C; c-basic-offset:4 ; -*- */
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /* 
  *
  *   Copyright (C) 1997 University of Chicago. 
@@ -8,7 +8,7 @@
 #include "ad_nfs.h"
 #include "adio_extern.h"
 
-void ADIOI_NFS_WriteContig(ADIO_File fd, void *buf, int count, 
+void ADIOI_NFS_WriteContig(ADIO_File fd, const void *buf, int count,
                      MPI_Datatype datatype, int file_ptr_type,
 		     ADIO_Offset offset, ADIO_Status *status, int *error_code)
 {
@@ -263,7 +263,7 @@ void ADIOI_NFS_WriteContig(ADIO_File fd, void *buf, int count,
 #endif
 
 
-void ADIOI_NFS_WriteStrided(ADIO_File fd, void *buf, int count,
+void ADIOI_NFS_WriteStrided(ADIO_File fd, const void *buf, int count,
                        MPI_Datatype datatype, int file_ptr_type,
                        ADIO_Offset offset, ADIO_Status *status, int
                        *error_code)
@@ -281,7 +281,7 @@ void ADIOI_NFS_WriteStrided(ADIO_File fd, void *buf, int count,
     ADIO_Offset userbuf_off;
     ADIO_Offset off, req_off, disp, end_offset=0, writebuf_off, start_off;
     char *writebuf, *value;
-    int flag, st_fwr_size, st_n_filetypes, writebuf_len, write_sz;
+    int st_fwr_size, st_n_filetypes, writebuf_len, write_sz;
     int new_bwr_size, new_fwr_size, err_flag=0, info_flag, max_bufsize;
     static char myname[] = "ADIOI_NFS_WRITESTRIDED";
 
@@ -290,6 +290,9 @@ void ADIOI_NFS_WriteStrided(ADIO_File fd, void *buf, int count,
 
     MPI_Type_size(fd->filetype, &filetype_size);
     if ( ! filetype_size ) {
+#ifdef HAVE_STATUS_SET_BYTES
+	MPIR_Status_set_bytes(status, datatype, 0);
+#endif
 	*error_code = MPI_SUCCESS; 
 	return;
     }
@@ -304,7 +307,7 @@ void ADIOI_NFS_WriteStrided(ADIO_File fd, void *buf, int count,
 /* get max_bufsize from the info object. */
 
     value = (char *) ADIOI_Malloc((MPI_MAX_INFO_VAL+1)*sizeof(char));
-    MPI_Info_get(fd->info, "ind_wr_buffer_size", MPI_MAX_INFO_VAL, value, 
+    ADIOI_Info_get(fd->info, "ind_wr_buffer_size", MPI_MAX_INFO_VAL, value, 
                  &info_flag);
     max_bufsize = atoi(value);
     ADIOI_Free(value);
@@ -381,25 +384,32 @@ void ADIOI_NFS_WriteStrided(ADIO_File fd, void *buf, int count,
 	disp = fd->disp;
 
 	if (file_ptr_type == ADIO_INDIVIDUAL) {
-	    offset = fd->fp_ind; /* in bytes */
-	    n_filetypes = -1;
-	    flag = 0;
-	    while (!flag) {
-                n_filetypes++;
-		for (i=0; i<flat_file->count; i++) {
-		    if (disp + flat_file->indices[i] + 
-                        (ADIO_Offset) n_filetypes*filetype_extent + flat_file->blocklens[i] 
-                            >= offset) {
-			st_index = i;
-			fwr_size = (int) (disp + flat_file->indices[i] + 
-			        (ADIO_Offset) n_filetypes*filetype_extent
-			         + flat_file->blocklens[i] - offset);
-			flag = 1;
-			break;
-		    }
-		}
-	    }
-	}
+       /* Wei-keng reworked type processing to be a bit more efficient */
+            offset       = fd->fp_ind - disp;
+            n_filetypes  = (offset - flat_file->indices[0]) / filetype_extent;
+            offset      -= (ADIO_Offset)n_filetypes * filetype_extent;
+            /* now offset is local to this extent */
+
+            /* find the block where offset is located, skip blocklens[i]==0 */
+            for (i=0; i<flat_file->count; i++) {
+                ADIO_Offset dist;
+                if (flat_file->blocklens[i] == 0) continue;
+                dist = flat_file->indices[i] + flat_file->blocklens[i] - offset;
+                /* fwr_size is from offset to the end of block i */
+                if (dist == 0) {
+                    i++;
+                    offset   = flat_file->indices[i];
+                    fwr_size = flat_file->blocklens[i];
+                    break;
+                }
+                if (dist > 0) {
+                    fwr_size = dist;
+                    break;
+                }
+            }
+            st_index = i;  /* starting index in flat_file->indices[] */
+            offset += disp + (ADIO_Offset)n_filetypes*filetype_extent;
+        }
 	else {
 	    n_etypes_in_filetype = filetype_size/etype_size;
 	    n_filetypes = (int) (offset / n_etypes_in_filetype);
@@ -419,10 +429,40 @@ void ADIOI_NFS_WriteStrided(ADIO_File fd, void *buf, int count,
 	    }
 
 	    /* abs. offset in bytes in the file */
-	    offset = disp + (ADIO_Offset) n_filetypes*filetype_extent + abs_off_in_filetype;
+	    offset = disp + (ADIO_Offset) n_filetypes*filetype_extent + 
+		abs_off_in_filetype;
 	}
 
         start_off = offset;
+       /* Wei-keng Liao:write request is within single flat_file contig block*/
+       /* this could happen, for example, with subarray types that are
+        * actually fairly contiguous */
+        if (buftype_is_contig && bufsize <= fwr_size) {
+            ADIO_WriteContig(fd, buf, bufsize, MPI_BYTE, ADIO_EXPLICIT_OFFSET,
+                             offset, status, error_code);
+
+           if (file_ptr_type == ADIO_INDIVIDUAL) {
+                /* update MPI-IO file pointer to point to the first byte 
+                * that can be accessed in the fileview. */
+                fd->fp_ind = offset + bufsize;
+                if (bufsize == fwr_size) {
+                    do {
+                        st_index++;
+                        if (st_index == flat_file->count) {
+                            st_index = 0;
+                            n_filetypes++;
+                        }
+                    } while (flat_file->blocklens[st_index] == 0);
+                    fd->fp_ind = disp + flat_file->indices[st_index]
+                               + (ADIO_Offset)n_filetypes*filetype_extent;
+                }
+            }
+           fd->fp_sys_posn = -1;   /* set it to null. */ 
+#ifdef HAVE_STATUS_SET_BYTES
+           MPIR_Status_set_bytes(status, datatype, bufsize);
+#endif 
+            return;
+        }
 
        /* Calculate end_offset, the last byte-offset that will be accessed.
          e.g., if start_offset=0 and 100 bytes to be write, end_offset=99*/
@@ -436,14 +476,15 @@ void ADIOI_NFS_WriteStrided(ADIO_File fd, void *buf, int count,
 	while (i < bufsize) {
 	    i += fwr_size;
 	    end_offset = off + fwr_size - 1;
-
-	    if (j < (flat_file->count - 1)) j++;
-	    else {
-		j = 0;
-		n_filetypes++;
+            j = (j+1) % flat_file->count;
+            n_filetypes += (j == 0) ? 1 : 0;
+            while (flat_file->blocklens[j]==0) {
+                j = (j+1) % flat_file->count;
+                n_filetypes += (j == 0) ? 1 : 0;
 	    }
 
-	    off = disp + flat_file->indices[j] + (ADIO_Offset) n_filetypes*filetype_extent;
+	    off = disp + flat_file->indices[j] + 
+		    (ADIO_Offset) n_filetypes*filetype_extent;
 	    fwr_size = ADIOI_MIN(flat_file->blocklens[j], bufsize-i);
 	}
 
@@ -509,13 +550,14 @@ void ADIOI_NFS_WriteStrided(ADIO_File fd, void *buf, int count,
                 /* did not reach end of contiguous block in filetype.
                    no more I/O needed. off is incremented by fwr_size. */
                 else {
-		    if (j < (flat_file->count - 1)) j++;
-		    else {
-			j = 0;
-			n_filetypes++;
-		    }
+                    j = (j+1) % flat_file->count;
+                    n_filetypes += (j == 0) ? 1 : 0;
+                    while (flat_file->blocklens[j]==0) {
+                        j = (j+1) % flat_file->count;
+                        n_filetypes += (j == 0) ? 1 : 0;
+                    }
 		    off = disp + flat_file->indices[j] + 
-                                        (ADIO_Offset) n_filetypes*filetype_extent;
+                                      (ADIO_Offset) n_filetypes*filetype_extent;
 		    fwr_size = ADIOI_MIN(flat_file->blocklens[j], bufsize-i);
 		}
 	    }
@@ -552,10 +594,11 @@ void ADIOI_NFS_WriteStrided(ADIO_File fd, void *buf, int count,
 
 		if (size == fwr_size) {
 /* reached end of contiguous block in file */
-		    if (j < (flat_file->count - 1)) j++;
-		    else {
-			j = 0;
-			n_filetypes++;
+                   j = (j+1) % flat_file->count;
+                   n_filetypes += (j == 0) ? 1 : 0;
+                   while (flat_file->blocklens[j]==0) {
+                       j = (j+1) % flat_file->count;
+                       n_filetypes += (j == 0) ? 1 : 0;
 		    }
 
 		    off = disp + flat_file->indices[j] + 
