@@ -1,4 +1,4 @@
-/* -*- Mode: C; c-basic-offset:4 ; -*- */
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /* 
  *   Copyright (C) 1997 University of Chicago. 
  *   See COPYRIGHT notice in top-level directory.
@@ -106,12 +106,12 @@
     defined(ROMIO_HAVE_STRUCT_STAT_WITH_ST_FSTYPE) 
 #ifndef ROMIO_NTFS
 #define ROMIO_NEEDS_ADIOPARENTDIR
-static void ADIO_FileSysType_parentdir(char *filename, char **dirnamep);
+static void ADIO_FileSysType_parentdir(const char *filename, char **dirnamep);
 #endif
 #endif 
-static void ADIO_FileSysType_prefix(char *filename, int *fstype, 
+static void ADIO_FileSysType_prefix(const char *filename, int *fstype,
 				    int *error_code);
-static void ADIO_FileSysType_fncall(char *filename, int *fstype, 
+static void ADIO_FileSysType_fncall(const char *filename, int *fstype,
 				    int *error_code);
 
 /*
@@ -152,7 +152,7 @@ Output Parameters:
  * Returns pointer to string in dirnamep; that string is allocated with
  * strdup and must be free()'d.
  */
-static void ADIO_FileSysType_parentdir(char *filename, char **dirnamep)
+static void ADIO_FileSysType_parentdir(const char *filename, char **dirnamep)
 {
     int err;
     char *dir = NULL, *slash;
@@ -204,15 +204,23 @@ static void ADIO_FileSysType_parentdir(char *filename, char **dirnamep)
 }
 #endif /* ROMIO_NTFS */
 
-#ifdef ROMIO_BGL   /* BlueGene support for pvfs through ufs */
+#ifdef ROMIO_BGL   /* BlueGene support for lockless i/o (necessary for PVFS.
+		      possibly beneficial for others, unless data sieving
+		      writes desired) */
+
+/* BlueGene environment variables can override lockless selection.*/
+extern void ad_bgl_get_env_vars();
+extern long bglocklessmpio_f_type;
+
 static void check_for_lockless_exceptions(long stat_type, int *fstype)
 {
-    /* exception for lockless PVFS file system.  PVFS is the only exception we
-     * make right now, but any future FS developers looking to override
-     * BlueGene fs detection can do it here */
-    if (stat_type == PVFS2_SUPER_MAGIC) 
-	/* use lock-free driver on bluegene to support pvfs */
-	*fstype = ADIO_BGLOCKLESS; 
+    /* exception for lockless file systems.  (PVFS2 is the default in ad_bgl_tuning.)
+     * The BGLOCKLESS_F_TYPE environment variable will override it by specifying
+     * the appropriate file system magic number here.
+     */
+    if (stat_type == bglocklessmpio_f_type)
+      /* use lock-free driver on bluegene to support specified fs (defaults to pvfs2) */
+      *fstype = ADIO_BGLOCKLESS;
 }
 #endif
 /*
@@ -232,10 +240,9 @@ Output Parameters:
  file system type.  Most other functions use the type which is stored when the 
  file is opened.
  */
-static void ADIO_FileSysType_fncall(char *filename, int *fstype, int *error_code)
+static void ADIO_FileSysType_fncall(const char *filename, int *fstype, int *error_code)
 {
-#ifndef ROMIO_NTFS
-    char *dir;
+#if defined (ROMIO_HAVE_STRUCT_STATVFS_WITH_F_BASETYPE) || defined (HAVE_STRUCT_STATFS) || defined (ROMIO_HAVE_STRUCT_STAT_WITH_ST_FSTYPE)
     int err;
 #endif
 
@@ -250,24 +257,36 @@ static void ADIO_FileSysType_fncall(char *filename, int *fstype, int *error_code
 #endif
     static char myname[] = "ADIO_RESOLVEFILETYPE_FNCALL";
 
+/* NFS can get stuck and end up returing ESTALE "forever" */
+#define MAX_ESTALE_RETRY 10000
+    int retry_cnt;
+
     *error_code = MPI_SUCCESS;
 
 #ifdef ROMIO_HAVE_STRUCT_STATVFS_WITH_F_BASETYPE
+    retry_cnt=0;
     do {
 	err = statvfs(filename, &vfsbuf);
-    } while (err && (errno == ESTALE));
+    } while (err && (errno == ESTALE) && retry_cnt++ < MAX_ESTALE_RETRY);
 
-    if (err && (errno == ENOENT)) {
+    if (err) {
 	/* ENOENT may be returned in two cases:
 	 * 1) no directory entry for "filename"
 	 * 2) "filename" is a dangling symbolic link
 	 *
 	 * ADIO_FileSysType_parentdir tries to deal with both cases.
 	 */
-	ADIO_FileSysType_parentdir(filename, &dir);
-	err = statvfs(dir, &vfsbuf);
+	if (errno == ENOENT) {
+	    char *dir;
+	    ADIO_FileSysType_parentdir(filename, &dir);
+	    err = statvfs(dir, &vfsbuf);
 
-	ADIOI_Free(dir);
+	    ADIOI_Free(dir);
+	}
+	else {
+	    *error_code = ADIOI_Err_create_code(myname, filename, errno);
+	    if(*error_code != MPI_SUCCESS) return;
+	}
     }
 
     /* --BEGIN ERROR HANDLING-- */
@@ -303,14 +322,22 @@ static void ADIO_FileSysType_fncall(char *filename, int *fstype, int *error_code
 #endif /* STATVFS APPROACH */
 
 #ifdef HAVE_STRUCT_STATFS
+    retry_cnt = 0;
     do {
 	err = statfs(filename, &fsbuf);
-    } while (err && (errno == ESTALE));
+    } while (err && (errno == ESTALE) && retry_cnt++ < MAX_ESTALE_RETRY);
 
-    if (err && (errno == ENOENT)) {
-	ADIO_FileSysType_parentdir(filename, &dir);
-	err = statfs(dir, &fsbuf);
-	ADIOI_Free(dir);
+    if (err) {
+	if(errno == ENOENT) {
+	    char *dir;
+	    ADIO_FileSysType_parentdir(filename, &dir);
+	    err = statfs(dir, &fsbuf);
+	    ADIOI_Free(dir);
+	}
+	else {
+	    *error_code = ADIOI_Err_create_code(myname, filename, errno);
+	    if(*error_code != MPI_SUCCESS) return;
+	}
     }
 
     /* --BEGIN ERROR HANDLING-- */
@@ -332,6 +359,10 @@ static void ADIO_FileSysType_fncall(char *filename, int *fstype, int *error_code
 #  ifdef ROMIO_BGL 
     /* BlueGene is a special case: all file systems are AD_BGL, except for
      * certain exceptions */
+
+    /* Bluegene needs to read enviroment variables before selecting the file system*/
+    ad_bgl_get_env_vars();
+
     *fstype = ADIO_BGL;
     check_for_lockless_exceptions(fsbuf.f_type, fstype);
     *error_code = MPI_SUCCESS;
@@ -411,14 +442,22 @@ static void ADIO_FileSysType_fncall(char *filename, int *fstype, int *error_code
 #endif /* STATFS APPROACH */
 
 #ifdef ROMIO_HAVE_STRUCT_STAT_WITH_ST_FSTYPE
+    retry_cnt = 0;
     do {
 	err = stat(filename, &sbuf);
-    } while (err && (errno == ESTALE));
+    } while (err && (errno == ESTALE) && retry_cnt++ < MAX_ESTALE_RETRY);
 
-    if (err && (errno == ENOENT)) {
-	ADIO_FileSysType_parentdir(filename, &dir);
-	err = stat(dir, &sbuf);
-	ADIOI_Free(dir);
+    if (err) {
+	if(errno == ENOENT) {
+	    char *dir;
+	    ADIO_FileSysType_parentdir(filename, &dir);
+	    err = stat(dir, &sbuf);
+	    ADIOI_Free(dir);
+	}
+	else{
+	    *error_code = ADIOI_Err_create_code(myname, filename, errno);
+	    if(*error_code != MPI_SUCCESS) return;
+	}
     }
     
     if (err) {
@@ -456,7 +495,7 @@ static void ADIO_FileSysType_fncall(char *filename, int *fstype, int *error_code
  * stat system calls (unless a fs prefix is given).  Cary out this file system
  * detection in a more scalable way by having rank 0 stat the file and broadcast the result (fs type and error code) to the other mpi processes */
 
-static void ADIO_FileSysType_fncall_scalable(MPI_Comm comm, char *filename, int * file_system, int * error_code)
+static void ADIO_FileSysType_fncall_scalable(MPI_Comm comm, const char *filename, int * file_system, int * error_code)
 {
     int rank;
     int buf[2];
@@ -490,7 +529,7 @@ Output Parameters:
   is considered an error. Except for on Windows systems where the default is NTFS.
 
  */
-static void ADIO_FileSysType_prefix(char *filename, int *fstype, int *error_code)
+static void ADIO_FileSysType_prefix(const char *filename, int *fstype, int *error_code)
 {
     static char myname[] = "ADIO_RESOLVEFILETYPE_PREFIX";
     *error_code = MPI_SUCCESS;
@@ -584,7 +623,7 @@ order to clean things up.  The goal is to separate all this "did we compile
 for this fs type" code from the MPI layer and also to introduce the ADIOI_Fns
 tables in a reasonable way. -- Rob, 06/06/2001
 @*/
-void ADIO_ResolveFileType(MPI_Comm comm, char *filename, int *fstype, 
+void ADIO_ResolveFileType(MPI_Comm comm, const char *filename, int *fstype,
 			  ADIOI_Fns **ops, int *error_code)
 {
     int myerrcode, file_system, min_code, max_code;
@@ -603,6 +642,8 @@ void ADIO_ResolveFileType(MPI_Comm comm, char *filename, int *fstype,
 	 * everyone else.  
 	 * - Note that we will not catch cases like
 	 * http://www.mcs.anl.gov/web-mail-archive/lists/mpich-discuss/2007/08/msg00042.html
+	 * (edit: now http://lists.mcs.anl.gov/pipermail/mpich-discuss/2007-August/002648.html)
+	 *
 	 * where file systems are not mounted or available on other processes,
 	 * but we'll catch those a few functions later in ADIO_Open 
 	 * - Note that if we have NFS enabled, we might have a situation where,
@@ -623,30 +664,28 @@ void ADIO_ResolveFileType(MPI_Comm comm, char *filename, int *fstype,
 	    }
 	} else {
 	    ADIO_FileSysType_fncall(filename, &file_system, &myerrcode);
-	    if (myerrcode != MPI_SUCCESS) {
-		*error_code = myerrcode;
 
-		/* the check for file system type will hang if any process got
-		 * an error in ADIO_FileSysType_fncall.  Processes encountering
-		 * an error will return early, before the collective file
-		 * system type check below.  This case could happen if a full
-		 * path exists on one node but not on others, and no prefix
-		 * like ufs: was provided.  see discussion at
-		 * http://www.mcs.anl.gov/web-mail-archive/lists/mpich-discuss/2007/08/msg00042.html 
-	         */
+	    /* the check for file system type will hang if any process got
+	     * an error in ADIO_FileSysType_fncall.  Processes encountering
+	     * an error will return early, before the collective file
+	     * system type check below.  This case could happen if a full
+	     * path exists on one node but not on others, and no prefix
+	     * like ufs: was provided.  see discussion at
+	     * http://www.mcs.anl.gov/web-mail-archive/lists/mpich-discuss/2007/08/msg00042.html
+	     * (edit: now
+	     * http://lists.mcs.anl.gov/pipermail/mpich-discuss/2007-August/002648.html)
+	     */
 
-	        MPI_Allreduce(error_code, &max_code, 1, MPI_INT, MPI_MAX, comm);
-		if (max_code != MPI_SUCCESS)  {
-		    *error_code = max_code;
-		    return;
-		} 
-		/* ensure everyone came up with the same file system type */
-		MPI_Allreduce(&file_system, &min_code, 1, MPI_INT, 
-			MPI_MIN, comm);
-		if (min_code == ADIO_NFS) file_system = ADIO_NFS;
+	    MPI_Allreduce(&myerrcode, &max_code, 1, MPI_INT, MPI_MAX, comm);
+	    if (max_code != MPI_SUCCESS)  {
+		*error_code = max_code;
+		return;
 	    }
+	    /* ensure everyone came up with the same file system type */
+	    MPI_Allreduce(&file_system, &min_code, 1, MPI_INT,
+		    MPI_MIN, comm);
+	    if (min_code == ADIO_NFS) file_system = ADIO_NFS;
 	}
-
     }
     else {
 	/* prefix specified; just match via prefix and assume everyone got 
