@@ -1,4 +1,4 @@
-/* -*- Mode: C; c-basic-offset:4 ; -*- */
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /* 
  *   Copyright (C) 1997 University of Chicago. 
  *   See COPYRIGHT notice in top-level directory.
@@ -103,6 +103,7 @@
 #   define ADIO_OFFSET OMPI_OFFSET_DATATYPE
 #else
 /* Open MPI: ignores all this stuff */
+
 #ifdef MPI_OFFSET_IS_INT
    typedef int ADIO_Offset;
 #  define ADIO_OFFSET MPI_INT
@@ -179,7 +180,7 @@ MPI_Info PMPI_Info_f2c(MPI_Fint info);
 char *strdup(const char *s);
 # endif
 #if defined(HAVE_READLINK) && defined(NEEDS_READLINK_DECL) && !defined(readlink)
-int readlink(const char *path, char *buf, size_t bufsiz);
+ssize_t readlink(const char *path, char *buf, size_t bufsiz);
 # endif
 #if defined(HAVE_LSTAT) && defined(NEEDS_LSTAT_DECL) && !defined(lstat)
 int lstat(const char *file_name, struct stat *buf);
@@ -244,6 +245,9 @@ typedef struct ADIOI_FileD {
     ADIO_Offset *file_realm_st_offs; /* file realm starting offsets */
     MPI_Datatype *file_realm_types;  /* file realm datatypes */
     int my_cb_nodes_index; /* my index into cb_config_list. -1 if N/A */
+    /* External32 */
+    int is_external32;      /* bool:  0 means native view */
+
 } ADIOI_FileD;
 
 typedef struct ADIOI_FileD *ADIO_File;
@@ -311,11 +315,16 @@ typedef struct {
 #define ADIO_FCNTL_GET_FSIZE     200
 
 /* file system feature tests */
-#define ADIO_LOCKS               300
-#define ADIO_SHARED_FP           301
-#define ADIO_ATOMIC_MODE         302
-#define ADIO_DATA_SIEVING_WRITES 303
-#define ADIO_SCALABLE_OPEN       304
+#define ADIO_LOCKS               300 /* file system supports fcntl()-style locking */
+#define ADIO_SHARED_FP           301 /* file system supports shared file pointers */
+#define ADIO_ATOMIC_MODE         302 /* file system supports atomic mode */
+#define ADIO_DATA_SIEVING_WRITES 303 /* file system supports data sieving for writes */
+#define ADIO_SCALABLE_OPEN       304 /* one process can open the file and
+					broadcast result to all other
+					processors */
+#define ADIO_UNLINK_AFTER_CLOSE  305 /* supports posix semantic of keeping a
+					deleted file around until all
+					processors have closed it */
 
 /* for default file permissions */
 #define ADIO_PERM_NULL           -1
@@ -330,10 +339,10 @@ typedef struct {
 
 void ADIO_Init(int *argc, char ***argv, int *error_code);
 void ADIO_End(int *error_code);
-MPI_File ADIO_Open(MPI_Comm orig_comm, MPI_Comm comm, char *filename, 
+MPI_File ADIO_Open(MPI_Comm orig_comm, MPI_Comm comm, const char *filename,
 		   int file_system, ADIOI_Fns *ops,
-		   int access_mode, ADIO_Offset disp, MPI_Datatype etype, 
-		   MPI_Datatype filetype, 
+		   int access_mode, ADIO_Offset disp, MPI_Datatype etype,
+		   MPI_Datatype filetype,
 		   MPI_Info info, int perm, int *error_code);
 void ADIOI_OpenColl(ADIO_File fd, int rank, int acces_mode, int *error_code);
 void ADIO_ImmediateOpen(ADIO_File fd, int *error_code);
@@ -371,7 +380,7 @@ void ADIO_ReadStrided(ADIO_File fd, void *buf, int count,
 		       MPI_Datatype datatype, int file_ptr_type,
 		       ADIO_Offset offset, ADIO_Status *status, int
 		       *error_code);
-void ADIO_WriteStrided(ADIO_File fd, void *buf, int count,
+void ADIO_WriteStrided(ADIO_File fd, const void *buf, int count,
 		       MPI_Datatype datatype, int file_ptr_type,
 		       ADIO_Offset offset, ADIO_Status *status, int
 		       *error_code);
@@ -397,7 +406,7 @@ void ADIO_Delete(char *filename, int *error_code);
 void ADIO_Flush(ADIO_File fd, int *error_code);
 void ADIO_Resize(ADIO_File fd, ADIO_Offset size, int *error_code);
 void ADIO_SetInfo(ADIO_File fd, MPI_Info users_info, int *error_code);
-void ADIO_ResolveFileType(MPI_Comm comm, char *filename, int *fstype, 
+void ADIO_ResolveFileType(MPI_Comm comm, const char *filename, int *fstype,
           ADIOI_Fns **ops, int *error_code);
 void ADIO_Get_shared_fp(ADIO_File fd, int size, ADIO_Offset *shared_fp, 
 			 int *error_code);
