@@ -1,4 +1,4 @@
-/* -*- Mode: C; c-basic-offset:4 ; -*- */
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /* 
  *
  *   Copyright (C) 1997 University of Chicago. 
@@ -89,4 +89,9 @@ extern struct ADIOI_Fns_struct ADIO_BGLOCKLESS_operations;
 extern struct ADIOI_Fns_struct ADIO_GRIDFTP_operations;
 #endif
 
+#ifdef ROMIO_ZOIDFS
+/* prototypes are in adio/ad_zoidfs/ad_zoidfs.h */
+extern struct ADIOI_Fns_struct ADIO_ZOIDFS_operations;
+#endif
+
 #endif
