@@ -1,8 +1,9 @@
-/* -*- Mode: C; c-basic-offset:4 ; -*- */
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /* 
  *
  *   Copyright (C) 1997 University of Chicago. 
  *   See COPYRIGHT notice in top-level directory.
+ *
  */
 
 #include "mpioimpl.h"
@@ -39,69 +40,81 @@ Output Parameters:
 
 .N fortran
 @*/
-int MPI_File_write_ordered(MPI_File mpi_fh, void *buf, int count, 
+int MPI_File_write_ordered(MPI_File fh, const void *buf, int count,
 			   MPI_Datatype datatype, MPI_Status *status)
 {
     int error_code, datatype_size, nprocs, myrank, incr;
     int source, dest;
     static char myname[] = "MPI_FILE_WRITE_ORDERED";
     ADIO_Offset shared_fp;
-    ADIO_File fh;
+    ADIO_File adio_fh;
+    void *e32buf=NULL;
+    const void *xbuf;
 
     MPIU_THREAD_CS_ENTER(ALLFUNC,);
 
-    fh = MPIO_File_resolve(mpi_fh);
+    adio_fh = MPIO_File_resolve(fh);
 
     /* --BEGIN ERROR HANDLING-- */
-    MPIO_CHECK_FILE_HANDLE(fh, myname, error_code);
-    MPIO_CHECK_COUNT(fh, count, myname, error_code);
-    MPIO_CHECK_DATATYPE(fh, datatype, myname, error_code);
+    MPIO_CHECK_FILE_HANDLE(adio_fh, myname, error_code);
+    MPIO_CHECK_COUNT(adio_fh, count, myname, error_code);
+    MPIO_CHECK_DATATYPE(adio_fh, datatype, myname, error_code);
     /* --END ERROR HANDLING-- */
 
     MPI_Type_size(datatype, &datatype_size);
 
     /* --BEGIN ERROR HANDLING-- */
-    MPIO_CHECK_INTEGRAL_ETYPE(fh, count, datatype_size, myname, error_code);
-    MPIO_CHECK_FS_SUPPORTS_SHARED(fh, myname, error_code);
-    MPIO_CHECK_COUNT_SIZE(fh, count, datatype_size, myname, error_code);
+    MPIO_CHECK_INTEGRAL_ETYPE(adio_fh, count, datatype_size, myname, error_code);
+    MPIO_CHECK_FS_SUPPORTS_SHARED(adio_fh, myname, error_code);
+    MPIO_CHECK_COUNT_SIZE(adio_fh, count, datatype_size, myname, error_code);
     /* --END ERROR HANDLING-- */
 
-    ADIOI_TEST_DEFERRED(fh, myname, &error_code);
+    ADIOI_TEST_DEFERRED(adio_fh, myname, &error_code);
 
-    MPI_Comm_size(fh->comm, &nprocs);
-    MPI_Comm_rank(fh->comm, &myrank);
+    MPI_Comm_size(adio_fh->comm, &nprocs);
+    MPI_Comm_rank(adio_fh->comm, &myrank);
 
-    incr = (count*datatype_size)/fh->etype_size;
+    incr = (count*datatype_size)/adio_fh->etype_size;
     /* Use a message as a 'token' to order the operations */
     source = myrank - 1;
     dest   = myrank + 1;
     if (source < 0) source = MPI_PROC_NULL;
     if (dest >= nprocs) dest = MPI_PROC_NULL;
-    MPI_Recv(NULL, 0, MPI_BYTE, source, 0, fh->comm, MPI_STATUS_IGNORE);
+    MPI_Recv(NULL, 0, MPI_BYTE, source, 0, adio_fh->comm, MPI_STATUS_IGNORE);
 
-    ADIO_Get_shared_fp(fh, incr, &shared_fp, &error_code);
+    ADIO_Get_shared_fp(adio_fh, incr, &shared_fp, &error_code);
 
     /* --BEGIN ERROR HANDLING-- */
     if (error_code != MPI_SUCCESS) {
 	error_code = MPIO_Err_create_code(MPI_SUCCESS, MPIR_ERR_FATAL,
 					  myname, __LINE__, MPI_ERR_INTERN, 
 					  "**iosharedfailed", 0);
-	error_code = MPIO_Err_return_file(fh, error_code);
+	error_code = MPIO_Err_return_file(adio_fh, error_code);
 	goto fn_exit;
     }
     /* --END ERROR HANDLING-- */
 
-    MPI_Send(NULL, 0, MPI_BYTE, dest, 0, fh->comm);
+    MPI_Send(NULL, 0, MPI_BYTE, dest, 0, adio_fh->comm);
 
-    ADIO_WriteStridedColl(fh, buf, count, datatype, ADIO_EXPLICIT_OFFSET,
+    xbuf = buf;
+    if (adio_fh->is_external32) {
+	error_code = MPIU_external32_buffer_setup(buf, count, datatype, &e32buf);
+	if (error_code != MPI_SUCCESS)
+	    goto fn_exit;
+
+	xbuf = e32buf;
+    }
+
+    ADIO_WriteStridedColl(adio_fh, xbuf, count, datatype, ADIO_EXPLICIT_OFFSET,
                           shared_fp, status, &error_code);
 
     /* --BEGIN ERROR HANDLING-- */
     if (error_code != MPI_SUCCESS)
-	error_code = MPIO_Err_return_file(fh, error_code);
+	error_code = MPIO_Err_return_file(adio_fh, error_code);
     /* --END ERROR HANDLING-- */
 
 fn_exit:
+    if (e32buf != NULL) ADIOI_Free(e32buf);
     MPIU_THREAD_CS_EXIT(ALLFUNC,);
 
     /* FIXME: Check for error code from WriteStridedColl? */
