@@ -1,4 +1,4 @@
-/* -*- Mode: C; c-basic-offset:4 ; -*- */
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /* 
  *
  *   Copyright (C) 1997 University of Chicago. 
@@ -35,36 +35,36 @@ Output Parameters:
 
 .N fortran
 @*/
-int MPI_File_write_ordered_end(MPI_File mpi_fh, void *buf, MPI_Status *status)
+int MPI_File_write_ordered_end(MPI_File fh, const void *buf, MPI_Status *status)
 {
     int error_code;
     static char myname[] = "MPI_FILE_WRITE_ORDERED_END";
-    ADIO_File fh;
+    ADIO_File adio_fh;
 
     MPIU_UNREFERENCED_ARG(buf);
 
     MPIU_THREAD_CS_ENTER(ALLFUNC,);
 
-    fh = MPIO_File_resolve(mpi_fh);
+    adio_fh = MPIO_File_resolve(fh);
 
     /* --BEGIN ERROR HANDLING-- */
-    MPIO_CHECK_FILE_HANDLE(fh, myname, error_code);
+    MPIO_CHECK_FILE_HANDLE(adio_fh, myname, error_code);
 
-    if (!(fh->split_coll_count))
+    if (!(adio_fh->split_coll_count))
     {
 	error_code = MPIO_Err_create_code(MPI_SUCCESS, MPIR_ERR_RECOVERABLE,
 					  myname, __LINE__, MPI_ERR_IO, 
 					  "**iosplitcollnone", 0);
-	error_code = MPIO_Err_return_file(fh, error_code);
+	error_code = MPIO_Err_return_file(adio_fh, error_code);
 	goto fn_exit;
     }
     /* --END ERROR HANDLING-- */
 
 #ifdef HAVE_STATUS_SET_BYTES
     if (status != MPI_STATUS_IGNORE)
-       *status = fh->split_status;
+       *status = adio_fh->split_status;
 #endif
-    fh->split_coll_count = 0;
+    adio_fh->split_coll_count = 0;
 
 
 fn_exit:
