@@ -20,18 +20,16 @@
 #ifndef MCA_IO_ROMIO_H
 #define MCA_IO_ROMIO_H
 
+#include "ompi_config.h"
 #include "opal/threads/mutex.h"
 #include "ompi/request/request.h"
 #include "ompi/file/file.h"
 #include "ompi/mca/io/io.h"
-#include "ompi/mca/io/base/io_base_request.h"
 #include "romio/adio/include/romioconf.h"
 #include "romio/include/mpio.h"
 
 
-#if defined(c_plusplus) || defined(__cplusplus)
-extern "C" {
-#endif
+BEGIN_C_DECLS
 
 OMPI_DECLSPEC extern mca_io_base_component_2_0_0_t mca_io_romio_component;
 
@@ -40,20 +38,8 @@ OMPI_DECLSPEC extern mca_io_base_component_2_0_0_t mca_io_romio_component;
  */
 extern opal_mutex_t mca_io_romio_mutex;
 extern mca_io_base_module_2_0_0_t mca_io_romio_module;
-extern opal_list_t mca_io_romio_pending_requests;
 OMPI_DECLSPEC extern mca_io_base_component_2_0_0_t mca_io_romio_component;
 
-/* 
- * The romio component will "inherit" from the mca_io_base_request_t,
- * adding its own stuff on to the end.
- */
-struct mca_io_romio_request_t {
-    mca_io_base_request_t super;
-    MPI_Request romio_rq;
-};
-typedef struct mca_io_romio_request_t mca_io_romio_request_t;
-
-
 /* 
  * Private data for ROMIO modules
  */
@@ -67,20 +53,6 @@ typedef struct mca_io_romio_data_t mca_io_romio_data_t;
  * Module functions
  */
 
-int mca_io_romio_progress(void);
-int mca_io_romio_request_free(ompi_request_t **req);
-int mca_io_romio_request_cancel(ompi_request_t *req, int flag);
-
-/**
- * Macro to add a request to the list of pending requests to be
- * progressed.  This macro is ONLY called when the ROMIO mutex is
- * already held!
- */
-#define MCA_IO_ROMIO_REQUEST_ADD(request)              \
-    ((ompi_request_t*) request)->req_state = OMPI_REQUEST_ACTIVE; \
-    opal_list_append(&mca_io_romio_pending_requests, (opal_list_item_t *) request); \
-    mca_io_base_request_progress_add();
-
 
 /*
  *  mca->ROMIO module routines:    
@@ -153,13 +125,13 @@ int mca_io_romio_file_iread_at (struct ompi_file_t *fh,
                                 void *buf,
                                 int count,
                                 struct ompi_datatype_t *datatype,
-                                mca_io_base_request_t * request);
+                                ompi_request_t **request);
 int mca_io_romio_file_iwrite_at (struct ompi_file_t *fh,
                                  MPI_Offset offset,
                                  void *buf,
                                  int count,
                                  struct ompi_datatype_t *datatype,
-                                 mca_io_base_request_t * request);
+                                 ompi_request_t **request);
 
 /* Section 9.4.3 */
 int mca_io_romio_file_read (struct ompi_file_t *fh,
@@ -186,12 +158,12 @@ int mca_io_romio_file_iread (struct ompi_file_t *fh,
                              void *buf,
                              int count,
                              struct ompi_datatype_t *datatype,
-                             mca_io_base_request_t * request);
+                             ompi_request_t **request);
 int mca_io_romio_file_iwrite (struct ompi_file_t *fh,
                               void *buf,
                               int count,
                               struct ompi_datatype_t *datatype,
-                              mca_io_base_request_t * request);
+                              ompi_request_t **request);
 int mca_io_romio_file_seek (struct ompi_file_t *fh,
                             MPI_Offset offset,
                             int whence);
@@ -216,12 +188,12 @@ int mca_io_romio_file_iread_shared (struct ompi_file_t *fh,
                                     void *buf,
                                     int count,
                                     struct ompi_datatype_t *datatype,
-                                    mca_io_base_request_t * request);
+                                    ompi_request_t **request);
 int mca_io_romio_file_iwrite_shared (struct ompi_file_t *fh,
                                      void *buf,
                                      int count,
                                      struct ompi_datatype_t *datatype,
-                                     mca_io_base_request_t * request);
+                                     ompi_request_t **request);
 int mca_io_romio_file_read_ordered (struct ompi_file_t *fh,
                                     void *buf,
                                     int count,
@@ -298,7 +270,5 @@ int mca_io_romio_file_sync (struct ompi_file_t *fh);
 
 /* End Prototypes */
 
-#if defined(c_plusplus) || defined(__cplusplus)
-}
-#endif
+END_C_DECLS
 #endif                          /* MCA_IO_ROMIO_H */
