@@ -11,7 +11,8 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2006-2007 Mellanox Technologies. All rights reserved.
- * Copyright (c) 2008-2009 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2008-2014 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2012      Los Alamos National Security, LLC.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -24,19 +25,17 @@
 
 #include <stdio.h>
 #include <string.h>
-#ifdef HAVE_MALLOC_H
-#include <malloc.h>
-#endif
 
 #include "opal/mca/mca.h"
 #include "opal/mca/base/base.h"
-#include "orte/util/show_help.h"
-#include "orte/util/name_fns.h"
-#include "orte/util/proc_info.h"
-#include "orte/runtime/orte_globals.h"
-#include "ompi/mca/mpool/mpool.h"
+#include "opal/util/show_help.h"
+
+#include "ompi/mca/rte/rte.h"
 #include "ompi/runtime/params.h"
+
+#include "ompi/mca/mpool/mpool.h"
 #include "ompi/mca/mpool/base/base.h"
+#include "opal/memoryhooks/memory.h"
 #include "mpool_base_mem_cb.h"
 
 
@@ -44,8 +43,8 @@ mca_mpool_base_component_t* mca_mpool_base_component_lookup(const char* name)
 {
     /* Traverse the list of available modules; call their init functions. */
     opal_list_item_t* item;
-    for (item = opal_list_get_first(&mca_mpool_base_components);
-         item != opal_list_get_end(&mca_mpool_base_components);
+    for (item = opal_list_get_first(&ompi_mpool_base_framework.framework_components);
+         item != opal_list_get_end(&ompi_mpool_base_framework.framework_components);
          item = opal_list_get_next(item)) {
          mca_base_component_list_item_t *cli = 
            (mca_base_component_list_item_t *) item;
@@ -66,25 +65,16 @@ mca_mpool_base_module_t* mca_mpool_base_module_create(
 {
     mca_mpool_base_component_t* component = NULL; 
     mca_mpool_base_module_t* module = NULL; 
-    opal_list_item_t* item;
+    mca_base_component_list_item_t *cli;
     mca_mpool_base_selected_module_t *sm;
 
-    for (item = opal_list_get_first(&mca_mpool_base_components);
-         item != opal_list_get_end(&mca_mpool_base_components);
-         item = opal_list_get_next(item)) {
-         mca_base_component_list_item_t *cli = 
-           (mca_base_component_list_item_t *) item;
-         component = 
-           (mca_mpool_base_component_t *) cli->cli_component;
+    OPAL_LIST_FOREACH(cli, &ompi_mpool_base_framework.framework_components, mca_base_component_list_item_t) {
+         component = (mca_mpool_base_component_t *) cli->cli_component;
          if(0 == strcmp(component->mpool_version.mca_component_name, name)) {
+             module = component->mpool_init(resources);
              break;
          }
     }
-
-    if (opal_list_get_end(&mca_mpool_base_components) == item) {
-        return NULL;
-    }
-    module = component->mpool_init(resources); 
     if ( NULL == module ) {
         return NULL;
     }
@@ -104,8 +94,10 @@ mca_mpool_base_module_t* mca_mpool_base_module_create(
            leave_pinned_pipeline is enabled (note that either of these
            leave_pinned variables may have been set by a user MCA
            param or elsewhere in the code base).  Yes, we could have
-           coded this more succinctly, but this is more clear. */
-        if (ompi_mpi_leave_pinned || ompi_mpi_leave_pinned_pipeline) {
+           coded this more succinctly, but this is more clear. Do not
+           check memory hooks if the mpool explicity asked us not to. */
+        if ((ompi_mpi_leave_pinned > 0 || ompi_mpi_leave_pinned_pipeline) &&
+            !(module->flags & MCA_MPOOL_FLAGS_NO_HOOKS)) {
             use_mem_hooks = 1;
         }
 
@@ -115,9 +107,9 @@ mca_mpool_base_module_t* mca_mpool_base_module_create(
                  opal_mem_hooks_support_level())) {
                 opal_mem_hooks_register_release(mca_mpool_base_mem_cb, NULL);
             } else {
-                orte_show_help("help-mpool-base.txt", "leave pinned failed",
-                               true, name, ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                               orte_process_info.nodename);
+                opal_show_help("help-mpool-base.txt", "leave pinned failed",
+                               true, name, OMPI_NAME_PRINT(OMPI_PROC_MY_NAME),
+                               ompi_process_info.nodename);
                 return NULL;
             }
 
@@ -132,13 +124,9 @@ mca_mpool_base_module_t* mca_mpool_base_module_create(
 
 mca_mpool_base_module_t* mca_mpool_base_module_lookup(const char* name)
 {
-    opal_list_item_t* item;
-    
-    for (item = opal_list_get_first(&mca_mpool_base_modules);
-         item != opal_list_get_end(&mca_mpool_base_modules);
-         item = opal_list_get_next(item)) {
-        mca_mpool_base_selected_module_t *mli =
-            (mca_mpool_base_selected_module_t *) item;
+    mca_mpool_base_selected_module_t *mli;
+
+    OPAL_LIST_FOREACH(mli, &mca_mpool_base_modules, mca_mpool_base_selected_module_t) {
         if(0 == strcmp(mli->mpool_component->mpool_version.mca_component_name,
                        name)) {
             return mli->mpool_module;
@@ -151,15 +139,11 @@ mca_mpool_base_module_t* mca_mpool_base_module_lookup(const char* name)
 
 int mca_mpool_base_module_destroy(mca_mpool_base_module_t *module)
 {
-    opal_list_item_t* item;
-    mca_mpool_base_selected_module_t *sm;
+    mca_mpool_base_selected_module_t *sm, *next;
 
-    for (item = opal_list_get_first(&mca_mpool_base_modules);
-            item != opal_list_get_end(&mca_mpool_base_modules);
-            item = opal_list_get_next(item)) {
-        sm = (mca_mpool_base_selected_module_t *) item;
+    OPAL_LIST_FOREACH_SAFE(sm, next, &mca_mpool_base_modules, mca_mpool_base_selected_module_t) {
         if (module == sm->mpool_module) {
-            opal_list_remove_item(&mca_mpool_base_modules,item);
+            opal_list_remove_item(&mca_mpool_base_modules, (opal_list_item_t*)sm);
             if (NULL != sm->mpool_module->mpool_finalize) {
                 sm->mpool_module->mpool_finalize(sm->mpool_module);
             }
