@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2013 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -12,6 +12,8 @@
  * Copyright (c) 2007      Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2007      Voltaire. All rights reserved.
  * Copyright (c) 2010      IBM Corporation.  All rights reserved.
+ * Copyright (c) 2012      Los Alamos National Security, LLC.
+ *                         All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -26,10 +28,9 @@
 #include "ompi_config.h"
 
 #include "opal/mca/mca.h"
-#include "orte/util/show_help.h"
-#include "orte/util/name_fns.h"
-#include "orte/util/proc_info.h"
-#include "orte/runtime/orte_globals.h"
+#include "opal/util/show_help.h"
+
+#include "ompi/mca/rte/rte.h"
 #include "ompi/runtime/params.h"
 #include "ompi/class/ompi_free_list.h"
 #include "ompi/class/ompi_rb_tree.h"
@@ -90,6 +91,17 @@ int mca_mpool_base_tree_init(void) {
     return rc;
 }
 
+/*
+ *
+ */
+int mca_mpool_base_tree_fini(void)
+{
+    OBJ_DESTRUCT(&mca_mpool_base_tree);
+    OBJ_DESTRUCT(&mca_mpool_base_tree_item_free_list);
+    OBJ_DESTRUCT(&tree_lock);
+    return OMPI_SUCCESS;
+}
+
 /* 
  * insert an item in the rb tree 
  */ 
@@ -142,11 +154,8 @@ mca_mpool_base_tree_item_t* mca_mpool_base_tree_find(void* base) {
  */
 mca_mpool_base_tree_item_t* mca_mpool_base_tree_item_get(void) { 
     ompi_free_list_item_t* item = NULL;
-    int rc;
-    OMPI_FREE_LIST_GET(&mca_mpool_base_tree_item_free_list, 
-                       item, 
-                       rc); 
-    if(OMPI_SUCCESS == rc) { 
+    OMPI_FREE_LIST_GET_MT(&mca_mpool_base_tree_item_free_list, item);
+    if(NULL != item) { 
         return (mca_mpool_base_tree_item_t*) item; 
     } else { 
         return NULL;
@@ -157,7 +166,7 @@ mca_mpool_base_tree_item_t* mca_mpool_base_tree_item_get(void) {
  * put an item back into the free list
  */
 void mca_mpool_base_tree_item_put(mca_mpool_base_tree_item_t* item) { 
-    OMPI_FREE_LIST_RETURN(&mca_mpool_base_tree_item_free_list,
+    OMPI_FREE_LIST_RETURN_MT(&mca_mpool_base_tree_item_free_list,
                           &(item->super));
 }
 
@@ -175,19 +184,22 @@ void mca_mpool_base_tree_print(void)
 
     num_leaks = 0;
     ompi_rb_tree_traverse(&mca_mpool_base_tree, condition, action);
+    if (0 == num_leaks) {
+        return;
+    }
 
     if (num_leaks <= ompi_debug_show_mpi_alloc_mem_leaks ||
         ompi_debug_show_mpi_alloc_mem_leaks < 0) {
-        orte_show_help("help-mpool-base.txt", "all mem leaks",
-                       true, ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                       orte_process_info.nodename,
-                       orte_process_info.pid, leak_msg);
+        opal_show_help("help-mpool-base.txt", "all mem leaks",
+                       true, OMPI_NAME_PRINT(OMPI_PROC_MY_NAME),
+                       ompi_process_info.nodename,
+                       ompi_process_info.pid, leak_msg);
     } else {
         int i = num_leaks - ompi_debug_show_mpi_alloc_mem_leaks;
-        orte_show_help("help-mpool-base.txt", "some mem leaks",
-                       true, ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                       orte_process_info.nodename, 
-                       orte_process_info.pid, leak_msg, i,
+        opal_show_help("help-mpool-base.txt", "some mem leaks",
+                       true, OMPI_NAME_PRINT(OMPI_PROC_MY_NAME),
+                       ompi_process_info.nodename, 
+                       ompi_process_info.pid, leak_msg, i,
                        (i > 1) ? "s were" : " was",
                        (i > 1) ? "are" : "is");
     }
