@@ -11,6 +11,8 @@
  *                         All rights reserved.
  * Copyright (c) 2007      Sun Microsystems, Inc.  All rights reserved.
  * Copyright (c) 2009      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2010-2012 Los Alamos National Security, LLC.  
+ *                         All rights reserved. 
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -25,26 +27,29 @@
 
 #include "ompi_config.h"
 
-#include "opal/event/event.h"
+#include "opal/mca/event/event.h"
+#include "opal/mca/shmem/shmem.h"
 
-#include "ompi/mca/common/sm/common_sm_mmap.h"
+#include "ompi/mca/common/sm/common_sm.h"
 #include "ompi/mca/mpool/mpool.h"
 #include "ompi/mca/allocator/allocator.h"
 
 BEGIN_C_DECLS
 
 struct mca_mpool_sm_component_t {
-  mca_mpool_base_component_t super;
-  /*      mca_allocator_base_module_t* sm_allocator; */
-  char*  sm_allocator_name;
-  int    verbose;
-  /*      struct mca_mpool_sm_mmap_t *sm_mmap; */
+    mca_mpool_base_component_t super;
+    /* mca_allocator_base_module_t* sm_allocator; */
+    char *sm_allocator_name;
+    int verbose;
+    /* struct mca_mpool_sm_mmap_t *sm_mmap; */
 };
 typedef struct mca_mpool_sm_component_t mca_mpool_sm_component_t;
 
 typedef struct mca_mpool_base_resources_t {
     size_t  size;
     int32_t mem_node;
+    /* backing store metadata */
+    opal_shmem_ds_t bs_meta_buf;
 } mca_mpool_base_resources_t;
 
 OMPI_MODULE_DECLSPEC extern mca_mpool_sm_component_t mca_mpool_sm_component;
@@ -52,9 +57,9 @@ OMPI_MODULE_DECLSPEC extern mca_mpool_sm_component_t mca_mpool_sm_component;
 typedef struct mca_mpool_sm_module_t {
     mca_mpool_base_module_t super;
     long sm_size;
-    mca_allocator_base_module_t * sm_allocator;
+    mca_allocator_base_module_t *sm_allocator;
     struct mca_mpool_sm_mmap_t *sm_mmap;
-    mca_common_sm_mmap_t *sm_common_mmap;
+    mca_common_sm_module_t *sm_common_module;
     int32_t mem_node;
 } mca_mpool_sm_module_t;
 
