@@ -1,5 +1,6 @@
 /*
  * Copyright (C) Mellanox Technologies Ltd. 2001-2011.  ALL RIGHTS RESERVED.
+ * Copyright (c) 2013      Intel, Inc. All rights reserved
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -10,16 +11,13 @@
 
 #include "ompi_config.h"
 
-#include "orte/util/show_help.h"
-#include "orte/util/proc_info.h"
 #include "ompi/mca/mtl/mtl.h"
 #include "ompi/runtime/ompi_module_exchange.h"
 #include "ompi/mca/mtl/base/mtl_base_datatype.h"
 #include "ompi/proc/proc.h"
-#include "orte/mca/ess/ess.h"
-#include "orte/runtime/orte_globals.h"
 #include "ompi/communicator/communicator.h"
 #include "opal/memoryhooks/memory.h"
+#include "opal/util/show_help.h"
 
 #include "mtl_mxm.h"
 #include "mtl_mxm_types.h"
@@ -39,6 +37,8 @@ mca_mtl_mxm_module_t ompi_mtl_mxm = {
        ompi_mtl_mxm_isend,
        ompi_mtl_mxm_irecv,
        ompi_mtl_mxm_iprobe,
+       ompi_mtl_mxm_imrecv,
+       ompi_mtl_mxm_improbe,
        ompi_mtl_mxm_cancel,
        ompi_mtl_mxm_add_comm,
        ompi_mtl_mxm_del_comm
@@ -50,24 +50,23 @@ mca_mtl_mxm_module_t ompi_mtl_mxm = {
 };
 
 
+#if MXM_API < MXM_VERSION(2,0)
 static uint32_t ompi_mtl_mxm_get_job_id(void)
 {
     uint8_t  unique_job_key[16];
     uint32_t job_key;
     unsigned long long *uu;
     char *generated_key;
-    uint16_t *jkp;
 
-    jkp = (uint16_t *) unique_job_key;
     uu = (unsigned long long *) unique_job_key;
 
     generated_key = getenv("OMPI_MCA_orte_precondition_transports");
     memset(uu, 0, sizeof(unique_job_key));
 
     if (!generated_key || (strlen(generated_key) != 33) || sscanf(generated_key, "%016llx-%016llx", &uu[0], &uu[1]) != 2) {
-        orte_show_help("help-mtl-mxm.txt", "no uuid present", true,
+        opal_show_help("help-mtl-mxm.txt", "no uuid present", true,
                        generated_key ? "could not be parsed from" :
-                                       "not present in", orte_process_info.nodename);
+                                       "not present in", ompi_process_info.nodename);
         return 0;
     }
 
@@ -83,6 +82,7 @@ static uint32_t ompi_mtl_mxm_get_job_id(void)
 
     return job_key;
 }
+#endif
 
 int ompi_mtl_mxm_progress(void);
 #if MXM_API >= MXM_VERSION(2,0)
@@ -90,7 +90,7 @@ static void ompi_mtl_mxm_mem_release_cb(void *buf, size_t length,
                                         void *cbdata, bool from_alloc);
 #endif
 
-#if MXM_API < MXM_VERSION(2, 0)
+#if MXM_API < MXM_VERSION(2,0)
 static int ompi_mtl_mxm_get_ep_address(ompi_mtl_mxm_ep_conn_info_t *ep_info, mxm_ptl_id_t ptlid)
 {
     size_t addrlen;
@@ -100,7 +100,7 @@ static int ompi_mtl_mxm_get_ep_address(ompi_mtl_mxm_ep_conn_info_t *ep_info, mxm
     err = mxm_ep_address(ompi_mtl_mxm.ep, ptlid,
                          (struct sockaddr *) &ep_info->ptl_addr[ptlid], &addrlen);
     if (MXM_OK != err) {
-        orte_show_help("help-mtl-mxm.txt", "unable to extract endpoint ptl address",
+        opal_show_help("help-mtl-mxm.txt", "unable to extract endpoint ptl address",
                        true, (int)ptlid, mxm_error_string(err));
         return OMPI_ERROR;
     }
@@ -127,7 +127,7 @@ static int ompi_mtl_mxm_get_ep_address(void **address_p, size_t *address_len_p)
 
     err = mxm_ep_get_address(ompi_mtl_mxm.ep, *address_p, address_len_p);
     if (MXM_OK != err) {
-        orte_show_help("help-mtl-mxm.txt", "unable to extract endpoint address",
+        opal_show_help("help-mtl-mxm.txt", "unable to extract endpoint ptl address",
                        true, mxm_error_string(err));
         return OMPI_ERROR;
     }
@@ -144,64 +144,13 @@ ompi_mtl_mxm_create_ep(mxm_h ctx, mxm_ep_h *ep, unsigned ptl_bitmap, int lr,
 {
     mxm_error_t err;
 
-#if MXM_API < MXM_VERSION(1,5)
-    mxm_ep_opts_t ep_opt;
-    struct sockaddr_mxm_local_proc sa_bind_self;
-    struct sockaddr_mxm_ib_local sa_bind_rdma;
-    struct sockaddr_mxm_shm_proc sa_bind_shm;
-
-    mxm_fill_ep_opts(&ep_opt);
-
-    sa_bind_self.sa_family = AF_MXM_LOCAL_PROC;
-    sa_bind_self.context_id = lr;
-
-    sa_bind_rdma.sa_family = AF_MXM_IB_LOCAL;
-    sa_bind_rdma.lid = 0;
-    sa_bind_rdma.pkey = 0;
-    sa_bind_rdma.qp_num = 0;
-    sa_bind_rdma.sl = 0;
-
-    sa_bind_shm.sa_family = AF_MXM_SHM_PROC;
-    sa_bind_shm.jobid = jobid;
-    sa_bind_shm.process_id = lr;
-    sa_bind_shm.context_id = mxlr;
-    sa_bind_shm.num_procs = nlps;
-
-    ep_opt.ptl_bind_addr[MXM_PTL_SELF] =
-            (ptl_bitmap & MXM_BIT(MXM_PTL_SELF)) ?
-                    (struct sockaddr*) &sa_bind_self : NULL;
-    ep_opt.ptl_bind_addr[MXM_PTL_RDMA] =
-            (ptl_bitmap & MXM_BIT(MXM_PTL_RDMA)) ?
-                    (struct sockaddr*) &sa_bind_rdma : NULL;
-    ep_opt.ptl_bind_addr[MXM_PTL_SHM] =
-            (ptl_bitmap & MXM_BIT(MXM_PTL_SHM)) ?
-                    (struct sockaddr*) &sa_bind_shm : NULL;
-
-    MXM_VERBOSE(1, "MXM version is old, consider to upgrade");
-    err = mxm_ep_create(ctx, &ep_opt, ep);
-#elif MXM_API < MXM_VERSION(2,0)
-    mxm_ep_opts_t *ep_opts;
-    err = mxm_config_read_ep_opts(&ep_opts);
-    if (err != MXM_OK) {
-        MXM_ERROR("Failed to parse MXM configuration");
-        return err;
-    }
-
-    ep_opts->job_id          = jobid;
-    ep_opts->local_rank      = lr;
-    ep_opts->num_local_procs = nlps;
-    err = mxm_ep_create(ctx, ep_opts, ep);
-    mxm_config_free(ep_opts);
+#if MXM_API < MXM_VERSION(2,0)
+    ompi_mtl_mxm.mxm_ep_opts->job_id          = jobid;
+    ompi_mtl_mxm.mxm_ep_opts->local_rank      = lr;
+    ompi_mtl_mxm.mxm_ep_opts->num_local_procs = nlps;
+    err = mxm_ep_create(ctx, ompi_mtl_mxm.mxm_ep_opts, ep);
 #else
-    mxm_ep_opts_t *ep_opts;
-    err = mxm_config_read_ep_opts(&ep_opts);
-    if (err != MXM_OK) {
-        MXM_ERROR("Failed to parse MXM configuration");
-        return err;
-    }
-
-    err = mxm_ep_create(ctx, ep_opts, ep);
-    mxm_config_free_ep_opts(ep_opts);
+    err = mxm_ep_create(ctx, ompi_mtl_mxm.mxm_ep_opts, ep);
 #endif
     return err;
 }
@@ -301,6 +250,9 @@ static int ompi_mtl_mxm_recv_ep_address(ompi_proc_t *source_proc, void **address
                                     &modex_cur_size);
         if (OMPI_SUCCESS != rc) {
             MXM_ERROR("Open MPI couldn't distribute EP connection details");
+            free(*address_p);
+            *address_p = NULL;
+            *address_len_p = 0;
             goto bail;
         }
 
@@ -326,7 +278,7 @@ int ompi_mtl_mxm_module_init(void)
     mxm_error_t err;
     uint32_t jobid;
     uint64_t mxlr;
-    ompi_proc_t *mp, **procs;
+    ompi_proc_t **procs;
     unsigned ptl_bitmap;
     size_t totps, proc;
     int lr, nlps;
@@ -334,14 +286,15 @@ int ompi_mtl_mxm_module_init(void)
 
     mxlr = 0;
     lr = -1;
-    nlps = 0;
+    jobid = 0;
 
-    mp = ompi_proc_local();
+#if MXM_API < MXM_VERSION(2,0)
     jobid = ompi_mtl_mxm_get_job_id();
     if (0 == jobid) {
     	MXM_ERROR("Failed to generate jobid");
     	return OMPI_ERROR;
     }
+#endif
 
     if (NULL == (procs = ompi_proc_world(&totps))) {
         MXM_ERROR("Unable to obtain process list");
@@ -349,29 +302,31 @@ int ompi_mtl_mxm_module_init(void)
     }
 
     if (totps < (size_t)ompi_mtl_mxm.mxm_np) {
-        MXM_VERBOSE(1, "MXM support will be disabled because of total number of processes"
-                "(%lu) is less then default (%u)",totps, ompi_mtl_mxm.mxm_np);
-                return OMPI_ERR_NOT_SUPPORTED;
+        MXM_VERBOSE(1, "MXM support will be disabled because of total number "
+                    "of processes (%lu) is less than the minimum set by the "
+                    "mtl_mxm_np MCA parameter (%u)", totps, ompi_mtl_mxm.mxm_np);
+        free(procs);
+        return OMPI_ERR_NOT_SUPPORTED;
     }
     MXM_VERBOSE(1, "MXM support enabled");
 
-    if ((lr = orte_ess.get_node_rank(ORTE_PROC_MY_NAME)) == ORTE_NODE_RANK_INVALID) {
+    if (ORTE_NODE_RANK_INVALID == (lr = ompi_process_info.my_node_rank)) {
         MXM_ERROR("Unable to obtain local node rank");
+        free(procs);
         return OMPI_ERROR;
     }
+    nlps = ompi_process_info.num_local_peers + 1;
 
     for (proc = 0; proc < totps; proc++) {
-        if(OPAL_PROC_ON_LOCAL_NODE(orte_ess.proc_get_locality(&procs[proc]->proc_name))) {
+        if (OPAL_PROC_ON_LOCAL_NODE(procs[proc]->proc_flags)) {
             mxlr = max(mxlr, procs[proc]->proc_name.vpid);
-            nlps++;
         }
     }
+    free(procs);
 
     /* Setup the endpoint options and local addresses to bind to. */
-#if MXM_API < MXM_VERSION(1,5)
-    ptl_bitmap = ompi_mtl_mxm.mxm_opts.ptl_bitmap;
-#elif MXM_API < MXM_VERSION(2,0)
-    ptl_bitmap = ompi_mtl_mxm.mxm_opts->ptl_bitmap;
+#if MXM_API < MXM_VERSION(2,0)
+    ptl_bitmap = ompi_mtl_mxm.mxm_ctx_opts->ptl_bitmap;
 #else
     ptl_bitmap = 0;
 #endif
@@ -380,7 +335,7 @@ int ompi_mtl_mxm_module_init(void)
     err = ompi_mtl_mxm_create_ep(ompi_mtl_mxm.mxm_context, &ompi_mtl_mxm.ep,
                                  ptl_bitmap, lr, jobid, mxlr, nlps);
     if (MXM_OK != err) {
-        orte_show_help("help-mtl-mxm.txt", "unable to create endpoint", true,
+        opal_show_help("help-mtl-mxm.txt", "unable to create endpoint", true,
         		mxm_error_string(err));
         return OMPI_ERROR;
     }
@@ -445,19 +400,19 @@ int ompi_mtl_mxm_finalize(struct mca_mtl_base_module_t* mtl)
 }
 
 int ompi_mtl_mxm_add_procs(struct mca_mtl_base_module_t *mtl, size_t nprocs,
-                           struct ompi_proc_t** procs, /*const*/
-                           struct mca_mtl_base_endpoint_t **mtl_peer_data)
+                           struct ompi_proc_t** procs)
 {
 #if MXM_API < MXM_VERSION(2,0)
     ompi_mtl_mxm_ep_conn_info_t *ep_info;
     mxm_conn_req_t *conn_reqs;
-    int timeout;
+    size_t ep_index = 0;
 #endif
-    void *ep_address;
+    void *ep_address = NULL;
     size_t ep_address_len;
     mxm_error_t err;
     size_t i;
     int rc;
+    mca_mtl_mxm_endpoint_t *endpoint;
 
     assert(mtl == &ompi_mtl_mxm.super);
 
@@ -473,8 +428,12 @@ int ompi_mtl_mxm_add_procs(struct mca_mtl_base_module_t *mtl, size_t nprocs,
 
     /* Get the EP connection requests for all the processes from modex */
     for (i = 0; i < nprocs; ++i) {
+        if (NULL != procs[i]->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_MTL]) {
+           continue; /* already connected to this endpoint */
+        }
         rc = ompi_mtl_mxm_recv_ep_address(procs[i], &ep_address, &ep_address_len);
         if (rc != OMPI_SUCCESS) {
+            free(ep_address);
             goto bail;
         }
 
@@ -482,35 +441,41 @@ int ompi_mtl_mxm_add_procs(struct mca_mtl_base_module_t *mtl, size_t nprocs,
         if (ep_address_len != sizeof(ep_info[i])) {
             MXM_ERROR("Invalid endpoint address length");
             rc = OMPI_ERROR;
+            free(ep_address);
             goto bail;
         }
 
         memcpy(&ep_info[i], ep_address, ep_address_len);
-        conn_reqs[i].ptl_addr[MXM_PTL_SELF] = (struct sockaddr *)&(ep_info[i].ptl_addr[MXM_PTL_SELF]);
-        conn_reqs[i].ptl_addr[MXM_PTL_SHM]  = (struct sockaddr *)&(ep_info[i].ptl_addr[MXM_PTL_SHM]);
-        conn_reqs[i].ptl_addr[MXM_PTL_RDMA] = (struct sockaddr *)&(ep_info[i].ptl_addr[MXM_PTL_RDMA]);
+        conn_reqs[ep_index].ptl_addr[MXM_PTL_SELF] = (struct sockaddr *)&(ep_info[i].ptl_addr[MXM_PTL_SELF]);
+        conn_reqs[ep_index].ptl_addr[MXM_PTL_SHM]  = (struct sockaddr *)&(ep_info[i].ptl_addr[MXM_PTL_SHM]);
+        conn_reqs[ep_index].ptl_addr[MXM_PTL_RDMA] = (struct sockaddr *)&(ep_info[i].ptl_addr[MXM_PTL_RDMA]);
+        ep_index++;
+
 #else
-        mtl_peer_data[i] = (mca_mtl_mxm_endpoint_t *) OBJ_NEW(mca_mtl_mxm_endpoint_t);
-        mtl_peer_data[i]->mtl_mxm_module = &ompi_mtl_mxm;
-        err = mxm_ep_connect(ompi_mtl_mxm.ep, ep_address, &mtl_peer_data[i]->mxm_conn);
+        endpoint = OBJ_NEW(mca_mtl_mxm_endpoint_t);
+        endpoint->mtl_mxm_module = &ompi_mtl_mxm;
+        err = mxm_ep_connect(ompi_mtl_mxm.ep, ep_address, &endpoint->mxm_conn);
         if (err != MXM_OK) {
             MXM_ERROR("MXM returned connect error: %s\n", mxm_error_string(err));
             rc = OMPI_ERROR;
+            free(ep_address);
             goto bail;
         }
+        procs[i]->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_MTL] = endpoint;
 #endif
         free(ep_address);
     }
 
 #if MXM_API < MXM_VERSION(2,0)
     /* Connect to remote peers */
-    timeout = (mxm_get_version() < MXM_VERSION(1,5)) ? 1000 : -1;
-    err = mxm_ep_connect(ompi_mtl_mxm.ep, conn_reqs, nprocs, timeout);
+    err = mxm_ep_connect(ompi_mtl_mxm.ep, conn_reqs, ep_index, -1);
     if (MXM_OK != err) {
         MXM_ERROR("MXM returned connect error: %s\n", mxm_error_string(err));
-        for (i = 0; i < nprocs; ++i) {
+        for (i = 0; i < ep_index; ++i) {
             if (MXM_OK != conn_reqs[i].error) {
-                MXM_ERROR("MXM EP connect to %s error: %s\n", procs[i]->proc_hostname,
+                MXM_ERROR("MXM EP connect to %s error: %s\n",
+                          (NULL == procs[i]->proc_hostname) ?
+                          "unknown" : procs[i]->proc_hostname,
                           mxm_error_string(conn_reqs[i].error));
             }
         }
@@ -519,12 +484,21 @@ int ompi_mtl_mxm_add_procs(struct mca_mtl_base_module_t *mtl, size_t nprocs,
     }
 
     /* Save returned connections */
-    for (i = 0; i < nprocs; ++i) {
-        mtl_peer_data[i] = (mca_mtl_mxm_endpoint_t *) OBJ_NEW(mca_mtl_mxm_endpoint_t);
-        mtl_peer_data[i]->mtl_mxm_module = &ompi_mtl_mxm;
-        mtl_peer_data[i]->mxm_conn = conn_reqs[i].conn;
+    for (i = 0; i < ep_index; ++i) {
+        endpoint = OBJ_NEW(mca_mtl_mxm_endpoint_t);
+        endpoint->mtl_mxm_module = &ompi_mtl_mxm;
+        endpoint->mxm_conn = conn_reqs[i].conn;
+        procs[i]->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_MTL] = endpoint;
+    }
+
+#endif
+
+#if MXM_API >= MXM_VERSION(3,1)
+    if (ompi_mtl_mxm.bulk_connect) {
+        mxm_ep_wireup(ompi_mtl_mxm.ep);
     }
 #endif
+
     rc = OMPI_SUCCESS;
 
 bail:
@@ -536,14 +510,36 @@ bail:
 }
 
 int ompi_mtl_mxm_del_procs(struct mca_mtl_base_module_t *mtl, size_t nprocs,
-                           struct ompi_proc_t** procs,
-                           struct mca_mtl_base_endpoint_t **mtl_peer_data)
+                           struct ompi_proc_t** procs)
 {
     size_t i;
 
+    ompi_communicator_t *comm_world;
+
+    if (ompi_mpi_finalized) {
+        comm_world = &ompi_mpi_comm_world.comm;
+        if (ompi_comm_size(comm_world) > 1) {
+            comm_world->c_coll.coll_barrier(comm_world, comm_world->c_coll.coll_barrier_module);
+        }
+    }
+
+#if MXM_API >= MXM_VERSION(3,1)
+    if (ompi_mtl_mxm.bulk_disconnect &&
+            nprocs == ompi_comm_size(&ompi_mpi_comm_world.comm)) {
+        mxm_ep_powerdown(ompi_mtl_mxm.ep);
+    }
+#endif
+
+    /* XXX: Directly accessing the obj_reference_count is an abstraction
+     * violation of the object system. We know this needs to be fixed, but
+     * are deferring the fix to a later time as it involves a design issue
+     * in the way we handle endpoints as objects
+     */
     for (i = 0; i < nprocs; ++i) {
-        mxm_ep_disconnect(mtl_peer_data[i]->mxm_conn);
-        OBJ_RELEASE(mtl_peer_data[i]);
+        mca_mtl_mxm_endpoint_t *endpoint = (mca_mtl_mxm_endpoint_t*)
+            procs[i]->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_MTL];
+        mxm_ep_disconnect(endpoint->mxm_conn);
+        OBJ_RELEASE(endpoint);
     }
     return OMPI_SUCCESS;
 }
@@ -559,7 +555,7 @@ int ompi_mtl_mxm_add_comm(struct mca_mtl_base_module_t *mtl,
 
     err = mxm_mq_create(ompi_mtl_mxm.mxm_context, comm->c_contextid, &mq);
     if (MXM_OK != err) {
-        orte_show_help("help-mtl-mxm.txt", "mxm mq create", true, mxm_error_string(err));
+        opal_show_help("help-mtl-mxm.txt", "mxm mq create", true, mxm_error_string(err));
         return OMPI_ERROR;
     }
 
@@ -583,7 +579,7 @@ int ompi_mtl_mxm_progress(void)
 
     err = mxm_progress(ompi_mtl_mxm.mxm_context);
     if ((MXM_OK != err) && (MXM_ERR_NO_PROGRESS != err) ) {
-        orte_show_help("help-mtl-mxm.txt", "errors during mxm_progress", true, mxm_error_string(err));
+        opal_show_help("help-mtl-mxm.txt", "errors during mxm_progress", true, mxm_error_string(err));
     }
     return 1;
 }
@@ -596,3 +592,9 @@ static void ompi_mtl_mxm_mem_release_cb(void *buf, size_t length,
                   from_alloc ? MXM_MEM_UNMAP_MARK_INVALID : 0);
 }
 #endif
+
+OBJ_CLASS_INSTANCE(
+        ompi_mtl_mxm_message_t,
+        ompi_free_list_item_t,
+        NULL,
+        NULL);
