@@ -33,13 +33,11 @@ BEGIN_C_DECLS
 /* MTL interface functions */
 extern int ompi_mtl_psm_add_procs(struct mca_mtl_base_module_t* mtl, 
                           size_t nprocs,
-                          struct ompi_proc_t** procs, 
-                          struct mca_mtl_base_endpoint_t **mtl_peer_data);
+                          struct ompi_proc_t** procs);
     
 extern int ompi_mtl_psm_del_procs(struct mca_mtl_base_module_t* mtl, 
                                  size_t nprocs,
-                                 struct ompi_proc_t** procs, 
-                                 struct mca_mtl_base_endpoint_t **mtl_peer_data);
+                                 struct ompi_proc_t** procs);
 
 int
 ompi_mtl_psm_send(struct mca_mtl_base_module_t* mtl, 
@@ -73,9 +71,28 @@ extern int ompi_mtl_psm_iprobe(struct mca_mtl_base_module_t* mtl,
                               int *flag,
                               struct ompi_status_public_t *status);
 
+extern int ompi_mtl_psm_imrecv(struct mca_mtl_base_module_t* mtl,
+                               struct opal_convertor_t *convertor,
+                               struct ompi_message_t **message,
+                               struct mca_mtl_request_t *mtl_request);
+
+extern int ompi_mtl_psm_improbe(struct mca_mtl_base_module_t *mtl,
+                                struct ompi_communicator_t *comm,
+                                int src,
+                                int tag,
+                                int *matched,
+                                struct ompi_message_t **message,
+                                struct ompi_status_public_t *status);
+
 extern int ompi_mtl_psm_cancel(struct mca_mtl_base_module_t* mtl,
                               struct mca_mtl_request_t *mtl_request, 
                               int flag);
+
+extern int ompi_mtl_psm_add_comm(struct mca_mtl_base_module_t *mtl,
+                                 struct ompi_communicator_t *comm);
+
+extern int ompi_mtl_psm_del_comm(struct mca_mtl_base_module_t *mtl,
+                                 struct ompi_communicator_t *comm);
     
 extern int ompi_mtl_psm_finalize(struct mca_mtl_base_module_t* mtl);
 
