@@ -10,6 +10,8 @@
  * Copyright (c) 2004-2006 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2006      QLogic Corporation. All rights reserved.
+ * Copyright (c) 2011      Los Alamos National Security, LLC.
+ *                         All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -18,7 +20,7 @@
  */
 
 #ifndef MTL_PSM_TYPES_H_HAS_BEEN_INCLUDED
-#define MTL_PSM_TYPS_H_HAS_BEEN_INCLUDED
+#define MTL_PSM_TYPES_H_HAS_BEEN_INCLUDED
 
 #include "ompi_config.h"
 #include "mtl_psm.h"
@@ -40,15 +42,17 @@ struct mca_mtl_psm_module_t {
 
     int32_t      connect_timeout;
   
-    uint32_t     debug_level;
+    int32_t      debug_level;
     int32_t      ib_unit;
     int32_t      ib_port;
     int32_t      ib_service_level;
     uint64_t     ib_pkey;
   
 #if PSM_VERNO >= 0x010d
-    uint64_t     ib_service_id;
-    psm_path_res_t path_res_type;
+    unsigned long long ib_service_id;
+    /* use int instead of psm_path_res_t so we can register this with
+       the MCA variable system */
+    int          path_res_type;
 #endif
 
     psm_ep_t	 ep;
