@@ -19,7 +19,7 @@
 #ifndef _MCA_PML_BASE_BSEND_H_
 #define _MCA_PML_BASE_BSEND_H_
 
-#include "ompi/mca/pml/pml.h"
+#include "ompi_config.h"
 #include "ompi/request/request.h"
 
 BEGIN_C_DECLS
