@@ -1,6 +1,7 @@
 /* -*- Mode: C; c-basic-offset:4 ; -*- */
 /*
  * Copyright (c) 2009      Sun Microsystems, Inc. All rights reserved.
+ * Copyright (c) 2011      Sandia National Laboratories. All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -24,7 +25,9 @@ typedef enum {
     MCA_PML_REQUEST_SEND,
     MCA_PML_REQUEST_RECV,
     MCA_PML_REQUEST_IPROBE,
-    MCA_PML_REQUEST_PROBE
+    MCA_PML_REQUEST_PROBE,
+    MCA_PML_REQUEST_IMPROBE,
+    MCA_PML_REQUEST_MPROBE
 } mca_pml_base_request_type_t;
 
 #endif /* MCA_PML_BASE_REQUEST_DBG_H */
