@@ -1,8 +1,9 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2008 The University of Tennessee and The University
+ * Copyright (c) 2004-2013 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
@@ -10,7 +11,9 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008      UT-Battelle, LLC. All rights reserved.
- * Copyright (c) 2010      Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 2010-2012 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 2011-2012 Los Alamos National Security, LLC.
+ *                         All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -24,7 +27,6 @@
 #include "ompi/constants.h"
 #include "ompi/mca/pml/pml.h"
 #include "ompi/mca/btl/btl.h"
-#include "orte/mca/errmgr/errmgr.h"
 #include "ompi/mca/mpool/mpool.h" 
 #include "pml_bfo.h"
 #include "pml_bfo_hdr.h"
@@ -189,7 +191,7 @@ mca_pml_bfo_match_completion_free( struct mca_btl_base_module_t* btl,
 #else /* PML_BFO */
         /* TSW - FIX */
         opal_output(0, "%s:%d FATAL", __FILE__, __LINE__);
-        orte_errmgr.abort(-1, NULL);
+        ompi_rte_abort(-1, NULL);
 #endif /* PML_BFO */
     }
 #if PML_BFO
@@ -231,7 +233,7 @@ mca_pml_bfo_rndv_completion( mca_btl_base_module_t* btl,
 {
     mca_pml_bfo_send_request_t* sendreq = (mca_pml_bfo_send_request_t*)des->des_cbdata;
     mca_bml_base_btl_t* bml_btl = (mca_bml_base_btl_t*)des->des_context;
-    size_t req_bytes_delivered = 0;
+    size_t req_bytes_delivered;
 
     /* check completion status */
     if( OPAL_UNLIKELY(OMPI_SUCCESS != status) ) {
@@ -241,7 +243,7 @@ mca_pml_bfo_rndv_completion( mca_btl_base_module_t* btl,
 #else /* PML_BFO */
         /* TSW - FIX */
         opal_output(0, "%s:%d FATAL", __FILE__, __LINE__);
-        orte_errmgr.abort(-1, NULL);
+        ompi_rte_abort(-1, NULL);
 #endif /* PML_BFO */
     }
 #if PML_BFO
@@ -254,10 +256,10 @@ mca_pml_bfo_rndv_completion( mca_btl_base_module_t* btl,
      * happens in one thread, the increase of the req_bytes_delivered does not
      * have to be atomic.
      */
-    MCA_PML_BFO_COMPUTE_SEGMENT_LENGTH( des->des_src,
-                                        des->des_src_cnt,
-                                        sizeof(mca_pml_bfo_rendezvous_hdr_t),
-                                        req_bytes_delivered );
+    req_bytes_delivered = mca_pml_bfo_compute_segment_length (btl->btl_seg_size,
+                                                              (void *) des->des_src,
+                                                              des->des_src_cnt,
+                                                              sizeof(mca_pml_bfo_rendezvous_hdr_t));
 
 #if PML_BFO
     MCA_PML_BFO_CHECK_SENDREQ_EAGER_BML_BTL(bml_btl, btl, sendreq, "RNDV");
@@ -278,14 +280,15 @@ mca_pml_bfo_rget_completion( mca_btl_base_module_t* btl,
 {
     mca_pml_bfo_send_request_t* sendreq = (mca_pml_bfo_send_request_t*)des->des_cbdata;
     mca_bml_base_btl_t* bml_btl = (mca_bml_base_btl_t*)des->des_context;
-    size_t req_bytes_delivered = 0;
+    size_t req_bytes_delivered;
 #if PML_BFO
     MCA_PML_BFO_RGET_COMPLETION_SENDREQ_ERROR_CHECK(sendreq, btl, des);
 #endif /* PML_BFO */
 
     /* count bytes of user data actually delivered and check for request completion */
-    MCA_PML_BFO_COMPUTE_SEGMENT_LENGTH( des->des_src, des->des_src_cnt,
-                                        0, req_bytes_delivered );
+    req_bytes_delivered = mca_pml_bfo_compute_segment_length (btl->btl_seg_size,
+                                                              (void *) des->des_src,
+                                                              des->des_src_cnt, 0);
     OPAL_THREAD_ADD_SIZE_T(&sendreq->req_bytes_delivered, req_bytes_delivered);
 
     send_request_pml_complete_check(sendreq);
@@ -336,7 +339,7 @@ mca_pml_bfo_frag_completion( mca_btl_base_module_t* btl,
 {
     mca_pml_bfo_send_request_t* sendreq = (mca_pml_bfo_send_request_t*)des->des_cbdata;
     mca_bml_base_btl_t* bml_btl = (mca_bml_base_btl_t*) des->des_context;
-    size_t req_bytes_delivered = 0;
+    size_t req_bytes_delivered;
 #if PML_BFO
     sendreq->req_events--;
 #endif /* PML_BFO */
@@ -348,15 +351,15 @@ mca_pml_bfo_frag_completion( mca_btl_base_module_t* btl,
 #else /* PML_BFO */
         /* TSW - FIX */
         opal_output(0, "%s:%d FATAL", __FILE__, __LINE__);
-        orte_errmgr.abort(-1, NULL);
+        ompi_rte_abort(-1, NULL);
 #endif /* PML_BFO */
     }
 
     /* count bytes of user data actually delivered */
-    MCA_PML_BFO_COMPUTE_SEGMENT_LENGTH( des->des_src,
-                                        des->des_src_cnt,
-                                        sizeof(mca_pml_bfo_frag_hdr_t),
-                                        req_bytes_delivered );
+    req_bytes_delivered = mca_pml_bfo_compute_segment_length (btl->btl_seg_size,
+                                                              (void *) des->des_src,
+                                                              des->des_src_cnt,
+                                                              sizeof(mca_pml_bfo_frag_hdr_t));
 
     OPAL_THREAD_ADD_SIZE_T(&sendreq->req_pipeline_depth, -1);
     OPAL_THREAD_ADD_SIZE_T(&sendreq->req_bytes_delivered, req_bytes_delivered);
@@ -618,15 +621,13 @@ int mca_pml_bfo_send_request_start_copy( mca_pml_bfo_send_request_t* sendreq,
         }
         return OMPI_SUCCESS;
     }
-    switch(rc) {
-        case OMPI_ERR_RESOURCE_BUSY:
-            /* No more resources. Allow the upper level to queue the send */
-            rc = OMPI_ERR_OUT_OF_RESOURCE;
-            break;
-        default:
-            mca_bml_base_free(bml_btl, des);
-            break;
+    if (OMPI_ERR_RESOURCE_BUSY == rc) {
+        /* No more resources. Allow the upper level to queue the send */
+        rc = OMPI_ERR_OUT_OF_RESOURCE;
     }
+
+    mca_bml_base_free (bml_btl, des);
+
     return rc;
 }
 
@@ -703,17 +704,16 @@ int mca_pml_bfo_send_request_start_rdma( mca_pml_bfo_send_request_t* sendreq,
      * operation is achieved.
      */
 
-    mca_btl_base_descriptor_t* des;
+    mca_btl_base_descriptor_t* des, *src = NULL;
     mca_btl_base_segment_t* segment;
     mca_pml_bfo_hdr_t* hdr;
     bool need_local_cb = false;
     int rc;
 
     bml_btl = sendreq->req_rdma[0].bml_btl;
-    if((sendreq->req_rdma_cnt == 1) && (bml_btl->btl_flags & MCA_BTL_FLAGS_GET)) {
+    if((sendreq->req_rdma_cnt == 1) && (bml_btl->btl_flags & (MCA_BTL_FLAGS_GET | MCA_BTL_FLAGS_CUDA_GET))) {
         mca_mpool_base_registration_t* reg = sendreq->req_rdma[0].btl_reg;
-        mca_btl_base_descriptor_t* src;
-        size_t i;
+        size_t seg_size;
         size_t old_position = sendreq->req_send.req_base.req_convertor.bConverted;
 
         MEMCHECKER(
@@ -731,7 +731,7 @@ int mca_pml_bfo_send_request_start_rdma( mca_pml_bfo_send_request_t* sendreq,
                                   MCA_BTL_NO_ORDER,
                                   0,
                                   &size,
-                                  0,
+                                  MCA_BTL_DES_FLAGS_GET,
                                   &src );
         MEMCHECKER(
             memchecker_call(&opal_memchecker_base_mem_noaccess,
@@ -747,10 +747,11 @@ int mca_pml_bfo_send_request_start_rdma( mca_pml_bfo_send_request_t* sendreq,
         src->des_cbfunc = mca_pml_bfo_rget_completion;
         src->des_cbdata = sendreq;
 
+        seg_size = bml_btl->btl->btl_seg_size * src->des_src_cnt;
+
         /* allocate space for get hdr + segment list */
         mca_bml_base_alloc(bml_btl, &des, MCA_BTL_NO_ORDER,
-                           sizeof(mca_pml_bfo_rget_hdr_t) +
-                           (sizeof(mca_btl_base_segment_t) * (src->des_src_cnt-1)),
+                           sizeof(mca_pml_bfo_rget_hdr_t) + seg_size,
                            MCA_BTL_DES_FLAGS_PRIORITY | MCA_BTL_DES_FLAGS_BTL_OWNERSHIP);
         if( OPAL_UNLIKELY(NULL == des) ) {
             opal_convertor_set_position( &sendreq->req_send.req_base.req_convertor,
@@ -779,11 +780,8 @@ int mca_pml_bfo_send_request_start_rdma( mca_pml_bfo_send_request_t* sendreq,
         bfo_hdr_hton(hdr, MCA_PML_BFO_HDR_TYPE_RGET,
                      sendreq->req_send.req_base.req_proc);
 
-        for( i = 0; i < src->des_src_cnt; i++ ) {
-            hdr->hdr_rget.hdr_segs[i].seg_addr.lval = ompi_ptr_ptol(src->des_src[i].seg_addr.pval);
-            hdr->hdr_rget.hdr_segs[i].seg_len       = src->des_src[i].seg_len;
-            hdr->hdr_rget.hdr_segs[i].seg_key.key64 = src->des_src[i].seg_key.key64;
-        }
+        /* copy segment data */
+        memmove (&hdr->hdr_rget + 1, src->des_src, seg_size);
 
         des->des_cbfunc = mca_pml_bfo_send_ctl_completion;
 
@@ -852,12 +850,16 @@ int mca_pml_bfo_send_request_start_rdma( mca_pml_bfo_send_request_t* sendreq,
         if (MCA_PML_BFO_HDR_TYPE_RNDV == hdr->hdr_common.hdr_type) {
             if (des->des_flags & MCA_BTL_DES_SEND_ALWAYS_CALLBACK) {
                 sendreq->req_events++;
-	    }
+            }
         }
 #endif /* PML_BFO */
         return OMPI_SUCCESS;
     }
     mca_bml_base_free(bml_btl, des);
+    if (NULL != src) {
+        mca_bml_base_free (bml_btl, src);
+    }    
+
     return rc;
 }
 
@@ -961,13 +963,13 @@ void mca_pml_bfo_send_request_copy_in_out( mca_pml_bfo_send_request_t *sendreq,
     ompi_free_list_item_t *i;
     mca_bml_base_endpoint_t* bml_endpoint = sendreq->req_endpoint;
     int num_btls = mca_bml_base_btl_array_get_size(&bml_endpoint->btl_send);
-    int rc, n;
+    int n;
     double weight_total = 0;
 
     if( OPAL_UNLIKELY(0 == send_length) )
         return;
 
-    OMPI_FREE_LIST_WAIT(&mca_pml_bfo.send_ranges, i, rc);
+    OMPI_FREE_LIST_WAIT_MT(&mca_pml_bfo.send_ranges, i);
 
     sr = (mca_pml_bfo_send_range_t*)i;
 
@@ -1021,7 +1023,7 @@ get_next_send_range(mca_pml_bfo_send_request_t* sendreq,
 {
     OPAL_THREAD_LOCK(&sendreq->req_send_range_lock);
     opal_list_remove_item(&sendreq->req_send_ranges, (opal_list_item_t *)range);
-    OMPI_FREE_LIST_RETURN(&mca_pml_bfo.send_ranges, &range->base);
+    OMPI_FREE_LIST_RETURN_MT(&mca_pml_bfo.send_ranges, &range->base);
     range = get_send_range_nolock(sendreq);
     OPAL_THREAD_UNLOCK(&sendreq->req_send_range_lock);
 
@@ -1060,7 +1062,7 @@ mca_pml_bfo_send_request_schedule_once(mca_pml_bfo_send_request_t* sendreq)
 
         assert(range->range_send_length != 0);
 #if PML_BFO
-	MCA_PML_BFO_CHECK_FOR_REMOVED_BTL(sendreq, range);
+        MCA_PML_BFO_CHECK_FOR_REMOVED_BTL(sendreq, range);
 #endif /* PML_BFO */
 
         if(prev_bytes_remaining == range->range_send_length)
@@ -1205,8 +1207,8 @@ static void mca_pml_bfo_put_completion( mca_btl_base_module_t* btl,
         sendreq->req_error++;
 #else /* PML_BFO */
         /* TSW - FIX */
-        ORTE_ERROR_LOG(status);
-        orte_errmgr.abort(-1, NULL);
+        OMPI_ERROR_LOG(status);
+        ompi_rte_abort(-1, NULL);
 #endif /* PML_BFO */
     }
 #if PML_BFO
@@ -1251,7 +1253,8 @@ int mca_pml_bfo_send_request_put_frag( mca_pml_bfo_rdma_frag_t* frag )
                               MCA_BTL_NO_ORDER,
                               0,
                               &frag->rdma_length,
-                              MCA_BTL_DES_FLAGS_BTL_OWNERSHIP,
+                              MCA_BTL_DES_FLAGS_BTL_OWNERSHIP |
+                              MCA_BTL_DES_FLAGS_PUT,
                               &des );
     
     if( OPAL_UNLIKELY(NULL == des) ) {
@@ -1287,8 +1290,8 @@ int mca_pml_bfo_send_request_put_frag( mca_pml_bfo_rdma_frag_t* frag )
         }
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
-    
-    des->des_dst = frag->rdma_segs;
+
+    des->des_dst = (mca_btl_base_segment_t *) frag->rdma_segs;
     des->des_dst_cnt = frag->rdma_hdr.hdr_rdma.hdr_seg_cnt;
     des->des_cbfunc = mca_pml_bfo_put_completion;
     des->des_cbdata = frag;
@@ -1307,13 +1310,13 @@ int mca_pml_bfo_send_request_put_frag( mca_pml_bfo_rdma_frag_t* frag )
             return OMPI_ERR_OUT_OF_RESOURCE;
         } else {
             /* TSW - FIX */
-            ORTE_ERROR_LOG(rc);
-            orte_errmgr.abort(-1, NULL);
+            OMPI_ERROR_LOG(rc);
+            ompi_rte_abort(-1, NULL);
         }
     }
 #if PML_BFO
     if (des->des_flags & MCA_BTL_DES_SEND_ALWAYS_CALLBACK) {
-	((mca_pml_bfo_send_request_t*)frag->rdma_req)->req_events++;
+        ((mca_pml_bfo_send_request_t*)frag->rdma_req)->req_events++;
     }
 #endif /* PML_BFO */
     return OMPI_SUCCESS;
@@ -1332,7 +1335,6 @@ void mca_pml_bfo_send_request_put( mca_pml_bfo_send_request_t* sendreq,
 {
     mca_bml_base_endpoint_t *bml_endpoint = sendreq->req_endpoint;
     mca_pml_bfo_rdma_frag_t* frag;
-    int rc;
     size_t i, size = 0;
 
     if(hdr->hdr_common.hdr_flags & MCA_PML_BFO_HDR_TYPE_ACK) { 
@@ -1343,28 +1345,29 @@ void mca_pml_bfo_send_request_put( mca_pml_bfo_send_request_t* sendreq,
     sendreq->req_recv = hdr->hdr_dst_req; /* only needed once, but it is OK */
 #endif /* PML_BFO */
 
-    MCA_PML_BFO_RDMA_FRAG_ALLOC(frag, rc); 
-
+    MCA_PML_BFO_RDMA_FRAG_ALLOC(frag); 
     if( OPAL_UNLIKELY(NULL == frag) ) {
         /* TSW - FIX */
-        ORTE_ERROR_LOG(rc);
-        orte_errmgr.abort(-1, NULL);
+        OMPI_ERROR_LOG(OMPI_ERR_OUT_OF_RESOURCE);
+        ompi_rte_abort(-1, NULL);
     }
 
+    assert (btl->btl_seg_size * hdr->hdr_seg_cnt <= sizeof (frag->rdma_segs));
+
     /* setup fragment */
+    memmove (frag->rdma_segs, hdr + 1, btl->btl_seg_size * hdr->hdr_seg_cnt);
+
     for( i = 0; i < hdr->hdr_seg_cnt; i++ ) {
-        frag->rdma_segs[i].seg_addr.lval = hdr->hdr_segs[i].seg_addr.lval;
-        frag->rdma_segs[i].seg_len       = hdr->hdr_segs[i].seg_len;
-        frag->rdma_segs[i].seg_key.key64 = hdr->hdr_segs[i].seg_key.key64;
+        mca_btl_base_segment_t *seg = (mca_btl_base_segment_t *) ((uintptr_t)(frag->rdma_segs) + i * btl->btl_seg_size);
 
 #if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
         if ((sendreq->req_send.req_base.req_proc->proc_arch & OPAL_ARCH_ISBIGENDIAN) !=
             (ompi_proc_local()->proc_arch & OPAL_ARCH_ISBIGENDIAN)) {
-            size += opal_swap_bytes4(frag->rdma_segs[i].seg_len);
+            size += opal_swap_bytes4(seg->seg_len);
         } else 
 #endif
         {
-            size += frag->rdma_segs[i].seg_len;
+            size += seg->seg_len;
         }
     }
 
