@@ -2,14 +2,16 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2010 The University of Tennessee and The University
+ * Copyright (c) 2004-2013 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2009-2010 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 2009-2012 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 2011-2012 Los Alamos National Security, LLC.
+ *                         All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -120,16 +122,14 @@ get_request_from_send_pending(mca_pml_bfo_send_pending_t *type)
 
 #define MCA_PML_BFO_SEND_REQUEST_ALLOC( comm,                           \
                                         dst,                            \
-                                        sendreq,                        \
-                                        rc)                             \
+                                        sendreq)                        \
     {                                                                   \
         ompi_proc_t *proc = ompi_comm_peer_lookup( comm, dst );         \
         ompi_free_list_item_t* item;                                    \
                                                                         \
-        rc = OMPI_ERR_OUT_OF_RESOURCE;                                  \
+        sendreq = NULL;                                                 \
         if( OPAL_LIKELY(NULL != proc) ) {                               \
-            rc = OMPI_SUCCESS;                                          \
-            OMPI_FREE_LIST_WAIT(&mca_pml_base_send_requests, item, rc); \
+            OMPI_FREE_LIST_WAIT_MT(&mca_pml_base_send_requests, item);     \
             sendreq = (mca_pml_bfo_send_request_t*)item;                \
             sendreq->req_send.req_base.req_proc = proc;                 \
         }                                                               \
@@ -212,7 +212,7 @@ do {
     do {                                                                \
     /*  Let the base handle the reference counts */                     \
     MCA_PML_BASE_SEND_REQUEST_FINI((&(sendreq)->req_send));             \
-    OMPI_FREE_LIST_RETURN( &mca_pml_base_send_requests,                 \
+    OMPI_FREE_LIST_RETURN_MT( &mca_pml_base_send_requests,                 \
                            (ompi_free_list_item_t*)sendreq);            \
     } while(0)
 
@@ -247,6 +247,10 @@ send_request_pml_complete(mca_pml_bfo_send_request_t *sendreq)
     if(false == sendreq->req_send.req_base.req_ompi.req_complete) {
         /* Should only be called for long messages (maybe synchronous) */
         MCA_PML_BFO_SEND_REQUEST_MPI_COMPLETE(sendreq, true);
+    } else {
+        if( MPI_SUCCESS != sendreq->req_send.req_base.req_ompi.req_status.MPI_ERROR ) {
+            ompi_mpi_abort(&ompi_mpi_comm_world.comm, MPI_ERR_REQUEST, true);
+        }
     }
     sendreq->req_send.req_base.req_pml_complete = true;
 #if PML_BFO
@@ -263,7 +267,7 @@ send_request_pml_complete(mca_pml_bfo_send_request_t *sendreq)
 static inline bool
 send_request_pml_complete_check(mca_pml_bfo_send_request_t *sendreq)
 {
-#if OPAL_HAVE_THREAD_SUPPORT
+#if OPAL_ENABLE_MULTI_THREADS
     opal_atomic_rmb();
 #endif
     /* if no more events are expected for the request and the whole message is
@@ -319,6 +323,13 @@ mca_pml_bfo_send_request_schedule(mca_pml_bfo_send_request_t* sendreq)
     mca_pml_bfo_send_request_schedule_exclusive(sendreq);
 }
 
+#if OPAL_CUDA_SUPPORT
+int mca_pml_bfo_send_request_start_cuda(
+    mca_pml_bfo_send_request_t* sendreq, 
+    mca_bml_base_btl_t* bml_btl,
+    size_t size);
+#endif /* OPAL_CUDA_SUPPORT */
+
 /**
  *  Start the specified request
  */
@@ -403,6 +414,11 @@ mca_pml_bfo_send_request_start_btl( mca_pml_bfo_send_request_t* sendreq,
                                                          MCA_PML_BFO_HDR_FLAGS_CONTIG);
             }
         } else {
+#if OPAL_CUDA_SUPPORT
+            if (sendreq->req_send.req_base.req_convertor.flags & CONVERTOR_CUDA) {
+                return mca_pml_bfo_send_request_start_cuda(sendreq, bml_btl, size);
+            }
+#endif /* OPAL_CUDA_SUPPORT */
             rc = mca_pml_bfo_send_request_start_rndv(sendreq, bml_btl, size, 0);
         }
     }
@@ -415,7 +431,7 @@ mca_pml_bfo_send_request_start( mca_pml_bfo_send_request_t* sendreq )
 {   
     mca_pml_bfo_comm_t* comm = sendreq->req_send.req_base.req_comm->c_pml_comm;
     mca_bml_base_endpoint_t* endpoint = (mca_bml_base_endpoint_t*)
-                                        sendreq->req_send.req_base.req_proc->proc_bml;
+                                        sendreq->req_send.req_base.req_proc->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_BML];
     size_t i;
 
     if( OPAL_UNLIKELY(endpoint == NULL) ) {
