@@ -2,13 +2,14 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2007 The University of Tennessee and The University
+ * Copyright (c) 2004-2013 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2006 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2012      Sandia National Laboratories.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -51,20 +52,20 @@ OBJ_CLASS_DECLARATION(mca_pml_cm_hvy_recv_request_t);
  *  @param rc (OUT)  OMPI_SUCCESS or error status on failure.
  *  @return          Receive request.
  */
-#define MCA_PML_CM_THIN_RECV_REQUEST_ALLOC(recvreq, rc)                        \
+#define MCA_PML_CM_THIN_RECV_REQUEST_ALLOC(recvreq)                            \
     do {                                                                       \
     ompi_free_list_item_t*item;                                                \
-    OMPI_FREE_LIST_GET(&mca_pml_base_recv_requests, item, rc);                 \
+    OMPI_FREE_LIST_GET_MT(&mca_pml_base_recv_requests, item);                     \
     recvreq = (mca_pml_cm_thin_recv_request_t*) item;                          \
     recvreq->req_base.req_pml_type = MCA_PML_CM_REQUEST_RECV_THIN;             \
     recvreq->req_mtl.ompi_req = (ompi_request_t*) recvreq;                     \
     recvreq->req_mtl.completion_callback = mca_pml_cm_recv_request_completion; \
  } while (0)
 
-#define MCA_PML_CM_HVY_RECV_REQUEST_ALLOC(recvreq, rc)                         \
+#define MCA_PML_CM_HVY_RECV_REQUEST_ALLOC(recvreq)                             \
 do {                                                                           \
     ompi_free_list_item_t*item;                                                \
-    OMPI_FREE_LIST_GET(&mca_pml_base_recv_requests, item, rc);                 \
+    OMPI_FREE_LIST_GET_MT(&mca_pml_base_recv_requests, item);                     \
     recvreq = (mca_pml_cm_hvy_recv_request_t*) item;                           \
     recvreq->req_base.req_pml_type = MCA_PML_CM_REQUEST_RECV_HEAVY;            \
     recvreq->req_mtl.ompi_req = (ompi_request_t*) recvreq;                     \
@@ -80,14 +81,13 @@ do {                                                                           \
  * @param count (IN)         Number of elements of indicated datatype.
  * @param datatype (IN)      User defined datatype.
  * @param src (IN)           Source rank w/in the communicator.
- * @param tag (IN)           User defined tag.
  * @param comm (IN)          Communicator.
  * @param persistent (IN)    Is this a ersistent request.
  */
+#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
 #define MCA_PML_CM_THIN_RECV_REQUEST_INIT( request,                     \
                                            ompi_proc,                   \
                                            comm,                        \
-                                           tag,                         \
                                            src,                         \
                                            datatype,                    \
                                            addr,                        \
@@ -115,7 +115,35 @@ do {                                                                    \
                                   0,                                    \
                                   &(request)->req_base.req_convertor ); \
 } while(0)
+#else
+#define MCA_PML_CM_THIN_RECV_REQUEST_INIT( request,                     \
+                                           ompi_proc,                   \
+                                           comm,                        \
+                                           src,                         \
+                                           datatype,                    \
+                                           addr,                        \
+                                           count )                      \
+do {                                                                    \
+    OMPI_REQUEST_INIT(&(request)->req_base.req_ompi, false);            \
+    (request)->req_base.req_ompi.req_mpi_object.comm = comm;            \
+    (request)->req_base.req_pml_complete = false;                       \
+    (request)->req_base.req_free_called = false;                        \
+    request->req_base.req_comm = comm;                                  \
+    request->req_base.req_datatype = datatype;                          \
+    OBJ_RETAIN(comm);                                                   \
+    OBJ_RETAIN(datatype);                                               \
+                                                                        \
+    opal_convertor_copy_and_prepare_for_recv(                           \
+        ompi_mpi_local_convertor,                                       \
+        &(datatype->super),                                             \
+        count,                                                          \
+        addr,                                                           \
+        0,                                                              \
+        &(request)->req_base.req_convertor );                           \
+} while(0)
+#endif
 
+#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
 #define MCA_PML_CM_HVY_RECV_REQUEST_INIT( request,                      \
                                           ompi_proc,                    \
                                           comm,                         \
@@ -152,7 +180,39 @@ do {                                                                    \
                                   0,                                    \
                                   &(request)->req_base.req_convertor ); \
  } while(0)
-
+#else
+#define MCA_PML_CM_HVY_RECV_REQUEST_INIT( request,                      \
+                                          ompi_proc,                    \
+                                          comm,                         \
+                                          tag,                          \
+                                          src,                          \
+                                          datatype,                     \
+                                          addr,                         \
+                                          count,                        \
+                                          persistent)                   \
+do {                                                                    \
+    OMPI_REQUEST_INIT(&(request)->req_base.req_ompi, persistent);       \
+    (request)->req_base.req_ompi.req_mpi_object.comm = comm;            \
+    (request)->req_base.req_pml_complete = OPAL_INT_TO_BOOL(persistent); \
+    (request)->req_base.req_free_called = false;                        \
+    request->req_base.req_comm = comm;                                  \
+    request->req_base.req_datatype = datatype;                          \
+    request->req_tag = tag;                                             \
+    request->req_peer = src;                                            \
+    request->req_addr = addr;                                           \
+    request->req_count = count;                                         \
+    OBJ_RETAIN(comm);                                                   \
+    OBJ_RETAIN(datatype);                                               \
+                                                                        \
+    opal_convertor_copy_and_prepare_for_recv(                           \
+        ompi_mpi_local_convertor,                                       \
+        &(datatype->super),                                             \
+        count,                                                          \
+        addr,                                                           \
+        0,                                                              \
+        &(request)->req_base.req_convertor );                           \
+ } while(0)
+#endif
 
 /**
  * Start an initialized request.
@@ -182,6 +242,26 @@ do {                                                                    \
                               &recvreq->req_mtl));                      \
 } while (0)
 
+#define MCA_PML_CM_THIN_RECV_REQUEST_MATCHED_START(request, message, ret) \
+do {                                                                    \
+    /* init/re-init the request */                                      \
+    request->req_base.req_pml_complete = false;                         \
+    request->req_base.req_ompi.req_complete = false;                    \
+    request->req_base.req_ompi.req_state = OMPI_REQUEST_ACTIVE;         \
+                                                                        \
+    /* always set the req_status.MPI_TAG to ANY_TAG before starting the \
+     * request. This field is used if cancelled to find out if the request \
+     * has been matched or not.                                         \
+     */                                                                 \
+    request->req_base.req_ompi.req_status.MPI_TAG = OMPI_ANY_TAG;       \
+    request->req_base.req_ompi.req_status.MPI_ERROR = OMPI_SUCCESS;     \
+    request->req_base.req_ompi.req_status._cancelled = 0;               \
+    ret = OMPI_MTL_CALL(imrecv(ompi_mtl,                                \
+                               &recvreq->req_base.req_convertor,        \
+                               message,                                 \
+                               &recvreq->req_mtl));                     \
+} while (0)
+
 
 #define MCA_PML_CM_HVY_RECV_REQUEST_START(request, ret)                 \
 do {                                                                    \
@@ -277,7 +357,7 @@ do {                                                                    \
     OBJ_RELEASE((recvreq)->req_base.req_datatype);                      \
     OMPI_REQUEST_FINI(&(recvreq)->req_base.req_ompi);                   \
     opal_convertor_cleanup( &((recvreq)->req_base.req_convertor) );     \
-    OMPI_FREE_LIST_RETURN( &mca_pml_base_recv_requests,                 \
+    OMPI_FREE_LIST_RETURN_MT( &mca_pml_base_recv_requests,                 \
                            (ompi_free_list_item_t*)(recvreq));          \
 }
 
@@ -290,7 +370,7 @@ do {                                                                    \
     OBJ_RELEASE((recvreq)->req_base.req_datatype);                      \
     OMPI_REQUEST_FINI(&(recvreq)->req_base.req_ompi);                   \
     opal_convertor_cleanup( &((recvreq)->req_base.req_convertor) );     \
-    OMPI_FREE_LIST_RETURN( &mca_pml_base_recv_requests,                 \
+    OMPI_FREE_LIST_RETURN_MT( &mca_pml_base_recv_requests,                 \
                            (ompi_free_list_item_t*)(recvreq));          \
 }
 
