@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2010 The University of Tennessee and The University
+ * Copyright (c) 2004-2013 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -56,47 +56,127 @@ typedef struct mca_pml_cm_hvy_send_request_t mca_pml_cm_hvy_send_request_t;
 OBJ_CLASS_DECLARATION(mca_pml_cm_hvy_send_request_t);
 
 
+#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
 #define MCA_PML_CM_THIN_SEND_REQUEST_ALLOC(sendreq, comm, dst,          \
-                                           ompi_proc, rc)               \
+                                           ompi_proc)                   \
 do {                                                                    \
     ompi_free_list_item_t* item;                                        \
     ompi_proc = ompi_comm_peer_lookup( comm, dst );                     \
                                                                         \
     if(OPAL_UNLIKELY(NULL == ompi_proc)) {                              \
-        rc = OMPI_ERR_OUT_OF_RESOURCE;                                  \
         sendreq = NULL;                                                 \
     } else {                                                            \
-        rc = OMPI_SUCCESS;                                              \
-        OMPI_FREE_LIST_WAIT(&mca_pml_base_send_requests,                \
-                            item, rc);                                  \
+        OMPI_FREE_LIST_WAIT_MT(&mca_pml_base_send_requests, item);         \
         sendreq = (mca_pml_cm_thin_send_request_t*)item;                \
         sendreq->req_send.req_base.req_pml_type = MCA_PML_CM_REQUEST_SEND_THIN; \
         sendreq->req_mtl.ompi_req = (ompi_request_t*) sendreq;          \
         sendreq->req_mtl.completion_callback = mca_pml_cm_send_request_completion; \
     }                                                                   \
 } while(0)
+#else
+#define MCA_PML_CM_THIN_SEND_REQUEST_ALLOC(sendreq, comm, dst,          \
+                                           ompi_proc)                   \
+do {                                                                    \
+    ompi_free_list_item_t* item;                                        \
+    OMPI_FREE_LIST_WAIT_MT(&mca_pml_base_send_requests, item);		\
+    sendreq = (mca_pml_cm_thin_send_request_t*)item;			\
+    sendreq->req_send.req_base.req_pml_type = MCA_PML_CM_REQUEST_SEND_THIN; \
+    sendreq->req_mtl.ompi_req = (ompi_request_t*) sendreq;              \
+    sendreq->req_mtl.completion_callback = mca_pml_cm_send_request_completion; \
+} while(0)
+#endif
 
 
+#if (OPAL_ENABLE_HETEROGENEOUS_SUPPORT)
 #define MCA_PML_CM_HVY_SEND_REQUEST_ALLOC(sendreq, comm, dst,           \
-                                          ompi_proc, rc)                \
+                                          ompi_proc)                    \
 {                                                                       \
     ompi_free_list_item_t* item;                                        \
     ompi_proc = ompi_comm_peer_lookup( comm, dst );                     \
     if(OPAL_UNLIKELY(NULL == ompi_proc)) {                              \
-        rc = OMPI_ERR_OUT_OF_RESOURCE;                                  \
         sendreq = NULL;                                                 \
     } else {                                                            \
-        rc = OMPI_SUCCESS;                                              \
-        OMPI_FREE_LIST_WAIT(&mca_pml_base_send_requests,                \
-                            item, rc);                                  \
+        OMPI_FREE_LIST_WAIT_MT(&mca_pml_base_send_requests, item);         \
         sendreq = (mca_pml_cm_hvy_send_request_t*)item;                 \
         sendreq->req_send.req_base.req_pml_type = MCA_PML_CM_REQUEST_SEND_HEAVY; \
         sendreq->req_mtl.ompi_req = (ompi_request_t*) sendreq;          \
         sendreq->req_mtl.completion_callback = mca_pml_cm_send_request_completion; \
     }                                                                   \
 }
+#else
+#define MCA_PML_CM_HVY_SEND_REQUEST_ALLOC(sendreq, comm, dst,           \
+                                          ompi_proc)                    \
+{                                                                       \
+    ompi_free_list_item_t* item;                                        \
+    OMPI_FREE_LIST_WAIT_MT(&mca_pml_base_send_requests, item);		\
+    sendreq = (mca_pml_cm_hvy_send_request_t*) item;			\
+    sendreq->req_send.req_base.req_pml_type = MCA_PML_CM_REQUEST_SEND_HEAVY; \
+    sendreq->req_mtl.ompi_req = (ompi_request_t*) sendreq;              \
+    sendreq->req_mtl.completion_callback = mca_pml_cm_send_request_completion; \
+}
+#endif
 
+#if (OPAL_ENABLE_HETEROGENEOUS_SUPPORT)
+#define MCA_PML_CM_HVY_SEND_REQUEST_INIT_COMMON(req_send,               \
+                                            ompi_proc,                  \
+                                            comm,                       \
+                                            tag,                        \
+                                            datatype,                   \
+                                            sendmode,                   \
+                                            buf,                        \
+                                            count)                      \
+{                                                                       \
+    OBJ_RETAIN(comm);                                                   \
+    OBJ_RETAIN(datatype);                                               \
+    (req_send)->req_base.req_comm = comm;                               \
+    (req_send)->req_base.req_datatype = datatype;                       \
+    opal_convertor_copy_and_prepare_for_send(                           \
+                                             ompi_proc->proc_convertor, \
+                                             &(datatype->super),        \
+                                             count,                     \
+                                             buf,                       \
+                                             0,                         \
+                                             &(req_send)->req_base.req_convertor ); \
+    (req_send)->req_base.req_ompi.req_mpi_object.comm = comm;           \
+    (req_send)->req_base.req_ompi.req_status.MPI_SOURCE =               \
+        comm->c_my_rank;                                                \
+    (req_send)->req_base.req_ompi.req_status.MPI_TAG = tag;             \
+    (req_send)->req_base.req_ompi.req_status._ucount = count;           \
+    (req_send)->req_send_mode = sendmode;                               \
+    (req_send)->req_base.req_free_called = false;                       \
+}
+#else
+#define MCA_PML_CM_HVY_SEND_REQUEST_INIT_COMMON(req_send,               \
+                                            ompi_proc,                  \
+                                            comm,                       \
+                                            tag,                        \
+                                            datatype,                   \
+                                            sendmode,                   \
+                                            buf,                        \
+                                            count)                      \
+{                                                                       \
+    OBJ_RETAIN(comm);                                                   \
+    OBJ_RETAIN(datatype);                                               \
+    (req_send)->req_base.req_comm = comm;                               \
+    (req_send)->req_base.req_datatype = datatype;                       \
+    opal_convertor_copy_and_prepare_for_send(                           \
+        ompi_mpi_local_convertor,                                       \
+        &(datatype->super),                                             \
+        count,                                                          \
+        buf,                                                            \
+        0,                                                              \
+        &(req_send)->req_base.req_convertor );                          \
+    (req_send)->req_base.req_ompi.req_mpi_object.comm = comm;           \
+    (req_send)->req_base.req_ompi.req_status.MPI_SOURCE =               \
+        comm->c_my_rank;                                                \
+    (req_send)->req_base.req_ompi.req_status.MPI_TAG = tag;             \
+    (req_send)->req_base.req_ompi.req_status._ucount = count;           \
+    (req_send)->req_send_mode = sendmode;                               \
+    (req_send)->req_base.req_free_called = false;                       \
+}
+#endif
 
+#if (OPAL_ENABLE_HETEROGENEOUS_SUPPORT)
 #define MCA_PML_CM_SEND_REQUEST_INIT_COMMON(req_send,                   \
                                             ompi_proc,                  \
                                             comm,                       \
@@ -126,6 +206,52 @@ do {                                                                    \
     (req_send)->req_base.req_free_called = false;                       \
 }
 
+#else
+#define MCA_PML_CM_SEND_REQUEST_INIT_COMMON(req_send,                   \
+                                            ompi_proc,                  \
+                                            comm,                       \
+                                            tag,                        \
+                                            datatype,                   \
+                                            sendmode,                   \
+                                            buf,                        \
+                                            count)                      \
+{                                                                       \
+    OBJ_RETAIN(comm);                                                   \
+    OBJ_RETAIN(datatype);                                               \
+    (req_send)->req_base.req_comm = comm;                               \
+    (req_send)->req_base.req_datatype = datatype;                       \
+    if (opal_datatype_is_contiguous_memory_layout(&datatype->super, count)) { \
+        (req_send)->req_base.req_convertor.remoteArch =                 \
+            ompi_mpi_local_convertor->remoteArch;                       \
+        (req_send)->req_base.req_convertor.flags      =                 \
+            ompi_mpi_local_convertor->flags;                            \
+        (req_send)->req_base.req_convertor.master     =                 \
+            ompi_mpi_local_convertor->master;                           \
+        (req_send)->req_base.req_convertor.local_size =                 \
+            count * datatype->super.size;                               \
+        (req_send)->req_base.req_convertor.pBaseBuf   =                 \
+            (unsigned char*)buf + datatype->super.true_lb;              \
+        (req_send)->req_base.req_convertor.count      = count;          \
+        (req_send)->req_base.req_convertor.pDesc      = &datatype->super; \
+    } else {                                                            \
+        opal_convertor_copy_and_prepare_for_send(                       \
+            ompi_mpi_local_convertor,                                   \
+            &(datatype->super),                                         \
+            count,                                                      \
+            buf,                                                        \
+            0,                                                          \
+            &(req_send)->req_base.req_convertor );                      \
+    }                                                                   \
+    (req_send)->req_base.req_ompi.req_mpi_object.comm = comm;           \
+    (req_send)->req_base.req_ompi.req_status.MPI_SOURCE =               \
+        comm->c_my_rank;                                                \
+    (req_send)->req_base.req_ompi.req_status.MPI_TAG = tag;             \
+    (req_send)->req_base.req_ompi.req_status._ucount = count;           \
+    (req_send)->req_send_mode = sendmode;                               \
+    (req_send)->req_base.req_free_called = false;                       \
+}
+#endif
+
 #define MCA_PML_CM_HVY_SEND_REQUEST_INIT( sendreq,                      \
                                           ompi_proc,                    \
                                           comm,                         \
@@ -144,7 +270,7 @@ do {                                                                    \
         sendreq->req_peer = dst;                                        \
         sendreq->req_addr = buf;                                        \
         sendreq->req_count = count;                                     \
-        MCA_PML_CM_SEND_REQUEST_INIT_COMMON( (&sendreq->req_send),      \
+        MCA_PML_CM_HVY_SEND_REQUEST_INIT_COMMON( (&sendreq->req_send),  \
                                              ompi_proc,                 \
                                              comm,                      \
                                              tag,                       \
@@ -313,7 +439,7 @@ do {
         OBJ_RELEASE(sendreq->req_send.req_base.req_comm);               \
         OMPI_REQUEST_FINI(&sendreq->req_send.req_base.req_ompi);        \
         opal_convertor_cleanup( &(sendreq->req_send.req_base.req_convertor) ); \
-        OMPI_FREE_LIST_RETURN( &mca_pml_base_send_requests,             \
+        OMPI_FREE_LIST_RETURN_MT( &mca_pml_base_send_requests,             \
                                (ompi_free_list_item_t*)sendreq);        \
     }
 
@@ -352,7 +478,7 @@ do {                                                                         \
         OBJ_RELEASE(sendreq->req_send.req_base.req_comm);               \
         OMPI_REQUEST_FINI(&sendreq->req_send.req_base.req_ompi);        \
         opal_convertor_cleanup( &(sendreq->req_send.req_base.req_convertor) ); \
-        OMPI_FREE_LIST_RETURN( &mca_pml_base_send_requests,             \
+        OMPI_FREE_LIST_RETURN_MT( &mca_pml_base_send_requests,             \
                                (ompi_free_list_item_t*)sendreq);        \
     }
 
