@@ -2,13 +2,14 @@
  * Copyright (c) 2004-2009 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2013 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2006 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2011      Sandia National Laboratories. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -45,6 +46,10 @@ mca_pml_crcpw_module_t mca_pml_crcpw_module = {
         mca_pml_crcpw_iprobe,
         mca_pml_crcpw_probe,
         mca_pml_crcpw_start,
+        mca_pml_crcpw_improbe,
+        mca_pml_crcpw_mprobe,
+        mca_pml_crcpw_imrecv,
+        mca_pml_crcpw_mrecv,
         mca_pml_crcpw_dump,
         mca_pml_crcpw_ft_event,
 
@@ -53,11 +58,11 @@ mca_pml_crcpw_module_t mca_pml_crcpw_module = {
     }
 };
 
-#define PML_CRCP_STATE_ALLOC(pml_state, rc)         \
+#define PML_CRCP_STATE_ALLOC(pml_state)             \
 do {                                                \
   if( !pml_crcpw_is_finalized ) {                   \
     ompi_free_list_item_t* item;                    \
-    OMPI_FREE_LIST_WAIT(&pml_state_list, item, rc); \
+    OMPI_FREE_LIST_WAIT_MT(&pml_state_list, item);     \
     pml_state = (ompi_crcp_base_pml_state_t*)item;  \
   }                                                 \
 } while(0); 
@@ -65,7 +70,7 @@ do {                                                \
 #define PML_CRCP_STATE_RETURN(pml_state)    \
 do {                                        \
   if( !pml_crcpw_is_finalized ) {           \
-    OMPI_FREE_LIST_RETURN(&pml_state_list,  \
+    OMPI_FREE_LIST_RETURN_MT(&pml_state_list,  \
     (ompi_free_list_item_t*)pml_state);     \
   }                                         \
 } while(0);
@@ -79,7 +84,7 @@ int mca_pml_crcpw_enable(bool enable)
         return mca_pml_crcpw_module.wrapped_pml_module.pml_enable(enable);
     }
 
-    PML_CRCP_STATE_ALLOC(pml_state, ret);
+    PML_CRCP_STATE_ALLOC(pml_state);
 
     pml_state->wrapped_pml_component = &(mca_pml_crcpw_module.wrapped_pml_component);
     pml_state->wrapped_pml_module    = &(mca_pml_crcpw_module.wrapped_pml_module);
@@ -121,7 +126,7 @@ int mca_pml_crcpw_add_comm(ompi_communicator_t* comm)
         return mca_pml_crcpw_module.wrapped_pml_module.pml_add_comm(comm);
     }
 
-    PML_CRCP_STATE_ALLOC(pml_state, ret);
+    PML_CRCP_STATE_ALLOC(pml_state);
 
     pml_state->wrapped_pml_component = &(mca_pml_crcpw_module.wrapped_pml_component);
     pml_state->wrapped_pml_module    = &(mca_pml_crcpw_module.wrapped_pml_module);
@@ -163,7 +168,7 @@ int mca_pml_crcpw_del_comm(ompi_communicator_t* comm)
         return mca_pml_crcpw_module.wrapped_pml_module.pml_del_comm(comm);
     }
 
-    PML_CRCP_STATE_ALLOC(pml_state, ret);
+    PML_CRCP_STATE_ALLOC(pml_state);
     if( NULL == pml_state ) {
         return mca_pml_crcpw_module.wrapped_pml_module.pml_del_comm(comm);
     }
@@ -204,7 +209,7 @@ int mca_pml_crcpw_add_procs(ompi_proc_t** procs, size_t nprocs)
     int ret;
     ompi_crcp_base_pml_state_t * pml_state = NULL;
 
-    PML_CRCP_STATE_ALLOC(pml_state, ret);
+    PML_CRCP_STATE_ALLOC(pml_state);
 
     pml_state->wrapped_pml_component = &(mca_pml_crcpw_module.wrapped_pml_component);
     pml_state->wrapped_pml_module    = &(mca_pml_crcpw_module.wrapped_pml_module);
@@ -242,7 +247,7 @@ int mca_pml_crcpw_del_procs(ompi_proc_t** procs, size_t nprocs)
     int ret;
     ompi_crcp_base_pml_state_t * pml_state = NULL;
 
-    PML_CRCP_STATE_ALLOC(pml_state, ret);
+    PML_CRCP_STATE_ALLOC(pml_state);
 
     pml_state->wrapped_pml_component = &(mca_pml_crcpw_module.wrapped_pml_component);
     pml_state->wrapped_pml_module    = &(mca_pml_crcpw_module.wrapped_pml_module);
@@ -280,7 +285,7 @@ int mca_pml_crcpw_iprobe(int dst, int tag, struct ompi_communicator_t* comm, int
     int ret;
     ompi_crcp_base_pml_state_t * pml_state = NULL;
 
-    PML_CRCP_STATE_ALLOC(pml_state, ret);
+    PML_CRCP_STATE_ALLOC(pml_state);
 
     pml_state->wrapped_pml_component = &(mca_pml_crcpw_module.wrapped_pml_component);
     pml_state->wrapped_pml_module    = &(mca_pml_crcpw_module.wrapped_pml_module);
@@ -323,7 +328,7 @@ int mca_pml_crcpw_probe( int dst, int tag, struct ompi_communicator_t* comm, omp
     int ret;
     ompi_crcp_base_pml_state_t * pml_state = NULL;
 
-    PML_CRCP_STATE_ALLOC(pml_state, ret);
+    PML_CRCP_STATE_ALLOC(pml_state);
 
     pml_state->wrapped_pml_component = &(mca_pml_crcpw_module.wrapped_pml_component);
     pml_state->wrapped_pml_module    = &(mca_pml_crcpw_module.wrapped_pml_module);
@@ -367,7 +372,7 @@ int mca_pml_crcpw_isend_init( void *buf, size_t count, ompi_datatype_t *datatype
     int ret;
     ompi_crcp_base_pml_state_t * pml_state = NULL;
 
-    PML_CRCP_STATE_ALLOC(pml_state, ret);
+    PML_CRCP_STATE_ALLOC(pml_state);
 
     pml_state->wrapped_pml_component = &(mca_pml_crcpw_module.wrapped_pml_component);
     pml_state->wrapped_pml_module    = &(mca_pml_crcpw_module.wrapped_pml_module);
@@ -406,7 +411,7 @@ int mca_pml_crcpw_isend( void *buf, size_t count, ompi_datatype_t *datatype, int
     int ret;
     ompi_crcp_base_pml_state_t * pml_state = NULL;
 
-    PML_CRCP_STATE_ALLOC(pml_state, ret);
+    PML_CRCP_STATE_ALLOC(pml_state);
 
     pml_state->wrapped_pml_component = &(mca_pml_crcpw_module.wrapped_pml_component);
     pml_state->wrapped_pml_module    = &(mca_pml_crcpw_module.wrapped_pml_module);
@@ -448,7 +453,7 @@ int mca_pml_crcpw_send(  void *buf, size_t count, ompi_datatype_t *datatype, int
     int ret;
     ompi_crcp_base_pml_state_t * pml_state = NULL;
 
-    PML_CRCP_STATE_ALLOC(pml_state, ret);
+    PML_CRCP_STATE_ALLOC(pml_state);
 
     pml_state->wrapped_pml_component = &(mca_pml_crcpw_module.wrapped_pml_component);
     pml_state->wrapped_pml_module    = &(mca_pml_crcpw_module.wrapped_pml_module);
@@ -490,7 +495,7 @@ int mca_pml_crcpw_irecv_init( void *buf, size_t count, ompi_datatype_t *datatype
     int ret;
     ompi_crcp_base_pml_state_t * pml_state = NULL;
 
-    PML_CRCP_STATE_ALLOC(pml_state, ret);
+    PML_CRCP_STATE_ALLOC(pml_state);
 
     pml_state->wrapped_pml_component = &(mca_pml_crcpw_module.wrapped_pml_component);
     pml_state->wrapped_pml_module    = &(mca_pml_crcpw_module.wrapped_pml_module);
@@ -529,7 +534,7 @@ int mca_pml_crcpw_irecv( void *buf, size_t count, ompi_datatype_t *datatype, int
     int ret;
     ompi_crcp_base_pml_state_t * pml_state = NULL;
 
-    PML_CRCP_STATE_ALLOC(pml_state, ret);
+    PML_CRCP_STATE_ALLOC(pml_state);
 
     pml_state->wrapped_pml_component = &(mca_pml_crcpw_module.wrapped_pml_component);
     pml_state->wrapped_pml_module    = &(mca_pml_crcpw_module.wrapped_pml_module);
@@ -574,7 +579,7 @@ int mca_pml_crcpw_recv(  void *buf, size_t count, ompi_datatype_t *datatype, int
     ompi_status_public_t* status = NULL;
     ompi_crcp_base_pml_state_t * pml_state = NULL;
 
-    PML_CRCP_STATE_ALLOC(pml_state, ret);
+    PML_CRCP_STATE_ALLOC(pml_state);
 
     pml_state->wrapped_pml_component = &(mca_pml_crcpw_module.wrapped_pml_component);
     pml_state->wrapped_pml_module    = &(mca_pml_crcpw_module.wrapped_pml_module);
@@ -631,7 +636,7 @@ int mca_pml_crcpw_dump( struct ompi_communicator_t* comm, int verbose )
     int ret;
     ompi_crcp_base_pml_state_t * pml_state = NULL;
 
-    PML_CRCP_STATE_ALLOC(pml_state, ret);
+    PML_CRCP_STATE_ALLOC(pml_state);
 
     pml_state->wrapped_pml_component = &(mca_pml_crcpw_module.wrapped_pml_component);
     pml_state->wrapped_pml_module    = &(mca_pml_crcpw_module.wrapped_pml_module);
@@ -673,7 +678,7 @@ int mca_pml_crcpw_progress(void)
         return mca_pml_crcpw_module.wrapped_pml_module.pml_progress();
     }
 
-    PML_CRCP_STATE_ALLOC(pml_state, ret);
+    PML_CRCP_STATE_ALLOC(pml_state);
 
     pml_state->wrapped_pml_component = &(mca_pml_crcpw_module.wrapped_pml_component);
     pml_state->wrapped_pml_module    = &(mca_pml_crcpw_module.wrapped_pml_module);
@@ -711,7 +716,7 @@ int mca_pml_crcpw_start( size_t count, ompi_request_t** requests )
     int ret;
     ompi_crcp_base_pml_state_t * pml_state = NULL;
 
-    PML_CRCP_STATE_ALLOC(pml_state, ret);
+    PML_CRCP_STATE_ALLOC(pml_state);
 
     pml_state->wrapped_pml_component = &(mca_pml_crcpw_module.wrapped_pml_component);
     pml_state->wrapped_pml_module    = &(mca_pml_crcpw_module.wrapped_pml_module);
@@ -749,12 +754,58 @@ int mca_pml_crcpw_start( size_t count, ompi_request_t** requests )
     return OMPI_SUCCESS;
 }
 
+
+int
+mca_pml_crcpw_improbe(int dst,
+                      int tag,
+                      struct ompi_communicator_t* comm,
+                      int *matched,
+                      struct ompi_message_t **message,
+                      ompi_status_public_t* status)
+{
+    return OMPI_ERR_NOT_SUPPORTED;
+}
+
+
+int
+mca_pml_crcpw_mprobe(int dst,
+                     int tag,
+                     struct ompi_communicator_t* comm,
+                     struct ompi_message_t **message,
+                     ompi_status_public_t* status)
+{
+    return OMPI_ERR_NOT_SUPPORTED;
+}
+
+
+int
+mca_pml_crcpw_imrecv(void *buf,
+                     size_t count,
+                     ompi_datatype_t *datatype,
+                     struct ompi_message_t **message,
+                     struct ompi_request_t **request)
+{
+    return OMPI_ERR_NOT_SUPPORTED;
+}
+
+
+int
+mca_pml_crcpw_mrecv(void *buf,
+                    size_t count,
+                    ompi_datatype_t *datatype,
+                    struct ompi_message_t **message,
+                    ompi_status_public_t* status)
+{
+    return OMPI_ERR_NOT_SUPPORTED;
+}
+
+
 int mca_pml_crcpw_ft_event(int state)
 {
     int ret;
     ompi_crcp_base_pml_state_t * pml_state = NULL;
 
-    PML_CRCP_STATE_ALLOC(pml_state, ret);
+    PML_CRCP_STATE_ALLOC(pml_state);
 
     pml_state->wrapped_pml_component = &(mca_pml_crcpw_module.wrapped_pml_component);
     pml_state->wrapped_pml_module    = &(mca_pml_crcpw_module.wrapped_pml_module);
