@@ -21,11 +21,8 @@
 
 #include "ompi_config.h"
 #include "ompi/constants.h"
-#include "orte/util/proc_info.h"
 #include "ompi/mca/pml/pml.h"
-#include "ompi/mca/btl/btl.h"
 #include "ompi/mca/bml/bml.h"
-#include "orte/types.h"
 #include "ompi/mca/mpool/mpool.h" 
 #include "pml_ob1.h"
 #include "pml_ob1_rdma.h"
@@ -61,10 +58,10 @@ size_t mca_pml_ob1_rdma_btls(
         mca_bml_base_btl_t* bml_btl =
             mca_bml_base_btl_array_get_index(&bml_endpoint->btl_rdma,
                     (bml_endpoint->btl_rdma_index + n) % num_btls); 
-        mca_mpool_base_registration_t* reg = NULL;
+        mca_mpool_base_registration_t* reg = &pml_ob1_dummy_reg;
         mca_mpool_base_module_t *btl_mpool = bml_btl->btl->btl_mpool;
 
-        if(NULL != btl_mpool) {
+        if( NULL != btl_mpool ) {
             if(!mca_pml_ob1.leave_pinned) {
                 /* look through existing registrations */
                 btl_mpool->mpool_find(btl_mpool, base, size, &reg);
@@ -74,18 +71,13 @@ size_t mca_pml_ob1_rdma_btls(
             }
 
             if(NULL == reg)
-                bml_btl = NULL; /* skip it */
-        } else {
-            /* if registration is not required use dummy registration */
-            reg = &pml_ob1_dummy_reg;
+                continue;
         }
 
-        if(bml_btl != NULL) {
-            rdma_btls[num_btls_used].bml_btl = bml_btl;
-            rdma_btls[num_btls_used].btl_reg = reg;
-            weight_total += bml_btl->btl_weight;
-            num_btls_used++;
-        }
+        rdma_btls[num_btls_used].bml_btl = bml_btl;
+        rdma_btls[num_btls_used].btl_reg = reg;
+        weight_total += bml_btl->btl_weight;
+        num_btls_used++;
     }
 
     /* if we don't use leave_pinned and all BTLs that already have this memory
@@ -95,7 +87,7 @@ size_t mca_pml_ob1_rdma_btls(
         return 0;
 
     mca_pml_ob1_calc_weighted_length(rdma_btls, num_btls_used, size,
-            weight_total);
+                                     weight_total);
 
     bml_endpoint->btl_rdma_index = (bml_endpoint->btl_rdma_index + 1) % num_btls;
     return num_btls_used;
