@@ -1,8 +1,9 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2009 The University of Tennessee and The University
+ * Copyright (c) 2004-2013 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2007 High Performance Computing Center Stuttgart, 
@@ -11,7 +12,9 @@
  *                         All rights reserved.
  * Copyright (c) 2008      UT-Battelle, LLC. All rights reserved.
  * Copyright (c) 2006-2008 University of Houston.  All rights reserved.
- * Copyright (c) 2009      Sun Microsystems, Inc.  All rights reserved.
+ * Copyright (c) 2009-2010 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 2012      Los Alamos National Security, LLC. All rights
+ *                         reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -28,18 +31,23 @@
 #include "opal/class/opal_list.h"
 #include "opal/threads/mutex.h"
 #include "opal/prefetch.h"
+
 #include "ompi/constants.h"
 #include "ompi/communicator/communicator.h"
 #include "ompi/mca/pml/pml.h"
+#include "ompi/peruse/peruse-internal.h"
+#include "ompi/memchecker.h"
+
 #include "pml_ob1.h"
 #include "pml_ob1_comm.h"
 #include "pml_ob1_recvfrag.h"
 #include "pml_ob1_recvreq.h"
 #include "pml_ob1_sendreq.h"
 #include "pml_ob1_hdr.h"
-#include "ompi/peruse/peruse-internal.h"
-#include "ompi/memchecker.h"
-
+#if OPAL_CUDA_SUPPORT
+#include "opal/datatype/opal_datatype_cuda.h"
+#include "ompi/mca/common/cuda/common_cuda.h"
+#endif /* OPAL_CUDA_SUPPORT */
 
 OBJ_CLASS_INSTANCE( mca_pml_ob1_buffer_t,
                     ompi_free_list_item_t,
@@ -65,10 +73,8 @@ append_frag_to_list(opal_list_t *queue, mca_btl_base_module_t *btl,
                     mca_pml_ob1_match_hdr_t *hdr, mca_btl_base_segment_t* segments,
                     size_t num_segments, mca_pml_ob1_recv_frag_t* frag)
 {
-    int rc;
-
     if(NULL == frag) {
-        MCA_PML_OB1_RECV_FRAG_ALLOC(frag, rc);
+        MCA_PML_OB1_RECV_FRAG_ALLOC(frag);
         MCA_PML_OB1_RECV_FRAG_INIT(frag, hdr, segments, num_segments, btl);
     }
     opal_list_append(queue, (opal_list_item_t*)frag);
@@ -101,7 +107,7 @@ void mca_pml_ob1_recv_frag_callback_match(mca_btl_base_module_t* btl,
                                           mca_btl_base_tag_t tag,
                                           mca_btl_base_descriptor_t* des,
                                           void* cbdata )
-{ 
+{
     mca_btl_base_segment_t* segments = des->des_dst;
     mca_pml_ob1_match_hdr_t* hdr = (mca_pml_ob1_match_hdr_t*)segments->seg_addr.pval;
     ompi_communicator_t *comm_ptr;
@@ -110,6 +116,8 @@ void mca_pml_ob1_recv_frag_callback_match(mca_btl_base_module_t* btl,
     mca_pml_ob1_comm_proc_t *proc;
     size_t num_segments = des->des_dst_cnt;
     size_t bytes_received = 0;
+
+    assert(num_segments <= MCA_BTL_DES_MAX_SEGMENTS);
     
     if( OPAL_UNLIKELY(segments->seg_len < OMPI_PML_OB1_MATCH_HDR_LEN) ) {
         return;
@@ -185,14 +193,14 @@ void mca_pml_ob1_recv_frag_callback_match(mca_btl_base_module_t* btl,
     
     /* release matching lock before processing fragment */
     OPAL_THREAD_UNLOCK(&comm->matching_lock);
-    
+
     if(OPAL_LIKELY(match)) {
         bytes_received = segments->seg_len - OMPI_PML_OB1_MATCH_HDR_LEN;
         match->req_recv.req_bytes_packed = bytes_received;
         
         MCA_PML_OB1_RECV_REQUEST_MATCHED(match, hdr);
         if(match->req_bytes_expected > 0) { 
-            struct iovec iov[2];
+            struct iovec iov[MCA_BTL_DES_MAX_SEGMENTS];
             uint32_t iov_count = 1;
             
             /*
@@ -247,7 +255,7 @@ void mca_pml_ob1_recv_frag_callback_rndv(mca_btl_base_module_t* btl,
                                          mca_btl_base_tag_t tag,
                                          mca_btl_base_descriptor_t* des,
                                          void* cbdata )
-{ 
+{
     mca_btl_base_segment_t* segments = des->des_dst;
     mca_pml_ob1_hdr_t* hdr = (mca_pml_ob1_hdr_t*)segments->seg_addr.pval;
     
@@ -264,7 +272,7 @@ void mca_pml_ob1_recv_frag_callback_rget(mca_btl_base_module_t* btl,
                                          mca_btl_base_tag_t tag,
                                          mca_btl_base_descriptor_t* des,
                                          void* cbdata )
-{ 
+{
     mca_btl_base_segment_t* segments = des->des_dst;
     mca_pml_ob1_hdr_t* hdr = (mca_pml_ob1_hdr_t*)segments->seg_addr.pval;
     
@@ -283,7 +291,7 @@ void mca_pml_ob1_recv_frag_callback_ack(mca_btl_base_module_t* btl,
                                         mca_btl_base_tag_t tag,
                                         mca_btl_base_descriptor_t* des,
                                         void* cbdata )
-{ 
+{
     mca_btl_base_segment_t* segments = des->des_dst;
     mca_pml_ob1_hdr_t* hdr = (mca_pml_ob1_hdr_t*)segments->seg_addr.pval;
     mca_pml_ob1_send_request_t* sendreq;
@@ -291,15 +299,22 @@ void mca_pml_ob1_recv_frag_callback_ack(mca_btl_base_module_t* btl,
     if( OPAL_UNLIKELY(segments->seg_len < sizeof(mca_pml_ob1_common_hdr_t)) ) {
          return;
     }
-    
+
     ob1_hdr_ntoh(hdr, MCA_PML_OB1_HDR_TYPE_ACK);
     sendreq = (mca_pml_ob1_send_request_t*)hdr->hdr_ack.hdr_src_req.pval;
     sendreq->req_recv = hdr->hdr_ack.hdr_dst_req;
-    
+
     /* if the request should be delivered entirely by copy in/out
      * then throttle sends */
-    if(hdr->hdr_common.hdr_flags & MCA_PML_OB1_HDR_FLAGS_NORDMA)
+    if(hdr->hdr_common.hdr_flags & MCA_PML_OB1_HDR_FLAGS_NORDMA) {
+        if (NULL != sendreq->src_des) {
+            /* release registered memory */
+            mca_bml_base_free (sendreq->req_rdma[0].bml_btl, sendreq->src_des);
+            sendreq->src_des = NULL;
+        }
+
         sendreq->req_throttle_sends = true;
+    }
     
     mca_pml_ob1_send_request_copy_in_out(sendreq,
                                          hdr->hdr_ack.hdr_send_offset,
@@ -319,35 +334,62 @@ void mca_pml_ob1_recv_frag_callback_ack(mca_btl_base_module_t* btl,
         OPAL_THREAD_ADD32(&sendreq->req_state, -1);
     }
 
+#if OPAL_CUDA_SUPPORT /* CUDA_ASYNC_SEND */
+    if ((sendreq->req_send.req_base.req_convertor.flags & CONVERTOR_CUDA) &&
+        (btl->btl_flags & MCA_BTL_FLAGS_CUDA_COPY_ASYNC_SEND)) {
+        /* The user's buffer is GPU and this BTL can support asynchronous copies,
+         * so adjust the convertor accordingly.  All the subsequent fragments will
+         * use the asynchronous copy. */
+        void *strm = mca_common_cuda_get_dtoh_stream();
+        opal_cuda_set_copy_function_async(&sendreq->req_send.req_base.req_convertor, strm);
+    }
+#endif /* OPAL_CUDA_SUPPORT */
+
     if(send_request_pml_complete_check(sendreq) == false)
         mca_pml_ob1_send_request_schedule(sendreq);
-    
+
     return;
 }
 
 void mca_pml_ob1_recv_frag_callback_frag(mca_btl_base_module_t* btl, 
                                          mca_btl_base_tag_t tag,
                                          mca_btl_base_descriptor_t* des,
-                                         void* cbdata ) { 
-     mca_btl_base_segment_t* segments = des->des_dst;
-     mca_pml_ob1_hdr_t* hdr = (mca_pml_ob1_hdr_t*)segments->seg_addr.pval;
-     mca_pml_ob1_recv_request_t* recvreq;
-     
-     if( OPAL_UNLIKELY(segments->seg_len < sizeof(mca_pml_ob1_common_hdr_t)) ) {
-         return;
-     }
-     ob1_hdr_ntoh(hdr, MCA_PML_OB1_HDR_TYPE_FRAG);
-     recvreq = (mca_pml_ob1_recv_request_t*)hdr->hdr_frag.hdr_dst_req.pval;
-     mca_pml_ob1_recv_request_progress_frag(recvreq,btl,segments,des->des_dst_cnt);
-     
-     return;
+                                         void* cbdata ) {
+    mca_btl_base_segment_t* segments = des->des_dst;
+    mca_pml_ob1_hdr_t* hdr = (mca_pml_ob1_hdr_t*)segments->seg_addr.pval;
+    mca_pml_ob1_recv_request_t* recvreq;
+
+    if( OPAL_UNLIKELY(segments->seg_len < sizeof(mca_pml_ob1_common_hdr_t)) ) {
+        return;
+    }
+    ob1_hdr_ntoh(hdr, MCA_PML_OB1_HDR_TYPE_FRAG);
+    recvreq = (mca_pml_ob1_recv_request_t*)hdr->hdr_frag.hdr_dst_req.pval;
+#if OPAL_CUDA_SUPPORT /* CUDA_ASYNC_RECV */
+    /* If data is destined for GPU buffer and convertor was set up for asynchronous
+     * copies, then start the copy and return.  The copy completion will trigger
+     * the next phase. */
+    if (recvreq->req_recv.req_base.req_convertor.flags & CONVERTOR_CUDA_ASYNC) {
+        assert(btl->btl_flags & MCA_BTL_FLAGS_CUDA_COPY_ASYNC_RECV);
+
+        /* This will trigger the opal_convertor_pack to start asynchronous copy. */
+        mca_pml_ob1_recv_request_frag_copy_start(recvreq,btl,segments,des->des_dst_cnt,des);
+        
+        /* Let BTL know that it CANNOT free the frag */
+        des->des_flags |= MCA_BTL_DES_FLAGS_CUDA_COPY_ASYNC;
+
+        return;
+    }
+#endif /* OPAL_CUDA_SUPPORT */
+    mca_pml_ob1_recv_request_progress_frag(recvreq,btl,segments,des->des_dst_cnt);
+
+    return;
 }
 
 
 void mca_pml_ob1_recv_frag_callback_put(mca_btl_base_module_t* btl, 
                                         mca_btl_base_tag_t tag,
                                         mca_btl_base_descriptor_t* des,
-                                        void* cbdata ) { 
+                                        void* cbdata ) {
     mca_btl_base_segment_t* segments = des->des_dst;
     mca_pml_ob1_hdr_t* hdr = (mca_pml_ob1_hdr_t*)segments->seg_addr.pval;
     mca_pml_ob1_send_request_t* sendreq;
@@ -367,7 +409,7 @@ void mca_pml_ob1_recv_frag_callback_put(mca_btl_base_module_t* btl,
 void mca_pml_ob1_recv_frag_callback_fin(mca_btl_base_module_t* btl, 
                                         mca_btl_base_tag_t tag,
                                         mca_btl_base_descriptor_t* des,
-                                        void* cbdata ) { 
+                                        void* cbdata ) {
     mca_btl_base_segment_t* segments = des->des_dst;
     mca_pml_ob1_hdr_t* hdr = (mca_pml_ob1_hdr_t*)segments->seg_addr.pval;
     mca_btl_base_descriptor_t* rdma;
@@ -479,6 +521,24 @@ match_one(mca_btl_base_module_t *btl,
                                                        num_segments);
                 /* attempt to match actual request */
                 continue;
+            } else if (MCA_PML_REQUEST_MPROBE == match->req_recv.req_base.req_type) {
+                /* create a receive frag and associate it with the
+                   request, which is then completed so that it can be
+                   restarted later during mrecv */
+                mca_pml_ob1_recv_frag_t *tmp;
+                if(NULL == frag) {
+                    MCA_PML_OB1_RECV_FRAG_ALLOC(tmp);
+                    MCA_PML_OB1_RECV_FRAG_INIT(tmp, hdr, segments, num_segments, btl);
+                } else {
+                    tmp = frag;
+                }
+
+                match->req_recv.req_base.req_addr = tmp;
+                mca_pml_ob1_recv_request_matched_probe(match, btl, segments,
+                                                       num_segments);
+                /* this frag is already processed, so we want to break out
+                   of the loop and not end up back on the unexpected queue. */
+                return NULL;
             }
 
             PERUSE_TRACE_COMM_EVENT(PERUSE_COMM_MSG_MATCH_POSTED_REQ,
