@@ -1,6 +1,7 @@
 /*
  * Copyright (c) 2004-2007 The Trustees of the University of Tennessee.
  *                         All rights reserved.
+ * Copyright (c) 2010      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -10,29 +11,26 @@
  
 #include "ompi_config.h"
 
-#include "opal/mca/base/mca_base_param.h"
 #include "opal/mca/base/base.h"
 #include "opal/mca/base/mca_base_component_repository.h"
 #include "ompi/constants.h"
 #include "ompi/mca/pml/base/base.h"
+#include "ompi/mca/vprotocol/vprotocol.h"
+#include "ompi/mca/vprotocol/base/base.h"
 #include "pml_v_output.h"
 #include "pml_v.h"
-#include "mca/vprotocol/vprotocol.h"
-#include "mca/vprotocol/base/base.h"
 
+static int mca_pml_v_component_register(void);
 static int mca_pml_v_component_open(void);
 static int mca_pml_v_component_close(void);
 static int mca_pml_v_component_parasite_close(void);
 
-static mca_pml_base_module_t *mca_pml_v_component_init(int* priority, bool enable_threads, bool enable_progress_threads);
+static mca_pml_base_module_t *mca_pml_v_component_init(int* priority, bool enable_progress_threads, bool enable_mpi_thread_multiple);
 static int mca_pml_v_component_finalize(void);
 static int mca_pml_v_component_parasite_finalize(void);
 
 static int mca_pml_v_enable(bool enable);
 
-static inline int mca_pml_v_param_register_int( const char* param_name, int default_value);
-static inline char *mca_pml_v_param_register_string( const char* param_name, char *default_value);
-
 mca_pml_base_component_2_0_0_t mca_pml_v_component = 
 {
   /* First, the mca_base_component_t struct containing meta
@@ -44,7 +42,9 @@ mca_pml_base_component_2_0_0_t mca_pml_v_component =
     OMPI_MINOR_VERSION,  /* MCA component minor version */
     OMPI_RELEASE_VERSION,  /* MCA component release version */
     mca_pml_v_component_open,
-    mca_pml_v_component_close
+    mca_pml_v_component_close,
+    NULL,
+    mca_pml_v_component_register
   },
   {
       MCA_BASE_METADATA_PARAM_NONE /* Component is not checkpointable */
@@ -54,57 +54,69 @@ mca_pml_base_component_2_0_0_t mca_pml_v_component =
   mca_pml_v_component_finalize   /* component finalize */
 };
 
-static bool pml_v_enable_progress_treads = OPAL_ENABLE_PROGRESS_THREADS;
-static bool pml_v_enable_mpi_threads = OMPI_ENABLE_THREAD_MULTIPLE;
+static bool pml_v_enable_progress_treads = OMPI_ENABLE_PROGRESS_THREADS;
+static bool pml_v_enable_mpi_thread_multiple = OMPI_ENABLE_THREAD_MULTIPLE;
+
+static char *ompi_pml_vprotocol_include_list;
+static char *ompi_pml_v_output;
+static int ompi_pml_v_verbose;
 
 /*******************************************************************************
  * MCA level functions - parasite setup
  */
+static int mca_pml_v_component_register(void)
+{
+    int var_id;
+
+    ompi_pml_v_output = "stderr";
+    (void) mca_base_component_var_register(&mca_pml_v_component.pmlm_version,
+                                           "output", NULL, MCA_BASE_VAR_TYPE_STRING,
+                                           NULL, 0, 0, OPAL_INFO_LVL_9,
+                                           MCA_BASE_VAR_SCOPE_READONLY,
+                                           &ompi_pml_v_output);
+
+    ompi_pml_v_verbose = 0;
+    (void) mca_base_component_var_register(&mca_pml_v_component.pmlm_version,
+                                           "verbose", "Verbosity of the pml v component",
+                                           MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
+                                           OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
+                                           &ompi_pml_v_verbose);
+
+    ompi_pml_vprotocol_include_list = "";
+    /* This parameter needs to go away if pml/v is unloaded so register it with a pml/v name */
+    var_id = mca_base_component_var_register(&mca_pml_v_component.pmlm_version,
+                                             "vprotocol", "Specify a specific vprotocol to use", 
+                                             MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
+                                             OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
+                                             &ompi_pml_vprotocol_include_list);
+    (void) mca_base_var_register_synonym(var_id, "ompi", "vprotocol", NULL, NULL, 0);
+
+    return OMPI_SUCCESS;
+}
+
 static int mca_pml_v_component_open(void)
 {
-    char *output;
-    int verbose;
-    int priority;
-    char *vprotocol_include_list;
-    
-    priority = mca_pml_v_param_register_int("priority", -1);
-    output = mca_pml_v_param_register_string("output", "stderr");
-    verbose = mca_pml_v_param_register_int("verbose", 0);
-
-    mca_base_param_reg_string_name("vprotocol", NULL, 
-                                   "Specify a specific vprotocol to use", 
-                                   false, false, "", &vprotocol_include_list);
-   
-    pml_v_output_open(output, verbose);
-
-    if(-1 != priority)
-        V_OUTPUT_ERR("pml_v: Overriding priority setting (%d) with -1. The PML V should NEVER be the selected component; even when enabling fault tolerance.", priority);
+    pml_v_output_open(ompi_pml_v_output, ompi_pml_v_verbose);
             
     V_OUTPUT_VERBOSE(500, "loaded");
 
-    return mca_vprotocol_base_open(vprotocol_include_list);
+    mca_vprotocol_base_set_include_list(ompi_pml_vprotocol_include_list);
+
+    return mca_base_framework_open(&ompi_vprotocol_base_framework, 0);
 }
  
 static int mca_pml_v_component_close(void)
 {
-    int ret;
-    
     /* Save original PML before making any changes  */
     mca_pml_v.host_pml_component = mca_pml_base_selected_component;
     mca_pml_v.host_pml = mca_pml;
     mca_pml_v.host_request_fns = ompi_request_functions;
     
     /* Do not load anything if no FT protocol is selected */
-    if(! mca_vprotocol_base_include_list[0])
+    if (NULL != mca_vprotocol_base_include_list && !mca_vprotocol_base_include_list[0]) {
         return mca_pml_v_component_parasite_close();
-        
-    V_OUTPUT_VERBOSE(500, "component_close: I don't want to be unloaded now.");
-    ret = mca_base_component_repository_retain_component("pml", "v");
-    if(OPAL_SUCCESS != ret)
-    {
-        V_OUTPUT_ERR("pml_v: component_close: can't retain myself. If Open MPI is build static you can ignore this error. Otherwise it should crash soon.");
     }
-    
+        
     /* Mark that we have changed something */ 
     snprintf(mca_pml_base_selected_component.pmlm_version.mca_component_name, 
              MCA_BASE_MAX_TYPE_NAME_LEN, "%s]v%s", 
@@ -135,7 +147,7 @@ static int mca_pml_v_component_parasite_finalize(void)
         mca_pml_v_component_parasite_close;
     cli = OBJ_NEW(mca_base_component_list_item_t);
     cli->cli_component = (mca_base_component_t *) &mca_pml_v_component;
-    opal_list_prepend(&mca_pml_base_components_available, 
+    opal_list_prepend(&ompi_pml_base_framework.framework_components,
                       (opal_list_item_t *) cli);
     
     /* finalize vprotocol component */
@@ -154,7 +166,7 @@ static int mca_pml_v_component_parasite_close(void)
                           mca_pml_v.host_pml_component.pmlm_version.mca_component_name);
     mca_pml_base_selected_component = mca_pml_v.host_pml_component;
 
-    mca_vprotocol_base_close();    
+    (void) mca_base_framework_close(&ompi_vprotocol_base_framework);
     pml_v_output_close();
 
     mca_pml.pml_enable = mca_pml_v.host_pml.pml_enable;
@@ -168,12 +180,12 @@ static int mca_pml_v_component_parasite_close(void)
  */
 static mca_pml_base_module_t *mca_pml_v_component_init(int *priority,
                                                       bool enable_progress_threads,
-                                                      bool enable_mpi_threads)
+                                                      bool enable_mpi_thread_multiple)
 {
     V_OUTPUT_VERBOSE(1, "init: I'm not supposed to be here until BTL loading stuff gets fixed!? That's strange...");
 
     pml_v_enable_progress_treads = enable_progress_threads;
-    pml_v_enable_mpi_threads = enable_mpi_threads;
+    pml_v_enable_mpi_thread_multiple = enable_mpi_thread_multiple;
     
     /* I NEVER want to be the selected PML, so I report less than possible 
      * priority and a NULL module 
@@ -209,7 +221,7 @@ static int mca_pml_v_enable(bool enable)
         /* Check if a protocol have been selected during init */
         if(! mca_vprotocol_base_selected())
             mca_vprotocol_base_select(pml_v_enable_progress_treads, 
-                                      pml_v_enable_mpi_threads);
+                                      pml_v_enable_mpi_thread_multiple);
 
         /* Check if we succeeded selecting a protocol */
         if(mca_vprotocol_base_selected()) {
@@ -235,25 +247,3 @@ static int mca_pml_v_enable(bool enable)
     ompi_request_functions = mca_pml_v.host_request_fns;
     return OMPI_SUCCESS;
 }
-
-
-/*******************************************************************************
- * utilities
- */
-static inline int mca_pml_v_param_register_int( const char* param_name,
-                                                  int default_value )
-{
-    int id = mca_base_param_register_int("pml", "v", param_name, NULL, default_value);
-    int param_value = default_value;
-    mca_base_param_lookup_int(id, &param_value);
-    return param_value;
-}
-
-static inline char *mca_pml_v_param_register_string( const char* param_name,
-                                                  char *default_value )
-{
-    int id = mca_base_param_register_string("pml", "v", param_name, NULL, default_value);
-    char *param_value = default_value;
-    mca_base_param_lookup_string(id, &param_value);
-    return param_value;
-}
