@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2013 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -30,10 +30,9 @@
  * @retval MPI_SUCCESS
  * @retval MPI_ERR_COMM
  */
-int mca_topo_base_cartdim_get (ompi_communicator_t* comm,
-                           int *ndims){
- 
-    *ndims = comm->c_topo_comm->mtc_ndims_or_nnodes;
+int mca_topo_base_cartdim_get(ompi_communicator_t* comm, int *ndims)
+{
+    *ndims = comm->c_topo->mtc.cart->ndims;
     return MPI_SUCCESS;
 }
 
