@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2012 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -33,27 +33,27 @@
  * @retval MPI_SUCCESS
  */                 
 
-int mca_topo_base_graph_get (ompi_communicator_t* comm,
-                         int maxindex,
-                         int maxedges,
-                         int *index,
-                         int *edges){
-    int i;
-    int *p;
+int mca_topo_base_graph_get(ompi_communicator_t* comm,
+                            int maxindex,
+                            int maxedges,
+                            int *index,
+                            int *edges)
+{
+    int i, *p;
     int nprocs = ompi_comm_size(comm);
 
     /*
      * Fill the nodes and edges arrays.
      */
-     p = comm->c_topo_comm->mtc_dims_or_index;
+     p = comm->c_topo->mtc.graph->index;
      for (i = 0; (i < nprocs) && (i < maxindex); ++i, ++p) {
          *index++ = *p;
       }
 
-      p = comm->c_topo_comm->mtc_periods_or_edges;
+      p = comm->c_topo->mtc.graph->edges;
 
       for (i = 0; 
-          (i < comm->c_topo_comm->mtc_dims_or_index[nprocs-1]) && (i < maxedges); 
+          (i < comm->c_topo->mtc.graph->index[nprocs-1]) && (i < maxedges); 
           ++i, ++p) {
           
          *edges++ = *p;
