@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -10,6 +11,10 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2007      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2012-2013 Los Alamos National Security, LLC.  All rights
+ *                         reserved. 
+ * Copyright (c) 2014      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -38,10 +43,10 @@
 static const char FUNC_NAME[] = "MPI_Alltoallw";
 
 
-int MPI_Alltoallw(void *sendbuf, int *sendcounts, int *sdispls, 
-                  MPI_Datatype *sendtypes,
-                  void *recvbuf, int *recvcounts, int *rdispls, 
-                  MPI_Datatype *recvtypes, MPI_Comm comm) 
+int MPI_Alltoallw(const void *sendbuf, const int sendcounts[],
+                  const int sdispls[], const MPI_Datatype sendtypes[],
+                  void *recvbuf, const int recvcounts[], const int rdispls[],
+                  const MPI_Datatype recvtypes[], MPI_Comm comm)
 {
     int i, size, err;   
     
@@ -49,21 +54,21 @@ int MPI_Alltoallw(void *sendbuf, int *sendcounts, int *sdispls,
         ptrdiff_t recv_ext;
         ptrdiff_t send_ext;
 
-        size = ompi_comm_remote_size(comm);
-
         memchecker_comm(comm);
-        for ( i = 0; i < size; i++ ) {
-            memchecker_datatype(sendtypes[i]);
-            memchecker_datatype(recvtypes[i]);
-            
-            ompi_datatype_type_extent(sendtypes[i], &send_ext);
-            ompi_datatype_type_extent(recvtypes[i], &recv_ext);
 
-            memchecker_call(&opal_memchecker_base_isdefined,
-                            (char *)(sendbuf)+sdispls[i]*send_ext,
-                            sendcounts[i], sendtypes[i]);
+        size = OMPI_COMM_IS_INTER(comm)?ompi_comm_remote_size(comm):ompi_comm_size(comm);
+        for ( i = 0; i < size; i++ ) {
+            if (MPI_IN_PLACE != sendbuf) {
+                memchecker_datatype(sendtypes[i]);
+                ompi_datatype_type_extent(sendtypes[i], &send_ext);
+                memchecker_call(&opal_memchecker_base_isdefined,
+                                (char *)(sendbuf)+sdispls[i]*send_ext,
+                                sendcounts[i], sendtypes[i]);
+            }
+            memchecker_datatype(recvtypes[i]);
+            ompi_datatype_type_extent(recvtypes[i], &recv_ext);
             memchecker_call(&opal_memchecker_base_isaddressable,
-                            (char *)(recvbuf)+sdispls[i]*recv_ext,
+                            (char *)(recvbuf)+rdispls[i]*recv_ext,
                             recvcounts[i], recvtypes[i]);
         }
     );
@@ -79,36 +84,43 @@ int MPI_Alltoallw(void *sendbuf, int *sendcounts, int *sdispls,
                                           FUNC_NAME);
         }
 
+        if (MPI_IN_PLACE == sendbuf) {
+            sendcounts = recvcounts;
+            sdispls    = rdispls;
+            sendtypes  = recvtypes;
+        }
+
         if ((NULL == sendcounts) || (NULL == sdispls) || (NULL == sendtypes) ||
             (NULL == recvcounts) || (NULL == rdispls) || (NULL == recvtypes) ||
-            MPI_IN_PLACE == sendbuf || MPI_IN_PLACE == recvbuf) {
+             MPI_IN_PLACE == recvbuf) {
             return OMPI_ERRHANDLER_INVOKE(comm, MPI_ERR_ARG, FUNC_NAME);
         }
 
-        /* We always define the remote group to be the same as the local
-           group in the case of an intracommunicator, so it's safe to
-           get the size of the remote group here for both intra- and
-           intercommunicators */
-
-        size = ompi_comm_remote_size(comm);
+        size = OMPI_COMM_IS_INTER(comm)?ompi_comm_remote_size(comm):ompi_comm_size(comm);
         for (i = 0; i < size; ++i) {
-            if (recvcounts[i] < 0) {
-                err = MPI_ERR_COUNT;
-            } else if (MPI_DATATYPE_NULL == recvtypes[i] || NULL == recvtypes[i]) {
-                err = MPI_ERR_TYPE;
-            } else {
-                OMPI_CHECK_DATATYPE_FOR_SEND(err, sendtypes[i], sendcounts[i]);
-            }
+            OMPI_CHECK_DATATYPE_FOR_SEND(err, sendtypes[i], sendcounts[i]);
             OMPI_ERRHANDLER_CHECK(err, comm, err, FUNC_NAME);
+            OMPI_CHECK_DATATYPE_FOR_RECV(err, recvtypes[i], recvcounts[i]);
+            OMPI_ERRHANDLER_CHECK(err, comm, err, FUNC_NAME);
+        }
+
+        if (MPI_IN_PLACE != sendbuf && !OMPI_COMM_IS_INTER(comm)) {
+            size_t sendtype_size, recvtype_size;
+            int me = ompi_comm_rank(comm);
+            ompi_datatype_type_size(sendtypes[me], &sendtype_size);
+            ompi_datatype_type_size(recvtypes[me], &recvtype_size);
+            if ((sendtype_size*sendcounts[me]) != (recvtype_size*recvcounts[me])) {
+                return OMPI_ERRHANDLER_INVOKE(comm, MPI_ERR_TRUNCATE, FUNC_NAME);
+            }
         }
     }
 
     OPAL_CR_ENTER_LIBRARY();
 
     /* Invoke the coll component to perform the back-end operation */
-
-    err = comm->c_coll.coll_alltoallw(sendbuf, sendcounts, sdispls, sendtypes, 
-                                      recvbuf, recvcounts, rdispls, recvtypes,
+    /* XXX -- CONST -- do not cast away const -- update mca/coll */
+    err = comm->c_coll.coll_alltoallw((void *) sendbuf, (int *) sendcounts, (int *) sdispls, (ompi_datatype_t **) sendtypes,
+                                      recvbuf, (int *) recvcounts, (int *) rdispls, (ompi_datatype_t **) recvtypes,
                                       comm, comm->c_coll.coll_alltoallw_module);
     OMPI_ERRHANDLER_RETURN(err, comm, err, FUNC_NAME);
 }
