@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
@@ -19,10 +19,13 @@
 #include <stdio.h>
 
 #include "ompi/mpi/c/bindings.h"
+#include "ompi/runtime/params.h"
+#include "ompi/communicator/communicator.h"
+#include "ompi/errhandler/errhandler.h"
 #include "ompi/mca/pml/pml.h"
 #include "ompi/mca/pml/base/pml_base_bsend.h"
 
-#if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
+#if OPAL_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
 #pragma weak MPI_Buffer_detach = PMPI_Buffer_detach
 #endif
 
@@ -35,6 +38,8 @@ static const char FUNC_NAME[] = "MPI_Buffer_detach";
 
 int MPI_Buffer_detach(void *buffer, int *size) 
 {
+    int ret = OMPI_SUCCESS;
+
   if (MPI_PARAM_CHECK) {
     OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
     if (NULL == buffer || NULL == size) {
@@ -42,5 +47,9 @@ int MPI_Buffer_detach(void *buffer, int *size)
     }
   }
 
-  return mca_pml_base_bsend_detach(buffer, size);
+  OPAL_CR_ENTER_LIBRARY();
+  ret = mca_pml_base_bsend_detach(buffer, size);
+
+  OPAL_CR_EXIT_LIBRARY();
+  return ret;
 }
