@@ -9,10 +9,12 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2012      Los Alamos National Security, LLC.  All rights
+ *                         reserved. 
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -42,8 +44,8 @@ static int getprimes(int num, int *pnprime, int **pprimes);
 /*
  * This is a utility function, no need to have anything in the lower
  * layer for this at all
- */ 
-int MPI_Dims_create(int nnodes, int ndims, int *dims) 
+ */
+int MPI_Dims_create(int nnodes, int ndims, int dims[])
 {
     int i;
     int freeprocs;
@@ -64,11 +66,16 @@ int MPI_Dims_create(int nnodes, int ndims, int *dims)
             return OMPI_ERRHANDLER_INVOKE (MPI_COMM_WORLD,
                                            MPI_ERR_ARG, FUNC_NAME);
         }
-        
+
         if (1 > ndims) {
             return OMPI_ERRHANDLER_INVOKE (MPI_COMM_WORLD, 
                                            MPI_ERR_DIMS, FUNC_NAME);
         }
+
+        if (1 > nnodes) {
+            return OMPI_ERRHANDLER_INVOKE (MPI_COMM_WORLD,
+                                           MPI_ERR_DIMS, FUNC_NAME);
+        }
     }
 
     /* Get # of free-to-be-assigned processes and # of free dimensions */
@@ -93,11 +100,7 @@ int MPI_Dims_create(int nnodes, int ndims, int *dims)
                                      FUNC_NAME);
     }
 
-    if (freeprocs < 1) {
-       return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_DIMS,
-                                     FUNC_NAME);
-    }
-    else if (freeprocs == 1) {
+    if (freeprocs == 1) {
         for (i = 0; i < ndims; ++i, ++dims) {
             if (*dims == 0) {
                *dims = 1;
@@ -111,7 +114,7 @@ int MPI_Dims_create(int nnodes, int ndims, int *dims)
        return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, err,
                                      FUNC_NAME);
     }
-    
+
     /* Factor the number of free processes */
     if (MPI_SUCCESS != (err = getfactors(freeprocs, nprimes, primes, &factors))) {
        return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, err,
@@ -307,4 +310,3 @@ getprimes(int num, int *pnprime, int **pprimes) {
    *pnprime = i;
    return MPI_SUCCESS;
 }
-
