@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -10,6 +11,8 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
+ * Copyright (c) 2013      Los Alamos National Security, LLC.  All rights
+ *                         reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -20,13 +23,15 @@
 #include "ompi_config.h"
 
 #include "ompi/mpi/c/bindings.h"
-#include "ompi/datatype/datatype.h"
+#include "ompi/runtime/params.h"
+#include "ompi/errhandler/errhandler.h"
+#include "ompi/datatype/ompi_datatype.h"
 #include "ompi/file/file.h"
 #include "ompi/mca/io/io.h"
 #include "ompi/mca/io/base/io_base_request.h"
 #include "ompi/memchecker.h"
 
-#if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
+#if OPAL_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
 #pragma weak MPI_File_iwrite_at = PMPI_File_iwrite_at
 #endif
 
@@ -37,12 +42,11 @@
 static const char FUNC_NAME[] = "MPI_File_iwrite_at";
 
 
-int MPI_File_iwrite_at(MPI_File fh, MPI_Offset offset, void *buf,
+int MPI_File_iwrite_at(MPI_File fh, MPI_Offset offset, const void *buf,
                        int count, MPI_Datatype datatype, 
                        MPI_Request *request)
 {
     int rc;
-    mca_io_base_request_t *io_request;
 
     MEMCHECKER(
         memchecker_datatype(datatype);
@@ -57,6 +61,8 @@ int MPI_File_iwrite_at(MPI_File fh, MPI_Offset offset, void *buf,
             rc = MPI_ERR_FILE;
         } else if (count < 0) {
             rc = MPI_ERR_COUNT;
+        } else if (NULL == request) {
+            rc = MPI_ERR_REQUEST;
         } else {
            OMPI_CHECK_DATATYPE_FOR_SEND(rc, datatype, count);
         }
@@ -65,21 +71,13 @@ int MPI_File_iwrite_at(MPI_File fh, MPI_Offset offset, void *buf,
 
     OPAL_CR_ENTER_LIBRARY();
 
-    /* Get a request */
-
-    if (OMPI_SUCCESS != mca_io_base_request_alloc(fh, &io_request)) {
-        OPAL_CR_EXIT_LIBRARY();
-        return OMPI_ERRHANDLER_INVOKE(fh, MPI_ERR_NO_MEM, FUNC_NAME);
-    }
-    *request = (ompi_request_t*) io_request;
-
     /* Call the back-end io component function */
-
     switch (fh->f_io_version) {
     case MCA_IO_BASE_V_2_0_0:
+        /* XXX -- CONST -- do not cast away const -- update mca/io */
         rc = fh->f_io_selected_module.v2_0_0.
-            io_module_file_iwrite_at(fh, offset, buf, count, datatype, 
-                                     io_request);
+            io_module_file_iwrite_at(fh, offset, (void *) buf, count, datatype,
+                                     request);
         break;
 
     default:
