@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -10,6 +11,8 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
+ * Copyright (c) 2013      Los Alamos National Security, LLC.  All rights
+ *                         reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -20,11 +23,13 @@
 #include "ompi_config.h"
 
 #include "ompi/mpi/c/bindings.h"
+#include "ompi/runtime/params.h"
+#include "ompi/errhandler/errhandler.h"
 #include "ompi/file/file.h"
-#include "ompi/datatype/datatype.h"
+#include "ompi/datatype/ompi_datatype.h"
 #include "ompi/memchecker.h"
 
-#if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
+#if OPAL_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
 #pragma weak MPI_File_write_at = PMPI_File_write_at
 #endif
 
@@ -35,8 +40,8 @@
 static const char FUNC_NAME[] = "MPI_File_write_at";
 
 
-int MPI_File_write_at(MPI_File fh, MPI_Offset offset, void *buf,
-                      int count, MPI_Datatype datatype, 
+int MPI_File_write_at(MPI_File fh, MPI_Offset offset, const void *buf,
+                      int count, MPI_Datatype datatype,
                       MPI_Status *status)
 {
     int rc;
@@ -66,8 +71,9 @@ int MPI_File_write_at(MPI_File fh, MPI_Offset offset, void *buf,
 
     switch (fh->f_io_version) {
     case MCA_IO_BASE_V_2_0_0:
+        /* XXX -- CONST -- do not cast away const -- update mca/io */
         rc = fh->f_io_selected_module.v2_0_0.
-            io_module_file_write_at(fh, offset, buf, count, datatype, status);
+          io_module_file_write_at(fh, offset, (void *) buf, count, datatype, status);
         break;
 
     default:
