@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2012 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
@@ -40,7 +40,6 @@ static const char FUNC_NAME[] = "MPI_Graph_neighbors_count";
 int MPI_Graph_neighbors_count(MPI_Comm comm, int rank, int *nneighbors) 
 {
     int err;
-    mca_topo_base_module_graph_neighbors_count_fn_t func;
 
     MEMCHECKER(
         memchecker_comm(comm);
@@ -57,10 +56,6 @@ int MPI_Graph_neighbors_count(MPI_Comm comm, int rank, int *nneighbors)
             return OMPI_ERRHANDLER_INVOKE (comm, MPI_ERR_COMM,
                                            FUNC_NAME);
         }
-        if (!OMPI_COMM_IS_GRAPH(comm)) {
-            return OMPI_ERRHANDLER_INVOKE (comm, MPI_ERR_TOPOLOGY,
-                                           FUNC_NAME);
-        }
         if ((0 > rank) || (rank > ompi_group_size(comm->c_local_group))) {
             return OMPI_ERRHANDLER_INVOKE (comm, MPI_ERR_RANK,
                                            FUNC_NAME);
@@ -71,18 +66,14 @@ int MPI_Graph_neighbors_count(MPI_Comm comm, int rank, int *nneighbors)
         }
     }
 
+    if (!OMPI_COMM_IS_GRAPH(comm)) {
+        return OMPI_ERRHANDLER_INVOKE (comm, MPI_ERR_TOPOLOGY,
+                                       FUNC_NAME);
+    }
     OPAL_CR_ENTER_LIBRARY();
 
-    /* get the function pointer to do the right thing */
-    func = comm->c_topo->topo_graph_neighbors_count;
-
-    /* call the function */
-    err = func(comm, rank, nneighbors);
+    err = comm->c_topo->topo.graph.graph_neighbors_count(comm, rank, nneighbors);
     OPAL_CR_EXIT_LIBRARY();
-    if ( MPI_SUCCESS != err ) {
-        return OMPI_ERRHANDLER_INVOKE(comm, err, FUNC_NAME);
-    }
-    
-    /* All done */
-    return MPI_SUCCESS;
+
+    OMPI_ERRHANDLER_RETURN(err, comm, err, FUNC_NAME);
 }
