@@ -21,12 +21,12 @@
 #include <stdio.h>
 
 #include "ompi/mpi/c/bindings.h"
+#include "ompi/runtime/params.h"
 #include "ompi/group/group.h"
 #include "ompi/errhandler/errhandler.h"
 #include "ompi/communicator/communicator.h"
-#include "ompi/proc/proc.h"
 
-#if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
+#if OPAL_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
 #pragma weak MPI_Group_intersection = PMPI_Group_intersection
 #endif
 
