@@ -22,7 +22,7 @@
 
 #include <stdlib.h>
 
-#include "orte/util/show_help.h"
+#include "opal/util/show_help.h"
 #include "ompi/mpi/c/bindings.h"
 #include "ompi/communicator/communicator.h"
 #include "ompi/errhandler/errhandler.h"
@@ -50,7 +50,7 @@ int MPI_Init(int *argc, char ***argv)
 
     if (ompi_mpi_finalized) {
         if (0 == ompi_comm_rank(MPI_COMM_WORLD)) {
-            orte_show_help("help-mpi-api.txt",
+            opal_show_help("help-mpi-api.txt",
                            "mpi-function-after-finalize", true, FUNC_NAME);
         }
         return ompi_errhandler_invoke(NULL, NULL,
@@ -58,7 +58,7 @@ int MPI_Init(int *argc, char ***argv)
                                       MPI_ERR_OTHER, FUNC_NAME);
     } else if (ompi_mpi_initialized) {
         if (0 == ompi_comm_rank(MPI_COMM_WORLD)) {
-            orte_show_help("help-mpi-api.txt", "mpi-initialize-twice",
+            opal_show_help("help-mpi-api.txt", "mpi-initialize-twice",
                            true, FUNC_NAME);
         }
         return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_OTHER,
