@@ -49,11 +49,7 @@ int MPI_Is_thread_main(int *flag)
 
     /* Compare this thread ID to the main thread ID */
 
-#if OMPI_ENABLE_THREAD_MULTIPLE
     *flag = (int) opal_thread_self_compare(ompi_mpi_main_thread);
-#else
-    *flag = 1;
-#endif
 
     return MPI_SUCCESS;
 }
