@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -5,14 +6,16 @@
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2013      Los Alamos National Security, LLC.  All rights
+ *                         reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -37,8 +40,8 @@
 static const char FUNC_NAME[] = "MPI_Issend";
 
 
-int MPI_Issend(void *buf, int count, MPI_Datatype type, int dest,
-               int tag, MPI_Comm comm, MPI_Request *request) 
+int MPI_Issend(const void *buf, int count, MPI_Datatype type, int dest,
+               int tag, MPI_Comm comm, MPI_Request *request)
 {
     int rc = MPI_SUCCESS;
 
@@ -67,7 +70,7 @@ int MPI_Issend(void *buf, int count, MPI_Datatype type, int dest,
         }
         OMPI_ERRHANDLER_CHECK(rc, comm, rc, FUNC_NAME);
     }
-    
+
     if (MPI_PROC_NULL == dest) {
         *request = &ompi_request_empty;
         return MPI_SUCCESS;
@@ -78,8 +81,9 @@ int MPI_Issend(void *buf, int count, MPI_Datatype type, int dest,
     MEMCHECKER (
         memchecker_call(&opal_memchecker_base_mem_noaccess, buf, count, type);
     );
-    rc = MCA_PML_CALL(isend(buf,count,type,dest,tag,
-                            MCA_PML_BASE_SEND_SYNCHRONOUS,comm,request));
+    /* XXX -- CONST -- do not cast away const -- update mca/pml */
+    rc = MCA_PML_CALL(isend((void *) buf, count, type, dest, tag,
+                            MCA_PML_BASE_SEND_SYNCHRONOUS, comm, request));
     OMPI_ERRHANDLER_RETURN(rc, comm, rc, FUNC_NAME);
 }
 
